/* ====================================================== */
/*                       XCFGFILE.H                       */
/*  Definiert das Klasseninterface fr die Klasse CfgFile */
/*          (C) 1993 Thomas Wlfer & toolbox              */
/*         Compiler: Microsoft Visual C++ 1.0             */
/* ====================================================== */

#ifndef _XCFGFILE_H
#define _XCFGFILE_H

/* ====================================================== */
/* XCfgFile: Eine komplette Datei mit allen fhrenden     */
/*           Kommentaren und allen Tags sowie ihren       */
/*           Eintrgen und Kommentaren. Die Tags werden   */
/*           dabei als Hashtabellen fr CStringlist       */
/*           verwendet. Da die Hashtabelle casesensitiv   */
/*           ist, mssen die Tags zweimal abgelegt werden:*/
/*           1. in der Hashtabelle und 2. in einer        */
/*           CStringlist, damit beim Zurckschreiben die  */
/*           selbe Reihenfolge wie beim Lesen verwendet   */
/*           wird.                                        */
/*                                                        */
/* Member:                                                */
/*   m_pLastTag:                                          */
/*       Zeiger auf den letzen verwendeten Tag; wird beim */
/*       Einlesen verwendet.                              */
/*  m_lstLeadingComments:                                 */
/*       Kommentare die vor dem 1. Tag stehen und damit   */
/*       damit keinem zugeordnet werden knnen.           */
/*  m_lstTags:                                            */
/*       Liste aller Tags in der Reihenfolge und der      */
/*       Schreibweise, wie sie im Ursprungsfile standen.  */
/*  m_fIsDirty:                                           */
/*       Wird bei allen Write..()-Zugriffen gesetzt und   */
/*       sorgt dafr, das die Datei am Ende nur dann      */
/*       zurckgeschrieben wird, wenn es ntig ist.       */
/*  IsTag():                                              */
/*       berprft, ob eine ankommende Zeile ein Tag ist  */
/*  AddTag():                                             */
/*       Hinzufgen eines Tags.                           */
/*                                                        */
/* public:                                                */
/*  XCfgFile()   : Konstruktor                            */
/*  ~XCfgFile()  : Destruktor                             */
/*  Flush()      : Schreibt das File zurck auf die Platte*/
/*  Fill()       : Einlesen des Files                     */
/*  WriteString(): Schreiben eines Eintrags in das File   */
/*  GetString()  : Lesen eines Eintrags aus dem File      */
/* ====================================================== */

class XTag;    // Forward-Deklaration fr die Verwendung in
               // XCfgFile

class XCfgFile : public CMapStringToOb
{
  XTag*       m_pLastTag;
  CStringList m_lstLeadingComments;
  CStringList m_lstTags;
  BOOL        m_fIsDirty;
  CString     m_strFileName;
  BOOL        IsTag(  const CString& szLine,
                      CString& szTag)        const;
  XTag*       AddTag( const char* szTag );
  BOOL        GetTag( CString& rszTag,
                      BOOL fIsFirst = FALSE) const;
public:
   XCfgFile(          const char * szFile);
   BOOL GetDirty()    const { return m_fIsDirty; }
   BOOL GetString(    const char* szSection,
                      const char* szKey,
                      const char* szDefault,
                      CString& szValue);
   void WriteString(  const char* szSection,
                      const char* szKey,
                      const char* szValue);
   void Fill(         const char* szOneLine);
   void Flush()       const;
   BOOL GetFirstTag(  CString& rszTag)       const;
   BOOL GetNextTag(   CString& rszTag)       const;
   BOOL GetFirstEntry(const char* pszTag,
                      CString& rszEntry)     const;
   BOOL GetNextEntry( const char* pszTag,
                      CString& rszEntry)     const;
   ~XCfgFile();
#ifdef _DEBUG
   void AssertValid() const
   {
     CMapStringToOb::AssertValid();
   }
#endif
};

#endif                                        // _XCFGFILE_H

/* ====================================================== */