(* ====================================================== *)
(*                     DBENGINE.PAS Version 1.1           *)
(*  OOP-Funktionen fr den Zugriff auf dBase-Datenbanken  *)
(*     Compiler: Borland Pascal 7.0 (Windows Target)      *)
(*          1993, 94 Peter Fliesges & DMV-Verlag        *)
(* ====================================================== *)
{$A+,B-,D-,F-,G+,I-,K-,L-,N-,P-,Q-,R-,S-,T-,V-,W+,X+,Y-}

UNIT dBEngine;
  
INTERFACE

USES Objects, WinTypes, WinProcs, Strings, WinDos;

TYPE
  tDate = RECORD
    Year, Month, Day : BYTE;
  END;

  tHeader = RECORD
    FirstByte   : BYTE;
    Date        : tDate;
    NumRec      : LongInt;
    HeaderSize  : WORD;
    DataSize    : WORD;
    Dummy1      : WORD;
    TransAction : BYTE;
    Cryption    : BYTE;
    Dummy2      : ARRAY [1..12] OF BYTE;
    MDX         : BYTE;
    Dummy3      : ARRAY [1..3] OF BYTE;
  END;

  tFieldDescr = RECORD
    FldName     : ARRAY [1..11] OF CHAR;
    Typ         : CHAR;
    Addr        : POINTER;
    Size        : BYTE;
    Dec         : BYTE;
    Dummy1      : WORD;
    WArea       : BYTE;
    Dummy2      : ARRAY [1..11] OF BYTE;
  END;

  tBuf4001 = ARRAY [0..4000] OF CHAR;

  pDBObject = ^tDBObject;
  tDBObject = OBJECT
    DBFName     : ARRAY [0..fsPathName] OF CHAR;
    f           : File;
    DBFError    : BOOLEAN;
    IsDirty     : BOOLEAN;
    Header      : tHeader;
    NumFields   : BYTE;
    Fields      : ARRAY [1..255] OF ^tFieldDescr;
    Buf4001     : ^tBuf4001;
    bFound      : BOOLEAN;

    PROCEDURE Open(FName : pChar);
    PROCEDURE Close;
    PROCEDURE Append;
    PROCEDURE Insert;
    PROCEDURE Delete;
    FUNCTION  Deleted : BOOLEAN;
    PROCEDURE Recall;
    PROCEDURE Pack;
    PROCEDURE Zap;
    PROCEDURE Skip;
    PROCEDURE Go(n : LongInt);
    PROCEDURE GoTop;
    PROCEDURE GoBottom;
    FUNCTION  BOF           : BOOLEAN;
    FUNCTION  EoF           : BOOLEAN;
    FUNCTION  Locate(n      : BYTE; s   : pChar) : BOOLEAN;
    FUNCTION  Found         : BOOLEAN;
    PROCEDURE PutField(n    : BYTE; s   : pChar);
    FUNCTION  GetField(n    : BYTE; Buf : pChar) : pChar;
    FUNCTION  Trim(aBuf     : pChar)    : pChar;
    FUNCTION  DBF           : pChar;
    FUNCTION  Version       : BYTE;
    FUNCTION  WithMemo      : BOOLEAN;
    FUNCTION  LUpDate(Buf10 : pChar)    : pChar;
    FUNCTION  RecCount      : LongInt;
    FUNCTION  RecNo         : LongInt;
    FUNCTION  RecSize       : INTEGER;
    FUNCTION  Field(n       : BYTE)     : pChar;
    FUNCTION  FieldType(n   : BYTE)     : CHAR;
    FUNCTION  FieldLength(n : BYTE)     : BYTE;
    FUNCTION  FldCount      : BYTE;
    FUNCTION  FileExists(FName : pChar) : BOOLEAN;
  END;

IMPLEMENTATION

PROCEDURE tDBObject.Open(FName : pChar);
VAR
  i, Idx : INTEGER;
BEGIN
  (* FName in Objektfeld bernehmen und Datei ffnen *)
  StrCopy(DBFName, StrUpper(FName));
  Assign(f, FName);
  {$I-}
  Reset(f, 1);
  {$I+}
  DBFError := (IOResult <> 0);
  IF DBFError THEN Exit;

  (* Header einlesen *)
  {$I-};
  BlockRead(f, Header, SizeOf(Header));
  {$I+};
  DBFError := (IOResult <> 0);

  (* Dateityp prfen *)
  WITH Header DO
    IF ((HeaderSize + (NumRec * DataSize) + 1)
         < (FileSize(f) - 1)) OR
       ((HeaderSize + (NumRec * DataSize) + 1)
         > (FileSize(f) + 1)) THEN BEGIN
      DBFError := TRUE;
      Exit;
    END;

  (* Platz fr Rohdatensatz schaffen *)
  GetMem(Buf4001, Header.DataSize);

  (* Anzahl Felder berechnen *)
  NumFields := (Header.HeaderSize - 33) DIV 32;

  (* Felder einlesen *)
  Idx := 1;
  (* Index fr Buf4001;                 *)
  (* Start bei 1 wegen des Deleted-Byte *)
  FOR i := 1 TO NumFields DO BEGIN
    New(Fields[i]);
    {$I-}
    BlockRead(f, Fields[i]^, SizeOf(Fields[i]^));
    {$I+}
    DBFError := (IOResult <> 0);
    Fields[i]^.Addr := Addr(Buf4001^[Idx]);
    Inc(Idx, Fields[i]^.Size);
  END;

  (* 1. Satz in Rohdatenpuffer einlesen *)
  IF RecCount > 0 THEN BEGIN
    Seek(f, FilePos(f) + 1);  (* $0D berspringen *)
    {$I-}
    BlockRead(f, Buf4001^, Header.DataSize);
    {$I+}
    DBFError := (IOResult <> 0);
    OEMToAnsi(Buf4001^, Buf4001^);
  END;
  bFound  := FALSE;
  IsDirty := FALSE;
END;

PROCEDURE tDBObject.Close;
VAR
  i              : BYTE;
  yy, mm, dd, dw : WORD;
BEGIN
  (* gegebenenfalls Datum der letzten Aktualisierung neu  *)
  (* setzen                                               *)
  IF IsDirty THEN BEGIN
    GetDate(yy, mm, dd, dw);
    Header.Date.Year  := BYTE(yy - 1900);
    Header.Date.Month := BYTE(mm);
    Header.Date.Day   := BYTE(dd);
    Seek(f, 0);
    BlockWrite(f, Header, SizeOf(Header));
  END;
  {$I-}
  System.Close(f);                     (* Datei schlieen *)
  {$I+}
  DBFError := (IOResult <> 0);
  (* Reservierten Heap wieder freigeben *)
  FOR i := 1 TO NumFields DO
    Dispose(Fields[i]);
  FreeMem(Buf4001, Header.DataSize);
END;

PROCEDURE tDBObject.Append;
VAR
  Buf : CHAR;
BEGIN
  (* Puffer mit Leerzeichen fllen; Entfernen von         *)
  (* FillChar ergibt CARRY ON                             *)
  FillChar(Buf4001^, Header.DataSize, Chr($20));
  (* Hier gegebenenfalls neue Daten in Puffer schreiben   *)
  Seek(f, FileSize(f) - 1);
  BlockWrite(f, Buf4001^, Header.DataSize);
  Buf := Chr($1A);
  {$I-}
  BlockWrite(f, Buf, 1);
  {$I+}
  DBFError := (IOResult <> 0);
  Inc(Header.NumRec);
  IsDirty := TRUE;
END;

PROCEDURE tDBObject.Insert;
VAR
  i, ActRec : LongInt;
  Buf       : CHAR;
BEGIN
  ActRec := RecNo; (* Aktuellen Satz merken               *)
  Go(RecCount);    (* Zum letzten Satz gehen und einlesen *)
  AnsiToOem(Buf4001^, Buf4001^);
  BlockWrite(f, Buf4001^, Header.DataSize);
                   (* --> letzter Satz jetzt zweimal da   *)
  Buf := Chr($1A); (* ^Z = Dateiende                      *)
  BlockWrite(f, Buf, 1);  (* Endekennzeichen schreiben    *)

  (* Vom vor-vorletzten Satz aus (NumRec noch nicht       *)
  (* erhht!) je einen Satz lesen und an der aktuellen    *)
  (* Position (also hinter dem gelesenen Satz) wieder     *)
  (* schreiben, so da vorbergehend die Stze doppelt    *)
  (* vorhanden sind, bis zur in ActRec gespeicherten      *)
  (* aktuellen Position.                                  *)
  FOR i := RecCount - 1 DOWNTO ActRec DO BEGIN
    Go(i);
    AnsiToOem(Buf4001^, Buf4001^);
    BlockWrite(f, Buf4001^, Header.DataSize);
  END;
  (* Jetzt stehen an der aktuellen Position zwei gleiche  *)
  (* Stze, von denen einer  berschrieben wird: an       *)
  (* Position ActRec --> Einfgen HINTER = INSERT oder    *)
  (* an Position (ActRec - 1) --> Einfgen VOR dem aktu-  *)
  (* ellen Satz = INSERT BEFORE                           *)
  Seek(f, (ActRec (* -1 *) ) *
          Header.DataSize + Header.HeaderSize);
  (* Mit Leerzeichen fllen = INSERT BLANK oder Buf4001   *)
  (* mit Daten versorgen. Ein Weglassen von FillChar      *)
  (* entspricht CARRY ON                                  *)
  FillChar(Buf4001^, Header.DataSize, Chr($20));
  {$I-}
  BlockWrite(f, Buf4001^, Header.DataSize);
  {$I+}
  DBFError := (IOResult <> 0);
  Inc(Header.NumRec);
  IsDirty  := TRUE;
END;

PROCEDURE tDBObject.Delete;
BEGIN
  IF Buf4001^[0] <> Chr($2A) THEN BEGIN
    Buf4001^[0] := Chr($2A);
    AnsiToOem(Buf4001^, Buf4001^);
    Seek(f, FilePos(f) - Header.DataSize);
    {$I-}
    BlockWrite(f, Buf4001^, Header.DataSize);
    {$I+}
    DBFError := (IOResult <> 0);
    IsDirty  := TRUE;
  END ELSE
    DBFError := FALSE;
END;

FUNCTION tDBObject.Deleted : BOOLEAN;
BEGIN
  Deleted := (Buf4001^[0] = Chr($2A));
END;

PROCEDURE tDBObject.Recall;
BEGIN
  IF Buf4001^[0] <> Chr($20) THEN BEGIN
    Buf4001^[0] := Chr($20);
    AnsiToOem(Buf4001^, Buf4001^);
    Seek(f, FilePos(f) - Header.DataSize);
    {$I-}
    BlockWrite(f, Buf4001^, Header.DataSize);
    {$I+}
    DBFError := (IOResult <> 0);
    IsDirty  := TRUE;
  END ELSE
    DBFError := FALSE;
END;

PROCEDURE tDBObject.Pack;
VAR
  i, CntRec : LongInt;
  p         : pChar;
  Buf       : CHAR;
  TmpF      : File;
  Ext       : ARRAY [0..4] OF CHAR;
BEGIN
  (* Alte Extension merken und temporre Datei ffnen     *)
  p := StrPos(DBFName, '.');
  IF p = NIL THEN BEGIN
    StrCopy(Ext, '');
    StrCat(DBFName, '.$$$');
  END ELSE BEGIN
    StrCopy(Ext, p);
    StrMove(p, '.$$$', 4);
  END;
  Assign(TmpF, DBFName);
  {$I-}
  Rewrite(TmpF, 1);
  {$I+}
  DBFError := (IOResult <> 0);
  IF DBFError THEN Exit;
  (* Header und Feldbeschreibung nach TmpF schreiben      *)
  BlockWrite(TmpF, Header, SizeOf(Header));
  FOR i := 1 TO NumFields DO
    BlockWrite(TmpF, Fields[i]^, SizeOf(Fields[i]^));
  Buf := Chr($0D);
  BlockWrite(TmpF, Buf, 1);
  (* Marker Ende der Feldbeschreibungen *)

  (* Jeden Satz der Quelle lesen und ggf. bernehmen      *)
  CntRec := 0;            (* Zhler fr bernommene Stze *)
  Seek(f, LongInt(Header.HeaderSize));
  FOR i := 1 TO Header.NumRec DO BEGIN
    BlockRead(f, Buf4001^, Header.DataSize);
    IF Buf4001^[0] <> Chr($2A) THEN BEGIN
      BlockWrite(TmpF, Buf4001^, Header.DataSize);
      Inc(CntRec);
    END;
  END;
  Buf := Chr($1A);                      (* ^Z = Dateiende *)
  BlockWrite(TmpF, Buf, 1);
  Header.NumRec := CntRec;
  Seek(TmpF, 0);
  BlockWrite(TmpF, Header, SizeOf(Header));

  (* Dateien schlieen, Original lschen und TmpF         *)
  (* umbenennen                                           *)
  System.Close(f);
  System.Close(TmpF);
  Erase(f);
  p := StrPos(DBFName, '.$$$');
  StrMove(p, Ext, 4);
  Rename(TmpF, DBFName);
  IsDirty := TRUE;                       (* Aktualisieren *)
  Reset(f, 1);
  IF RecCount > 0 THEN Go(1)
                  ELSE Seek(f, LongInt(Header.HeaderSize));
END;

PROCEDURE tDBObject.Zap;
VAR
  i   : BYTE;
  Buf : ARRAY [1..2] OF CHAR;
BEGIN
  System.Close(f);

  {$I-}
  Rewrite(f, 1); (* Neue Datei ffnen, die alte zerstren *)
  {$I+}
  (* Wichtig: wird bei Rewrite kein zweiter Parameter     *)
  (* angegeben (hier 1), wird als Blockgre 128          *)
  (* angenommen, und dann werden bei BlockWrite(..32)     *)
  (* nicht 32, sondern 32 * 128 Bytes bertragen...       *)

  DBFError := (IOResult <> 0);
  IF DBFError THEN Exit;
  Header.NumRec := 0;
  IsDirty       := TRUE;
  BlockWrite(f, Header, 32);
                     (* genderten Header zurckschreiben *)

  FOR i := 1 TO NumFields DO (* Feldliste zurckschreiben *)
    BlockWrite(f, Fields[i]^, SizeOf(Fields[i]^));

  Buf[1] := Chr($0D);        (* Ende der Feldbeschreibung *)
  Buf[2] := Chr($1A);        (* DateiEnde fr Dos         *)
  BlockWrite(f, Buf, 2);
END;

PROCEDURE tDBObject.Skip;
BEGIN
  {$I-}
  BlockRead(f, Buf4001^, Header.DataSize);
  {$I+}
  DBFError := (IOResult <> 0);
  OEMToAnsi(Buf4001^, Buf4001^);
END;

PROCEDURE tDBObject.Go(n : LongInt);
BEGIN
  DBFError := FALSE;
  IF (n < 0) OR (n > Header.NumRec) THEN DBFError := TRUE
  ELSE BEGIN
    Seek(f, (n - 1) * Header.DataSize + Header.HeaderSize);
    {$I-}
    BlockRead(f, Buf4001^, Header.DataSize);
    {$I+}
    DBFError := (IOResult <> 0);
    OEMToAnsi(Buf4001^, Buf4001^);
  END;
END;

PROCEDURE tDBObject.GoTop;
BEGIN
  Go(1);
END;

PROCEDURE tDBObject.GoBottom;
BEGIN
  Go(RecCount);
END;

FUNCTION tDBObject.BOF : BOOLEAN;
BEGIN
  IF RecNo = 0 THEN
    BOF := TRUE
  ELSE
    BOF := FALSE;
END;

FUNCTION tDBObject.EoF : BOOLEAN;
BEGIN
  EoF := System.EoF(f);
END;

FUNCTION tDBObject.Locate(n : BYTE; s : pChar) : BOOLEAN;
VAR
  nPos : pChar;
  Buf  : ARRAY [0..255] OF CHAR;
BEGIN
  DBFError := FALSE;
  IF (n < 1) OR (n > NumFields) THEN DBFError := TRUE
  ELSE
    WHILE NOT (EoF) AND
              (StrPos(GetField(n, Buf), s) = NIL) DO
      Skip;
  nPos   := StrPos(GetField(n, s), s);
  bFound := (nPos <> NIL);
  Locate := bFound;
END;

FUNCTION tDBObject.Found : BOOLEAN;
BEGIN
  Found := bFound;
END;

PROCEDURE tDBObject.PutField(n : BYTE; s : pChar);
VAR
  Li  : LongInt;
  Err : INTEGER;
  Ft  : CHAR;
  v   : REAL;
  Buf : ARRAY [0..256] OF CHAR;
BEGIN
  DBFError := FALSE;
  Ft       := FieldType(n);
  IF DBFError THEN Exit;
  CASE Ft OF
  'C' : BEGIN
          IF StrLen(s) > Fields[n]^.Size THEN
            s[StrLen(s)] := #0             (* abschneiden *)
          ELSE
            WHILE StrLen(s) < Fields[n]^.Size DO
              StrCat(s, ' ');   (* mit Leerzeichen fllen *)
        END;
  'N' : BEGIN
          Val(s, v, Err);
          DBFError := (Err <> 0);
          IF DBFError THEN Exit;
          IF Fields[n]^.Dec = 0 THEN
            Str(v:Fields[n]^.Size:0, Buf)
          ELSE
            Str(v : Fields[n]^.Size-Fields[n]^.Dec - 1 :
                Fields[n]^.Dec, Buf);
          StrCopy(s, Buf);
          WHILE StrLen(s) < Fields[n]^.Size DO
            StrCat(s, ' '); (* mit Leerzeichen fllen *)
        END;
  'D' : BEGIN
          Val(s, Li, Err);
          IF (Err <> 0) OR (StrLen(s) <> 8) THEN BEGIN
            DBFError := TRUE; Exit;
          END;
        END;
  'L' : BEGIN
          IF (StrLen(s) <> 1) OR NOT (UpCase(s[0])
            IN ['T', 'F', 'W', 'F', '?'] )THEN
            DBFError := TRUE
          ELSE
            DBFError := FALSE;
          IF DBFError THEN Exit;
        END;

  'M' : BEGIN
          Val(s, Li, Err);
          IF (Err <> 0) OR (Li <= 0) THEN BEGIN
            DBFError := TRUE; Exit;
          END ELSE BEGIN
            Str(Li: 10, Buf); StrCopy(s, Buf);
          END;
        END;
  END;

  Move(s[0], Fields[n]^.Addr^, Fields[n]^.Size);

  (* Satzzeiger an den Anfang des aktuellen Satzes        *)
  (* zurckstellen                                        *)
  Seek(f, FilePos(f) - Header.DataSize);

  (* Satz mit gendertem Feldinhalt zurckschreiben.      *)
  (* Weil AnsiToOem das  einfach verschluckt und nur ein *)
  (* Leerzeichen zurcklt, wurde folgender Trick        *)
  (* erforderlich:                                        *)
  FOR Li := 0 TO Header.DataSize DO
    IF  Buf4001^[Li] =#225 THEN Buf4001^[Li] := #223{''};
  FOR Li := 0 TO Header.DataSize DO
  CASE Buf4001^[Li] OF
    '': Buf4001^[Li] := #142{''};  (* .. und ist        *)
    '': Buf4001^[Li] := #153{''};  (* AnsiToOem nicht   *)
    '': Buf4001^[Li] := #154{''};  (* willig, dann      *)
    '': Buf4001^[Li] := #132{''};  (* geht's eben mit   *)
    '': Buf4001^[Li] := #148{''};  (* Gewalt!           *)
    '': Buf4001^[Li] := #129{''};  (* Die ganze Sache   *)
   #223: Buf4001^[Li] := #225{''};  (* funktioniert aber *)
    '': Buf4001^[Li] := #175{''};  (* nur mit der DOS   *)
    '': Buf4001^[Li] := #174{''};  (* Codepage 437      *)
    '': Buf4001^[Li] := #130{''};
    '': Buf4001^[Li] := #138{''};
    '': Buf4001^[Li] := #136{''};
    '': Buf4001^[Li] := #144{''};
    '': Buf4001^[Li] := #160{''};
    '': Buf4001^[Li] := #133{''};
    '': Buf4001^[Li] := #131{''};
    '': Buf4001^[Li] := #162{''};
    '': Buf4001^[Li] := #149{''};
    '': Buf4001^[Li] := #131{''};
    '': Buf4001^[Li] := #163{''};
    '': Buf4001^[Li] := #133{''};
    '': Buf4001^[Li] := #151{''};
    '': Buf4001^[Li] := #161{''};
    '': Buf4001^[Li] := #141{''};
    '': Buf4001^[Li] := #140{''};
    '': Buf4001^[Li] := #164{''};
    '': Buf4001^[Li] := #165{''};
    '': Buf4001^[Li] := #145{''};
    '': Buf4001^[Li] := #146{''};
    '': Buf4001^[Li] := #135{''};
    '': Buf4001^[Li] := #128{''};
    '': Buf4001^[Li] := #137{''};
    '': Buf4001^[Li] := #021{''};
    '': Buf4001^[Li] := #237{''};
    '': Buf4001^[Li] := #134{''};
    '': Buf4001^[Li] := #143{''};
    (* hier kann man noch erweitern, falls es unbedingt   *)
    (* sein mu                                           *)
  END;
  (* Anm.: es_OEMConvert fr tEdit half auch nicht weiter *)
  {$I-}
    BlockWrite(f, Buf4001^, Header.DataSize);
  {$I+}
  DBFError := (IOResult <> 0);
  IsDirty := TRUE;
END;

FUNCTION tDBObject.GetField(n : BYTE; Buf : pChar) : pChar;
VAR
  p : pChar;
BEGIN
  DBFError := FALSE;
  IF (n < 1) OR (n > NumFields) THEN
    DBFError := TRUE
  ELSE
    CASE Fields[n]^.Typ OF
    'C',
    'N',
    'L': BEGIN
           Move(Fields[n]^.Addr^, Buf[0], Fields[n]^.Size);
           Buf[Fields[n]^.Size] := #0;
           GetField := Buf;
         END;
    'M': StrCopy(Buf, 'Memo');
    'D': BEGIN
           p := pChar(Fields[n]^.Addr);
           Move(pChar(p + 6)^, Buf[0], 2);
           Buf[2] := '.';
           Move(pChar(p + 4)^, Buf[3], 2);
           Buf[5] := '.';
           Move(pChar(p + 0)^, Buf[6], 4);
           Buf[10] := #0;
         END;
    END;
END;

FUNCTION tDBObject.Trim(aBuf : pChar) : pChar;
BEGIN
  WHILE (StrLen(aBuf) - 1 >= 0) AND
        (aBuf[StrLen(aBuf) - 1] = #32) DO
    aBuf[StrLen(aBuf) - 1] := #0;
  Trim := @aBuf;
END;

FUNCTION tDBObject.DBF : pChar;
BEGIN
  DBF := @DBFName;
END;

FUNCTION tDBObject.Version : BYTE;
BEGIN
  Version := (Header.FirstByte AND $7);
END;

FUNCTION tDBObject.WithMemo : BOOLEAN;
BEGIN
  WithMemo := (Header.FirstByte AND $80 <> 0);
END;

FUNCTION tDBObject.LUpDate(Buf10 : pChar) : pChar;
VAR
  s : ARRAY [0..2] OF CHAR;
BEGIN
  WITH Header.Date DO BEGIN
    Str(Day, s);
    StrCopy(Buf10, s);
    StrCat(Buf10, '.');
    Str(Month, s);
    StrCat(Buf10, s);
    StrCat(Buf10, '.19');
    IF Year < 10 THEN
      StrCat(Buf10, '0');
    Str(Year, s);
    StrCat(Buf10, s);
  END;
  LUpDate := Buf10;
END;

FUNCTION tDBObject.RecCount : LongInt;
BEGIN
  RecCount := Header.NumRec;
END;

FUNCTION tDBObject.RecNo : LongInt;     (* Aktueller Satz *)
BEGIN
  WITH Header DO
    RecNo := (FilePos(f) - HeaderSize) DIV DataSize;
END;

FUNCTION tDBObject.RecSize : INTEGER;
BEGIN
  RecSize := Header.DataSize;
END;

FUNCTION tDBObject.Field(n : BYTE) : pChar;
VAR
  k : INTEGER;
BEGIN
  DBFError := FALSE;
  IF (n < 1) OR (n > NumFields) THEN
    DBFError := TRUE
  ELSE BEGIN
    WITH Fields[n]^ DO BEGIN
      FOR k := 1 TO Length(FLDName) DO BEGIN
        CASE FLDName[k] OF
          #142: FLDName[k] :=  '';  (* und ist OemToAnsi *)
          #153: FLDName[k] :=  '';  (* nicht willig,     *)
          #154: FLDName[k] :=  '';  (* dann geht's eben  *)
          #132: FLDName[k] :=  '';  (* auch hier mit     *)
          #148: FLDName[k] :=  '';  (* Gewalt! Die ganze *)
          #129: FLDName[k] :=  '';  (* Sache klappt aber *)
          #225: FLDName[k] := #223;  (* auch hier nur mit *)
          #175: FLDName[k] :=  '';  (* der DOS Codepage  *)
          #174: FLDName[k] :=  '';  (* 437               *)
          #130: FLDName[k] :=  '';
          #138: FLDName[k] :=  '';
          #136: FLDName[k] :=  '';
          #144: FLDName[k] :=  '';
          #160: FLDName[k] :=  '';
          #133: FLDName[k] :=  '';
          #131: FLDName[k] :=  '';
          #162: FLDName[k] :=  '';
          #149: FLDName[k] :=  '';
          #131: FLDName[k] :=  '';
          #163: FLDName[k] :=  '';
          #133: FLDName[k] :=  '';
          #151: FLDName[k] :=  '';
          #161: FLDName[k] :=  '';
          #141: FLDName[k] :=  '';
          #140: FLDName[k] :=  '';
          #164: FLDName[k] :=  '';
          #165: FLDName[k] :=  '';
          #145: FLDName[k] :=  '';
          #146: FLDName[k] :=  '';
          #135: FLDName[k] :=  '';
          #128: FLDName[k] :=  '';
          #137: FLDName[k] :=  '';
          #021: FLDName[k] :=  '';
          #237: FLDName[k] :=  '';
          #134: FLDName[k] :=  '';
          #143: FLDName[k] :=  '';
    (* hier kann man noch erweitern, falls es unbedingt   *)
    (* sein mu                                           *)
        END;
      END;
    END;
    Field := @Fields[n]^.FldName;
  END;
END;

FUNCTION tDBObject.FieldType(n : BYTE) : CHAR;
BEGIN
  DBFError := FALSE;
  IF (n < 1) OR (n > NumFields) THEN
    DBFError := TRUE
  ELSE
    FieldType := Fields[n]^.Typ;
END;

FUNCTION tDBObject.FieldLength(n : BYTE) : BYTE;
BEGIN
  DBFError := FALSE;
  IF (n < 1) OR (n > NumFields) THEN
    DBFError := TRUE
  ELSE
    FieldLength := Fields[n]^.Size;
END;

FUNCTION tDBObject.FldCount : BYTE;
BEGIN
  FldCount := NumFields;
END;

FUNCTION tDBObject.FileExists(FName : pChar) : BOOLEAN;
                                         (* dBase: FILE() *)
VAR
  TmpF : File;
BEGIN
  Assign(TmpF, FName);
  {$I-}
  Reset(TmpF);
  {$I+}
  IF IOResult = 0 THEN BEGIN
    System.Close(TmpF);
    FileExists := StrLen(FName) <> 0;
  END ELSE
    FileExists := FALSE;
END;

END.
(* ====================================================== *)
(*                  Ende von DBENGINE.PAS                 *)
