// dlgbox.h
// implementiert die Dialogbox-klasse
// und die benoetigten sonstigen Klassen
// fuer das einfache Dialogboxbeispiel
//
// Update: 1.00 29-Apr-93 tw

#ifndef _DLGBOX_H
#define _DLGBOX_H   

#include <afxcoll.h>

#define IDOK         1
#define IDCANCEL     2
#define IDHELP       3
                      
typedef struct tagPOINT2DGK
{
   short x;
   short y;
   
}  point2dGK;                    
                      
class XWnd : public CObject
{
protected:
   static short m_cxChar; 
   static short m_cyChar;
   static const short m_cxTxtBorder;
   static const short m_cyTxtBorder;

   static short m_colBtnFace;        // GRAU
   static short m_colBtnShadow;      // D_GRAU
   static short m_colBtnHilight;     // WEISS
   static short m_colBtnFrame;       // SCHWARZ

   CString  m_strCaption;
   short    m_cxCaption;
   short    m_cx;
   short    m_cy;

   short    m_xUpperLeft;  
   short    m_yUpperLeft;   
   short    m_id;     
   
public:
   XWnd() {}
   XWnd( const point2dGK pntUpperLeft, short id, const char * pszCaption );
   virtual ~XWnd() {};
   void Create( const point2dGK pntUpperLeft, short id, const char * pszCaption );
   short ID() const;
   short FontXSize() const;
   short FontYSize() const;
   short TxtBorderXSize() const;
   short TxtBorderYSize() const;
};

class XButton : public XWnd
{
private:
   BOOL     m_fIsChecked;

public:
   XButton( const point2dGK pntUpperLeft, short id, const char * pszCaption,
            BOOL fIsChecked = FALSE );
   virtual ~XButton() {};
   const CString& Caption() const;
   short X() const; 
   short Y() const; 
   short Length() const;
   short Heigth() const;
   short CharLength() const;
   short CharHeigth() const;
   short ID() const;
   
   virtual void OnPaint() const;
   void Caption( short x, short y ) const; 
   void Caption( const char * pszCaption );
   BOOL CheckState() const;
   void CheckState( BOOL fCheck );
   virtual XButton * GotHit( short x, short y );
               
   virtual void OnBtnDown() = NULL;
   virtual void OnBtnUp() const = NULL;
               
};
  
  
class XPushButton : public XButton
{
public:
   XPushButton( const point2dGK pntUpperLeft, short id, const char * pszCaption );
   ~XPushButton() {}
   void OnPaint() const;
   virtual void OnBtnDown(); 
   virtual void OnBtnUp() const;  
};

class XUpArrowButton : public XPushButton
{
public:
   XUpArrowButton( const point2dGK pntUpperLeft, short id );
   ~XUpArrowButton();
};


class XStatic : public XButton
{
public:
   XStatic( const point2dGK pntUpperLeft, short id, const char * pszCaption );
   ~XStatic() {}
   void OnPaint() const;
   virtual void OnBtnDown() {};
   virtual void OnBtnUp() const {};
};

class XRadioButton : public XButton
{
public:
   XRadioButton( const point2dGK pntUpperLeft, short id, const char * pszCaption,
                 BOOL fIsChecked );
   ~XRadioButton() {}
   void OnPaint() const;
   virtual void OnBtnUp() const {};
   virtual void OnBtnDown();
};

class XCheckButton : public XButton
{
public:
   XCheckButton( const point2dGK pntUpperLeft, short id, const char * pszCaption,
                 BOOL fIsChecked );
   ~XCheckButton() {}
   void OnPaint() const;
   virtual void OnBtnUp() const {}
   virtual void OnBtnDown();
};

class XGroupBox : public XButton
{
protected:
   CObList  m_lstButtons;

public:
   XGroupBox( const point2dGK pntUpperLeft, short id, const char * pszCaption );
   ~XGroupBox();
   void AddButton( XButton * pNewButton );
   BOOL GetDlgItemCheckState( short idDlgItem ) const;
   virtual XButton * GotHit( short x, short y);
   virtual void OnPaint() const;
   virtual void OnBtnDown();
   virtual void OnBtnUp() const;
};

class XRadioBox : public XGroupBox
{
public:
   XRadioBox( const point2dGK pntUpperLeft, short id, const char * pszCaption );
   virtual XButton * GotHit( short x, short y);
};

class XButtonList : public CObList
{
public:
   XButtonList() {}
   ~XButtonList();
   void Add( XButton * pButton );
   void OnPaint();
   virtual XButton * GotHit( short x, short y );
};

class XDialogBox : public XWnd
{
private:
   short m_xLowerRight,
         m_yLowerRight;

   char __huge * m_pImage;
   
   CString  m_strDescription;
   XButtonList    m_lstButtons;
   
   BOOL (*m_pDlgProc)(XButton * pButton, XDialogBox * pDlgCurrent );

public:
   XDialogBox( const point2dGK wndPos, const point2dGK wndSize,
               short id, const char * pszCaption,
               BOOL(*pDlgProc)(XButton *pButton, XDialogBox * pDlgCurrent)=0 );
   ~XDialogBox();
   void AddItem( XButton * pButton );
   BOOL GetDlgItemCheckState( short idDlgItem, short idDlgItemGroup=0 ) const;
   BOOL Create( const char * pszFont = 0 );
   BOOL DoModal();
   XButton * GetDlgItem( const short idDlgItem ) const;
};


#endif //_DLGBOX_H









   

                 
   

                 

   
  

 
