{$A+,B-,D+,E+,F-,G-,I+,L+,N+,O-,P-,Q-,R-,S+,T-,V+,X+}
{$M 16384,0,655360}

PROGRAM Berg;
{
Literatur:
'Computer-Kurzweil' von A.K. Dewney
in 'Spektrum der Wissenschaft', Mrz 1987

Eingaben:
Level:
  Je hher, desto mehr Details.
Bereich:
  Je hher, desto steiler.
Meer:
  Wasser kann das Land mit einer bestimmten
  Wasserhhe fluten.
Feste Kanten:
  Den Kanten der GrunMutelche kann hiermit eine
   Anfangshhe zugewiesen werden, die whrend des
  Programms nicht mehr verndert wird. Ansonsten
  haben nur die vier Eckpunkte des Feldes unver-
  nderbare Hhe.
Zufallsalg.:
  Es gibt zwei verschiedene, ausprobieren! Sie
  knnen auch ein bichen mit Mute und dt spielen }

USES
  Crt, Dos, Graph, PCXTools;

CONST
  MaxDots = 64;                   (* 2 ^ MaximalLevel / 2 *)
  MinDots = -MaxDots;

VAR
  RandBuf, Mute : Double;
  Pixel         : ARRAY[MinDots..MaxDots,
                        MinDots..MaxDots] OF INTEGER;
  StartFixZ,
  StartFlexZ,
  PixelDiff,
  Iteration,
  Range,
  SeaLevel,
  Algorithm,
  PlaneColor,
  TopoNetColor,
  SeaFillColor,
  WaveColor,
  GraphDriver   : INTEGER;

FUNCTION Power(Base, Exp: INTEGER): INTEGER;
VAR
  i, h: INTEGER;
BEGIN
  IF Exp = 0 THEN Power := 1 ELSE BEGIN
    h := Base;
    FOR i := 2 TO Exp DO h := h * Base;
    Power := h;
  END;
END;

PROCEDURE InitPixelArray(Edges, Land : BOOLEAN);
VAR
  i, j, w: INTEGER;
BEGIN
  Randomize;
  FOR i := MinDots + 1 TO MaxDots - 1 DO
    FOR j := MinDots + 1 TO MaxDots - 1 DO
      Pixel[i, j] := StartFixZ;
  IF Edges THEN w := StartFlexZ ELSE w := StartFixZ;
  FOR i := MinDots TO MaxDots DO BEGIN
    Pixel[i, MinDots] := w;
    Pixel[i, MaxDots] := w;
    Pixel[MinDots, i] := w;
    Pixel[MaxDots, i] := w;
  END;
  IF (((GraphDriver > 1) AND (GraphDriver < 5))
                          OR (GraphDriver = 9)) THEN
    PlaneColor := Brown
  ELSE
    PlaneColor := Black;
  TopoNetColor := White;
  SeaFillColor := Blue;
  WaveColor    := LightBlue;
  Land         := TRUE;
  SetFillStyle(SolidFill, PlaneColor);
  SetColor(TopoNetColor);
END;

FUNCTION RandPos(z, n: INTEGER) : INTEGER;
(* Erzeugt Zufallszahl nach Algorithmus n. *)
VAR
  i: SINGLE;
BEGIN
  CASE n OF
    1: RandPos := Round(Random(z) - z * RandBuf);
       (* Alg. 1: [-z * RandBuf.. z - z * RandBuf] *)
    2: BEGIN
         IF Random - RandBuf < 0 THEN i := - Mute
                                 ELSE i := 1;
         RandPos := Round(Random(z) * i);
         (* Alg. 2: [-z * Mute + 1 .. z]    *)
       END;
  END;
END;

PROCEDURE Calculate(Level, Range: INTEGER);
(* Erzeugt im Feld Pixel die Hhen der einzelnen *)
(* Punkte abhngig von Level und Range.          *)
VAR
  i, j, l, d : INTEGER;
BEGIN
  FOR l := 1 TO level DO BEGIN
    d := (MaxDots - MinDots) DIV Power(2, l);
    j := MinDots;
    REPEAT
      i := MinDots;
      REPEAT
        IF Pixel[i, j + d] = StartFixZ THEN
          Pixel[i, j + d] :=
               (Pixel[i, j] +
                Pixel[i, j + 2 * d]) DIV 2 +
                RandPos(Range, Algorithm);
        IF Pixel[i + d, j] = StartFixZ THEN
          Pixel[i + d, j] :=
               (Pixel[i, j] +
                Pixel[i + 2 * d, j]) DIV 2 +
                RandPos(Range, Algorithm);
        IF Pixel[i + d, j + d] = StartFixZ THEN
           Pixel[i + d, j + d] :=
                (Pixel[i, j] +
                 Pixel[i + 2 * d, j + 2 * d] +
                 Pixel[i + 2 * d , j] +
                 Pixel [i, j + 2 * d]) DIV 4 +
                 RandPos(Range, Algorithm);
        IF Pixel[i + d, j + 2 * d] = StartFixZ THEN
          Pixel[i + d, j + 2 * d] :=
               (Pixel[i, j + 2 * d] +
                Pixel[i + 2 * d, j + 2 * d]) DIV 2 +
                RandPos(Range, Algorithm);
        IF Pixel[i + 2 * d, j + d] = StartFixZ THEN
          Pixel[i + 2 * d, j + d] :=
               (Pixel[i + 2 * d, j] +
                Pixel [i + 2 * d, j + 2 * d]) DIV 2 +
                RandPos(Range, Algorithm);
        Inc(i, 2 * d)
      UNTIL i = MaxDots;
      Inc(j, 2 * d)
    UNTIL j = MaxDots;
    Range := Round (Range / PixelDiff);
  END;
END;

PROCEDURE Display(Level: INTEGER; Land, Sea: BOOLEAN);
CONST
  rho   = 8;
  theta = 1.1;
  phi   = 1.8;
TYPE
  tLine = ARRAY[MinDots..MaxDots] OF PointType;
VAR
  LineArray   : ARRAY[1..2] OF tLine;
  SquareArray : ARRAY[1..4] OF PointType;
  i, j,
  j1, j2, d   : INTEGER;
  s1, s2,
  c1, c2,
  cx, cy,
  dx, dy      : Double;
  m           : BOOLEAN;

  PROCEDURE CalcTopo(i, j, n: INTEGER);
  VAR
    x,  y,  z,
    xe, ye, ze,
    xn, yn     : SINGLE;
  BEGIN
    x  := i / MaxDots;
    y  := j / MaxDots;
    z  := - Pixel[i, j] / 300;
    xe := - x * s1 + y * c1;
    ye := - x * c1 * c2 - y * s1 * c2 + z * s2;
    ze := - x * s2 * c1 - y * s2 * s1 - z * c2 + rho;
    xn := dx * xe / ze + cx;
    yn := dy * ye / ze + cy;
    LineArray[n, i].x := Round(xn);
    LineArray[n, i].y := Round(yn);
  END;

  PROCEDURE DrawLine(j, n: INTEGER);
  BEGIN
    i := MinDots;
    REPEAT
      CalcTopo(i, j, n);
      Inc(i, d)
    UNTIL (i = MaxDots + d) OR KeyPressed
  END;

  PROCEDURE Paint(Land, Sea : BOOLEAN);
  BEGIN
    i := MinDots;
    REPEAT
      SquareArray[1] := LineArray[1, i];
      SquareArray[2] := LineArray[1, i + d];
      SquareArray[3] := LineArray[2, i + d];
      SquareArray[4] := LineArray[2, i];
      IF Sea THEN BEGIN
        m := (Pixel[i,     j1] = SeaLevel) AND
             (Pixel[i + d, j2] = SeaLevel) AND
             (Pixel[i + d, j1] = SeaLevel) AND
             (Pixel[i,     j2] = SeaLevel);
        IF m AND Land THEN BEGIN
          SetColor(WaveColor);
          SetFillStyle(Solidfill, SeaFillColor);
          Land := FALSE;
        END ELSE BEGIN
          IF (NOT m) AND (NOT Land) THEN BEGIN
            SetColor(TopoNetColor);
            SetFillStyle(SolidFill, PlaneColor);
            Land := TRUE;
          END;
        END;
      END;
      FillPoly(4, SquareArray);
      Inc(i, d)
    UNTIL (i = MaxDots) OR KeyPressed
  END;

BEGIN
  cx := GetMaxX DIV 2;
  cy := GetMaxY DIV 2;
  dx := 2.8 * GetMaxX;
  dy := 4.5 * GetMaxY;
  s1 := Sin(theta);
  c1 := Cos(theta);
  s2 := Sin(phi);
  c2 := Cos(phi);
  d  := (MaxDots - MinDots) DIV Power(2, Level);
  j  := MinDots;
  DrawLine(j, 1);
  j1 := j;
  IF Level = 0 THEN BEGIN
    DrawLine (j + d, 2);
    j2 := j + d;
    Paint(Land, Sea);
    j := MaxDots;
  END;
  WHILE (j < MaxDots) AND (NOT KeyPressed) DO BEGIN
    Inc(j, d * 2);
    DrawLine (j - d, 2);
    j2 := j - d;
    Paint(Land, Sea);
    DrawLine(j, 1);
    j1 := j;
    Paint(Land, Sea);
  END;
END;

PROCEDURE CalcSea(Level, h:INTEGER);
VAR
  i, j: INTEGER;
BEGIN
  FOR i := MinDots TO MaxDots DO BEGIN
    FOR j := MinDots TO MaxDots DO
      IF Pixel [i, j] < h THEN Pixel [i, j] := h;
  END;
END;

VAR
  str         : STRING;
  PCXDir      : DirStr;
  PCXName     : NameStr;
  PCXExt      : ExtStr;
  again       : CHAR;
  Edges,
  Sea,
  Land        : BOOLEAN;
  PCXFileName : PathStr;
  GraphMode   : INTEGER;

BEGIN
  DetectGraph(GraphDriver, GraphMode);
  InitGraph(GraphDriver, GraphMode, GetEnv('BGIPATH'));
  PCXTools.XMin   := 0;
  PCXTools.XMax   := GetMaxX;
  PCXTools.YMin   := 0;
  PCXTools.YMax   := GetMaxY;
  PCXTools.Screen := 0;
  PCXTools.Col    := GetMaxColor;
  StartFixZ       := -100;
  StartFlexZ      := -StartFixZ - 1;
  RandBuf         := 0.34;
  Mute            := 0.6;
  PixelDiff       := 2;
  REPEAT
    ClearDevice;
    DirectVideo := FALSE;
    Edges       := FALSE;
    Sea         := FALSE;
    GotoXY(1, 1);
    Write('Interationstiefe (max 7): ');
    ReadLn(Iteration);
    IF Iteration > MaxDots DIV 10 THEN
      Iteration := MaxDots DIV 10;
    Write('Schwankungbereich       : ');
    ReadLn(Range);
    IF Range <= 0 THEN Range := 1;
    Write('Meereshhe (-3 .. +3)   : ');
    ReadLn(SeaLevel);
    REPEAT
      Write('Algorithmus (1 oder 2)  : ');
      ReadLn(Algorithm);
    UNTIL Algorithm IN [1..2];
    Write('Feste Kanten (j/[n])    : ');
    ReadLn(str);
    IF Length(str) > 0 THEN
      IF UpCase(str[1]) = 'J' THEN Edges := TRUE;
    Write('Meer darstellen (j/[n]) : ');
    ReadLn(str);
    IF Length(str) > 0 THEN
      IF UpCase(str[1]) = 'J' THEN Sea := TRUE;
    Write('Name fr die Bilddatei  : ');
    ReadLn(PCXFileName);
    IF PCXFileName <> '' THEN BEGIN
      FSplit(PCXFileName, PCXDir, PCXName, PCXExt);
      IF PCXExt = '' THEN
        PCXFileName := PCXFileName + '.PCX';
    END;
    ClearDevice;
    DirectVideo := TRUE;
    InitPixelArray(Edges, Land);
    Calculate(Iteration, Range);
    IF Sea THEN CalcSea(Iteration, SeaLevel);
    Display(Iteration, Land, Sea);
    Write(^G);
    IF PCXFileName <> '' THEN
      BGItoPCX(GraphDriver, GraphMode, PCXFileName);
    ReadKey;
    MoveTo(5, 20);
    SetColor(LightRed);
    OutText('Noch ein Bild zeichnen? [J/N]: ');
    again := UpCase(ReadKey);
    OutText(again);
  UNTIL again = 'N';
  CloseGraph;
END.

