(* ====================================================== *)
(*                       FRAKLAND.PAS                     *)
(*      Copyright (C) 1993, 1994 J. Braun & toolbox       *)
(*                Compiler: Turbo Pascal 7.0              *)
(* ====================================================== *)
{$A+,B+,D-,E+,F-,G-,I-,L-,N+,O-,P-,Q-,R-,S-,T-,V-,X+,Y-}
{$M 16384,0,655360}

PROGRAM MandelLand;

USES
  Dos, Crt, Graph, PCXTools;

VAR
  XOrigin, YOrigin, Height, Angle, Degrees : Double;

(* ------------------------------------------------------ *)

PROCEDURE SavePicture(GraphDriver, GraphMode: INTEGER;
                      PCXFileName: PathStr);
VAR
  f        : Text;
  Dir      : DirStr;
  Name     : NameStr;
  Ext      : ExtStr;
  DataName : PathStr;
BEGIN
  PCXTools.XMin   := 0; PCXTools.XMax := GetMaxX;
  PCXTools.YMin   := 0; PCXTools.YMax := GetMaxY;
  PCXTools.Screen := 0; PCXTools.Col  := GetMaxColor;
  IF PCXFileName <> '' THEN BEGIN
    FSplit(PCXFileName, Dir, Name, Ext);
    IF Ext = '' THEN PCXFileName := PCXFileName + '.PCX';
    BGItoPCX(GraphDriver, GraphMode, PCXFileName);
    DataName := Dir + Name + '.DAT';
    Assign(f, DataName);
    Rewrite(f);
    WriteLn(f, 'Koordinaten fr ', PCXFileName, ':');
    WriteLn(f, 'X-Ursprung         : ', XOrigin:3:3);
    WriteLn(f, 'Y-Ursprung         : ', YOrigin:3:3);
    WriteLn(f, 'Hhe (Z-Koord.)    : ', Height :3:3);
    WriteLn(f, 'Blickwinkel in Grad: ', Degrees:3:3);
    Close(f);
  END;
END;

(* ------------------------------------------------------ *)

FUNCTION ReadDouble(VAR num: Double): BOOLEAN;
VAR
  s    : STRING;
  test : INTEGER;
BEGIN
  ReadDouble := TRUE;
  REPEAT
    ReadLn(s);
    IF s = '' THEN BEGIN
      ReadDouble := FALSE;
      Exit;
    END;
    IF s[1] = '.' THEN s := '0' + s;
    Val(s, num, test);
  UNTIL test = 0;
END;

(* ------------------------------------------------------ *)

FUNCTION Sign(x: Double): INTEGER;
BEGIN
  IF x < 0 THEN Sign := -1 ELSE Sign := 1;
END;

(* ------------------------------------------------------ *)

PROCEDURE PaintSun(MaxX, MaxY: INTEGER);
(* Sonne zeichnen *)
BEGIN
  Randomize;
  SetFillStyle(SolidFill, EGAYellow);
  SetColor(EGAYellow);
  PieSlice(Random(MaxX DIV 16) + MaxX DIV 32 + 1,
           Random(MaxY DIV  4) + MaxX DIV 32 + 1,
           0, 360,
           MaxX DIV 20);
END;

(* ------------------------------------------------------ *)

PROCEDURE Help;
(* Hilfe bei /? /H                                        *)
BEGIN
  HighVideo;
  WriteLn('Frakland v1.0. Generator fr fraktale Landschaf'
          + 'ten'^J);
  NormVideo;
  WriteLn('Es wird eine dreidimensionele Landschaft aus de'
        + 'n Daten der Mandel-');
  WriteLn('brot-Menge erzeugt. Das errechnete Bild kann al'
        + 's PCX-Datei abge-');
  Writeln('speichert werden. Der Name fr das Bild mu hie'
        + 'rfr auf der Komman-');
  WriteLn('dozeile angegegen werden. Die Endung .PCX und e'
        + 'in Pfad fr das Bild');
  Writeln('sind optional.');
  WriteLn('Dem Programm mssen folgende Daten angegeben we'
        + 'rden:');
  WriteLn('Der Startpunkt (x- und y-Koordinate und Standh'
        + 'he des Betrachters,');
  WriteLn('und zudem der Blickwinkel in Grad. Hierbei befi'
        + 'ndet sich (0,0,0)');
  WriteLn('genau im Zentrum des Apfelmnnchens. Die Quelle'
        + ' des Apfelsees');
  WriteLn('liegt in Richtung 270 '#248'.');
  WriteLn('Das Zeichnen des Bilds kann jederzeit mit [ESC]'
        + ' oder Enter abge-');
  WriteLn('brochen werden. Bei Abbruch mit [Enter] wird da'
        + 's Bild als PCX-Da-');
  WriteLn('tei gespeichert, auch wenn die Landschaft nicht'
        + ' fertig berechnet');
  WriteLn('ist.');
  WriteLn('Das Programm bentigt den BGI-Treiber im mit S'
        + 'ET BGIPATH=<name>');
  WriteLn('angegebenen oder im aktuellen Verzeichnis.'^J);
END;

(* ------------------------------------------------------ *)

PROCEDURE PaintClouds(MaxX, MaxY: INTEGER);
(* Wolken darstellen *)
VAR
  id, jd, br, yb, yc : Double;
  as, hr, i, xb, c1  : INTEGER;
  h0                 : INTEGER;
BEGIN
  Randomize;
  FOR i := 8 TO  92 DO BEGIN
    id  := i * 4;
    jd  := Ln(id / 92);
    jd  := jd * 90;
    br  := Random + 0.7;
    br  := br * id * 0.1;
    as  := Round(Random * (GetMaxX + 1) * 2 - MaxY DIV 2);
    hr  := Round(br * (2 - Random * 1.4) * 2);
    SetColor(EGALightGray);
    FOR h0 := Round(-br) TO Round(br) DO BEGIN
      xb := h0 + as;
      IF Abs(xb - MaxX DIV 2) <= MaxX DIV 2 THEN BEGIN
        c1 := Round(br * br - h0 * h0) DIV hr;
        IF Round(c1 + jd) >= MaxY DIV 2 THEN
          c1 := MaxY DIV 2 - Round(jd);
        yb := c1 * jd * 0.006;
        yc := MaxY DIV 2 - 10 - jd;
        Line(xb, Round(yc - c1) - MaxY DIV 8,
             xb, Round(yc + yb) - MaxY DIV 8);
        PutPixel(xb, Round(yc - yb) - MaxY DIV 8,
                 EGADarkGray);
      END;
    END;
  END;
END;

(* ------------------------------------------------------ *)

PROCEDURE Stop(Message: STRING; Code: WORD);
BEGIN
  CloseGraph;
  WriteLn(^M^J, Message);
  Halt(Code);
END;

(* ====================================================== *)

VAR
  MaxX, MaxY, MaxColor     : INTEGER;
  VertLines, Color,
  m0, m1, d8, d9, h0       : INTEGER;
  c, ca, co,
  x2, xa, xc, xn,
  y2, ya, yb, yc, yn,
  z, za, r, Si, e, hc,
  ix, id                   : Double;
  PCXFileName              : STRING;
  ch                       : CHAR;
  GraphDriver, GraphMode   : INTEGER;
BEGIN
  IF ParamCount > 0 THEN PCXFileName := ParamStr(1);
  IF (Pos('?', PCXFileName) > 0) OR
     (Pos('/', PCXFileName) > 0) OR
     (Pos('*', PCXFileName) > 0) THEN Help
  ELSE BEGIN
    DetectGraph(GraphDriver, GraphMode);
    DirectVideo := FALSE;
    InitGraph(GraphDriver, GraphMode, GetEnv('BGIPATH'));
    IF GraphResult = grOk THEN BEGIN
      MaxX := GetMaxX + 1;
      MaxY := GetMaxY + 1;
      ClearDevice;
      GotoXY(1, 1);
      WriteLn('Werte fr Ursprung und Hhe eingeben:');
      REPEAT
        WriteLn;
        Write('x-Koordinate: x = ');
        IF NOT ReadDouble(XOrigin) THEN
          Stop('... abgebrochen', 1);
        Write('y-Koordinate: y = ');
        IF NOT ReadDouble(YOrigin) THEN
          Stop('... abgebrochen', 1);
        z  := 2;
        xc := XOrigin;
        yc := YOrigin;
        xa := 0;
        ya := 0;
        REPEAT
          x2 := Abs(xa);
          y2 := Abs(ya);
          xn := x2 - y2 - xc;
          ya := xa * ya;
          ya := ya + ya - yc;
          xa := xn;
          z  := z - 0.05;
          IF z < 0 THEN Break;
          IF x2 + y2 > MaxY * 2 THEN Break;
        UNTIL FALSE;
        Write('Blickwinkel : w = ');
        IF NOT ReadDouble(Angle) THEN
          Stop('... abgebrochen', 1);
        Degrees := Angle;
        Angle   := Angle * Pi / 180;
        Si      := Sin(Angle);
        Co      := Cos(Angle);
        WriteLn('Hhe bei ', XOrigin:3:2, ' / ',
                             YOrigin:3:2, ' : ',
                             (z * 1000 + 0.5) / 1000: 3:3);
        Write('z-Koordinate: z = ');
        IF NOT ReadDouble(Height) THEN
          Stop('... abgebrochen', 1);
        IF Height = 0.0 THEN Height := 0.01;    (* DIV/0! *)
        IF Height < z THEN BEGIN
          ClearDevice;
          GotoXY(1, 1);
          WriteLn('Die Hhe mu grer als z sein, bitte ',
                  'Eingaben wiederholen:');
        END;
      UNTIL Height >= z;
      MaxColor := GetMaxColor;
      SetBkColor(EGABlue);
      ClearDevice;
      SetWriteMode(NormalPut);
      DirectVideo := TRUE;
      PaintSun(MaxX, MaxY);
      PaintClouds(MaxX, MaxY);
      SetColor(Green);
      Line (0, MaxY DIV 2 - 2, MaxX , MaxY DIV 2 - 2);

      (* Berechnung der Landschaft: *)
      FOR VertLines := 1 TO MaxX DO BEGIN
            (* VertLines = Anzahl der gezeichneten Linien *)
        IF KeyPressed THEN BEGIN
          ch := ReadKey;
          IF ch IN [#13, #27] THEN BEGIN
            IF GraphDriver <= VGA THEN
              IF ch = #13 THEN
                SavePicture(GraphDriver, GraphMode,
                            PCXFileName);  (* abspeichern *)
            Stop('', 0);
          END;
        END;
        e := Height / VertLines (* DIV/0 {!} *) * 10 * 2;
        FOR h0 := -(MaxX DIV 2) TO MaxX DIV 2 DO BEGIN
          Color := Green;
          z     := 2;
          hc    := e * h0 * 0.00935;
          xa    := 0;
          ya    := 0;
          xc    := XOrigin + hc * Co + e * Si;
          yc    := YOrigin - hc * Si + e * Co;
          REPEAT
            x2 := Abs(xa);
            y2 := Abs(ya);
            xn := x2 - y2 - xc;
            ya := xa * ya;
            ya := ya + ya - yc;
            xa := xn;
            z  := z - 0.05;
            IF z < 0 THEN Break;
            IF x2 + y2 > MaxX * 2 THEN Break;
          UNTIL FALSE;
          IF h0 > - MaxX DIV 2 THEN
            z := za * 0.7 + z * 0.3;
          IF z < 0 THEN z := -0.0001;
          c     := z - Height;
          c     := c * VertLines * 2 / Height;
          za    := z;
          IF Abs(c) > MaxX DIV 2 + 2 THEN
            c   := (MaxX DIV 2 + 2) * Sign(c);
          m1    := MaxX DIV 2 + 1 + h0;
          m0    := MaxX DIV 2     + h0;

          IF d9 > Round(MaxY DIV 2 - 3 - c) THEN BEGIN
            IF za <= 0.0001 THEN Color := EGADarkGray  ELSE
            IF za <= 0.001  THEN Color := EGABlue      ELSE
            IF za <= 0.01   THEN Color := EGALightBlue ELSE
            IF za <= 0.1    THEN Color := EGAYellow    ELSE
            IF za <= 1.0    THEN Color := EGARed       ELSE
            IF za <= 1.25   THEN Color := EGACyan      ELSE
            IF za <= 1.75   THEN Color := EGAGreen
                            ELSE Color := EGALightGreen
                                           (* aufsteigend *)
          END ELSE IF d9 = Round(MaxY DIV 2 -3-c) THEN BEGIN
            IF za <= 0.0001 THEN Color := EGADarkGray  ELSE
            IF za <= 0.001  THEN Color := EGABlue      ELSE
            IF za <= 0.01   THEN Color := EGALightBlue ELSE
            IF za <= 0.1    THEN Color := EGAYellow    ELSE
            IF za <= 1.0    THEN Color := EGARed       ELSE
            IF za <= 1.25   THEN Color := EGACyan      ELSE
            IF za <= 1.75   THEN Color := EGAGreen
                            ELSE Color := EGAWhite;
          END ELSE IF d9 < Round(MaxY DIV 2 -3-c) THEN BEGIN
            IF za <= 0.0001 THEN Color := EGADarkGray  ELSE
            IF za <= 0.001  THEN Color := EGABlue      ELSE
            IF za <= 0.01   THEN Color := EGALightBlue ELSE
            IF za <= 0.1    THEN Color := EGAYellow    ELSE
            IF za <= 1.0    THEN Color := EGALightGray ELSE
            IF za <= 1.25   THEN Color := EGALightRed  ELSE
            IF za <= 1.75   THEN Color := EGAGreen
                            ELSE Color := EGADarkGray;
                                            (* absteigend *)
          END;
        (*  Ausgabe des Imaginrteils. Es mu dann aber   *)
        (*  DirectVideo auf FALSE gesetzt sein:           *)
        { GotoXY(1, 1);                                    }
        { Write('Imaginr za = ', za:2:3);                 }
          d8    := Round(MaxY DIV 2 - 2 - c);
          d9    := Round(MaxY DIV 2 - 1 - c);
          SetColor(Color);
          Line(m1, MaxY, m1, d8);     (* waagrechte Linie *)
          SetColor(EGABrown);         (* Hintergrundfarbe *)
          Line(m0, MaxX, m0, d9);     (* senkrechte Linie *)
          IF c < 0 THEN PutPixel (m0, d9, Color);

          IF c > ca + 0.5 THEN BEGIN
            IF h0 > 0 THEN BEGIN
              SetColor(EGAWhite);
              Line(m0, MaxY DIV 2 - 1 - Round(ca), m0, d9)
            END;
          END ELSE IF (c <= ca) THEN BEGIN
            SetColor(Color);
            Line(m0, MaxY DIV 2 - 1 - Round(ca), m0, d9-1);
          END ELSE
             PutPixel(m0, d8, Color); (*nachgezogene Linie*)
          ca := c;
        END;
      END;
      Write(^G);
      IF GraphDriver <= VGA THEN
        SavePicture(GraphDriver, GraphMode, PCXFileName);
      ReadKey;
      Stop('... regulr beendet', 0);
    END ELSE
      Stop('Fehler bei der Initialisierung der Grafik!', 2);
  END;
END.

(* ====================================================== *)
(*                    Ende von FRAKLAND.PAS               *)
