(* ====================================================== *)
(*                      JULIA.PAS                         *)
(*     Berechnen und Zeichnen der Julia-Menge aus dem     *)
(*                    Apfelmnnchen                       *)
(*           Compiler: Turbo/Borland Pascal 7.0           *)
(* ====================================================== *)
{$A+,B-,D-,E+,F-,G-,I-,L-,N+,O-,P-,Q-,R-,S-,T-,V-,X+,Y-}
{$M 16384,0,655360}

PROGRAM JuliaFractal;

USES
  Crt, Dos, Graph, PCXTools;

TYPE
  tJuliaData = RECORD
                 xPos, yPos : INTEGER;
                 PosX, PosY : STRING;
              END;
VAR
  CtrX, CtrY, MaxX, MaxY, MaxColor : INTEGER;

(* ------------------------------------------------------ *)
{$F+}
PROCEDURE MandelPicProc; EXTERNAL;           {$L MANDEL.OBJ}
                         (* eingelinkte Datei MANDEL.BIN  *)
{$F-}
(* ------------------------------------------------------ *)

PROCEDURE SavePicture(GraphDriver, GraphMode: INTEGER;
                      PCXFileName           : PathStr;
                      XStr, YStr            : STRING);
VAR
  f        : Text;
  Dir      : DirStr;
  Name     : NameStr;
  Ext      : ExtStr;
  DataName : PathStr;
BEGIN
  PCXTools.XMin   := 0;
  PCXTools.XMax   := GetMaxX;
  PCXTools.YMin   := 0;
  PCXTools.YMax   := GetMaxY;
  PCXTools.Screen := 0;
  PCXTools.Col    := GetMaxColor;
  FSplit(PCXFileName, Dir, Name, Ext);
  IF Ext = '' THEN Ext := '.PCX';
  BGItoPCX(GraphDriver, GraphMode, PCXFileName);
  DataName := Dir + Name + '.DAT';
  Assign(f, DataName);
  Rewrite(f);
  WriteLn(f, 'Koordinaten fr ', PCXFileName, ':');
  WriteLn(f, 'X-Ursprung     : ', XStr);
  WriteLn(f, 'Y-Ursprung     : ', YStr);
  Close(f);
END;

(* ------------------------------------------------------ *)

PROCEDURE Mandelbrot(VAR JuliaData: tJuliaData);
VAR
  Cursor             : Pointer;
  i, j, x, y,
  PicSize,
  MaxXMand, MaxYMand : INTEGER;
  ch, ch1            : CHAR;
BEGIN
  (* Aufbau der eingelinkten Bilddatei:         *)
  (* 1. Integerzahl = Lnge der X-Achse (0..    *)
  (* 2. Integerzahl = Lnge der Y-Achse (0..    *)
  (* 3. Integerzahl = Dateilnge / 2            *)
  (* anschlieend folgen bis zum Dateiende die  *)
  (* Koordinatenpaare, davon zuerst der X- und  *)
  (* dann der jeweilige Y-Wert                  *)
  MaxXMand := Integer(Ptr(Seg(MandelPicProc),
                          Ofs(MandelPicProc))^);
  MaxYMand := Integer(Ptr(Seg(MandelPicProc),
                          Ofs(MandelPicProc) + 2)^);
  PicSize  := Integer(Ptr(Seg(MandelPicProc),
                          Ofs(MandelPicProc) + 4)^);
  FOR i := 6 TO PicSize DIV 2 - 2 DO BEGIN
    j := i * 4 + 6;
    x := Integer(Ptr(Seg(MandelPicProc),
                     Ofs(MandelPicProc) + j)^);
    y := Integer(Ptr(Seg(MandelPicProc),
                     Ofs(MandelPicProc) + j + 2)^);
    PutPixel(Round(x / MaxXMand * MaxX),
             Round(y / MaxYMand * MaxY), 15);
    PutPixel(Round(x / MaxXMand * MaxX),
             MaxY - Round(y / MaxYMand * MaxY + 1), 15);
  END;
  SetLineStyle(SolidLn, 0, NormWidth);
  Line(CtrX, 0, CtrX, MaxX);
  Line(0, CtrY, MaxX, CtrY);
  OutTextXY(0, MaxY - 20, 'Pfeiltasten whlen Punkt');
  OutTextXY(0, MaxY - 10, 'Besttigen mit [Enter]');
  OutTextXY(0,         0, 'X-Koordinate:');
  OutTextXY(0,        20, 'Y-Koordinate:');
  Line(CtrX - 10, CtrY +  6, CtrX - 10, CtrY +  9);
  Line(CtrX - 10, CtrY + 11, CtrX - 10, CtrY + 14);
  Line(CtrX - 15, CtrY + 10, CtrX - 12, CtrY + 10);
  Line(CtrX -  8, CtrY + 10, CtrX -  5, CtrY + 10);
  GetMem(Cursor, ImageSize(CtrX - 15, CtrY +  6,
                           CtrX -  5, CtrY + 14));
  GetImage(CtrX - 15, CtrY + 6, CtrX - 5, CtrY + 14,
           Cursor^);
  WITH JuliaData DO BEGIN
    XPos := CtrX - 15;
    YPos := CtrY + 6;
    REPEAT
      Str(( XPos + 5 - CtrX) * 1.6 / CtrX:3:7, PosX);
      Str((-YPos - 4 + CtrY) * 1.2 / CtrY:3:7, PosY);
      OutTextXY(10, 10, PosX);
      OutTextXY(10, 30, PosY);
      ch := ReadKey;
      IF NOT (ch IN [#13, #27]) THEN ch1 := ReadKey;
      IF ch = #27 THEN BEGIN
        CloseGraph;
        Halt;
      END;
      SetColor(EGABlack);
      OutTextXY(10, 10, PosX);
      OutTextXY(10, 30, PosY);
      SetColor(GetMaxColor);
      PutImage(XPos, YPos, Cursor^, XORPut);
      IF ch = #0 THEN BEGIN
        CASE ch1 OF
        (* einfache Cursortasten *)
          #77: IF XPos < MaxX-10 THEN Inc(XPos);
          #75: IF XPos > 0       THEN Dec(XPos);
          #80: IF YPos < MaxY-10 THEN Inc(YPos);
          #72: IF YPos > 0       THEN Dec(YPos);
         (* Home, PgUp, End, PgDn *)
          #71: IF (YPos > 0) OR (XPos > 0) THEN BEGIN
                 YPos :=0;
                 XPos := 0;
               END;
          #79: IF (XPos > 0) OR (YPos < MaxY-10) THEN BEGIN
                 XPos := 0;
                 YPos := MaxY - 10;
               END;
          #73: IF (XPos < MaxX-10) OR (YPos > 0) THEN BEGIN
                 XPos := MaxX - 10;
                 YPos := 0;
               END;
          #81: IF (XPos < MaxX-10) OR
                  (YPos < MaxY-10) THEN BEGIN
                 XPos := MaxX - 10;
                 YPos := MaxY - 10;
               END;
         (* Ctrl + Cursor -> <- *)
         #116: IF XPos < MaxX - 20 THEN Inc(XPos, 10);
         #115: IF XPos > 9         THEN Dec(XPos, 10);
         (* Ctrl+Home, Ctrl+End ... *)
         #119: IF (YPos > 0) AND (XPos > 0) THEN BEGIN
                 Dec(XPos);
                 Dec(YPos);
               END;
         #117: IF (XPos > 0) AND (YPos < MaxY-10) THEN BEGIN
                 Dec(XPos);
                 Inc(YPos);
               END;
         #132: IF (XPos < MaxX-10) AND (YPos > 0) THEN BEGIN
                 Inc(XPos);
                 Dec(YPos);
               END;
         #118: IF (XPos < MaxX-10) AND
                  (YPos < MaxY-10) THEN BEGIN
                 Inc(XPos);
                 Inc(YPos);
               END;
        END;
        PutImage(XPos, YPos, Cursor^, XORPut);
      END;
    UNTIL ch = #13;
    Inc(XPos, 5);
    Inc(YPos, 4);
  END;
  ClearDevice;
END;

(* ------------------------------------------------------ *)

PROCEDURE Julia(JuliaData: tJuliaData);
VAR
  i, j, k       : INTEGER;
  a, b, x, y, z : Double;
BEGIN
  WITH JuliaData DO BEGIN
    a := ( XPos - CtrX) * 1.6 / CtrX;
    b := (-YPos + CtrY) * 1.2 / CtrY;
    OutTextXY( 0,  0, 'Julia-Menge fr');
    OutTextXY( 0, 10, 'X-Koordinate:');
    OutTextXY(10, 20, PosX);
    OutTextXY( 0, 30, 'Y-Koordinate:');
    OutTextXY(10, 40, PosY);
  END;
  OutTextXY(0, MaxY - 10, 'Ende mit beliebiger Taste');
  FOR j := 0 TO CtrX DO BEGIN
    FOR k := - CtrY TO CtrY DO BEGIN
      x := j * 2.4 / CtrX;
      y := k * 1.6 / CtrY;
      i := 0;
      WHILE (i < 100) AND ((x * x + y * y) < 4) DO BEGIN
        z := x * x - y * y + a;
        y := 2 * x * y + b;
        x := z;
        Inc(i);
      END;
      PutPixel(j + CtrX, k + CtrY, i AND MaxColor);
      PutPixel(CtrX - j, CtrY - k, i AND MaxColor);
    END;
    IF KeyPressed THEN BEGIN
      CloseGraph;
      Halt;
    END;
  END;
END;

(* ====================================================== *)

VAR
  ch             : CHAR;
  StartX, StartY,
  GraphDriver,
  GraphMode      : INTEGER;
  PCXFileName    : PathStr;
  JuliaData      : tJuliaData;
  Message        : STRING;

BEGIN
  IF ParamCount = 0 THEN PCXFileName := 'JULIA'
                    ELSE PCXFileName := ParamStr(1);
  IF Pos('?', PCXFileName) > 0 THEN BEGIN
    WriteLn('Berechnung von JULIA-Mengen.');
    WriteLn('Aufruf: JULIA <PCXDateiname>');
    Halt;
  END;
  DirectVideo := TRUE;
  DetectGraph(GraphDriver, GraphMode);
  InitGraph(GraphDriver, GraphMode, GetEnv('BGIPATH'));
  IF GraphResult = grOk THEN BEGIN
    MaxX     := GetMaxX;
    MaxY     := GetMaxY;
    CtrX     := MaxX DIV 2;
    CtrY     := MaxY DIV 2;
    MaxColor := GetMaxColor;
    SetTextStyle(DefaultFont, HorizDir, 1);
    REPEAT
      SetColor(GetMaxColor);
      Mandelbrot(JuliaData);
      Julia(JuliaData);
      (*  nur vollstndige Bilder werden abgespeichert:   *)
      SavePicture(GraphDriver, GraphMode, PCXFileName,
                  JuliaData.PosX, JuliaData.PosY);
      Write(^G);
      Message := 'Nochmal? (j/n) _';
      SetFillStyle(SolidFill, Blue);
      Bar(CtrX - TextWidth(Message),
          CtrY - TextHeight(Message) * 2,
          CtrX + TextWidth(Message),
          CtrY + TextHeight(Message) * 2);
      SetColor(GetMaxColor);
      Rectangle(CtrX - TextWidth(Message),
                CtrY - TextHeight(Message) * 2,
                CtrX + TextWidth(Message),
                CtrY + TextHeight(Message) * 2);
      OutTextXY(CtrX - TextWidth(Message)  DIV 2 ,
                CtrY - TextHeight(Message) DIV 2, Message);
      REPEAT
        ch := UpCase(ReadKey);
        IF ch = #27 THEN ch := 'N';
      UNTIL ch IN ['J', 'N'];
      ClearDevice;
    UNTIL ch = 'N';
    CloseGraph;
  END ELSE WriteLn('Grafik-Initialisierungsfehler!');
END.

(* ====================================================== *)
(*                   Ende von JULIA.PAS                   *)
