(* ====================================================== *)
(*                      PCXTOOLS.PAS                      *)
(*            Copyright (C) 1990, 1994 toolbox            *)
(*         Abspeichern von 16-Farben-PCX-Dateien          *)
(* ====================================================== *)
{$A+,B-,D-,E-,F-,G-,I-,L-,N-,O-,P-,Q-,R-,S-,T-,V-,X-}

UNIT PCXTools;

INTERFACE

CONST
  ActivePage : WORD = 0;

VAR
  XMin, XMax,              (* Diese Variablen mssen im   *)
  YMin, YMax  : WORD;      (* Hauptprogramm fr das Bild  *)
  Screen, Col : BYTE;      (* festgelegt werden           *)

FUNCTION BGItoPCX (gd, gm : INTEGER;
                   Name   : STRING): INTEGER;

IMPLEMENTATION

USES Dos;

TYPE
  tPlaneType = ARRAY[0..767] OF BYTE;
  pPlaneType = ^tPlaneType;
  tScanLine  = ARRAY[0..3] OF pPlaneType;
  tPCXHeader = RECORD
                 Creator    : BYTE;
                 Version    : BYTE;
                 Encoding   : BYTE;
                 Bits       : BYTE;
                 XMin, YMin,
                 XMax, YMax : INTEGER;
                 HRes, VRes : INTEGER;
                 Palette    : ARRAY[0..15, 0..2] OF BYTE;
                 VMode      : BYTE;
                 Planes     : BYTE;
                 BytePerLine: INTEGER;
                 PaletteInfo: INTEGER;
                 Dummy      : ARRAY[0..57] OF BYTE;
               END;

CONST
  BlockSize : WORD = 512;

  PCXDefaultPalette : ARRAY[0..15, 0..2] OF BYTE =
   ((  0,   0,   0), (  0,   0, 170), (  0, 170,   0),
    (  0, 170, 170), (170,   0,   0), (170,   0, 170),
    (170, 170,   0), (170, 170, 170), ( 85,  85,  85),
    ( 85,  85, 255), ( 85, 255,  85), ( 85, 255, 255),
    (255,  85,  85), (255,  85, 255), (255, 255,  85),
    (255, 255, 255));

VAR
  PCXBuf    : ARRAY[1..512] OF BYTE;
  i, j      : WORD;
  SPtr      : Pointer;
  PCXHeader : tPCXHeader;
  ScanLine  : tScanLine;



PROCEDURE SetReadPlane(nr: BYTE);
BEGIN
  Port[$3CE] := 4;
  Port[$3CF] := nr;
END;

PROCEDURE SetWritePlane(nr: BYTE);
BEGIN
  Port[$3CE] := 2;
  Port[$3C5] := 1 SHL nr;
END;

PROCEDURE SetEGAReg(nr, Value : BYTE);
BEGIN
  Port[$3CE] := nr;
  Port[$3CF] := Value;
END;

FUNCTION GetPCXHeader(VAR PCXHeader : tPCXHeader;
                          Name      : STRING): INTEGER;
VAR
  f: File;
BEGIN
  FillChar(PCXHeader, 128, 0);
  Assign(f, Name);
  Reset(f, 1);
  DosError := IOResult;
  IF DosError <> 0 THEN BEGIN
    GetPCXHeader := DosError;
    Exit;
  END;
  BlockRead(f, PCXHeader, 128);
  DosError := IOResult;
  IF DosError <> 0 THEN BEGIN
    GetPCXHeader := DosError;
    Close(f);
    Exit;
  END;
  Close(f);
  GetPCXHeader := IOResult;
  IF (PCXHeader.Version  > 5) OR
     (PCXHeader.Encoding > 1) THEN GetPCXHeader := - 1;
END;

FUNCTION WritePCXHeader(VAR PCXHeader: tPCXHeader;
                            Name     : STRING): INTEGER;
VAR
  f: File;
BEGIN
  Assign(f, Name);
  Rewrite(f, 1);
  DosError := IOResult;
  IF DosError <> 0 THEN BEGIN
    WritePCXHeader := DosError;
    Exit;
  END;
  BlockWrite(f, PCXHeader, 128);
  DosError := IOResult;
  IF DosError <> 0 THEN BEGIN
    WritePCXHeader := DosError;
    Close(f);
    IF IOResult <> 0 THEN Exit;
  END;
  Close(f);
  WritePCXHeader := IOResult
END;

FUNCTION GetPCXByte(VAR f: File): BYTE;
CONST
  Count   : BYTE    = 0;
  Value   : BYTE    = 0;
  p       : WORD    = 512;
  EndFile : BOOLEAN = FALSE;
VAR
  Temp    : BYTE;

  PROCEDURE ReadBlock;
  VAR
    Result: WORD;
  BEGIN
    IF EoF(f) THEN EndFile := TRUE ELSE BEGIN
      BlockRead(f, PCXBuf, BlockSize, Result);
      IF Result < BlockSize THEN BlockSize := Result;
      p := 1;
    END;
  END;

  FUNCTION GetByte: BYTE;
  BEGIN
    IF EndFile THEN GetByte := 0 ELSE BEGIN
      IF p = BlockSize THEN ReadBlock
                       ELSE Inc(p);
      GetByte := PCXBuf[p];
    END;
  END;

BEGIN
  IF Count > 0 THEN BEGIN
    Dec(Count);
    GetPCXByte := Value;
    Exit;
  END;
  Temp := GetByte;
  IF Temp AND $C0 = $C0 THEN BEGIN
    Count := Temp AND $3F - 1;
    Value := GetByte;
  END ELSE BEGIN
    Count := 0;
    Value := Temp;
  END;
  GetPCXByte := Value;
END;

FUNCTION WritePCXByte(VAR f        : File;
                      Value, Count : BYTE): INTEGER;
CONST
  Total : LongInt = 0;
BEGIN
  IF (Count = 1) AND ($C0 <> $C0 AND Value) THEN BEGIN
    BlockWrite(f, Value, 1);
    WritePCXByte := IOResult;
    Inc(Total);
  END ELSE BEGIN
    Count := $C0 OR Count;
    BlockWrite(f, Count, 1);
    WritePCXByte := IOResult;
    BlockWrite(f, Value, 1);
    WritePCXByte := IOResult;
    Inc(Total, 2);
  END;
END;

FUNCTION WritePCXLine(VAR f  :   File;
                      VAR Buf: pPlaneType;
                      Count  : BYTE): INTEGER;
VAR
  This, Last     : BYTE;
  CPtr, RunCount : BYTE;
BEGIN
  WritePCXLine := 0;
  Last         := Buf^[0];
  RunCount     := 1;
  FOR CPtr := 1 TO Count - 1 DO BEGIN
    IF Buf^[CPtr] = Last THEN BEGIN
      Inc(RunCount);
      IF RunCount = 63 THEN BEGIN
        DosError := WritePCXByte(f, Last, RunCount);
        IF DosError <> 0 THEN BEGIN
          WritePCXLine := DosError;
          Exit;
        END;
        RunCount := 0;
      END;
    END ELSE BEGIN
      DosError := WritePCXByte(f, Last, RunCount);
      IF DosError <> 0 THEN BEGIN
        WritePCXLine := DosError;
        Exit;
      END;
      Last := Buf^[CPtr];
      RunCount := 1;
    END;
  END;
  IF RunCount > 0 THEN BEGIN
    DosError := WritePCXByte(f, Last, RunCount);
    IF DosError <> 0 THEN WritePCXLine := DosError;
  END;
END;

PROCEDURE DefPCXPalette(VAR PCXHeader: tPCXHeader;
                            ColType  : BYTE);
VAR
  i, j: INTEGER;
BEGIN
  CASE ColType OF
   0: BEGIN
        FillChar(PCXHeader.Palette, 48, 255);
        FillChar(PCXHeader.Palette,  3,   0);
      END;
   1: FOR i := 0 TO 15 DO BEGIN
        IF Odd(i) THEN
          FOR j := 0 TO 2 DO PCXHeader.Palette[i, j] := 240
        ELSE
          FOR j := 0 TO 2 DO PCXHeader.Palette[i, j] := 0;
      END;
   2: Move(PCXDefaultPalette, PCXHeader.Palette, 48);
  END;
END;

FUNCTION BGItoPCX(gd, gm:INTEGER; Name: STRING): INTEGER;
VAR
  f    : File;
  Page : INTEGER;

  PROCEDURE ErrorCheck;
  BEGIN
    IF DosError <> 0 THEN BEGIN
      BGItoPCX := DosError;
      Exit;
    END;
  END;

  PROCEDURE ReopenFile;
  BEGIN
    Assign(f, Name);
    Reset(f, 1);
    DosError := IOResult;
    ErrorCheck;
    Seek(f, 128);
    DosError := IOResult;
    ErrorCheck;
  END;

BEGIN
  FillChar(PCXHeader, 128, 0);
  PCXHeader.Creator     := 10;
  PCXHeader.Version     := 3;
  PCXHeader.Encoding    := 1;
  PCXHeader.Bits        := 1;
  PCXHeader.XMin        := XMin;
  PCXHeader.XMax        := XMax;
  PCXHeader.YMin        := YMin;
  PCXHeader.YMax        := YMax;
  PCXHeader.HRes        := 75;
  PCXHeader.VRes        := 75;
  PCXHeader.PaletteInfo := 1;
  CASE gd OF
    3..5, 9:
      BEGIN
        CASE gm OF
          0: BEGIN
               PCXHeader.Planes      := 4;
               PCXHeader.BytePerLine := 80;
               DefPCXPalette(PCXHeader, 2);
               DosError := WritePCXHeader(PCXHeader, Name);
               ErrorCheck;
               ReopenFile;
               FOR i := 0 TO 199 DO BEGIN
                 SPtr := Ptr(SegA000 + $400 * ActivePage,
                             i * 80);
                 FOR Page := 0 TO 3 DO BEGIN
                   SetReadPlane(Page);
                   Move(SPtr^, ScanLine[0]^, 80);
                   DosError := WritePCXLine(f, ScanLine[0],
                                               80);
                   ErrorCheck;
                 END;
               END;
             END;
          1: BEGIN
               PCXHeader.Planes      := 4;
               PCXHeader.BytePerLine := 80;
               DefPCXPalette(PCXHeader, 2);
               DosError := WritePCXHeader(PCXHeader, Name);
               ErrorCheck;
               ReopenFile;
               FOR i := 0 TO 349 DO BEGIN
                 SPtr := Ptr(SegA000 + $800 * ActivePage,
                             i * 80);
                 FOR Page := 0 TO 3 DO BEGIN
                   SetReadPlane(Page);
                   Move(SPtr^, ScanLine[0]^, 80);
                   DosError := WritePCXLine(f, ScanLine[0],
                                            80);
                   ErrorCheck;
                 END;
               END;
             END;
          2: BEGIN
               PCXHeader.Planes      := 4;
               PCXHeader.BytePerLine := 80;
               DefPCXPalette(PCXHeader, 2);
               DosError := WritePCXHeader(PCXHeader, Name);
               ErrorCheck;
               ReopenFile;
               FOR i := 0 TO 479 DO BEGIN
                 SPtr := Ptr(SegA000 + $960 * ActivePage,
                             i * 80);
                 FOR Page := 0 TO 3 DO BEGIN
                   SetReadPlane(Page);
                   Move(SPtr^, ScanLine[0]^, 80);
                   DosError := WritePCXLine(f, ScanLine[0],
                                            80);
                   ErrorCheck;
                 END;
               END;
             END;
          3: BEGIN
               PCXHeader.Planes      := 1;
               PCXHeader.BytePerLine := 80;
               PCXHeader.Version     := 2;
               DefPCXPalette(PCXHeader, 0);
               DosError := WritePCXHeader(PCXHeader, Name);
               ErrorCheck;
               ReopenFile;
               SetReadPlane(0);
               FOR i := 0 TO 349 DO BEGIN
                 SPtr := Ptr(SegA000 + $800 * ActivePage,
                             i * 80);
                 Move(SPtr^, ScanLine[0]^, 80);
                 BlockWrite(f, ScanLine[0]^, 80);
                 DosError := WritePCXLine(f, ScanLine[0],
                                          80);
                 ErrorCheck;
               END;
             END;
            END;
          END;
   7: BEGIN
        PCXHeader.Planes      := 1;
        PCXHeader.BytePerLine := 90;
        PCXHeader.Version     := 2;
        DefPCXPalette(PCXHeader, 0);
        DosError := WritePCXHeader(PCXHeader, Name);
        ErrorCheck;
        ReopenFile;
        FOR i := 0 TO 347 DO BEGIN
          SPtr := Ptr(SegB000, WORD((i AND 3)
                      SHL 13 + 90 * (i SHR 2)));
          Move (SPtr^, ScanLine[0]^, 90);
          DosError := WritePCXLine(f, ScanLine[0], 90);
          ErrorCheck;
        END;
      END;
1, 2: BEGIN
        PCXHeader.Planes      := 1;
        PCXHeader.Bits        := 2;
        PCXHeader.BytePerLine := 80;
        IF (gd = 2) AND (gm = 3) THEN BEGIN
          j         := 479;
          PCXHeader.Bits := 1;
        END ELSE j := 199;
        IF gm = 4 THEN PCXHeader.Bits := 1;
        PCXHeader.Version := 5;
        DefPCXPalette(PCXHeader, 1);
        DosError := WritePCXHeader(PCXHeader, Name);
        ErrorCheck;
        ReopenFile;
        FOR i := 0 TO j DO BEGIN
          SPtr := Ptr(SegB800, WORD((i AND 1) SHL 13 + 80 *
                      (i SHR 1)));
          Move(SPtr^, ScanLine[0]^, 80);
          DosError := WritePCXLine(f, ScanLine[0], 80);
          ErrorCheck;
        END;
      END;
  END;
  Close(f);
  IF IOResult <> 0 THEN ;
END;


PROCEDURE EncodePCX(VAR PCXHeader : tPCXHeader;
                        Name      : STRING);
VAR
  f    : File;
  i, j : INTEGER;
  l    : WORD;
  n, w : BYTE;
  p    : Pointer;
BEGIN
  Assign(f, Name);
  Reset(f, 1);
  Seek(f, 128);
  SetEGAReg(5, 0);
  SetEGAReg(1, 0);
  FOR l := 0 TO PCXHeader.YMax - PCXHeader.YMin DO BEGIN
    FOR j := 0 TO PCXHeader.Planes - 1 DO BEGIN
      FOR i := 0 TO PCXHeader.BytePerLine - 1 DO BEGIN
        ScanLine[j]^[i] := GetPCXByte(f);
      END;
    END;
    CASE Screen OF
      0: BEGIN
           p := Ptr($A000, l * 80);
           FOR j := 0 TO PCXHeader.Planes - 1 DO BEGIN
             SetWritePlane(j);
             Move(ScanLine[j]^, p^, 80);
           END;
         END;
      1: BEGIN
           p := Ptr($B800, WORD((l AND 3) SHL 13 + 90 *
                           (l SHR 2)));
           Move(ScanLine[0]^, p^, 80);
         END;
      2: BEGIN
           p := Ptr($B800, WORD((l AND 1) SHL 13 + 80 *
                           (l SHR 1)));
           Move(ScanLine[0]^, p^, 80);
         END;
    END;
  END;
  Close(f);
  SetWritePlane($F);
END;

BEGIN
  GetMem(ScanLine[0], 90);
END.

(* ====================================================== *)
(*                   Ende von PCXTOOLS.PAS                *)
