(* ====================================================== *)
(*                     SCHNEEFL.PAS                       *)
(*   Copyright (C) 1989, 1993 Uwe Peter Schmit & toolbox  *)
(*         Compiler: Turbo/Borland Pascal 7.0             *)
(* ====================================================== *)
(* gute Werte fr [S]chneflocke sind beispielsweise       *)
(*     1: -100, 100 und 2: 100, -100                      *)
(*     1:  150,-150 und 2: -150, 150 ...                  *)
(* ====================================================== *)
{$A+,B-,D-,E+,F-,G-,I-,L-,N+,O-,P-,Q-,R-,S-,T-,V-,X+,Y-}
{$M 16384,0,$A0000}

PROGRAM Schneeflocke;

USES Crt, Dos, Graph;

TYPE
  tDots    = RECORD
              x, y : INTEGER
             END;
  pCorner  = ^tCorner;
  tCorner  = RECORD
              Dot  : tDots;
              Next : pCorner
             END;
VAR
  IterMax                   : BYTE;
  GraphDriver, GraphMode, i : INTEGER;
  YDif                      : Double;
  p1, p2, p3                : tDots;
  s                         : pCorner;
  p                         : Pointer;
  MaxX, MaxY                : INTEGER;
  Center                    : tDots;

  PROCEDURE Triangle;
  VAR
    h1, h2 : pCorner;
  BEGIN
    Mark(p);
    New(s);
    s^.Dot   := p1;
    New(h1);
    h1^.Dot  := p3;
    s^.Next  := h1;
    New(h2);
    h2^.Dot  := p2;
    h1^.Next := h2;
    h2^.Next := s;
  END;

  PROCEDURE PaintTriangle;
  VAR
    h : pCorner;
  BEGIN
    SetColor(White);
    h := s^.Next;
    MoveTo(s^.Dot.x + MaxX DIV 2,
           Round(YDif*s^.Dot.y + MaxY DIV 2));
    WHILE h <> s DO BEGIN
      LineTo(h^.Dot.x + MaxX DIV 2,
             Round(YDif*h^.Dot.y + MaxY DIV 2));
      h := h^.Next
    END;
    LineTo(s^.Dot.x + MaxX DIV 2,
           Round(YDif*s^.Dot.y + MaxY DIV 2));
  END;

  PROCEDURE CalculateP3;
  CONST
    m = 0.8660254;   (* 0.5 * SQRT(3) / Hhe des Dreiecks *)
  VAR
    h, hs  : tDots;
  BEGIN
    h.x  := p2.x-p1.x;        (* h-senkrecht: (-h.y, h.x) *)
    h.y  := p2.y-p1.y;        (* p3 = p1+h+m*h-senkrecht  *)
    p3.x := Round(p1.x + 0.5 * h.x - m * h.y);
    p3.y := Round(p1.y + 0.5 * h.y + m * h.x)
  END;

  PROCEDURE Koch;
  VAR
    h1, h2,
    e1, e2  : pCorner;
    i, j    : ShortInt;
    x, y    : INTEGER;
  BEGIN
    h1 := s;
    h2 := h1^.Next;
    REPEAT
      x    := h2^.Dot.x - h1^.Dot.x;
      y    := h2^.Dot.y - h1^.Dot.y;
      p1.x := Round(h1^.Dot.x + x / 3);
      p1.y := Round(h1^.Dot.y + y / 3);
      p2.x := Round(h2^.Dot.x - x / 3);
      p2.y := Round(h2^.Dot.y - y / 3);
      SetColor(Black);
      FOR i := -1 TO 1 DO
        FOR j := -1 TO 1 DO BEGIN
          MoveTo(p1.x + MaxX DIV 2,
                 Round(YDif * p1.y) + i + MaxY DIV 2);
          LineTo(p2.x + MaxX DIV 2,
                 Round(YDif * p2.y) + j + MaxY DIV 2);
        END;                        (* alte Linie lschen *)
      CalculateP3;
      SetColor(Random(White));
      MoveTo(p1.x + MaxX DIV 2,
             Round(YDif * p1.y) + MaxY DIV 2); (* neues   *)
      LineTo(p3.x + MaxX DIV 2,
             Round(YDif * p3.y) + MaxY DIV 2); (* Dreieck *)
      LineTo(p2.x + MaxX DIV 2,
             Round(YDif * p2.y) + MaxY DIV 2); (* zeichnen*)
      New(e1);
      e1^.Dot := p1;
      h1^.Next  := e1;
      New(e2);
      e2^.Dot := p3;
      e1^.Next  := e2;
      New(e1);
      e1^.Dot := p2;
      e2^.Next  := e1;
      e1^.Next  := h2;
      h1        := h2;
      h2        := h1^.Next;
    UNTIL h1 = s;
  END;

  PROCEDURE CalcMid;
  CONST
    m1 = 0.8660254;                        (* SQRT(3) / 2 *)
    m2 = 0.2886751;                        (* SQRT(3) / 6 *)
  VAR
    h, hs  : tDots;
  BEGIN
    h.x      := p2.x-p1.x;
    h.y      := p2.y-p1.y;    (* h-senkrecht: (-h.y, h.x) *)
    p3.x     := Round(p1.x + h.x / 2 - m1 * h.y);
    p3.y     := Round(p1.y + h.y / 2 + m1 * h.x);
    Center.x := Round(p1.x + h.x / 2 - m2 * h.y);
    Center.y := Round(p1.y + h.y / 2 + m2 * h.x)
  END;

  PROCEDURE Snow;
  VAR
    i,    j    : BYTE;
    h1,   h2,
    Pkt1, Pkt2 : tDots;

  BEGIN
    CalcMid;
    Pkt1 := p1;
    Pkt2 := p2;
    h1.x := Center.x - p1.x;
    h1.y := Center.y - p1.y;
    h2.x := Center.x - p2.x;
    h2.y := Center.y - p2.y;
    FOR i := 0 TO 9 DO BEGIN
      p1.x := Round(Pkt1.x + i / 10 * h1.x);
      p1.y := Round(Pkt1.y + i / 10 * h1.y);
      p2.x := Round(Pkt2.x + i / 10 * h2.x);
      p2.y := Round(Pkt2.y + i / 10 * h2.y);
      CalculateP3;
      Triangle;
      PaintTriangle;
      FOR j := 1 TO IterMax DO Koch;
      Release(p);
    END;
  END;

  PROCEDURE Menu;
  VAR
    ch : CHAR;
    i  : BYTE;

    PROCEDURE ReadData;
    BEGIN
      WriteLn(^J^J^J^J);
      WriteLn('------------------------------------------',
              ^J);
      Write('   Punkt 1 (X-Koordinate) : ');
      ReadLn(p1.x);
      Write('   Punkt 1 (Y-Koordinate) : ');
      ReadLn(p1.y);
      WriteLn;
      Write('   Punkt 2 (X-Koordinate) : ');
      ReadLn(p2.x);
      Write('   Punkt 2 (Y-Koordinate) : ');
      ReadLn(p2.y);
      CalculateP3;
      WriteLn;
      WriteLn('   Punkt 3 (X-Koordinate) : ', p3.x);
      WriteLn('   Punkt 3 (Y-Koordinate) : ', p3.y);
      Write(^J, '   Iterationen (max.6)    :');
      ReadLn(IterMax);
      IF IterMax > 6 THEN IterMax := 6;
                        (* --> sonst luft der Hpf ber! *)
    END;
  BEGIN
    REPEAT
      RestoreCrtMode;
      WriteLn(^J'------------------------------------------'
            + ^J);
      WriteLn('   [K]och''sche Schneeflocke',^J);
      WriteLn('   [S]chneeflocke',^J);
      WriteLn('   [E]nde',^J);
      WriteLn('------------------------------------------',
              ^J);
      REPEAT
        ch := UpCase(ReadKey);
      UNTIL ch IN ['K', 'S', 'E', #27];
      CASE ch OF
        'K' : BEGIN
                ReadData;
                SetGraphMode(GraphMode);
                Triangle;
                PaintTriangle;
                FOR i := 1 TO IterMax DO Koch;
                Release(p);
                Write(^G);
                ReadKey;
              END;
        'S' : BEGIN
                ReadData;
                SetGraphMode(GraphMode);
                Snow;
                Write(^G);
                ReadKey;
              END;
      END;
    UNTIL ch IN ['E', #27];
  END;

VAR
  XAsp, YAsp : WORD;

BEGIN
  GraphDriver := Detect;
  InitGraph(GraphDriver, GraphMode, GetEnv('BGIPATH'));
  IF GraphResult = grOk THEN BEGIN
    DirectVideo := FALSE;
    MaxX        := GetMaxX;
    MaxY        := GetMaxY;
    Randomize;
    GetAspectRatio(XAsp, YAsp);
    YDif        := XAsp / YAsp;
    Menu;
    CloseGraph;
  END ELSE WriteLn('Grafik-Initialisierungsfehler!');
END.

(* ====================================================== *)
(*                Ende von SCHNEEFL.PAS                   *)
