(*========================================================*)
(*                      HELPTOOL.PAS                      *)
(*           (C) 1993 Kai Jouen & DMV-Verlag             *)
(*      Compiler: Borland Pascal 7.0, Windows Target      *)
{*       Verndert fr Turbo Pascal fr Windows 1.5       *}
(*========================================================*)
{$A+,B-,D-,F-,G+,I-,L-,N-,R-,S+,V-,W+,X+}
{$M 8192,8192}

PROGRAM HelpTool;

{$R HelpTool.RES}

USES
  WinTypes, WinProcs, WinDos,
{$IFNDEF Ver70}
  WObjects, StdDlgs, StdWnds,
{$ELSE}
  oWindows, oStdDlgs, oStdWnds, oDialogs,
{$ENDIF}
  Strings, RTFGen;

CONST
  cm_GenerateFiles     = 101;
  cm_GenerateHelp      = 102;
  cm_EditFiles         = 103;
  EditorCount: INTEGER = 0;

TYPE
  tMenuState = (msEnable, msDisable);

  thtApplication = OBJECT(tApplication)
    PROCEDURE InitMainWindow; VIRTUAL;
    PROCEDURE InitInstance;   VIRTUAL;
  END;

  pErrorWindow = ^tErrorWindow;
  tErrorWindow = OBJECT(tFileWindow)
    CONSTRUCTOR Init(aParent : pWindowsObject;
                     aTitle, aFileName : pChar);
    FUNCTION    CanClose : BOOLEAN; VIRTUAL;
    DESTRUCTOR  Done; VIRTUAL;
  END;

  phtWindow = ^thtWindow;
  thtWindow = OBJECT(tMDIWindow)
    Name        : ARRAY[0..80] OF CHAR;
    ErrorWindow : pErrorWindow;
    HPJWindow   : pWindowsObject;
    IncWindow   : pWindowsObject;
    CONSTRUCTOR Init(aTitle: pChar; aMenu: hMenu);
    PROCEDURE   SetupWindow; VIRTUAL;
    PROCEDURE   GetWindowClass(VAR Fensterklasse: tWndClass);
      VIRTUAL;
    PROCEDURE   NewFile(VAR Msg: tMessage);
      VIRTUAL cm_First + cm_MDIFileNew;
    PROCEDURE OpenFile(VAR Msg: tMessage);
      VIRTUAL cm_First + cm_MDIFileOpen;
    PROCEDURE CMFileSaveAs(VAR Msg: tMessage);
      VIRTUAL cm_First + cm_FileSaveAs;
    PROCEDURE GenFiles(VAR Msg: tMessage);
      VIRTUAL cm_First + cm_GenerateFiles;
    PROCEDURE GenHelp(VAR Msg: tMessage);
      VIRTUAL cm_First + cm_GenerateHelp;
    PROCEDURE EditFiles(VAR Msg: tMessage);
      VIRTUAL cm_First + cm_EditFiles;
  END;

  pFileEditor = ^tFileEditor;
  tFileEditor = OBJECT(tFileWindow)
    CONSTRUCTOR Init(aParent: pWindowsObject;
                     aFileName: pChar);
    DESTRUCTOR  Done; VIRTUAL;
    PROCEDURE   GetWindowClass(VAR aWndClass: tWndClass);
      VIRTUAL;
    FUNCTION    GetClassName: pChar; VIRTUAL;
  END;

VAR
  InputFile,
  RTFName,
  HPJName,
  IncName   : STRING;

PROCEDURE MenuItems(State: tMenuState);

  PROCEDURE ModifyCommand(Command: WORD);
  VAR
    NewState: WORD;
  BEGIN
    NewState := mf_ByCommand;
    IF State = msEnable THEN
      Inc(NewState, mf_Enabled)
    ELSE
      Inc(NewState, mf_Disabled + mf_Grayed);
    EnableMenuItem(pWindow(
                    Application^.MainWindow)^.Attr.Menu,
                    Command, NewState);
  END;

BEGIN
  IF Application^.MainWindow^.hWindow = 0 THEN Exit;
  ModifyCommand(cm_FileSave);
  ModifyCommand(cm_FileSaveAs);
  ModifyCommand(cm_ArrangeIcons);
  ModifyCommand(cm_TileChildren);
  ModifyCommand(cm_CascadeChildren);
  ModifyCommand(cm_CloseChildren);
  ModifyCommand(cm_EditCut);
  ModifyCommand(cm_EditCopy);
  ModifyCommand(cm_EditPaste);
  ModifyCommand(cm_EditDelete);
  ModifyCommand(cm_EditClear);
  ModifyCommand(cm_EditUndo);
  ModifyCommand(cm_EditFind);
  ModifyCommand(cm_EditReplace);
  ModifyCommand(cm_EditFindNext);
  ModifyCommand(cm_GenerateFiles);
  ModifyCommand(cm_GenerateHelp);
  ModifyCommand(cm_EditFiles);
END;

PROCEDURE IncEditors;
BEGIN
  IF EditorCount = 0 THEN MenuItems(msEnable);
  Inc(EditorCount);
END;

PROCEDURE DecEditors;
BEGIN
  Dec(EditorCount);
  IF EditorCount = 0 THEN BEGIN
    MenuItems(msDisable);
    InputFile := '';
    RTFName   := '';
    HPJName   := '';
    IncName   := '';
  END;
END;

FUNCTION Exists(Name : STRING): BOOLEAN;
VAR
  f : Text;
BEGIN
  Exists := TRUE;
  {$I-}
  Assign(f,Name);
  Reset(f);
  IF IOResult <> 0 THEN Exists := FALSE ELSE Close(f);
  {$I+}
END;

CONSTRUCTOR tErrorWindow.Init(aParent : pWindowsObject;
                              aTitle, aFileName: pChar);
BEGIN
  tFileWindow.Init(aParent, aTitle, aFileName);
  phtWindow(Parent)^.ErrorWindow := @Self;
  Attr.H := 120;
  Editor^.Attr.Style := Editor^.Attr.Style XOR ws_HScroll;
END;

FUNCTION tErrorWindow.CanClose : BOOLEAN;
BEGIN
  CanClose := TRUE;
END;

DESTRUCTOR tErrorWindow.Done;
BEGIN
  phtWindow(Parent)^.ErrorWindow := NIL;
  tFileWindow.Done;
END;

CONSTRUCTOR tFileEditor.Init(aParent  : pWindowsObject;
                             aFileName: pChar);
BEGIN
  tFileWindow.Init(aParent, '', aFileName);
  IncEditors;
END;

DESTRUCTOR tFileEditor.Done;
BEGIN
  DecEditors;
  tFileWindow.Done;
END;

PROCEDURE tFileEditor.GetWindowClass(
          VAR aWndClass: tWndClass);
BEGIN
  tFileWindow.GetWindowClass(aWndClass);
  aWndClass.hIcon := LoadIcon(hInstance, 'FileIcon');
END;

FUNCTION tFileEditor.GetClassName: pChar;
BEGIN
  GetClassName := 'FileEditor';
END;

CONSTRUCTOR thtWindow.Init(aTitle:pChar; aMenu:hMenu);
BEGIN
  tMDIWindow.Init(aTitle,aMenu);
  ErrorWindow := NIL;
  HPJWindow   := NIL;
  IncWindow   := NIL;
END;

Procedure   thtWindow.GetWindowClass(Var Fensterklasse: TWndClass);
     Begin
        TWindow.GetWindowClass(Fensterklasse);
        Fensterklasse.hIcon :=  LoadIcon(hInstance,'ICON2');
     End;
            {Ende GetWindowClass}


PROCEDURE thtWindow.NewFile(VAR Msg: tMessage);
BEGIN
  Application^.MakeWindow(New(pFileEditor, Init(@Self, '')));
  InputFile := '';
  RTFName := '';
  HPJName := '';
  IncName := '';
END;

PROCEDURE thtWindow.SetupWindow;
BEGIN
  tMDIWindow.SetupWindow;
  MenuItems(msDisable);
END;

PROCEDURE thtWindow.OpenFile(VAR Msg: tMessage);
VAR
  FileName: ARRAY[0..fsPathName] OF CHAR;
BEGIN
  StrCopy(FileName,'*.Txt');
  IF Application^.ExecDialog(New(pFileDialog,
     Init(@Self, pChar(sd_FileOpen),FileName))) = id_OK THEN
    Application^.MakeWindow(New(pFileEditor,
                            Init(@Self, FileName)));
  InputFile := StrPas(FileName);
  FileName[StrLen(FileName) -
           StrLen(StrRScan(FileName,'.'))] := #0;
  RTFName := StrPas(FileName) + '.Rtf';
  HPJName := StrPas(FileName) + '.Hpj';
  IncName := StrPas(FileName) + '.Inc';
END;

PROCEDURE thtWindow.cmFileSaveAs(VAR Msg: tMessage);
VAR
  FileName: ARRAY[0..fsPathName] OF CHAR;
BEGIN
  Application^.ExecDialog(New(pFileDialog,
                 Init(@Self, pChar(sd_FileSave),FileName)));
  InputFile := StrPas(FileName);
  FileName[StrLen(FileName) -
           StrLen(StrRScan(FileName,'.'))] := #0;
  RTFName := StrPas(FileName) + '.Rtf';
  HPJName := StrPas(FileName) + '.Hpj';
  IncName := StrPas(FileName) + '.Inc';
END;

PROCEDURE thtWindow.GenFiles(VAR Msg: tMessage);
VAR
  P : pDialog;
BEGIN
  P := New(pDialog,Init(@Self,'WaitDlg'));
  P^.Create;
  EnableWindow(hWindow,FALSE);
  P^.Show(sw_Normal);
  SetDlgItemText(P^.hWindow, 101,
                    'Generating Help-Source-Files ...');
  SetDlgItemText(P^.hWindow, 102, 'Please Wait');
  DoIt(InputFile);
  EnableWindow(hWindow,TRUE);
  P^.Destroy;
END;

PROCEDURE thtWindow.GenHelp(VAR Msg : tMessage);
VAR
  IH : tHandle;
  M : tMsg;
  ErrorFileName : ARRAY[0..80] OF CHAR;
  FileInfo : WinDos.tSearchRec;
BEGIN
  IF Exists(HPJName) THEN BEGIN
    StrPCopy(Name,('HC31.Exe ' + HPJName));
                  (* oder HC30.EXE *)
    StrCat(Name, ' > Errors.Log');
    IH := WinExec(Name, sw_Normal);
    IF IH <= 32 THEN
      MessageBox(0, Name, 'Failed to EXEC',
                 mb_OK + mb_IconHand)
    ELSE BEGIN
      REPEAT
        WHILE PeekMessage(M, 0, 0, 0, pm_Remove) DO BEGIN
          IF M.Message = wm_Quit THEN BEGIN
            PostQuitMessage(M.wParam);
            Exit;
          END ELSE BEGIN
            TranslateMessage(M);
            DispatchMessage(M);
          END;
        END;
      UNTIL GetModuleUsage(IH) = 0;
    END;
  END ELSE
    MessageBox(hWindow,
      'HPJ-File not existing. Generate Source Files first.',
      'Error', mb_IconHand);
  StrCopy(ErrorFileName, 'Errors.Log');
  FindFirst(ErrorFileName, faAnyFile, FileInfo);
  IF (DosError = 0) AND (FileInfo.Size > 2) THEN
    IF ErrorWindow <> NIL THEN BEGIN
      SendMessage(ErrorWindow^.hWindow, wm_Close, 0, 0);
      Application^.MakeWindow(New(PErrorWindow,
                   Init(@Self, 'HC Error-Output',
                   ErrorFileName)));
    END;
END;

PROCEDURE thtWindow.EditFiles(VAR Msg: tMessage);
BEGIN
  IF (Exists(HPJName) AND Exists(IncName)) THEN BEGIN
    StrPCopy(Name,HPJName);
    IF HPJWindow <> NIL THEN
      SendMessage(HPJWindow^.hWindow, wm_Close, 0, 0);
    hpjWindow := Application^.MakeWindow(New(pFileEditor,
                                        Init(@Self, Name)));
    StrPCopy(Name,IncName);
    IF IncWindow <> NIL THEN
      SendMessage(IncWindow^.hWindow, wm_Close, 0, 0);
    IncWindow := Application^.MakeWindow(New(pFileEditor,
                                        Init(@Self, Name)));
  END
    ELSE MessageBox(hWindow,
                    'HPJ-and Include-File not existing. ' +
                    'Generate Source Files first.', 'Error',
                    mb_IconHand);
END;

PROCEDURE thtApplication.InitMainWindow;
BEGIN
  MainWindow := New(phtWindow, Init('Hilfe_Generator',
                    LoadMenu(hInstance, 'Commands')));
  phtWindow(MainWindow)^.ChildMenuPos := 3;
END;
                                                 
PROCEDURE thtApplication.InitInstance;
BEGIN
  tApplication.InitInstance;
  IF Status = 0 THEN BEGIN
    hAccTable := LoadAccelerators(hInstance,
                                  'FileCommands');
    IF hAccTable = 0 THEN Status := em_InvalidWindow;
  END;
END;

VAR
  htApplication : thtApplication;

BEGIN
  htApplication.Init('HelpTool');
  htApplication.Run;
  htApplication.Done;
END.

(*========================================================*)
(*                  Ende von HELPTOOL.PAS                 *)
