(* ====================================================== *)
(*                       BLASE.PAS                        *)
(* Berechnung der minimalen Spannung einer Seifenblase.   *)
(* Eingesetzt wird das Odd-Even-Verfahren. Visualisierung *)
(* erfolgt in Form einer farblichen Darstellung.          *)
(*      (c) 1993 Dr. Lothar Wenzel & toolbox              *)
(* ====================================================== *)
{$A-,B+,D-,E+,F-,G+,I-,L+,N+,O-,P-,Q-,R-,S-,T-,V+,X+,Y+}
{$M 16384,0,655360}

PROGRAM Blase;

USES Dos, Graph, Crt;

CONST
  Dimension_X  =  240; { Breite }
  Dimension_Y  =  240; { Tiefe  }
  Links_Oben_X =   10;
  Links_Oben_Y =   10;

TYPE
  Spalte       = ARRAY [1..Dimension_Y] OF Double;

VAR
  Rahmen       : ARRAY [1..Dimension_X] OF ^Spalte;

  PROCEDURE Graph_Init;
  VAR
    gd, gm : INTEGER;
  BEGIN
    gd := Detect;
    InitGraph(gd, gm, GetEnv('BGIPATH'));
    IF GraphResult <> GrOk THEN Halt(1);
  END;

  PROCEDURE Initial;
  VAR
    i, j : INTEGER;
  BEGIN
    FOR i := 1 TO Dimension_X DO New(Rahmen[i]);
    FOR i := 1 TO Dimension_X DO BEGIN
      Rahmen[i]^[1]          :=8+8*Sin(2*Pi*i/Dimension_X);
      Rahmen[i]^[Dimension_Y]:=8+8*Sin(2*Pi*i/Dimension_X);
    END;
    FOR i := 1 TO Dimension_Y DO BEGIN
      Rahmen[1]^[i]          :=8+8*Sin(2*Pi*I/Dimension_Y);
      Rahmen[Dimension_X]^[i]:=8+8*Sin(2*Pi*I/Dimension_Y);
    END;
    FOR i := 2 TO Dimension_X-1 DO
      FOR j := 2 TO Dimension_Y-1 DO BEGIN
        Rahmen[I]^[J]:=(Rahmen[1]^[j]           +
                        Rahmen[Dimension_X]^[j] +
                        Rahmen[i]^[1]           +
                        Rahmen[i]^[Dimension_Y])/4;
      END;
  END;

  PROCEDURE Iteration;
    (* Das ODD-EVEN-Verfahren *)
  VAR
    i, j : INTEGER;
  BEGIN
    (* ODD-Verfahren *)
    FOR i := 2 TO Dimension_X-1 DO
      FOR j := 2 TO Dimension_Y-1 DO BEGIN
        IF ((i+j) MOD 2 = 1) THEN
          Rahmen[i]^[j] :=
           (Rahmen[i-1]^[j] + Rahmen[i+1]^[j] +
            Rahmen[i]^[j-1] + Rahmen[i]^[j+1])/4;
      END;

    (* EVEN-Verfahren *)
    FOR i := 2 TO Dimension_X-1 DO
      FOR j := 2 TO Dimension_Y-1 DO BEGIN
        IF ((i+j) MOD 2 = 0) THEN
          Rahmen[i]^[j]:=
           (Rahmen[i-1]^[j]+Rahmen[i+1]^[j] +
            Rahmen[i]^[j-1]+Rahmen[i]^[j+1])/4;
      END;
  END;

  PROCEDURE Darstellung;
  VAR
    i, j : INTEGER;
  BEGIN
    FOR i := 1 TO Dimension_X DO
      FOR j := 1 TO Dimension_Y DO BEGIN
        PutPixel(i+Links_Oben_X, j+Links_Oben_Y,
                 Round(Rahmen[i]^[j]));
      END;
  END;

BEGIN
  Initial;
  Graph_Init;
  Rectangle(Links_Oben_X - 1,
            Links_Oben_Y - 1,
            Links_Oben_X + Dimension_X + 1,
            Links_Oben_Y + Dimension_Y + 1);
  Darstellung;
  REPEAT
    Iteration;
    Darstellung;
  UNTIL KeyPressed;
  CloseGraph;
END.
(* ====================================================== *)
(*                  Ende von BLASE.PAS                    *)
