(* ====================================================== *)
(*                       RS232.PAS                        *)
(* Modifizierte Version der AuxInOut von Borland,Inc.     *)
(* Der von Borland eingeschlagene Weg ueber Systemrufe    *)
(* fhrt zu groen Problemen. Der Ausweg liegt in der     *)
(* direkten Programmierung der Schnittstelle.             *)
(*          (c) 1993 R. Nikschik & toolbox                *)
(* ====================================================== *)
{$A-,B+,D-,E+,F-,G+,I-,L+,N+,O-,P-,Q-,R-,S-,T-,V+,X+,Y+}
{$M 16384,0,655360}

UNIT RS232;

INTERFACE

USES Dos, Crt;

  PROCEDURE AuxInit(Com_Nr                 : WORD;
                    Baud                   : Real;
                    Parit, DatBit, Stopbit : BYTE);
  FUNCTION  AuxInReady(Com_Nr : WORD) : BOOLEAN;
  PROCEDURE AssignAux(VAR F                      : Text;
                          Port                   : BYTE;
                          Baud                   : Real;
                          Parit, DatBit, Stopbit : BYTE);

 { Com    : 1 - COM 1
            2 - COM 2
   Baud   :    50.0;    75.0;   110.0;    134.5;    150.0
              300.0;   600.0;  1200.0;   1800.0;   2000.0;
             2400.0;  3600.0;  4800.0;   7200.0;   9600.0;
            19200.0; 38400.0; 57600.0; 115200.0;

   Parit  : 0 - keine;  Paritt
            1 - odd;      -"-
            3 - even;     -"-

   DatBit : 5; 6; 7; 8 Datenbit;

   StopBit: 1; 2  Stoppbit;                                }

IMPLEMENTATION

{$R-,S-}

TYPE
  TextBuf = ARRAY [0..127] OF CHAR;
  TextRec = RECORD
              Handle     : WORD;
              Mode       : WORD;
              BufSize    : WORD;
              PRIVATE    : WORD;
              BufPos     : WORD;
              BufEnd     : WORD;
              BufPtr     : ^TextBuf;
              OpenProc   : Pointer;
              InOutProc  : Pointer;
              FlushProc  : Pointer;
              CloseProc  : Pointer;
              ComPort    : WORD;
              ComBaud    : Real;
              ComParit   : BYTE;
              ComDatBit  : BYTE;
              ComStopBit : BYTE;
              UserFill   : ARRAY [1..5]  OF BYTE;
              Name       : ARRAY [0..79] OF CHAR;
              Buffer     : TextBuf;
            END;

CONST
  fmClosed = $D7B0;
  fmInput  = $D7B1;
  fmOutput = $D7B2;
  fmInOut  = $D7B3;

  RBR = 00;  THR  = 00;
  IER = 01;  LIR  = 02;
  LCR = 03;  MCR  = 04;
  LSR = 05;  MSR  = 06;
  SCR = 07;
  DLL = 00;  DLM  = 01;

  RTS  = 02; DTR  = 01;
  LOOP = 04;

  CTS  = 04; DSR  = 05;
  RI   = 06; DCD  = 07;

  DR   = 00; THRE = 05;

  COM1 = $03F8; COM2 = $02F8;

  IRQ_MASK = $21;

  BaudKonst = 115200.0; BaudNorm = 9600.0;
  BaudPar   =   $03;    BaudIntr =     00;

  MaxCount  = $FFFF;

(* ====================================================== *)

  PROCEDURE AuxInit(Com_Nr                 : WORD;
                    Baud                   : Real;
                    Parit, DatBit, Stopbit : BYTE);
  VAR
    Divisor, ParDat : BYTE;
  BEGIN
    ParDat := 0;
    ParDat := ((Parit AND 3) SHL 2)  OR
              ((Stopbit AND 1) SHL 1 ) OR
              (((DatBit-5) AND 7));
    Port[IRQ_MASK]   := Port[IRQ_MASK] OR $18;
    Divisor          := Round(BaudKonst/Baud);
    Port[Com_Nr+LCR] := $80;
    Port[Com_Nr+DLL] := Divisor AND $00FF;
    Port[Com_Nr+DLM] := ((Divisor AND $FF00) SHR 8);
    Port[Com_Nr+LCR] := ParDat;
    Port[Com_Nr+IER] := BaudIntr;
    Port[Com_Nr+MCR] := $0B;
    Divisor          := Port[Com_Nr+RBR];
  END;

  FUNCTION AuxInReady(Com_Nr : WORD) : BOOLEAN;
  VAR
    OutTime : LongInt;
  BEGIN
    OutTime := 60000;
    WHILE ((OutTime > 0) AND
          ((Port[Com_Nr+LSR] AND 1) = 0))  DO
      Dec(OutTime);
    IF OutTime = 0 THEN
      AuxInReady := FALSE
    ELSE
      AuxInReady := TRUE;
  END;

  FUNCTION AuxInChar(Com_Nr : INTEGER) : CHAR;
  VAR
    c : CHAR;
  BEGIN
    c := Char(Port[Com_Nr+RBR]);
    AuxInChar := c;
  END;

  PROCEDURE AuxOutChar(Com_Nr : INTEGER; Wert : CHAR);
  VAR
    Count  : LongInt;
    Status : BYTE;
  BEGIN
    { Count := Port[Com_Nr+RBR]; }   { Dummy-Wert einlesen }
    Count  := 60000;
    Status := Port[Com_Nr+LSR] AND $20;
    WHILE (Count > 0) AND (Status = 0) DO BEGIN
      Dec(Count);
      Status := Port[Com_Nr+LSR] AND $20;
    END;
    Port[Com_Nr+THR] := Ord(Wert);
 END;

{$F+}
  FUNCTION AuxInput(VAR F : TextRec) : INTEGER;
  VAR
    P : INTEGER;
  BEGIN
    WITH F DO BEGIN
      P := 0;
      WHILE AuxInReady(ComPort) AND (P < BufSize) DO BEGIN
        BufPtr^[P] := AuxInChar(ComPort);
        Inc(P);
      END;
      BufPos := 0; BufEnd := P;
    END;
    AuxInput := 0;
  END;

  FUNCTION AuxOutput(VAR F : TextRec): INTEGER;
  VAR
    P : INTEGER;
  BEGIN
    WITH F DO BEGIN
      FOR P := 0 TO BufPos-1 DO BEGIN
        AuxOutChar(ComPort, BufPtr^[P]);
        Delay(1);
      END;
      BufPos := 0;
    END;
    AuxOutput := 0;
  END;

  FUNCTION AuxIgnore(VAR F : TextRec) : INTEGER;
  BEGIN
    AuxIgnore := 0;
  END;

  FUNCTION AuxOpen(VAR F : TextRec) : INTEGER;
  BEGIN
    WITH F DO BEGIN
      AuxInit(ComPort, ComBaud, ComParit,
              ComDatBit, ComStopBit);
      IF Mode = fmInput THEN BEGIN
        InOutProc := @AuxInput;
        FlushProc := @AuxIgnore;
      END ELSE BEGIN
        Mode      := fmOutput;
        InOutProc := @AuxOutput;
        FlushProc := @AuxOutput;
      END;
      CloseProc := @AuxIgnore;
    END;
    AuxOpen := 0;
  END;
{$F-}

  PROCEDURE AssignAux;
  BEGIN
    WITH TextRec(F) DO BEGIN
      Handle       := $FFFF;
      Mode         := fmClosed;
      BufSize      := SizeOf(Buffer);
      BufPtr       := @Buffer;
      OpenProc     := @AuxOpen;
      ComBaud      := Baud;
      ComParit     := Parit;
      ComDatBit    := DatBit;
      ComStopBit   := Stopbit;
      Name[0]      := #0;
      CASE Port OF
        1 : ComPort := COM1;
        2 : ComPort := COM2;
      END;
    END;
  END;

END.
(* ====================================================== *)
(*                  Ende von RS232.PAS                    *)
