(* ====================================================== *)
(*                        RUND.PAS                        *)
(*     Lsung des Rundreise-Problems mit Lin-Methode      *)
(*    Berechnung einer suboptimalen Rundreise mit der     *)
(*    Lin-Methode. Aufruf mit 3 Parametern:               *)
(*      1. Zahl der Stdte,                               *)
(*      2. Zahl der zu bestimmenden suboptimalen Touren,  *)
(*      3. Dateiname fr die Ergebnisse                   *)
(* ====================================================== *)
{$A-,B+,D+,E+,F-,G+,I-,L+,N+,O-,P-,Q-,R-,S-,T-,V+,X+,Y+}
{$M 16384,0,655360}

PROGRAM Rund;

USES Dos, Crt, Graph;

CONST
  Breite   =  600;             { Breite des Grafikfensters }
  Tiefe    =  300;             { Tiefe des Grafikfensters  }
  Radius   =    5;
  Maximal  =  300;

TYPE
  Abstand  = ARRAY [1..Maximal] OF Double;

VAR
  X_Koordinate  : ARRAY [1..Maximal] OF Double;
  Y_Koordinate  : ARRAY [1..Maximal] OF Double;
  Matrix        : ARRAY [1..Maximal] OF ^Abstand;
                                       { Entfernungsmatrix }
  i, i1, i2     : INTEGER;
  Zwischen      : INTEGER;
  Stand         : ARRAY [1..Maximal] OF INTEGER;
  Optimum       : ARRAY [1..Maximal] OF INTEGER;
  Glob_Optimum  : ARRAY [1..Maximal] OF INTEGER;
                                 { verschiedene Rundreisen }
  Opt_Wert      : Double;
  Glob_Opt_Wert : Double;

  Runde         : INTEGER;
  Opt_String    : STRING;
  Ende          : INTEGER;
                        { Zahl der suboptimalen Rundreisen }
  Tausch        : INTEGER;
  Code          : INTEGER;

  Daten         : FILE OF BYTE;
  Staedte,Name  : STRING;
  Anzahl        : INTEGER;

  Botschaft     : FILE OF INTEGER;

  PROCEDURE Graph_Init;
  VAR
    gd, gm : INTEGER;
  BEGIN
    gd := Detect;
    InitGraph(gd, gm, GetEnv('BGIPATH'));
    IF GraphResult <> GrOk THEN Halt(1);
  END;

  PROCEDURE Punkte;
  VAR
    i : INTEGER;
    j : BYTE;
  BEGIN
    FOR i := 1 TO Ende DO BEGIN
      X_Koordinate[i] := 2 * Radius +
                         2 * Random(Round(Breite/2)-Radius);
      Y_Koordinate[i] := 2 * Radius+Random(Tiefe-2*Radius);
      New(Matrix[i]);
    END;
  END;

  PROCEDURE Entfernung;
  VAR
    i,  j  : INTEGER;
    hx, hy : Double;
  BEGIN
    FOR i := 1 TO Ende DO BEGIN
      FOR j := 1 TO Ende DO BEGIN
        hx := (X_Koordinate[i] - X_Koordinate[j]);
        hy := (Y_Koordinate[i] - Y_Koordinate[j]);
        Matrix[i]^[j] := Sqrt(hx * hx + hy * hy);
      END;
    END;
  END;

  FUNCTION Laenge : Double;
  VAR
    i     : INTEGER;
    Summe : Double;
  BEGIN
    Summe := 0;
    FOR i := 1 TO Ende-1 DO
      Summe := Summe + Matrix[Optimum[i]]^[Optimum[i+1]];
    Summe  := Summe + Matrix[Optimum[Ende]]^[Optimum[1]];
    Laenge := Summe;
  END;

  PROCEDURE Darstellung;
  VAR
    i        : INTEGER;
    P_String : STRING [4];
  BEGIN
    ClearDevice;
    Rectangle(0, 0, Breite, Tiefe);
    FOR i := 1 TO Ende DO BEGIN
      Circle(Round(X_Koordinate[i]),
             Round(Y_Koordinate[i]), Radius);
      Str(i:2, P_String);
      OutTextXY(Round(X_Koordinate[i]) + 6,
                Round(Y_Koordinate[i]), P_String);
    END;
    FOR i := 1 TO Ende DO BEGIN
      Line(Round(X_Koordinate[Optimum[i]]),
           Round(Y_Koordinate[Optimum[i]]),
           Round(X_Koordinate[Optimum[i+1]]),
           Round(Y_Koordinate[Optimum[i+1]]));
    END;
  END;

  PROCEDURE Lin_2_Opt;
    (* Realisierung der LIN-Methode. Luft solange ab,   *)
    (* bis keine Verbesserungen mehr auftreten.          *)
  VAR
    i, j, k : INTEGER;
    Delta   : Double;
    Merk    : BOOLEAN;
  BEGIN
    Darstellung;
    REPEAT
      Merk := FALSE;
      FOR k := 1 TO Ende+1 DO Stand[k] := Optimum[k];
      FOR i := 1 TO Ende DO BEGIN
        FOR j := 1 TO Ende DO BEGIN
          IF ((Stand[i+1] <> Stand[j])   AND
              (Stand[i]   <> Stand[j+1]) AND
              (Stand[i]   <> Stand[j])   AND
              (Stand[i+1] <> Stand[j+1])) THEN BEGIN
            Delta := Matrix[Stand[i]]^[Stand[i+1]]  +
                     Matrix[Stand[j]]^[Stand[j+1]]  -
                     Matrix[Stand[i]]^[Stand[j]]    -
                     Matrix[Stand[i+1]]^[Stand[j+1]];
            IF Delta > 0 THEN BEGIN
              Opt_Wert := Opt_Wert - Delta;
              FOR k := j DOWNTO i+1 DO
                Optimum[i - k + j + 1] := Stand[k];
              Optimum[Ende+1] := Optimum[1];
              Darstellung;
              Str(Opt_Wert:8:2, Opt_String);
              OutTextXY(50, 330, 'Laenge:' + Opt_String);
              Merk := TRUE;
              i := Ende;
              j := Ende;
            END;
          END;
        END;
      END;
    UNTIL NOT Merk;
  END;

BEGIN
  Val(ParamStr(1), Ende,   Code);
  Val(ParamStr(2), Anzahl, Code);
  Name          := ParamStr(3);
  Glob_Opt_Wert := 1000000;
  Tausch        := Ende DIV 2;
  Punkte;
  Entfernung;
  Graph_Init;
  Runde := 0;
  FOR i := 1 TO Ende DO Stand[i] := i;
  Stand[Ende+1] := Stand[1];
  REPEAT
    Inc(Runde);
    FOR i := 1 TO Tausch DO BEGIN
      i1 := 1 + Random(Ende);
      REPEAT
        i2 := 1 + Random(Ende);
      UNTIL (i1 <> i2);
      Zwischen  := Stand[i2];
      Stand[i2] := Stand[i1];
      Stand[i1] := Zwischen;
    END;
    FOR i := 1 TO Ende+1 DO Optimum[i] := Stand[i];
    Optimum[Ende+1] := Optimum[1];
    Stand[Ende+1]   := Stand[1];
    Opt_Wert        := Laenge;
    Lin_2_Opt;
    Write(^G);
    Darstellung;
    Str(Opt_Wert:8:2, Opt_String);
    OutTextXY(50, 330, 'Laenge:' + Opt_String);
    IF Opt_Wert < Glob_Opt_Wert THEN BEGIN
      Glob_Opt_Wert := Opt_Wert;
      FOR i := 1 TO Ende+1 DO
        Glob_Optimum[i] := Optimum[i];
    END;
  UNTIL (Runde = Anzahl);
  ReadLn;
  CloseGraph;

  Assign(Botschaft, Name);
  Rewrite(Botschaft);
  FOR i := 1 TO Ende DO
    Write(Botschaft, Glob_Optimum[i]);
  Close(Botschaft);
END.
(* ====================================================== *)
(*                  Ende von RUND.PAS                     *)

