/* ====================================================== */
/*                       PATCHIT.CPP                      */
/*          (C) 1993 Thomas Wlfer & toolbox              */
/*     Dieses Modul enthlt das Programm, mit dem die     */
/*        selbst-abspielenden Bilder erzeugt werden.      */
/*         Compiler: Microsoft Visual C++ 1.0             */
/* ====================================================== */

#include "playit.h"
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <dos.h>

static void ReadImageHeader(const char *pszImageFile,
                            IMAGEHEADER *ih);
static void PatchPlayer(    char *pszPlayerCode,
                            IMAGEHEADER &rIh,
                            long cbPlayerCode);
static void WritePlayer(    const char *pszPlayerCode,
                            int fhDest,
                            long cbPlayerCode);
static void WriteImage(     const char *pszImageName,
                            int fhDest);


void main(int argc, char **argv)
{
  // als 1. Parameter bekommen wir den Namen des
  // einzubettenden Players, als 2. Parameter den Namen
  // des Bilds, als 3. Parameter den Namen des Exe-Files,
  // das wir schreiben.

  // ? == Argv[1]
  if (argc != 4)
    exit(printf("Aufruf: patchit player image result\n"));

  // ist datei argv[2] vorhanden?
  // ist datei argv[3] vorhanden?
  struct find_t aFile;
   if(_dos_findfirst( argv[1], _A_NORMAL, &aFile))
     exit(printf("Patchit: kann %s nicht finden\n",
                 argv[1]));
   if(_dos_findfirst(argv[2], _A_NORMAL, &aFile))
     exit(printf("Patchit: kann %s nicht finden\n",
                 argv[2] ));
   if(! _dos_findfirst( argv[3], _A_NORMAL, &aFile))
     exit(printf("patchit: %s existiert bereits\n",
                 argv[3]));

   // Lesen von argv[1]
   // sicherstellen, da dies auch ein Player ist,
   // der verwendet werden kann ....
   int fhPlayer = _open(argv[1], _O_BINARY | _O_RDONLY);
   if (fhPlayer == -1)
     exit(printf("Fehler beim ffnen von %s", argv[1]));
   long cbFile = _filelength(fhPlayer);
   // Sicherstellen, da wir in ein Segment passen
   if (cbFile > 64000)
    exit(printf("%s ist nicht der Playit-Player", argv[1]));

   // Speicher fr den Player allozieren
   char *pszPlayer = (char *)malloc((size_t)cbFile);
   if (! pszPlayer)
    exit(printf("Nicht genug Speicher um Player zu lesen"));

   // Player einlesen
   if (_read(fhPlayer, (void*)pszPlayer,
      (unsigned int)cbFile )== -1 )
     exit(printf("Fehler beim Lesen von %s", argv[1]));
   _close( fhPlayer );

   // dann wird das Bild geffnet und der Header des Bilds
   // gelesen. Dieser enthlt eine struct videoconfig und
   // die Gre des Bilds
   IMAGEHEADER ih;
   ReadImageHeader(argv[2], &ih);

   // im Playercode, der sich dann im Speicher befindet, den
   // Patchstring suchen und mit den bentigten Infos auf-
   // fllen
   PatchPlayer(pszPlayer, ih, cbFile);

   // jetzt wird das Executable unter seinem neuen Namen
   // (argv[3] zurckgeschrieben
   int fhDest = _open(argv[3], _O_WRONLY | _O_BINARY
                             | _O_CREAT);
   if (fhDest == -1)
     exit(printf("Fehler beim Anlegen von %s", argv[3] ));
   if (_write(fhDest, (void*)pszPlayer,
      (unsigned int)cbFile ) == -1)
     exit(printf("Fehler beim Schreiben des Players"));

   // und hinten hngen wird dann das Bild dran.
   WriteImage(argv[2], fhDest);
   _close(fhDest);

   // fertig ....
   printf("done");
}

static void ReadImageHeader(const char *pszImageFile,
                            IMAGEHEADER *ih)
{
  int fhImage = _open(pszImageFile, _O_BINARY | _O_RDONLY);
  if (fhImage == -1)
    exit(printf("Fehler beim ffnen von %s", pszImageFile));

  _read(fhImage, (void*)ih, sizeof(IMAGEHEADER));
  _close(fhImage);
}

static void PatchPlayer(char * pszPlayerCode,
                        IMAGEHEADER &rIh, long cbPlayerCode)
{
  // im Playercode wird der szFindPatchAreaString gesucht.
  char * pC        = pszPlayerCode;
  long iPlayerCode = 0;

  while( iPlayerCode < cbPlayerCode )
  {
    if(*pC       == szFindPatchAreaString[0] &&
       *(pC + 1) == szFindPatchAreaString[1] )
    {
      if (! memcmp( pC, szFindPatchAreaString, 15))
      {
        // gefunden! Information eintragen
        PATCHINFO *pPI = (PATCHINFO*)pC;
        pPI->cbPlayer  = cbPlayerCode;
        pPI->ih        = rIh;
        return;
      }
    }
    pC++;
    iPlayerCode++;
  }

  // Patch-Area nicht gefunden
  exit(printf("Dieser Player enthlt keine Patcharea"));
}

static void WriteImage(const char *pszImageName, int fhDest)
{
  int fhImage = _open(pszImageName, _O_BINARY | _O_RDONLY);
  if (fhImage == -1)
    exit(printf("Fehler beim ffnen von %s", pszImageName));

  IMAGEHEADER ih;
  _read(fhImage, (void*)&ih, sizeof(IMAGEHEADER));
  char *pImage = (char*)malloc((size_t)ih.cbImageSize);
  if (! pImage)
    exit(printf("Nicht genug Speicher um Bild zu laden"));
  _read(fhImage, (void*)pImage,
        (unsigned int)ih.cbImageSize);
  _close(fhImage);

  if (_write(fhDest, (void*)pImage,
      (unsigned int)ih.cbImageSize) == -1)
    exit(printf("Fehler beim Schreiben des Bilds"));
  free(pImage);
}

/* ====================================================== */
/*                   Ende von PATCHIT.CPP                 */

