(* ====================================================== *)
(*                      MFILEAPP.PAS                      *)
(*              Erweiterung der Borland-Demo              *)
(*            (c) 1993 Martin Wlker & toolbox            *)
(*    Portions Copyright (c) 1991 Borland International   *)
(* ====================================================== *)
PROGRAM MDIFileEditor;

{$R MFILEAPP.RES}

USES WinTypes, WinProcs, WinDos, Objects, OWindows,
     ODialogs, OMemory, OStdDlgs, Validate, MStdWnds,
     Mpaint, MyWin, Strings, BWCC;

CONST
  cm_SaveState    = 905;
  cm_RestoreState = 906;
  cm_ConfigFile   = 907; { Einstellen des Dateinamens }
  cm_AutoSave     = 908; { Sichern alle n Minuten }
  cm_About        = 333;

TYPE
  { Declare tMDIFileApp, a tApplication descendant }
  tMDIFileApp = OBJECT (tApplication)
    PROCEDURE InitMainWindow; VIRTUAL;
  END;

  { Declare tMDIFileWindow, a tMDIWindow descendant }
  pMDIFileWindow = ^tMDIFileWindow;
  tMDIFileWindow = OBJECT (tMDIWindow)
    DskFile, FileMenustr,
    SaveDelay, DelayName : pChar;
    SaveTime             : LongInt;
    PromptOnsave         : Bool;

    CONSTRUCTOR Init(aTitle, aFileName : pChar;
                     aMenu             : hMenu);
    DESTRUCTOR  Done;                               VIRTUAL;
    PROCEDURE   SetupWindow;                        VIRTUAL;
    PROCEDURE   NewFile(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_MDIFileNew;
    PROCEDURE   OpenFile(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_MDIFileOpen;
    PROCEDURE   SaveState(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_SaveState;
    PROCEDURE   RestoreState(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_RestoreState;
    PROCEDURE   DefWndProc(VAR Msg : tMessage);     VIRTUAL;
    PROCEDURE   SetConfigFile(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_ConfigFile;
    PROCEDURE   AutoSave(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_autosave;
    PROCEDURE   cmAbout(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_about;
  END;

  { Declare TFileEditor, a TFileWindow desendant }
  pFileEditor = ^tFileEditor;
{$IFDEF Editor}
  tFileEditor = OBJECT (tFileWindow)
{$ENDIF}
{$IFDEF Paint}
  tFileEditor = OBJECT (tPaintWin)
{$ENDIF}
    CONSTRUCTOR Init(aParent   : pWindowsObject;
                     aFileName : pChar);
    DESTRUCTOR  Done;                               VIRTUAL;
    PROCEDURE   GetWindowClass(VAR aWndClass : tWndClass);
                                                    VIRTUAL;
    FUNCTION    GetClassName : pChar;               VIRTUAL;
  END;

CONST
  rFileEditor: tStreamRec = (
    ObjType : 1000;
    vmtLink : Ofs(TypeOf(tFileEditor)^);
    Load    : @tFileEditor.Load;
    Store   : @tFileEditor.Store);

(* ================ Start =============================== *)
(*     Dialog fr Einstellung des Autosave                *)

CONST
  id_Nummer      = 100;
  id_Prompt      = 103;
  id_ja          = 6;
  id_nein        = 7;

TYPE
  pAutosaveDlg = ^tAutosaveDlg;
  tAutosaveDlg = OBJECT (tDialog)
    e : pEdit;

    CONSTRUCTOR Init(aParent : pWindowsObject; VAR Zeit);
    PROCEDURE   BAutoSaveOn(VAR Msg : tMessage);
                VIRTUAL id_First + id_ja;
    PROCEDURE   BAutoSaveOff(VAR Msg : tMessage);
                VIRTUAL id_First + id_nein;
  END;

  CONSTRUCTOR tAutosaveDlg.Init(aParent : pWindowsObject;
                                VAR Zeit);
  VAR
    c : pWindowsObject;
  BEGIN
    INHERITED Init(aParent, 'autosave');
    e := New(pEdit, InitResource(@Self, id_Nummer, 3));
    c := New(pCheckBox, InitResource(@Self, id_Prompt));
    e^.SetValidator(New(pRangeValidator, Init(0, 99)));
    TransferBuffer := @Zeit;
  END;

  PROCEDURE tAutosaveDlg.BAutoSaveOn(VAR Msg : tMessage);
  BEGIN
    Ok(Msg);
  END;

  PROCEDURE tAutosaveDlg.BAutoSaveOff(VAR Msg : tMessage);
  BEGIN
    Cancel(Msg);
  END;

CONST
  EditorCount : INTEGER = 0;

TYPE
  tMenuState = (msEnable, msDisable);

  PROCEDURE MenuItems(State : tMenuState);

    PROCEDURE ModifyCommand(Command : WORD);
    VAR
      NewState : WORD;
    BEGIN
      NewState := mf_ByCommand;
      IF State = msEnable THEN
        Inc(NewState, mf_Enabled)
      ELSE
        Inc(NewState, mf_Disabled + mf_Grayed);
      EnableMenuItem
        (pWindow(Application^.MainWindow)^.Attr.Menu,
                 Command, NewState);
    END;

  BEGIN
    { Bail out if the window is already closed }
    IF Application^.MainWindow^.hWindow = 0 THEN Exit;

    ModifyCommand(cm_FileSave);
    ModifyCommand(cm_FileSaveAs);
    ModifyCommand(cm_ArrangeIcons);
    ModifyCommand(cm_TileChildren);
    ModifyCommand(cm_CascadeChildren);
    ModifyCommand(cm_CloseChildren);
    ModifyCommand(cm_EditCut);
    ModifyCommand(cm_EditCopy);
    ModifyCommand(cm_EditPaste);
    ModifyCommand(cm_EditDelete);
    ModifyCommand(cm_EditUndo);
    ModifyCommand(cm_EditFind);
    ModifyCommand(cm_EditReplace);
    ModifyCommand(cm_EditFindNext);
    ModifyCommand(cm_ClearAll);
    ModifyCommand(cm_Color);
    ModifyCommand(cm_Font);
  END;

  PROCEDURE IncEditors;
  BEGIN
    IF EditorCount = 0 THEN MenuItems(msEnable);
    Inc(EditorCount);
  END;

  PROCEDURE DecEditors;
  BEGIN
    Dec(EditorCount);
    IF EditorCount = 0 THEN MenuItems(msDisable);
  END;

  CONSTRUCTOR tFileEditor.Init(aParent   : pWindowsObject;
                               aFileName : pChar);
  BEGIN
    INHERITED Init(aParent, '', aFileName);
    IncEditors;
  END;

  DESTRUCTOR tFileEditor.Done;
  BEGIN
    DecEditors;
    INHERITED Done;
  END;

  PROCEDURE tFileEditor.GetWindowClass
                        (VAR aWndClass : tWndClass);
  BEGIN
    INHERITED GetWindowClass(aWndClass);
    aWndClass.hIcon := LoadIcon(hInstance, 'FILEICON');
  END;

  FUNCTION tFileEditor.GetClassName : pChar;
  BEGIN
    GetClassName := 'FileEditor';
  END;


  { initializes the MDIWindow and activates
    the Autosave option }
  CONSTRUCTOR tMDIFileWindow.Init(aTitle, aFileName : pChar;
                                  aMenu : hMenu);
  VAR
    Puffer : ARRAY [0..30] OF CHAR;
  BEGIN
    INHERITED Init(aTitle, aMenu);
    DskFile := StrNew(aFileName);
    IF DskFile = NIL THEN
      DskFile := StrNew('desktop.cfg');
    GetMenuString(aMenu, cm_ConfigFile, @Puffer,
                  30, mf_ByCommand);
    FileMenustr := StrNew(Puffer);
    IF FileMenustr = NIL THEN
      FileMenustr := StrNew('Desktop File:');
    StrCat(Puffer, DskFile);
    ModifyMenu(aMenu, cm_ConfigFile,
               mf_ByCommand OR mf_Enabled OR mf_String,
               cm_ConfigFile, @Puffer);
    GetMenuString(aMenu, cm_autosave, @Puffer,
                  30, mf_ByCommand);
    SaveDelay    := StrNew('10');
    PromptOnsave := FALSE;
    DelayName    := StrNew(Puffer);
    IF DelayName = NIL THEN
      DelayName := StrNew('Autosave');
    StrCat(Puffer, SaveDelay);
    ModifyMenu(aMenu, cm_autosave,
               mf_ByCommand OR mf_Enabled OR mf_String,
               cm_autosave, @Puffer);
    SaveTime := MaxLongInt;
  END;

  { inherited init and dispose of the desktop file name }
  DESTRUCTOR tMDIFileWindow.Done;
  BEGIN
    StrDispose(DskFile);   StrDispose(FileMenustr);
    StrDispose(SaveDelay); StrDispose(DelayName);
    INHERITED Done;
  END;

  { Respond to "New" command by constructing,
    creating, and setting up a
    new tFileWindow MDI child }
  PROCEDURE tMDIFileWindow.NewFile(VAR Msg : tMessage);
  BEGIN
    Application^.MakeWindow(New(pFileEditor,
                                Init(@Self, '')));
  END;

  PROCEDURE tMDIFileWindow.SetupWindow;
  BEGIN
    tMDIWindow.SetupWindow;
    MenuItems(msDisable);
  END;

  { Respond to "Open" command by constructing,
    creating, and setting up a
    new TFileWindow MDI child }
  PROCEDURE tMDIFileWindow.OpenFile(VAR Msg : tMessage);
  VAR
    FileName: ARRAY [0..fsPathName] OF CHAR;
  BEGIN
    IF Application^.ExecDialog
       (New(pFileDialog, Init(@Self, pChar(sd_FileOpen),
{$IFDEF Editor}
        StrCopy(FileName, '*.*')))) = id_OK THEN
{$ENDIF}
{$IFDEF Paint}
        StrCopy(FileName, '*.bmp')))) = id_OK THEN
{$ENDIF}
      Application^.MakeWindow(New(pFileEditor,
                                  Init(@Self, FileName)));
  END;

  { Save the the position and contents of the windows to the
    "desk top" file. }
  PROCEDURE tMDIFileWindow.SaveState(VAR Msg : tMessage);
  VAR
    s        : pStream;
    ErrorMsg : pChar;

    FUNCTION FileDelete(Name : pChar) : INTEGER; ASSEMBLER;
    ASM
      PUSH  DS
      LDS   DX,Name
      MOV   AH,41H
      INT   21H
      JC    @@1
      XOR   AX,AX
    @@1:
      NEG   AX
      POP   DS
    END;

  BEGIN
    ErrorMsg := NIL;
    s := New(pBufStream, Init(DskFile, stCreate, 1024));
    IF s <> NIL THEN BEGIN
      PutChildren(s^);
      s^.Write(EditorCount, SizeOf(EditorCount));
      IF s^.Status <> stOk THEN BEGIN
        MessageBox(hWindow, Word(s^.Status), 0,
                   mb_OK OR mb_IconExclamation);
        Dispose(s, Done);
        FileDelete(DskFile);
      END ELSE
        Dispose(s, Done);
    END ELSE
      MessageBox(hWindow, 1, 0,
                 mb_OK OR mb_IconExclamation);
  END;

  { Read windows positions and contents from
    the "desk top" file. }
  PROCEDURE tMDIFileWindow.RestoreState(VAR Msg : tMessage);
  VAR
    s        : pStream;
    ErrorMsg : pChar;
    TmpName  : ARRAY [0..fsPathName] OF CHAR;
  BEGIN
    ErrorMsg := NIL;
    StrCopy(TmpName, DskFile);
    s := New(pBufStream, Init(DskFile, stOpenRead, 1024));
    WHILE s^.Status <> stOk DO BEGIN
      CASE Application^.ExecDialog
           (New(pFileDialog,
               Init(@Self, pChar(sd_FileOpen), TmpName))) OF
        id_OK :
          s := New(pBufStream, Init(DskFile,
                                    stOpenRead, 1024));
        id_Cancel : Exit;
      END;
      IF s^.Status = id_OK THEN BEGIN
        StrDispose(DskFile);
        DskFile := StrNew(TmpName);
        StrCopy(TmpName, FileMenustr);
        StrCat(TmpName, DskFile);
        ModifyMenu
          (GetMenu(Application^.MainWindow^.hWindow),
                   cm_ConfigFile,
                   mf_ByCommand OR mf_Enabled OR mf_String,
                   cm_ConfigFile, @TmpName);
      END;
    END;
    CloseChildren;
    GetChildren(s^);
    s^.Read(EditorCount, SizeOf(EditorCount));
    IF EditorCount > 0 THEN MenuItems(msEnable);
    IF s^.Status <> stOk THEN
      MessageBox(hWindow, 3, 0,
                 mb_OK OR mb_IconExclamation);
    IF LowMemory THEN BEGIN
      CloseChildren;
      MessageBox(hWindow, 4, 0,
                 mb_OK OR mb_IconExclamation);
    END ELSE
      CreateChildren;
  END;

  { Checks if its time for autosave }
  PROCEDURE tMDIFileWindow.DefWndProc(VAR Msg : tMessage);
  VAR
    Wert, Aktwert : LongInt;
    Code          : INTEGER;
  BEGIN
    IF SaveTime < MaxLongInt THEN BEGIN
      Aktwert := GetCurrentTime;
      IF Aktwert > SaveTime THEN BEGIN
        Val(SaveDelay, Wert, Code);
        SaveTime := Wert*60000 + Aktwert;
    {$B-}
        IF NOT PromptOnsave OR
           (MessageBox(hWindow, 20, 16,
               mb_OkCancel OR mb_IconQuestion) = id_OK) THEN
          SaveState(Msg);
      END;
    END;
    INHERITED DefWndProc(Msg);
  END;

  { Change the current desktop filename }
  PROCEDURE tMDIFileWindow.SetConfigFile(VAR Msg: tMessage);
  VAR
    TmpName : ARRAY [0..fsPathName] OF CHAR;
    Puffer  : ARRAY [0..fsPathName+15] OF CHAR;
  BEGIN
    IF DskFile <> NIL THEN
      StrCopy(TmpName, DskFile)
    ELSE
      TmpName[0] := #0;
    WHILE Application^.ExecDialog
          (New(pFileDialog, Init(@Self,
             pChar(sd_FileSave), TmpName))) = id_OK DO BEGIN
      CASE Application^.ExecDialog
           (New(pDoubleDialog,Init(@Self, TmpName))) OF
        id_Retry : Continue; { Neuen Filenamen abfragen}
        id_No    : Break;    { Abfrageschleife verlassen }
        id_OK,   { olles ok!  Speichern und verlassen }
        id_Yes   : BEGIN
          StrDispose(DskFile);
          DskFile := StrNew(TmpName);
          StrCopy(Puffer, FileMenustr);
          StrCat(Puffer, DskFile);
          ModifyMenu(GetMenu
                     (Application^.MainWindow^.hWindow),
                      cm_ConfigFile,
                    mf_ByCommand OR mf_Enabled OR mf_String,
                      cm_ConfigFile, @Puffer);
          Break;
        END;
      END;
    END;
  END;

  { Brings up a dialog in order to select a autosave time }
  PROCEDURE tMDIFileWindow.Autosave(VAR Msg : tMessage);
  VAR
    Wert, Aktwert : LongInt;
    Code          : INTEGER;
    Puffer        : ARRAY [0..30] OF CHAR;
    Buf           : RECORD
                      Delay  : ARRAY [0..2] OF CHAR;
                      Prompt : Bool;
                    END;
  BEGIN
    StrCopy(Buf.Delay, SaveDelay);
    Buf.Prompt := PromptOnsave;
    IF Application^.ExecDialog
       (New(pAutosaveDlg,
            Init(@Self, Buf))) = id_OK THEN BEGIN
      StrCopy(SaveDelay, Buf.Delay);
      PromptOnsave := Buf.Prompt;
      StrCopy(Puffer, DelayName);
      StrCat(Puffer, SaveDelay);
      ModifyMenu(GetMenu(Application^.MainWindow^.hWindow),
                 cm_autosave,
                 mf_ByCommand OR mf_Enabled OR mf_String,
                 cm_autosave, @Puffer);
      CheckMenuItem
        (GetMenu(Application^.MainWindow^.hWindow),
                 cm_autosave, mf_Checked);
      Aktwert := GetCurrentTime;
      Val(SaveDelay, Wert, Code);
      SaveTime := Wert*60000 + Aktwert;
    END ELSE BEGIN
      CheckMenuItem
        (GetMenu(Application^.MainWindow^.hWindow),
                 cm_autosave, mf_Unchecked);
      SaveTime := MaxLongInt;
    END;
  END;

  PROCEDURE tMDIFileWindow.cmAbout(VAR Msg : tMessage);
  BEGIN
{$IFDEF Editor}
    Application^.ExecDialog(New(pDialog,
                                Init(@Self, 'Info')));
{$ENDIF}
{$IFDEF Paint}
    Application^.ExecDialog(New(pDialog,
                            Init(@Self, 'AboutDialog')));
{$ENDIF}
  END;

  PROCEDURE tMDIFileApp.InitMainWindow;
  BEGIN
{$IFDEF Editor}
    MainWindow := New(pMDIFileWindow,
                      Init('MDI Files', NIL,
                          LoadMenu(hInstance, 'Commands')));
{$ENDIF}
{$IFDEF Paint}
    MainWindow := New(pMDIFileWindow,
                      Init('MDI Paint', NIL,
                         LoadMenu(hInstance, 'Paintmenu')));
{$ENDIF}
    pMDIFileWindow(MainWindow)^.ChildMenuPos := 3;

    { Register types to be written to stream }
    RegisterType(rWindow);
    RegisterType(rEdit);
    RegisterType(rFileEditor);
  END;

  { Declare a variable of type tFileApp }
VAR
  MDIFileApp : tMDIFileApp;

{ Run the FileApp }
BEGIN
  MDIFileApp.Init('MDIFileApp');
  MDIFileApp.Run;
  MDIFileApp.Done;
END.
(* ====================================================== *)
(*                 Ende von MFILEAPP.PAS                  *)
