(* ====================================================== *)
(*                       MPAINT.PAS                       *)
(*           Demo for using the Container Window          *)
(*            (c) 1993 Martin Wlker & toolbox            *)
(*          Portions (c) 1992 Borland International       *)
(*                 adaption of PaintDemo                  *)
(*       now for use as MDI-Child with tContainerWin      *)
(*       Compiler: Turbo-/Borland-Pascal for Windows      *)
(*   PaintDef... im Verzeichnis \BP\EXAMPLES\WIN\PAINT    *)
(* ====================================================== *)
UNIT MPaint;

INTERFACE

USES PaintDef, ResDef, PaintDlg, ToolBar, LineBar, Palette,
     Canvas, WinTypes, WinProcs, OWindows, ODialogs,
     Strings, MyWin {tContainerWin declared here}, Objects;

TYPE
  pPaintWin = ^tPaintWin;
  tPaintWin = OBJECT (tContainerWin)
    State   : tState;   { Drawing state of the program     }
    Palette : pPalette; { Color palette                    }
    ToolBar : pToolBar; { Palette of available tools       }
    LineBar : pLineBar; { Palette of available line widths }
    Canvas  : pCanvas;  { Window to actually draw on       }
    CBChainNext : hWnd;
                      { Next window in the clipboard chain }

    { Creation }
    CONSTRUCTOR Init(aParent           : pWindowsObject;
                     aTitle, aFileName : pChar);
    FUNCTION    GetClassName : pChar;               VIRTUAL;
    PROCEDURE   GetWindowClass(VAR wndClass : tWndClass);
                                                    VIRTUAL;
    PROCEDURE   SetupWindow;                        VIRTUAL;

    { Utility }
    FUNCTION    Size2Child(VAR Msg : tMessage) : BOOLEAN;
                                                    VIRTUAL;
      { vorher UpdateChildren }

      { Window manager interface routines }
    PROCEDURE   WMChangeCBChain(VAR Msg : tMessage);
                VIRTUAL wm_First + wm_ChangeCBChain;
    PROCEDURE   WMDrawClipBoard(VAR Msg : tMessage);
                VIRTUAL wm_First + wm_DrawClipboard;
    PROCEDURE   WMDestroy(VAR Msg : tMessage);
                VIRTUAL wm_First + wm_Destroy;

      { Menu routines }
    PROCEDURE   cmEditUndo(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_EditUndo;
    PROCEDURE   cmEditCut(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_EditCut;
    PROCEDURE   cmEditCopy(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_EditCopy;
    PROCEDURE   cmEditPaste(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_EditPaste;
    PROCEDURE   cmEditDelete(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_EditDelete;

    FUNCTION    Clear : BOOLEAN;                    VIRTUAL;
    FUNCTION    ClearModify : BOOLEAN;              VIRTUAL;
    FUNCTION    IsModified  : BOOLEAN;              VIRTUAL;
    FUNCTION    Write_Daten : BOOLEAN;              VIRTUAL;
    FUNCTION    Read_Daten  : BOOLEAN;              VIRTUAL;

    PROCEDURE   CMOptionsSize(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_OptionsSize;
  END;

IMPLEMENTATION

{ tPaintWin }

  { Create a drawing window, initializing its drawing state
    and associated windows. }
  CONSTRUCTOR tPaintWin.Init(aParent : pWindowsObject;
                             aTitle, aFileName : pChar);
  BEGIN
    INHERITED Init(aParent, aTitle, aFileName);
    { Initialize the drawing state }
    WITH State DO BEGIN
      PaintTool     := NIL;
      MemDC         := 0;
      IsDirtyBitmap := FALSE;
      SetRectEmpty(Selection);
      SelectionBM   := 0;
      PenSize       := 3;
      PenColor      := $000000;
      BrushColor    := $FFFFFF;
      BitmapSize.x  := 640;
      BitmapSize.y  := 480;
    END;

    { Create the associated windows }
    Palette := New(pPalette, Init(@Self, @State));
    ToolBar := New(pToolBar, Init(@Self, @State));
    LineBar := New(pLineBar, Init(@Self, @State));
    Canvas  := New(pCanvas,  Init(@Self, @State));
    CBChainNext := 0;
  END;

  FUNCTION tPaintWin.GetClassName : pChar;
  BEGIN
    GetClassName := 'TPaintWin';
  END;

  PROCEDURE tPaintWin.GetWindowClass
                      (VAR wndClass : tWndClass);
  BEGIN
    INHERITED GetWindowClass(wndClass);
    wndClass.hbrBackground := color_AppWorkSpace + 1;
    wndClass.hIcon := LoadIcon(hInstance, 'PaintIcon');
  END;

  PROCEDURE tPaintWin.SetupWindow;
  BEGIN
    INHERITED SetupWindow;
    IF IsClipboardFormatAvailable(cf_Bitmap) THEN
      EnableMenuItem(Attr.Menu, cm_EditPaste, mf_Enabled);
    CBChainNext := SetClipboardViewer(hWindow);
  END;

  FUNCTION tPaintWin.Size2Child;
  VAR
    s : INTEGER;  { Lower coordinates of Palette }
    R : tRect;	  { Window client area }
  BEGIN
    GetClientRect(hWindow, R);
    s := ((R.Bottom - 8) DIV 17) * 3 + 1;
    MoveWindow(Palette^.hWindow,
               4, 4, s, R.Bottom - 8, TRUE);
    MoveWindow(ToolBar^.hWindow,
               s + 8, 4, (Ord(MaxTool) + 1) * 31 + 1,
               32, TRUE);
    MoveWindow(LineBar^.hWindow,
               s + (Ord(MaxTool) + 1) * 31 + 13, 4,
               LineBarWidth, 32, TRUE);
    Canvas^.MoveSelf(s + 8, 40,
                     R.Right - s - 12, R.Bottom - 44, TRUE);
  END;

{ Window manager interface routines }

  { Update the clipboard chain link,
    or pass down the message }
  PROCEDURE tPaintWin.WMChangeCBChain(VAR Msg : tMessage);
  BEGIN
    IF Msg.wParam = CBChainNext THEN
      CBChainNext := Msg.lParamLo
    ELSE IF CBChainNext <> 0 THEN
      SendMessage(CBChainNext,
                  Msg.Message, Msg.wParam, Msg.lParam);
  END;

  { Enable/disable menus to reflect a change in the state
    of the clipboard. }
  PROCEDURE tPaintWin.WMDrawClipBoard(VAR Msg : tMessage);
  BEGIN
    IF IsClipboardFormatAvailable(cf_Bitmap) THEN
      EnableMenuItem(Attr.Menu, cm_EditPaste, mf_Enabled)
    ELSE
      EnableMenuItem(Attr.Menu, cm_EditPaste, mf_Grayed);
    IF CBChainNext <> 0 THEN
      SendMessage(CBChainNext, Msg.Message, 0, 0);
  END;

  { Notify the clipboard chain before dying. }
  PROCEDURE tPaintWin.WMDestroy(VAR Msg : tMessage);
  BEGIN
    ChangeClipboardChain(hWindow, CBChainNext);
    INHERITED WMDestroy(Msg);
  END;

  { Read a bitmap from file. }
  FUNCTION tPaintWin.Read_Daten;
  BEGIN
    Read_Daten := 0 <> Canvas^.Load(FileName);
  END;

  { Save the current bitmap to file. }
  FUNCTION tPaintWin.Write_Daten;
  BEGIN
    Canvas^.Store(FileName);
  END;

  FUNCTION tPaintWin.ClearModify : BOOLEAN;
  BEGIN
    State.IsDirtyBitmap := FALSE;
  END;

  { Make sure the bitmap is saved if desired before
    dying or cancel if desired. }
  FUNCTION tPaintWin.IsModified : BOOLEAN;
  BEGIN
    IsModified := State.IsDirtyBitmap;
  END;

  { Edit menu functions }

  { Undo the last change to the bitmap.}
  PROCEDURE tPaintWin.cmEditUndo(VAR Msg : tMessage);
  BEGIN
    Canvas^.Undo;
  END;

  { Copy the current selection to the clipboard
    and clear it from the screen. }
  PROCEDURE tPaintWin.cmEditCut(VAR Msg : tMessage);
  BEGIN
    Canvas^.Cut;
  END;

  { Copy the current selection to the clipboard. }
  PROCEDURE tPaintWin.cmEditCopy(VAR Msg : tMessage);
  BEGIN
    Canvas^.Copy;
  END;

  { Retrieve the contents of the clipboard and make
    it the current selection. }
  PROCEDURE tPaintWin.cmEditPaste(VAR Msg : tMessage);
  BEGIN
    ToolBar^.ToolSelect(SelectTool);
    Canvas^.Paste;
  END;

  { Clear the current selection from the screen. }
  PROCEDURE tPaintWin.cmEditDelete(VAR Msg : tMessage);
  BEGIN
    Canvas^.Delete;
  END;

  { Clear the entire drawing area.}
  FUNCTION tPaintWin.Clear;
  BEGIN
    Canvas^.ClearAll;
  END;

  { Options }
  PROCEDURE tPaintWin.CMOptionsSize(VAR Msg : tMessage);
  VAR
    SizeBMInfo : tSizeBMInfo;
  BEGIN
    WITH SizeBMInfo, State DO BEGIN
      Width         := BitmapSize.x;
      Height        := BitmapSize.y;
      CurrentBMFlag := id_PadBM;
    END;
    IF Application^.ExecDialog
       (New(pSizeBMDialog,
            Init(@Self, 'SizeBMDialog',
                 @SizeBMInfo))) = id_OK THEN BEGIN
      WITH State, SizeBMInfo DO BEGIN
        BitmapSize.x := Width;
        BitmapSize.y := Height;
      END;
      Canvas^.Resize(SizeBMInfo.CurrentBMFlag);
      WMSize(Msg);
    END;
  END;

END.
(* ====================================================== *)
(*                  Ende von MPAINT.PAS                   *)
