(* ====================================================== *)
(*                       MPAINT.PAS                       *)
(*      Standard windows unit for ObjectWindows           *)
(*            (c) 1993 Martin Wlker & toolbox            *)
(*          Portions (c) 1992 Borland International       *)
(* define trace: Wenn Trace definiert ist, dann geben     *)
(*               verschiedene Methoden Meldungen aus.     *)
(* ====================================================== *)
UNIT MStdWnds;

INTERFACE

USES WinTypes, WinProcs, WinDos, Objects, OWindows,
     ODialogs, Strings, WinCrt, OMemory, MyWin;

TYPE
  pEditWindow = ^tEditWindow;
  tEditWindow = OBJECT (tContainerWin)
    Editor : pEdit;
    PROCEDURE   SetupWindow;                        VIRTUAL;
    PROCEDURE   GetWindowClass(VAR aWndClass : tWndClass);
                                                    VIRTUAL;
    FUNCTION    GetClassName : pChar;               VIRTUAL;
    CONSTRUCTOR Init(aParent   : pWindowsObject;
                     aTitle    : pChar;
                     aFileName : pChar);
    CONSTRUCTOR Load(VAR s : tStream);
    PROCEDURE   Store(VAR s : tStream);
    FUNCTION    Clear : BOOLEAN;                    VIRTUAL;
    FUNCTION    ClearModify : BOOLEAN;              VIRTUAL;
    FUNCTION    IsModified : BOOLEAN;               VIRTUAL;
    FUNCTION    Suchen(SearchText : pChar;
                       CaseSensitive : Bool) : INTEGER;
                                                    VIRTUAL;
    FUNCTION    Ersetzen(ReplaceText : pChar) : BOOLEAN;
                                                    VIRTUAL;
    FUNCTION    GeheZu(Position : INTEGER) : BOOLEAN;
                                                    VIRTUAL;
  END;

  pFileWindow = ^tFileWindow;
  tFileWindow = OBJECT (tEditWindow)
    FUNCTION Read_Daten : BOOLEAN;                  VIRTUAL;
    FUNCTION Write_Daten : BOOLEAN;                 VIRTUAL;
  END;

CONST
  rEditWindow : tStreamRec = (
    ObjType : 180;
    vmtLink : Ofs(TypeOf(tEditWindow)^);
    Load    : @tEditWindow.Load;
    Store   : @tEditWindow.Store);
  rFileWindow : tStreamRec = (
    ObjType : 181;
    vmtLink : Ofs(TypeOf(tFileWindow)^);
    Load    : @tFileWindow.Load;
    Store   : @tFileWindow.Store);

  PROCEDURE RegisterStdWnds;

IMPLEMENTATION

  FUNCTION tEditWindow.IsModified : BOOLEAN;
  BEGIN
    IsModified := Editor^.IsModified;
  END;

  PROCEDURE tEditWindow.GetWindowClass
                        (VAR aWndClass : tWndClass);
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWindow.GetWindowClass ..':30);
{$ENDIF}
    INHERITED GetWindowClass(aWndClass);
    aWndClass.hIcon := LoadIcon(hInstance, 'FileIcon');
{$IFDEF Trace}
    WriteLn('tEditWindow.GetWindowClass ok':30);
{$ENDIF}
  END;

  FUNCTION tEditWindow.GetClassName : pChar;
  BEGIN
    GetClassName := 'Editor';
  END;

  { Constructor for a tEditWindow.
    Initializes its data fields using passed parameters and
    default values.  Constructs its child edit control. }
  CONSTRUCTOR tEditWindow.Init(aParent   : pWindowsObject;
                               aTitle    : pChar;
                               aFileName : pChar);
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWindow.Init ..':30);
{$ENDIF}
    INHERITED Init(aParent, aTitle, aFileName);
    Editor := New(pEdit,Init(@Self,300,NIL,0,0,0,0,0,TRUE));
    WITH Editor^.Attr DO
      Style := Style OR es_NoHideSel;
{$IFDEF Trace}
    WriteLn('tEditWindow.Init ok':30);
{$ENDIF}
  END;

  PROCEDURE tEditWindow.SetupWindow;
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWindow.SetUpWindow ..':30);
{$ENDIF}
    INHERITED SetupWindow;
    SendMessage(Editor^.hWindow, wm_SetFont, theFont, 0);
{$IFDEF Trace}
    WriteLn('tEditWindow.SetUpWindow ok':30);
{$ENDIF}
  END;

  { Load a tEditWindow from the given stream }
  CONSTRUCTOR tEditWindow.Load(VAR s : tStream);
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWindow.Load ..':30);
{$ENDIF}
    INHERITED Load(s);
    GetChildPtr(s, Editor);
{$IFDEF Trace}
    WriteLn('tEditWindow.Load ok':30);
{$ENDIF}
  END;

  { Store a tEditWindow to the given stream }
  PROCEDURE tEditWindow.Store(VAR s : tStream);
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWindow.Store ..':30);
{$ENDIF}
    INHERITED Store(s);
    PutChildPtr(s, Editor);
{$IFDEF Trace}
    WriteLn('tEditWindow.Store ok':30);
{$ENDIF}
  END;

  FUNCTION tEditWindow.Suchen(SearchText  : pChar;
                            CaseSensitive : Bool) : INTEGER;
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWindow.Suchen ..':30);
{$ENDIF}
    Suchen := Editor^.Search(-1, SearchText, CaseSensitive);
{$IFDEF Trace}
    WriteLn('tEditWindow.Suchen ok':30);
{$ENDIF}
  END;

  FUNCTION tEditWindow.Ersetzen
                       (ReplaceText : pChar) : BOOLEAN;
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWindow.Ersetzen ..':30);
{$ENDIF}
    Editor^.Insert(ReplaceText);
{$IFDEF Trace}
    WriteLn('tEditWindow.Ersetzen ok':30);
{$ENDIF}
  END;

  FUNCTION tEditWindow.GeheZu(Position : INTEGER) : BOOLEAN;
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWindow.Gehezu ..':30);
{$ENDIF}
    WITH Editor^ DO
      IF Position>-1 THEN
        SetSelection(Position, Position) { positionieren }
      ELSE
        SetSelection(GetLineIndex(GetNumLines) +
                     GetLineLength(GetNumLines),
        {goto bottom}GetLineIndex(GetNumLines) +
                     GetLineLength(GetNumLines));
{$IFDEF Trace}
    WriteLn('tEditWindow.Gehezu ok':30);
{$ENDIF}
  END;

  FUNCTION tEditWindow.ClearModify : BOOLEAN;
  BEGIN
    Editor^.ClearModify;
  END;

  FUNCTION tEditWindow.Clear : BOOLEAN;
  BEGIN
    Editor^.Clear;
  END;

{ tFileWindow }

  { Reads the contents of a previously-specified file into
    the tEdit child control.}
  FUNCTION tFileWindow.Read_Daten : BOOLEAN;
  CONST
    BufferSize = 1024;
  VAR
    CharsToRead : LongInt;
    BlockSize   : INTEGER;
    aStream     : pDosStream;
    aBuffer     : pChar;
  BEGIN
{$IFDEF Trace}
    WriteLn('tFileWindow.Read_Daten ..':30);
{$ENDIF}
    aStream := New(pDosStream, Init(FileName, stOpen));
    aBuffer := MemAlloc(BufferSize + 1);
    CharsToRead := aStream^.GetSize;
    IF aBuffer <> NIL THEN BEGIN
      Editor^.Clear;
      WHILE CharsToRead > 0 DO BEGIN
        IF CharsToRead > BufferSize THEN
          BlockSize := BufferSize
        ELSE
          BlockSize := CharsToRead;
        aStream^.Read(aBuffer^, BlockSize);
        aBuffer[BlockSize] := Char(0);
        Editor^.Insert(aBuffer);
        CharsToRead := CharsToRead - BlockSize;
      END;
      IsNewFile := FALSE;
      Editor^.ClearModify;
      Editor^.SetSelection(0, 0);
      FreeMem(aBuffer, BufferSize + 1);
    END;
    Dispose(aStream, Done);
    Read_Daten := TRUE;
{$IFDEF Trace}
    WriteLn('tFileWindow.Read_Daten ok':30);
{$ENDIF}
  END;

  { Writes the contents of the tEdit child control to a
    previously-specified file. If the operation will cause
    truncation of the text, first confirms (through
    displaying a message box) that it is OK to proceed. }
  FUNCTION tFileWindow.Write_Daten : BOOLEAN;
  CONST
    BufferSize = 1024;
  VAR
    CharsToWrite, CharsWritten : LongInt;
    BlockSize                  : INTEGER;
    aStream                    : pDosStream;
    aBuffer                    : POINTER;
    NumLines                   : INTEGER;
  BEGIN
{$IFDEF Trace}
    WriteLn('tFileWindow.Write_Daten ..':30);
{$ENDIF}
    NumLines := Editor^.GetNumLines;
    CharsToWrite := Editor^.GetLineIndex(NumLines-1) +
                    Editor^.GetLineLength(NumLines-1);
    aStream := New(pDosStream, Init(FileName, stCreate));
    aBuffer := MemAlloc(BufferSize + 1);
    CharsWritten := 0;
    IF aBuffer <> NIL THEN BEGIN
      WHILE CharsWritten < CharsToWrite DO BEGIN
        IF CharsToWrite - CharsWritten > BufferSize THEN
          BlockSize := BufferSize
        ELSE
          BlockSize := CharsToWrite - CharsWritten;
        Editor^.GetSubText(aBuffer, CharsWritten,
                           CharsWritten + BlockSize);
        aStream^.Write(aBuffer^, BlockSize);
        CharsWritten := CharsWritten + BlockSize;
      END;
      IsNewFile := FALSE;
      Editor^.ClearModify;
      FreeMem(aBuffer, BufferSize + 1);
    END;
    Dispose(aStream, Done);
{$IFDEF Trace}
    WriteLn('tFileWindow.Write_Daten ok':30);
{$ENDIF}
  END;

  PROCEDURE RegisterStdWnds;
  BEGIN
    RegisterType(rEditWindow);
    RegisterType(rFileWindow);
    RegisterType(rEdit);
    RegisterType(rScroller);
  END;

END.
(* ====================================================== *)
(*                Ende von MPAINT.PAS                     *)
