(* ====================================================== *)
(*                         MYWIN.PAS                      *)
(*                                                        *)
(*            (c) 1993 Martin Wlker & toolbox            *)
(*                                                        *)
(* Containerfenster fr beliebige Datenobjekte in MDI-Apps*)
(* Aufgabenteilung: MDI-Rahmenfenster cm_MDIFileNew       *)
(*                                    cm_MDIFileOpen      *)
(*                                    cm_Exit             *)
(*                                    cm_ArrangeIcons     *)
(*                                    cm_TileChildren     *)
(*                                    cm_CascadeChildren  *)
(*                                    cm_CloseChildren    *)
(*                  Container-Fenster cm_FileNew          *)
(*                                    cm_FileOpen         *)
(*                                    cm_FileSave         *)
(*                                    cm_FileSaveAs       *)
(*                                    cm_EditFind         *)
(*                                    cm_EditReplace      *)
(*                                    cm_EditFindNext     *)
(*                                    cm_EditClear        *)
(*                  Daten-Objekt      cm_EditCut          *)
(*                                    cm_EditCopy         *)
(*                                    cm_EditPaste        *)
(*                                    cm_EditDelete       *)
(*                                                        *)
(* ====================================================== *)
(* tWindow                                                *)
(*      |                                                 *)
(* tColFontWin   - Zustzlich Schriftart/Farbe            *)
(*     |           einstellbar                            *)
(*     |                                                  *)
(* tEditWin      + Verwaltungsroutinen fr folgende Typen *)
(*    |          |                                        *)
(* tSearchWin    + Suchfuntionen fr beliebige Daten      *)
(*   |           |                                        *)
(* tContainerWin + Filefuntionen fr beliebige Daten      *)
(*               |                                        *)
(*             Abstrakte Klassen mit Defaut-Routinen      *)
(*                                                        *)
(* ====================================================== *)
(* define trace   Wenn Trace Definiert ist, dann geben    *)
(*                verschiedene Methoden Meldungen aus.    *)
(* ====================================================== *)
UNIT MyWin;

INTERFACE

USES  ODialogs, Objects, MMSystem, OStdDlgs,
      WinTypes, WinProcs, CommDlg, OMemory,
      Strings, OWindows, WinDos, BWCC;

{ Neue Kommandos fr alle Fenster }
CONST
  cm_ClearAll = 900; { Clear All abfangen }
  cm_font     = 901; { Fontauswahl }
  cm_color    = 902; { Farbwahl }

  { Liefert das aktive Child Window }
  FUNCTION GetActiveMDIChild: pWindowsObject;

  { Liest den Meldungstext aus einer Stringresource }
  FUNCTION MessageBox(hWindow       : hWnd;
                      Text, Caption,
                      TextStyle     : WORD) : INTEGER;
  { Liest die Formatanweisung aus einer Stringresource }
  FUNCTION wvSprintF(Dest : pChar; Format : WORD;
                     VAR ArgList) : INTEGER;

TYPE { Neue Fenstertypen }

  pColFontWin = ^tColFontWin;
  { Erweitert jedes Fenster um einen Zeichensatz und
    eine Farbe }
  tColFontWin = OBJECT (tWindow)
    theFont  : hFont;
    aLogFont : tLogFont;
    ColorRef : LongInt;
    CONSTRUCTOR Init(aParent : pWindowsObject;
                     aName   : pChar);
    DESTRUCTOR  Done;                               VIRTUAL;
    CONSTRUCTOR Load(VAR s : tStream);
    PROCEDURE   Store(VAR s : tStream);
    { -> Datenobjekt ist neu zu zeichnen }
    FUNCTION    Show : BOOLEAN;                     VIRTUAL;
    { Stellt die userdefinierte Schrift und Farbe ein }
    PROCEDURE   Paint(PaintDC : hDC;
                      VAR PaintInfo: tPaintStruct); VIRTUAL;
    { Zwei neue Standard-Kommandos }
    PROCEDURE   CMColor(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_color;
    PROCEDURE   CMFonts(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_font;
 { Einige Windowsbotschaften erfordern nderungen am Child }
    PROCEDURE   WMControlColor(VAR Msg : tMessage);
                VIRTUAL wm_First + wm_CtlColor;
    PROCEDURE   WMSize(VAR Msg : tMessage);
                VIRTUAL wm_First + wm_Size;
    PROCEDURE   WMSetFocus(VAR Msg : tMessage);
                VIRTUAL wm_First + wm_SetFocus;
    { Auch Childwindow mit diesen Informationen versorgen }
    { Voreingestellt ist hier das zuletzt hinzugefgte
      Objekt. }
    FUNCTION    Font2Child : BOOLEAN;               VIRTUAL;
    FUNCTION    Focus2Child : BOOLEAN;              VIRTUAL;
    FUNCTION    Size2Child(VAR Msg : tMessage) : BOOLEAN;
                                                    VIRTUAL;
  END;

  pEditWin = ^tEditWin;
  { Smtliche Botschaften zum Editieren werden von dem
    Datenobjekt selbst verwaltet. Der Container stellt
    nur wichtige Routinen zur Verwaltung zur Verfgung. }
  tEditWin = OBJECT (tColFontWin)
    FUNCTION  CanClear : BOOLEAN;                   VIRTUAL;
    PROCEDURE wmKeyDown(VAR Msg : tMessage);
              VIRTUAL wm_First + wm_KeyDown;
    { Clear all aus Sicherheitsgrnden hier schon abfangen }
    PROCEDURE cmEditClear (VAR Msg : tMessage);
              VIRTUAL cm_First + cm_ClearAll;
    FUNCTION  Clear : BOOLEAN;                      VIRTUAL;
    { -> Datenobjekt }
    FUNCTION  ClearModify : BOOLEAN;                VIRTUAL;
    { -> Datenobjekt ob es gendert wurde }
    FUNCTION  IsModified : BOOLEAN;                 VIRTUAL;
  END;

  pSearchWin = ^tSearchWin;
  { Erweitert jedes Fenster um Suchen und Ersetzen }
  tSearchWin = OBJECT (tEditWin)
  { Kommandos }
    PROCEDURE CMEditFind(VAR Msg : tMessage);
              VIRTUAL cm_First + cm_EditFind;
    PROCEDURE CMEditFindNext(VAR Msg : tMessage);
              VIRTUAL cm_First + cm_EditFindNext;
    PROCEDURE CMEditReplace(VAR Msg : tMessage);
              VIRTUAL cm_First + cm_EditReplace;
    { Diese Routine fhrt alle Aktionen durch }
    PROCEDURE DoSearch;                             VIRTUAL;
  { die Suchroutine DoSearch  ruft die abstrakten Routinen }
  { suchen, ersetzen und gehezu, die von Nachfahren        }
  { dieser Klasse berschrieben werden mssen              }
    FUNCTION  Suchen(SearchText : pChar;
                     CaseSensitive: Bool): INTEGER; VIRTUAL;
    FUNCTION  Ersetzen(ReplaceText: pChar):BOOLEAN; VIRTUAL;
    FUNCTION  GeheZu(Position : INTEGER) : BOOLEAN; VIRTUAL;
  END;

  pContainerWin = ^tContainerWin;
  { Erweitert jedes Fenster um die Logik fr Filezugriffe }
  tContainerWin = OBJECT (tSearchWin)
    IsNewFile : BOOLEAN;
    FileName  : pChar;
    CONSTRUCTOR Init(aParent : pWindowsObject;
                     aTitle, aFileName : pChar);
    DESTRUCTOR  Done;                               VIRTUAL;
    CONSTRUCTOR Load(VAR s : tStream);
    PROCEDURE   Store(VAR s : tStream);
    FUNCTION    CanClear : BOOLEAN;                 VIRTUAL;
    FUNCTION    CanClose : BOOLEAN;                 VIRTUAL;
    PROCEDURE   NewFile;
    PROCEDURE   Open;
    PROCEDURE   SetFileName(aFileName : pChar);
    PROCEDURE   ReplaceWith(aFileName : pChar);
    FUNCTION    Save : BOOLEAN;
    FUNCTION    SaveAs : BOOLEAN;
    PROCEDURE   SetupWindow;                        VIRTUAL;
    PROCEDURE   CMFileNew(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_FileNew;
    PROCEDURE   cmFileOpen(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_FileOpen;
    PROCEDURE   CMFileSave(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_FileSave;
    PROCEDURE   CMFileSaveAs(VAR Msg : tMessage);
                VIRTUAL cm_First + cm_FileSaveAs;
    { Routinen mssen berschrieben werden, da sie }
    { auf die bearbeiteten Daten wirken.           }
    FUNCTION    Read_Daten : BOOLEAN;               VIRTUAL;
    FUNCTION    Write_Daten : BOOLEAN;              VIRTUAL;
  END;

  pDoubleDialog = ^tDoubleDialog;
  tDoubleDialog = OBJECT (tDialog)
    FilePath : ARRAY [0..fsPathName] OF CHAR;
    CONSTRUCTOR Init(aParent : pWindowsObject;
                     aFilePath : pChar);
    PROCEDURE   SetupWindow;                        VIRTUAL;
    PROCEDURE   BJa(VAR Msg : tMessage);
                VIRTUAL id_First + id_Yes;
    PROCEDURE   BNein(VAR Msg : tMessage);
                VIRTUAL id_First + id_No;
    PROCEDURE   BSaveAs(VAR Msg : tMessage);
                VIRTUAL id_First + 103;
  END;

CONST
  rColFontWin : tStreamRec = (
    ObjType : 4101;
    vmtLink : Ofs(TypeOf(tColFontWin)^);
    Load    : @tColFontWin.Load;
    Store   : @tColFontWin.Store);
  rSearchWin : tStreamRec = (
    ObjType : 4102;
    vmtLink : Ofs(TypeOf(tSearchWin)^);
    Load    : @tSearchWin.Load;
    Store   : @tSearchWin.Store);
  rContainerWin : tStreamRec = (
    ObjType : 4103;
    vmtLink : Ofs(TypeOf(tContainerWin)^);
    Load    : @tContainerWin.Load;
    Store   : @tContainerWin.Store);

IMPLEMENTATION

CONST
  Modul_Name : pChar = 'MyWin';
  Version            = '1.7';
  Date               = '10-18-1993';

{ Globale Variable }
VAR
  HourGlass  : hCursor;
  StdFont    : hFont;
  StdLogFont : tLogFont;

  PROCEDURE MakeDefaultFont;
  { erzeugt fr alle Fenster einen global verfgbaren Font }
  BEGIN
    FillChar(StdLogFont, SizeOf(tLogFont), #0);
    WITH StdLogFont DO BEGIN
      lfHeight         :=  14;  { Make a large font         }
      lfWeight         := 100;  { indicate a Bold attribute }
      lfItalic         :=   0;  { <> 0  --> italic          }
      lfUnderline      :=   0;  { <> 0  --> underline       }
      lfOutPrecision   := out_Stroke_Precis;
      lfClipPrecision  := clip_Stroke_Precis;
      lfQuality        := Default_Quality;
      lfPitchAndFamily := Variable_Pitch;
      StrCopy(lfFaceName, 'Courier');
    END;
    StdFont := CreateFontIndirect(StdLogFont);
  END;

  FUNCTION wvSprintF(Dest : pChar; Format : WORD;
                     VAR ArgList) : INTEGER;
  { gleiches Verhalten, ld aber stringresource }
  VAR
    Puffer : ARRAY [0..255] OF CHAR;
  BEGIN
    StrCopy(Puffer, '');
    LoadString(hInstance, Format, @Puffer, 255);
    wvSprintF := WinProcs.wvSprintF(Dest, Puffer, ArgList);
  END;

  FUNCTION MessageBox(hWindow       : hWnd;
                      Text, Caption,
                      TextStyle     : WORD) : INTEGER;
  { gleiches Verhalten, ld aber Stringresource }
  VAR
    Puffer : ARRAY [0..255] OF CHAR;
    Titel  : ARRAY [0..60] OF CHAR;
  BEGIN
    StrCopy(Puffer, '');  StrCopy(Titel, '');
    LoadString(hInstance, Text, @Puffer, 255);
    LoadString(hInstance, Caption, @Titel, 60);
    MessageBox := BWCCMessageBox(hWindow,
                                 Puffer, Titel, TextStyle);
  END;

  FUNCTION GetActiveMDIChild : pWindowsObject;
  VAR
    L : LongInt;
    h : hWnd;
  BEGIN
    L := SendMessage
         (Application^.MainWindow^.GetClient^.hWindow,
          wm_MDIGetActive, 0, 0);
    h := LoWord(L);
    IF h <> 0 THEN
      GetActiveMDIChild := GetObjectPtr(h)
    ELSE
      GetActiveMDIChild := NIL;
  END;

CONST
  id_FileName = 100;
  id_PathName = 101;

  CONSTRUCTOR tDoubleDialog.Init(aParent : pWindowsObject;
                                 aFilePath : pChar);
  BEGIN
    INHERITED Init(aParent, 'OVERWRITE');
    StrCopy(FilePath, aFilePath);
  END;

  PROCEDURE tDoubleDialog.BJa(VAR Msg : tMessage);
  BEGIN
    EndDlg(id_Yes);
  END;

  PROCEDURE tDoubleDialog.BNein(VAR Msg : tMessage);
  BEGIN
    EndDlg(id_No);
  END;

  PROCEDURE tDoubleDialog.BSaveAs(VAR Msg : tMessage);
  BEGIN
    EndDlg(id_Retry);
  END;

  PROCEDURE tDoubleDialog.SetupWindow;
  VAR
    ReOpenBuff : tOfStruct;
    Name       : ARRAY [0..fsFileName+4] OF CHAR;
    Dir        : ARRAY [0..fsDirectory] OF CHAR;
    Ext        : ARRAY [0..fsExtension] OF CHAR;
    OldCursor  : hCursor;
  BEGIN
{$IFDEF Trace}
    WriteLn('tDoubleDlg.SetupWindow ..':30);
{$ENDIF}
    IF OpenFile(FilePath, ReOpenBuff,
                of_Exist) <> -1 THEN BEGIN
      INHERITED SetupWindow;
      FileSplit(FilePath, Dir, Name, Ext);
      StrCat(Name, Ext);
      SetDlgItemText(hWindow, id_FileName, AnsiLower(Name));
      SetDlgItemText(hWindow, id_PathName, AnsiLower(Dir));
    END ELSE
      EndDlg(id_OK);
{$IFDEF Trace}
    WriteLn('tDoubleDlg.SetupWindow ok':30);
{$ENDIF}
  END;

CONST
  id_SearchText      = 100;
  id_CaseSensitive   = 101;
  id_ReplaceText     = 102;
  id_ReplaceAll      = 103;
  id_PromptOnReplace = 104;
  id_For             = 604;
  id_Back            = 605;
  id_Akt             = 608;
  id_Start           = 609;

TYPE
  tSearchRec = RECORD
                 SearchText      : ARRAY [0..80] OF CHAR;
                 CaseSensitive   : Bool;
                 Vorwaerts,
                 Raewaerts       : Bool;
                 AbCursor,
                 Vollstaendig    : Bool;
                 ReplaceText     : ARRAY [0..80] OF CHAR;
                 ReplaceAll      : Bool;
                 PromptOnReplace : Bool;
                 IsReplace       : BOOLEAN;
               END;

VAR
  SearchRec : tSearchRec;      { globaler Searchrecord }
    { Daher kann in einem Fenster gesucht werden       }
    { und derselbe Suchstring auch in anderen Fenstern }
    { ohne Neueingabe verwendet werden                 }

TYPE
  pSearchDlg = ^tSearchDlg;
  tSearchDlg = OBJECT (tDialog)
    CONSTRUCTOR Init(aParent  : pWindowsObject;
                     Template : pChar;
                     VAR SearchRec);
  END;

  CONSTRUCTOR tSearchDlg.Init(aParent  : pWindowsObject;
                              Template : pChar;
                              VAR SearchRec);
  VAR
    c : pWindowsObject;
  BEGIN
{$IFDEF Trace}
    WriteLn('tSearchDlg.init ..':30);
{$ENDIF}
    tDialog.Init(aParent, Template);
    c := New(pEdit, InitResource(@Self, id_SearchText, 81));
    c := New(pCheckBox,
             InitResource(@Self, id_CaseSensitive));
    c := New(pRadioButton, InitResource(@Self, id_For));
    c := New(pRadioButton, InitResource(@Self, id_Back));
    c := New(pRadioButton, InitResource(@Self, id_Akt));
    c := New(pRadioButton, InitResource(@Self, id_Start));
    IF (Template^ = 'r') THEN BEGIN
      c := New(pEdit,
               InitResource(@Self, id_ReplaceText, 81));
      c := New(pCheckBox,
               InitResource(@Self, id_ReplaceAll));
      c := New(pCheckBox,
               InitResource(@Self, id_PromptOnReplace));
    END;
    TransferBuffer := @SearchRec;
{$IFDEF Trace}
    WriteLn('tSearchDlg.init ok':30);
{$ENDIF}
  END;

{****************************}
{ Fenster mit Farbe und Font }
{****************************}

  CONSTRUCTOR tColFontWin.Init(aParent : pWindowsObject;
                               aName   : pChar);
  BEGIN
{$IFDEF Trace}
    WriteLn('tColorFontWinw.init ..':30);
{$ENDIF}
    INHERITED Init(aParent, aName);
    ColorRef := RGB(0, 0, 64);
    aLogFont := StdLogFont;
    theFont  := StdFont;
{$IFDEF Trace}
    WriteLn('tColorFontWinw.init ok':30);
{$ENDIF}
  END;

  DESTRUCTOR tColFontWin.Done;
  BEGIN
{$IFDEF Trace}
    WriteLn('tColorFontWinw.done ..':30);
{$ENDIF}
    { individuelle Schrift fr das Fenster lschen }
    IF theFont <> StdFont THEN DeleteObject(theFont);
    INHERITED Done;
{$IFDEF Trace}
    WriteLn('TColorFontWinw.done ok':30);
{$ENDIF}
  END;

  CONSTRUCTOR tColFontWin.Load(VAR s : tStream);
  BEGIN
{$IFDEF Trace}
    WriteLn('tColorFontWinw.Load ..':30);
{$ENDIF}
    INHERITED Load(s);
    s.Read(ColorRef, SizeOf(ColorRef));
    s.Read(aLogFont, SizeOf(aLogFont));
    theFont := CreateFontIndirect(aLogFont);
{$IFDEF Trace}
    WriteLn('tColorFontWinw.Load ok':30);
{$ENDIF}
  END;

  PROCEDURE tColFontWin.Store(VAR s : tStream);
  BEGIN
{$IFDEF Trace}
    WriteLn('tColorFontWinw.Store ..':30);
{$ENDIF}
    INHERITED Store(s);
    s.Write(ColorRef, SizeOf(ColorRef));
    s.Write(aLogFont, SizeOf(aLogFont));
{$IFDEF Trace}
    WriteLn('TColorFontWinw.Store ok':30);
{$ENDIF}
  END;

  FUNCTION tColFontWin.Show : BOOLEAN;
  BEGIN
    InvalidateRect(ChildList^.hWindow, NIL, FALSE);
  END;

  PROCEDURE tColFontWin.Paint(PaintDC : hDC;
                              VAR PaintInfo : tPaintStruct);
  BEGIN
    INHERITED Paint(PaintDC, PaintInfo);
    SelectObject(PaintDC, theFont);
    SetTextColor(PaintDC, ColorRef);
  END;

  { Untergeordnete Fenster bentigen ebenfalls Farbe.}
  PROCEDURE tColFontWin.WMControlColor(VAR Msg : tMessage);
  BEGIN
    DefWndProc(Msg);
    SetTextColor(Msg.wParam, ColorRef);
  END;

  { Displays the "Choose Color" dialog from the common
    dialogs unit. Permits an initial value to be inserted
    and custom colors to be developed. Note, custom colors
    are not used by the "ChooseFont" dialog. }
  PROCEDURE tColFontWin.CMColor(VAR Msg : tMessage);
  TYPE
    tLongAry = ARRAY [0..15] OF LongInt;
  CONST
    { Establishes a set of custom colors in
      15 shades of blue }
    CustColors : tLongAry = (
      $000000, $100000, $200000, $300000,
      $400000, $500000, $600000, $700000,
      $800000, $900000, $A00000, $B00000,
      $C00000, $D00000, $E00000, $F00000);
  VAR
    ChooseClr : tChooseColor;
    i         : INTEGER;
  BEGIN
    WITH ChooseClr DO BEGIN
      hWndOwner    := hWindow;
      lStructSize  := SizeOf(tChooseColor);
      RgbResult    := ColorRef;
      lpCustColors := @CustColors;
      Flags        := cc_FullOpen OR cc_RGBInit;
    END;
    IF NOT ChooseColor(ChooseClr) THEN Exit;
    ColorRef := ChooseClr.RgbResult;
    Show;
  END;

  { Displays the ChooseFont dialog to permit the selection
    of a font which is returned as a TLogFont. Then a font
    handle is created from this logical font information.}
  PROCEDURE tColFontWin.CMFonts(VAR Msg : tMessage);
  VAR
    ChooseRec : tChooseFont;
    Colors    : LongInt;
    Style     : ARRAY [0..100] OF CHAR;
    TempFont  : tLogFont;
  BEGIN
    FillChar(ChooseRec, SizeOf(ChooseRec), #0);
    WITH ChooseRec DO BEGIN
      lStructSize := SizeOf(tChooseFont);
      hWndOwner   := hWindow;
      lpLogFont   := @aLogFont;
      Flags       := cf_ScreenFonts OR cf_Effects OR
                     cf_InitToLogFontStruct;
      RgbColors   := ColorRef;
      lpszStyle   := Style;
    END;
    IF NOT ChooseFont(ChooseRec) THEN Exit;

    { Update the Font and Color data fields,
      then cause the window to be repainted.}
    IF theFont <> StdFont THEN DeleteObject(theFont);
    ColorRef:= ChooseRec.RgbColors;
    theFont := CreateFontIndirect(aLogFont);
    Font2Child;
  END;

  FUNCTION tColFontWin.Font2Child : BOOLEAN;
  BEGIN
    SendMessage(ChildList^.hWindow, wm_SetFont, theFont, 0);
    Show;
  END;

  { Responds to an incoming wm_Size message by resizing
    the child edit control according to the size of the
    tColFontWins client area. }
  PROCEDURE tColFontWin.WMSize(VAR Msg : tMessage);
  BEGIN
    INHERITED WMSize(Msg);
    Size2Child(Msg);
  END;

  FUNCTION tColFontWin.Size2Child
           (VAR Msg : tMessage) : BOOLEAN;
  BEGIN
    SetWindowPos(ChildList^.hWindow, 0, -1, -1,
                 Msg.lParamLo+2, Msg.lParamHi+2,
                 swp_NoZOrder);
  END;

  { Responds to an incoming wm_SetFocus message
    by setting the focus to the child edit control. }
  PROCEDURE tColFontWin.WMSetFocus(VAR Msg : tMessage);
  BEGIN
    Focus2Child;
  END;

  FUNCTION tColFontWin.Focus2Child : BOOLEAN;
  BEGIN
    SetFocus(ChildList^.hWindow);
  END;

{**************************}
{ Fenster mit EditFunktion }
{**************************}

  FUNCTION tEditWin.CanClear : BOOLEAN;
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWin.CanClear ..':30);
{$ENDIF}
    CanClear := IsModified;
{$IFDEF Trace}
    WriteLn('tEditWin.CanClear ok':30);
{$ENDIF}
  END;

  FUNCTION tEditWin.IsModified : BOOLEAN;
  BEGIN
    MessageBox(hWindow, 49, 48, mb_OK + mb_IconExclamation);
  END;

  PROCEDURE tEditWin.cmEditClear(VAR Msg : tMessage);
  BEGIN
{$IFDEF Trace}
    WriteLn('tEditWin.CMEditClear ..':30);
{$ENDIF}
    IF CanClear THEN Clear;
{$IFDEF Trace}
    WriteLn('tEditWin.CMEditClear ok':30);
{$ENDIF}
  END;

  FUNCTION tEditWin.Clear : BOOLEAN;
  BEGIN
    MessageBox(hWindow, 52, 48, mb_OK + mb_IconExclamation);
  END;

  FUNCTION tEditWin.ClearModify : BOOLEAN;
  BEGIN
    MessageBox(hWindow, 51, 48, mb_OK + mb_IconExclamation);
  END;

  PROCEDURE tEditWin.wmKeyDown(VAR Msg : tMessage);
  BEGIN
    CASE Msg.wParam OF
      vk_Down:
        SendMessage(hWindow, wm_VScroll, sb_LineDown, 0);
      vk_Up:
        SendMessage(hWindow, wm_VScroll, sb_LineUp,   0);
      vk_Next:
        SendMessage(hWindow, wm_VScroll, sb_PageDown, 0);
      vk_Prior:
        SendMessage(hWindow, wm_VScroll, sb_PageUp,   0);
      vk_Home:
        SendMessage(hWindow, wm_VScroll, sb_Top,      0);
      vk_End:
        SendMessage(hWindow, wm_VScroll, sb_Bottom,   0);
    END;
  END;

{************************}
{ Search Fenstermethoden }
{************************}

  PROCEDURE tSearchWin.CMEditFind(VAR Msg : tMessage);
  BEGIN
    IF Application^.ExecDialog
       (New(pSearchDlg,
            Init(@Self, 'search',
                        SearchRec))) = id_OK THEN BEGIN
      IF StrLen(SearchRec.SearchText) = 0 THEN BEGIN
        MessageBox(hWindow, 65, 64, mb_IconStop OR mb_OK);
        Exit;
      END ELSE BEGIN
        SearchRec.IsReplace := FALSE;
        DoSearch;
      END;
    END;
  END;

  PROCEDURE tSearchWin.CMEditFindNext(VAR Msg : tMessage);
  BEGIN
    DoSearch;
  END;

  PROCEDURE tSearchWin.CMEditReplace(VAR Msg : tMessage);
  VAR
    Puffer : ARRAY [0..30] OF CHAR;
  BEGIN
    IF Application^.ExecDialog
       (New(pSearchDlg,
            Init(@Self,'replace', SearchRec))) = id_OK THEN
      IF StrLen(SearchRec.SearchText) = 0 THEN BEGIN
        MessageBox(hWindow, 65, 64, mb_IconStop OR mb_OK);
        Exit;
      END ELSE BEGIN
        SearchRec.IsReplace := TRUE;
        DoSearch;
      END;
  END;

  PROCEDURE tSearchWin.DoSearch;
  VAR
    s, T : ARRAY [0..80] OF CHAR;
    p    : POINTER;
    Rslt : INTEGER;
  BEGIN
{$IFDEF Trace}
    WriteLn('tSearchWin.DoSearch ..':30);
{$ENDIF}
    Rslt := 0;
    WITH SearchRec DO
      REPEAT
        IF Vollstaendig THEN BEGIN
          IF Vorwaerts THEN
            GeheZu(0)  { goto top }
          ELSE
            GeheZu(-1);{ goto bottom }
          AbCursor     := TRUE;
          Vollstaendig := FALSE;
        END;
        Rslt := Suchen(SearchText, CaseSensitive);
        IF Rslt = -1 THEN BEGIN
          IF NOT IsReplace OR NOT ReplaceAll THEN BEGIN
            p := @SearchText;
            wvSprintF(s, 66, p);
            LoadString(hInstance, 64, @T, 80);
            BWCCMessageBox(hWindow, s, T,
                           mb_OK + mb_IconExclamation);
          END;
        END ELSE IF IsReplace THEN
          IF NOT PromptOnReplace THEN
            Ersetzen(ReplaceText)
          ELSE BEGIN
            Rslt := MessageBox(hWindow, 68, 67,
                          mb_YesNoCancel + mb_IconQuestion);
            IF Rslt = id_Yes THEN
              Ersetzen(ReplaceText)
            ELSE IF Rslt = id_Cancel THEN Exit;
          END;
      UNTIL (Rslt = -1) OR NOT ReplaceAll OR NOT IsReplace;
{$IFDEF Trace}
    WriteLn('tSearchWin.DoSearch ok':30);
{$ENDIF}
  END;

  FUNCTION tSearchWin.Suchen(SearchText : pChar;
                             CaseSensitive: Bool) : INTEGER;
  BEGIN
    MessageBox(hWindow, 69, 64, mb_OK + mb_IconExclamation);
  END;

  FUNCTION tSearchWin.Ersetzen(ReplaceText : pChar):BOOLEAN;
  BEGIN
    MessageBox(hWindow, 70, 64, mb_OK + mb_IconExclamation);
  END;

  FUNCTION tSearchWin.GeheZu(Position : INTEGER) : BOOLEAN;
  BEGIN
    MessageBox(hWindow, 71, 64, mb_OK + mb_IconExclamation);
  END;

{*************************}
{ ContainerFenstermethoden}
{*************************}

  FUNCTION tContainerWin.Read_Daten : BOOLEAN;
  BEGIN
    MessageBox(hWindow, 33, 32, mb_OK + mb_IconExclamation);
  END;

  FUNCTION tContainerWin.Write_Daten : BOOLEAN;
  BEGIN
    MessageBox(hWindow, 34, 32, mb_OK + mb_IconExclamation);
  END;

  { Constructor for a tContainerWin.
    Initializes its data fields using
    passed parameters and default values. }
  CONSTRUCTOR tContainerWin.Init(aParent : pWindowsObject;
                                 aTitle, aFileName : pChar);
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.Init ..':30);
{$ENDIF}
    INHERITED Init(aParent, aTitle);
    IsNewFile := TRUE;
    FileName := StrNew(aFileName);
{$IFDEF Trace}
    WriteLn('tContainerWin.Init ok':30);
{$ENDIF}
  END;

  { Dispose of the file name }
  DESTRUCTOR tContainerWin.Done;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.Done ..':30);
{$ENDIF}
    StrDispose(FileName);
    INHERITED Done;
{$IFDEF Trace}
    WriteLn('tContainerWin.Done ok':30);
{$ENDIF}
  END;

{ Load a tContainerWin from the stream }
  CONSTRUCTOR tContainerWin.Load(VAR s : tStream);
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.Load ..':30);
{$ENDIF}
    INHERITED Load(s);
    FileName := s.StrRead;
    IsNewFile := FileName = NIL;
{$IFDEF Trace}
    WriteLn('tContainerWin.Load ok':30);
{$ENDIF}
  END;

  { Store a tContainerWin from the stream }
  PROCEDURE tContainerWin.Store(VAR s : tStream);
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.Store ..':30);
{$ENDIF}
    INHERITED Store(s);
    s.StrWrite(FileName);
{$IFDEF Trace}
    WriteLn('tContainerWin.Store ok':30);
{$ENDIF}
  END;

  { Performs setup for a tContainerWin,
    appending 'Untitled' to its caption }
  PROCEDURE tContainerWin.SetupWindow;
  VAR
    OldCursor : hCursor;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.SetupWindow ..':30);
{$ENDIF}
    INHERITED SetupWindow;
    IF FileName <> NIL THEN BEGIN
      OldCursor := SetCursor(HourGlass);
      IF Read_Daten THEN
        SetFileName(FileName);
      SetCursor(OldCursor);
      ClearModify;
      IsNewFile := FALSE;
    END ELSE
      SetFileName(FileName);
{$IFDEF Trace}
    WriteLn('tContainerWin.SetupWindow ok':30);
{$ENDIF}
  END;

  { Sets the file name of the window and updates the
    caption. Assumes that the AFileName parameter and
    the FileName instance variable were allocated
    by StrNew. }
  PROCEDURE tContainerWin.SetFileName(aFileName : pChar);
  VAR
    NewCaption : ARRAY [0..80] OF CHAR;
    p          : ARRAY [0..1] OF pChar;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.SetFileName ..':30);
{$ENDIF}
    IF FileName <> aFileName THEN BEGIN
      StrDispose(FileName);
      FileName := StrNew(aFileName);
    END;
    p[0] := Attr.Title;
    IF FileName = NIL THEN p[1] := '(Untitled)'
                      ELSE p[1] := aFileName;
    IF Attr.Title = NIL THEN
      SetWindowText(hWindow, p[1])
    ELSE BEGIN
      WinProcs.wvSprintF(NewCaption,
                         '%0.40s - %0.37s', p[0]);
      SetWindowText(hWindow, NewCaption);
    END;
{$IFDEF Trace}
    WriteLn('tContainerWin.SetFileName ok':30);
{$ENDIF}
  END;

  { Begins the edit of a new file, after determining that it
    is Ok to clear the TEdit's text. }
  PROCEDURE tContainerWin.NewFile;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.NewFile ..':30);
{$ENDIF}
    IF CanClear THEN BEGIN
      Clear;
      Show;
      ClearModify;
      IsNewFile := TRUE;
      SetFileName(NIL);
    END;
{$IFDEF Trace}
    WriteLn('tContainerWin.NewFile ok':30);
{$ENDIF}
  END;

  { Replaces the current file with the given file. }
  PROCEDURE tContainerWin.ReplaceWith(aFileName : pChar);
  VAR
    OldCursor : hCursor;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.ReplaceWith ..':30);
{$ENDIF}
    OldCursor := SetCursor(HourGlass);
    IF Read_Daten THEN
      SetFileName(aFileName);
    SetCursor(OldCursor);
    ClearModify;
    Show;
{$IFDEF Trace}
    WriteLn('tContainerWin.ReplaceWith ok':30);
{$ENDIF}
  END;

  { Brings up a dialog allowing the user to open a file into
    this window.
    Save as selecting File|Open from the menus.}
  PROCEDURE tContainerWin.Open;
  VAR
    TmpName : ARRAY [0..fsPathName] OF CHAR;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.Open ..':30);
{$ENDIF}
    IF CanClear AND
       (Application^.ExecDialog(New(pFileDialog,
                     Init(@Self,
                     pChar(sd_FileOpen),
                    StrCopy(TmpName, '*.*')))) = id_OK) THEN
      ReplaceWith(TmpName);
    IsNewFile := FALSE;
{$IFDEF Trace}
    WriteLn('tContainerWin.Open ok':30);
{$ENDIF}
  END;

  { Saves the contents of the tEdit child control into the
    file currently being editted.  Returns true if the file
    was saved. }
  FUNCTION tContainerWin.Save : BOOLEAN;
  VAR
    OldCursor : hCursor;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.Save ..':30);
{$ENDIF}
    Save := FALSE;
    IF IsModified THEN
      IF IsNewFile THEN BEGIN
        Save := SaveAs;
      END ELSE BEGIN
        OldCursor := SetCursor(HourGlass);
        Write_Daten;
        SetCursor(OldCursor);
        ClearModify;
        Save := TRUE;
      END;
{$IFDEF Trace}
    WriteLn('tContainerWin.Save ok':30);
{$ENDIF}
  END;

  { Saves the contents of the TEdit child control into a
    file whose name is retrieved from the user, through
    execution of a "Save" file dialog.
    Returns true if the file was saved. }
  FUNCTION tContainerWin.SaveAs : BOOLEAN;
  VAR
    TmpName   : ARRAY [0..fsPathName] OF CHAR;
    OldCursor : hCursor;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.SaveAs ..':30);
{$ENDIF}
    SaveAs := FALSE;
    IF FileName <> NIL THEN
      StrCopy(TmpName, FileName)
    ELSE
      TmpName[0] := #0;
    WHILE Application^.ExecDialog(New(pFileDialog,
        Init(@Self, pChar(sd_FileSave), TmpName))) = id_OK
    DO BEGIN
      CASE Application^.ExecDialog(
           New(pDoubleDialog,Init(@Self,TmpName))) OF
        id_Retry : Continue; { Neuen Filenamen abfragen}
        id_No    : Break;    { Abfrageschleife verlassen }
        id_OK,   { olles ok!  Speichern und verlassen }
        id_Yes   : BEGIN
                     SetFileName(TmpName);
                     OldCursor := SetCursor(HourGlass);
                     Write_Daten;
                     SetCursor(OldCursor);
                     ClearModify;
                     IsNewFile := FALSE;
                     SaveAs := TRUE;
                     Break;
                   END;
       END;
    END;
{$IFDEF Trace}
    WriteLn('tContainerWin.SaveAs ok':30);
{$ENDIF}
  END;

  { Returns a Boolean value indicating whether or not it is
    Ok to clear the TEdit's text.  Returns True if the text
    has not been changed, or if the user Oks the clearing
    of the text. }
  FUNCTION tContainerWin.CanClear : BOOLEAN;
  VAR
    Name   : ARRAY [0..fsFileName] OF CHAR;
    Dir    : ARRAY [0..fsDirectory] OF CHAR;
    Ext    : ARRAY [0..fsExtension] OF CHAR;
    Puffer : ARRAY [0..fsPathName+50] OF CHAR;
    p      : ARRAY [0..2] OF pChar;
    Rslt   : INTEGER;
  BEGIN
{$IFDEF Trace}
    WriteLn('tContainerWin.CanClear ..':30);
{$ENDIF}
    CanClear := TRUE;
    IF IsModified THEN BEGIN
      IF FileName = NIL THEN
        LoadString(hInstance, 38, @Puffer, fsPathName)
      ELSE BEGIN
        FileSplit(FileName, Dir, Name, Ext);
        p[0] := Name;
        p[1] := Ext;
        p[2] := Dir;
        wvSprintF(Puffer, 39, p);
      END;
      LoadString(hInstance, 37, @Dir, fsPathName);
      CASE BWCCMessageBox(hWindow,
         Puffer, Dir, mb_YesNoCancel OR mb_IconQuestion) OF
        id_Yes    : CanClear := Save;
        id_Cancel : CanClear := FALSE;
      END;
    END;
{$IFDEF Trace}
    WriteLn('tContainerWin.CanClear ok':30);
{$ENDIF}
  END;

  { Returns a Boolean value indicating whether or not it is
    Ok to close the TEdit's text.  Returns the result of a
    call to Self.CanClear. }
  FUNCTION tContainerWin.CanClose : BOOLEAN;
  BEGIN
    CanClose := CanClear;
  END;

  { Responds to an incoming "New" command (with a cm_FileNew
    command identifier) by calling Self.New. }
  PROCEDURE tContainerWin.CMFileNew(VAR Msg : tMessage);
  BEGIN
    NewFile;
  END;

  { Responds to an incoming "Open" command (with a
    cm_FileOpen command identifier) by calling Self.Open. }
  PROCEDURE tContainerWin.cmFileOpen(VAR Msg : tMessage);
  BEGIN
    Open;
  END;

  { Responds to an incoming "Save" command (with a
    cm_FileSave command identifier) by calling Self.Save. }
  PROCEDURE tContainerWin.CMFileSave(VAR Msg : tMessage);
  BEGIN
    Save;
  END;

  { Responds to an incoming "SaveAs" command (with a
    cm_FileSaveAs command identifier) by calling Self.SaveAs. }
  PROCEDURE tContainerWin.CMFileSaveAs(VAR Msg : tMessage);
  BEGIN
    SaveAs;
  END;

  { Defaulteinstellungen beim Einbinden der Unit }
BEGIN
  MakeDefaultFont;
  HourGlass := LoadCursor(0, idc_Wait);
  WITH SearchRec DO BEGIN
    SearchText[0] := #0;      ReplaceText[0]  := #0;
    CaseSensitive := TRUE;
    Vorwaerts     := TRUE;    Raewaerts       := FALSE;
    AbCursor      := TRUE;    Vollstaendig    := FALSE;
    ReplaceAll    := FALSE;   PromptOnReplace := TRUE;
  END;
END.
(* ====================================================== *)
(*                  Ende von MYWIN.PAS                    *)
