(* ====================================================== *)
(*                       RAMSTM.PAS                       *)
(*          (c) 1993 Peter M. Abuja & toolbox             *)
(* ====================================================== *)
{$C PRELOAD MOVEABLE DISCARDABLE}
UNIT RAMStm;

INTERFACE

USES WinTypes,
{$IFNDEF Ver70}
WObjects;
{$ELSE}
Objects, oMemory, oWindows;
{$ENDIF}

CONST
  MaxByteArray =  $FFF0;
  MaxRAMPage   = $10000;
  HalfRAMPage  =  $7FFF;
  stxMemError  =  4;

TYPE
  pByteArray   = ^tByteArray;
  tByteArray   = ARRAY [0..MaxByteArray] OF BYTE;

TYPE
  pRAMStream   = ^tRAMStream;
  tRAMStream   = OBJECT (tStream)
    Position,
    Size,
    AllocSize,
    MinReAlloc : LongInt;
    Buffer     : pByteArray;
    hMem       : tHandle;
    FixedSize  : BOOLEAN;

    CONSTRUCTOR Init(aSize : LongInt);
    CONSTRUCTOR Open(aBuffer : Pointer;
                     aHandle : tHandle;
                     aSize   : LongInt);
    DESTRUCTOR  Done;                               VIRTUAL;
    DESTRUCTOR  Close;                              VIRTUAL;

    FUNCTION    GetHandle : tHandle;
    FUNCTION    GetBuffer : pByteArray;

    PROCEDURE   Read(VAR Buf; Count : WORD);        VIRTUAL;
    PROCEDURE   Write(VAR Buf; Count : WORD);       VIRTUAL;

    PROCEDURE   Seek(Pos : LongInt);                VIRTUAL;
    PROCEDURE   Truncate;                           VIRTUAL;

    FUNCTION    GetPos        : LongInt;            VIRTUAL;
    FUNCTION    GetSize       : LongInt;            VIRTUAL;
    FUNCTION    GetAllocSize  : LongInt;
    FUNCTION    GetMinReAlloc : LongInt;
    PROCEDURE   SetMinReAlloc(NewMinReAlloc : LongInt);
 END;

IMPLEMENTATION

USES WinProcs;

CONST
  AllocMin = 1024;

TYPE
  LongType = RECORD
               CASE WORD OF
                 0 : (Ptr    : Pointer);
                 1 : (Long   : LongInt);
                 2 : (Lo, Hi : WORD);
             END;

  PROCEDURE aHIncr; FAR;
    EXTERNAL 'KERNEL' Index 114;

  CONSTRUCTOR tRAMStream.Init(aSize : LongInt);
  BEGIN
    tStream.Init;
    Position   := 0;
    Size       := 0;
    AllocSize  := Abs(aSize);
    MinReAlloc := AllocSize;
    IF MinReAlloc < AllocMin  THEN MinReAlloc := AllocMin;
    IF MaxAvail   < AllocSize THEN Error(stInitError, 0);
                              { memory cannot be allocated }
    IF (aSize > MaxByteArray) OR (aSize <= 0) THEN BEGIN
                                    { get mem from windows }
      FixedSize := FALSE;
      Buffer    := NIL;
      IF aSize < AllocMin THEN AllocSize := MinReAlloc;
      hMem := GlobalAlloc(GMEM_MOVEABLE OR GMEM_ZEROINIT OR
                          GMEM_SHARE, AllocSize);
      IF hMem = 0 THEN Error(stInitError, 0);
                                    { mem alloc failed     }
      AllocSize := GlobalSize(hMem);
    END ELSE BEGIN                  { get only one segment }
      FixedSize := TRUE;
      hMem      := 0;
      Buffer    := MemAlloc(AllocSize);
      FillChar(Buffer^, AllocSize, 0);
    END;
  END;

  CONSTRUCTOR tRAMStream.Open(aBuffer : Pointer;
                              aHandle : tHandle;
                              aSize   : LongInt);
  BEGIN
    tStream.Init;
    IF (aHandle = 0) OR (GlobalSize(aHandle) = 0) THEN BEGIN
                                      {  fixed size stream }
      FixedSize := TRUE;
      hMem      := 0;
      Buffer    := aBuffer;
      Position  := 0;
      Size      := aSize;
      AllocSize := aSize;
    END ELSE BEGIN                  { variable size stream }
      FixedSize := FALSE;
      hMem      := aHandle;
      Buffer    := NIL;
      AllocSize := GlobalSize(hMem);
      IF AllocSize = 0 THEN Error(stInitError, 0);
      MinReAlloc := Abs(aSize);
      IF MinReAlloc < AllocMin THEN MinReAlloc := AllocMin;
      Size     := AllocSize;
      Position := 0;
    END;
  END;

  DESTRUCTOR tRAMStream.Done;
  BEGIN
    IF FixedSize THEN FreeMem(Buffer, AllocSize)
                 ELSE GlobalFree(hMem);
    tStream.Done;
  END;

  DESTRUCTOR tRAMStream.Close;
  BEGIN
    tStream.Done;
  END;

  PROCEDURE tRAMStream.Read(VAR Buf; Count : WORD);
  VAR
    StartPos, BufPtr, pBuf : LongType;
    MaxBytes               : LongInt;
  BEGIN
    IF (Position+Count > Size) OR
       (Status <> stOk) THEN BEGIN
      Error(stReadError, 0);
                            { read past EOF or other error }
      FillChar(Buf, Count, 0);
    END ELSE BEGIN
      IF FixedSize THEN BEGIN          { fixed size stream }
        Move(Buffer^[Position], Buf, Count);
        Inc(Position, Count);
      END ELSE BEGIN               { variable sized stream }
        Buffer := GlobalLock(hMem);
                              { pointer to start of stream }
        IF Buffer = NIL THEN BEGIN
          Error(stReadError, 0);   { cannot obtain pointer }
          Exit;
        END;
        pBuf.Ptr   := Addr(Buf);
        BufPtr.Ptr := Buffer;

        WHILE Count > 0 DO BEGIN
            { obtain a correct pointer for the read buffer }
          StartPos.Long := Position;
          MaxBytes      := MaxRAMPage - StartPos.Lo;
          StartPos.Hi   := BufPtr.Hi +
                           StartPos.Hi * Ofs(aHIncr);
          Buffer        := StartPos.Ptr;

          IF Count < MaxBytes THEN MaxBytes := Count;
          IF MaxBytes > HalfRAMPage THEN
            MaxBytes := HalfRAMPage;
          Move(Buffer^[0], pBuf.Ptr^, MaxBytes);
          Dec(Count,     MaxBytes);
          Inc(Position,  MaxBytes);
          Inc(pBuf.Long, MaxBytes);
        END;
        Buffer := NIL;
        GlobalUnlock(hMem);
      END;
    END;
  END;

  PROCEDURE tRAMStream.Write(VAR Buf; Count : WORD);
  VAR
    StartPos, BufPtr, pBuf : LongType;
    Alloc, OldAlloc        : LongInt;
    MaxBytes               : LongInt;
  BEGIN
    IF Status = stOk THEN
      IF FixedSize THEN               { fixed  size stream }
        IF Position+Count > AllocSize THEN
          Error(stWriteError, 0)               { too large }
        ELSE BEGIN
          Move(Buf, Buffer^[Position], Count);
          Inc(Position, Count);
          IF Position > Size THEN Size := Position;
        END
      ELSE BEGIN                   { variable sized stream }
        IF Position+Count > AllocSize THEN BEGIN
          IF Count < MinReAlloc THEN Alloc := MinReAlloc
                                ELSE Alloc := Count;
          Inc(Alloc, Position);
          hMem := GlobalRealloc(hMem, Alloc, GMEM_MOVEABLE);
          IF hMem = 0 THEN BEGIN
            Error(stWriteError, stxMemError);
            Exit;
          END;
          OldAlloc  := AllocSize;
          AllocSize := GlobalSize(hMem);
                      { adjust MinReAlloc for a power of 2 }
          IF Count < MinReAlloc THEN
            MinReAlloc := AllocSize - OldAlloc;
        END;
        Buffer := GlobalLock(hMem);
                              { pointer to start of stream }
        IF Buffer = NIL THEN BEGIN
          Error(stWriteError, 0);
          Exit;
        END;
        pBuf.Ptr   := Addr(Buf);
        BufPtr.Ptr := Buffer;
        WHILE Count > 0 DO BEGIN
            { obtain a correct pointer for the read buffer }
          StartPos.Long := Position;
          MaxBytes      := MaxRAMPage - StartPos.Lo;
          StartPos.Hi   := BufPtr.Hi +
                           StartPos.Hi*Ofs(aHIncr);
          Buffer        := StartPos.Ptr;
          IF Count < MaxBytes THEN MaxBytes := Count;
          IF MaxBytes > HalfRAMPage THEN
            MaxBytes := HalfRAMPage;
          Move(pBuf.Ptr^, Buffer^[0], MaxBytes);
          Dec(Count,     MaxBytes);
          Inc(Position,  MaxBytes);
          Inc(pBuf.Long, MaxBytes);
        END;
        IF Size < Position THEN Size := Position;
        Buffer := NIL;
        GlobalUnlock(hMem);
      END;
  END;

  FUNCTION tRAMStream.GetHandle : tHandle;
  BEGIN
    GetHandle := hMem;
  END;

  FUNCTION tRAMStream.GetBuffer : pByteArray;
  BEGIN
    GetBuffer := Buffer;
  END;

  PROCEDURE tRAMStream.Seek(Pos : LongInt);
  BEGIN
    IF Pos > Size THEN Error(stReadError, 0)
                  ELSE Position := Pos;
  END;

  PROCEDURE tRAMStream.Truncate;
  BEGIN
    Size := Position;
    IF NOT FixedSize THEN BEGIN
      hMem      := GlobalRealloc(hMem, Size, GMEM_MOVEABLE);
      AllocSize := GlobalSize(hMem);
    END;
  END;

  FUNCTION tRAMStream.GetPos : LongInt;
  BEGIN
    GetPos := Position;
  END;

  FUNCTION tRAMStream.GetSize : LongInt;
  BEGIN
    GetSize := Size;
  END;

  FUNCTION tRAMStream.GetAllocSize : LongInt;
  BEGIN
    GetAllocSize := AllocSize;
  END;

  FUNCTION tRAMStream.GetMinReAlloc : LongInt;
  BEGIN
    GetMinReAlloc := MinReAlloc;
  END;

  PROCEDURE tRAMStream.SetMinReAlloc
                                  (NewMinReAlloc : LongInt);
  BEGIN
    MinReAlloc := NewMinReAlloc;
  END;

END.
(* ====================================================== *)
(*              Ende von RAMSTM.PAS                       *)
