(* ====================================================== *)
(*                        TESTDLG.PAS                     *)
(*            (c) 1993 Peter M. Abuja & toolbox           *)
(* ====================================================== *)
{$C PRELOAD MOVEABLE DISCARDABLE}

UNIT TestDlg;

{
  SOURCE  00000
  CODE    00000
  DATA    00000
  STACK   00000
  LOCALH  00000
}
{$R RAMSTR.RES }

INTERFACE

USES WinTypes,
{$IFNDEF Ver70}
WObjects,
{$ELSE}
Objects, oDialogs, oWindows,
{$ENDIF}
BWCC, RAMStm;

CONST
  WMStreamData = 'wm_StreamData';

VAR
  wm_StreamData : WORD;     { define a new windows message }

{$I RAMSTR.INC }

TYPE
  pTestDlg = ^tTestDlg;
  tTestDlg = OBJECT (tDialog)
    TheCollection : pStrCollection;
    TheStream     : pRAMStream;
    TheBox        : pListBox;
    MyType        : INTEGER;

    CONSTRUCTOR Init(aParent : pWindowsObject;
                     aName : pChar);
    DESTRUCTOR  Done;                               VIRTUAL;
    PROCEDURE   SetupWindow;                        VIRTUAL;
    PROCEDURE   CreateStrings(VAR Msg : tMessage);
                VIRTUAL id_First + id_Create;
    PROCEDURE   SendStrings(VAR Msg : tMessage);
                VIRTUAL id_First + id_Send;
    PROCEDURE   DefWndProc(VAR Msg : tMessage);     VIRTUAL;
  END;

IMPLEMENTATION

USES WinProcs, Strings;

CONST
  TestString : ARRAY [0..17] OF CHAR = 'String %05u ...';

  CONSTRUCTOR tTestDlg.Init(aParent : pWindowsObject;
                            aName   : pChar);
  VAR
    AnEdit : pEdit;
  BEGIN
    tDialog.Init(aParent, aName);
    TheCollection := NIL;
    TheStream     := NIL;
    TheBox        := New(pListBox,
                         InitResource(@Self, id_List));

    IF StrIComp(Attr.Name, 'SEND_DLG') = 0 THEN
      AnEdit := New(pEdit,
                    InitResource(@Self, id_NumStrings, 20));
  END;

  DESTRUCTOR tTestDlg.Done;
  BEGIN
    IF TheCollection <> NIL THEN
      Dispose(TheCollection, Done);
    IF TheStream <> NIL THEN Dispose(TheStream, Done);
    tDialog.Done;
  END;

  PROCEDURE tTestDlg.SetupWindow;
  VAR
    AnEdit : pEdit;
  BEGIN
    tDialog.SetupWindow;

    IF StrIComp(Attr.Name, 'SEND_DLG') = 0 THEN BEGIN
      AnEdit := pEdit(ChildWithID(id_NumStrings));
      AnEdit^.SetText('10');
    END;
  END;

  PROCEDURE tTestDlg.CreateStrings(VAR Msg : tMessage);
  VAR
    NumStr, Error, i : INTEGER;
    S                : ARRAY [0..20] OF CHAR;
    AnEdit           : pEdit;
  BEGIN
    AnEdit := pEdit(ChildWithID(id_NumStrings));
    AnEdit^.GetText(S, 20);
    Val(S, NumStr, Error);
    IF (Error <> 0) OR (NumStr < 1) THEN NumStr := 10;
    IF NumStr > 65536 DIV Succ(SizeOf(TestString)) THEN
      NumStr := 65536 DIV Succ(SizeOf(TestString));

    TheBox^.ClearList;
    IF TheCollection <> NIL THEN
      Dispose(TheCollection, Done);
    TheCollection := New(pStrCollection, Init(NumStr, 1));
    FOR i := 1 TO NumStr DO BEGIN
      wvsprintf(S, TestString, i);
      TheCollection^.Insert(StrNew(S));
      TheBox^.AddString(S);
    END;
  END;

  PROCEDURE tTestDlg.SendStrings(VAR Msg : tMessage);
  VAR
    aHandle     : tHandle;
    Destination : hWnd;
    aSize       : LongInt;
  BEGIN
    IF (TheCollection = NIL) OR (TheStream <> NIL) THEN
      Exit;
    TheStream := New(pRAMStream, Init(-8192));
    IF TheStream^.Status <> stOk THEN BEGIN
      MessageBox(hWindow, 'Could not create stream',
                 'ERROR', mb_OK OR mb_IconExclamation);
      Exit;
    END;
    TheCollection^.Store(TheStream^);
      { obtain handle and allocation limit before closing }
    aHandle := TheStream^.GetHandle;
    aSize   := TheStream^.GetAllocSize;
      { obtain a handle to the target window }
    Destination := FindWindow('BorDlg', 'Receive');

    IF Destination <> 0 THEN BEGIN
        { close stream but leave memory }
      Dispose(TheStream, Close);
      TheStream := NIL;

      { post handle and allocation limit - receiver is now
        owner of the memory and responsible for freeing it }
      PostMessage(Destination, wm_StreamData,
                  aHandle, aSize);
    END ELSE BEGIN
      MessageBox(hWindow, 'Could not find receiver!',
                 'ERROR', mb_OK OR mb_IconExclamation);

      {  no receiver - so clean up yourself }
      Dispose(TheStream, Done);
      TheStream := NIL;
    END;
  END;

  PROCEDURE tTestDlg.DefWndProc(VAR Msg : tMessage);

    PROCEDURE LoadBox(P : pChar); FAR;
    BEGIN
      TheBox^.AddString(P);
    END;

  BEGIN

    { this message should be only posted to a receiver ! }
    IF Msg.Message = wm_StreamData THEN BEGIN
      IF TheStream <> NIL THEN Dispose(TheStream, Done);

        { create the stream from the memory handle }
      TheStream := New(pRAMStream,
                       Open(NIL, Msg.wParam, Msg.lParam));
      IF TheCollection <> NIL THEN
        Dispose(TheCollection, Done);
      TheCollection := New(pStrCollection,
                           Load(TheStream^));
      TheBox^.ClearList;

      TheCollection^.ForEach(@LoadBox);

      { the owner must dispose the memory now }
      Dispose(TheStream, Done);
      TheStream := NIL;
    END;

    tDialog.DefWndProc(Msg);
  END;

BEGIN
   RegisterWObjects;

   { as at least two applications will use this message
     it must be registered }
   wm_StreamData := RegisterWindowMessage(WMStreamData);
END.
(* ====================================================== *)
(*                  Ende von TESTDLG.PAS                  *)

