/* ====================================================== */
/*                      RIPMENU.CPP                       */
/*      (c) 1993 Raimond Reichert & toolbox               */
/* ====================================================== */

#define Uses_TRipMenuBar
#define Uses_TRipMenuBox
#define Uses_TPinMenuBox

#include <ripmenu.h>
#include <tv.h>

// newSubView, in der Dokumentation zu Turbo Vision
// genauso wie bei der Turbo-Pascal-Ausfhrung bersehen,
// erstellt eine Menbox und ist hier berschrieben, so da
// die Menboxen vom Typ TRipMenuBox sind.
// ACHTUNG: TMenuView::newSubView ist aus unverstndlichen
// Grnden nicht virtuell (in Turbo Pascal ist diese Methode
// virtuell), so da vor dem Einsatz dieser Methode
// virtual in die Definition in MENUS.H eingefgt und das
// Ganze mit make neu bersetzt werden mu.

TMenuView *TRipMenuBar::newSubView (const TRect& bounds,
                                    TMenu *aMenu,
                                    TMenuView *aParentMenu)
{
  return new TRipMenuBox (bounds, aMenu, aParentMenu);
}


TStreamable *TRipMenuBar::build (void)
{
  return new TRipMenuBar (streamableInit);
}

const char * const TRipMenuBar::name = "TRipMenuBar";

TStreamableClass RRipMenuBar (TRipMenuBar::name,
                              TRipMenuBar::build,
                              __DELTA (TRipMenuBar));

// execute() fngt doPin ab, das von getEvent auf True
// gesetzt wird, falls die Menbox verschoben wurde, und
// reagiert mit dem Einfgen einer neuen TPinMenuBox an
// der Stelle, an der das alte Men eben geschlossen
// wurde. Dazu mu die Position ber
// TProgramm::deskTop->makeLocal berechnet werden, denn
// die geschlossene Menbox gehrte zuvor zu
// TProgram::application.

ushort TRipMenuBox::execute (void)
{
  ushort result = TMenuBox::execute ();
  if (doPin)
  {
    TRect menuBounds = getBounds ();
    menuBounds.a =
      TProgram::deskTop->makeLocal (menuBounds.a);
    menuBounds.b =
      TProgram::deskTop->makeLocal (menuBounds.b);
    TMenuBox *pinBox = newPinBox (menuBounds, menu);
    TProgram::deskTop->insert (pinBox);
    doPin = False;
  }
  return result;
}

// Sofern getEvent ein Anklicken des oberen Rahmens
// registriert, werden zunchst ber owner->makeLocal
// aus denselben Grnden wie in execute zunchst die
// Bewegungsgrenzen umgerechnet, die an dragView bergeben
// werden. doPin = True weist execute an, da die Box
// verschoben wurde. Anschlieend wird das letzte Event
// durch ein Kuckucksei ersetzt, damit das Men den Focus
// aufgibt. Sonst kann es passieren, da beim Loslassen der
// Maustaste ein Menbefehl abgesetzt wird.

void TRipMenuBox::getEvent (TEvent& event)
{
  TMenuBox::getEvent (event);

  if ((event.what == evMouseDown &&
       makeLocal (event.mouse.where).y == 0))
  {
    TRect limits = TProgram::deskTop->getBounds ();
    limits.a = owner->makeLocal (limits.a);
    limits.b = owner->makeLocal (limits.b);
    dragView(event, dmDragMove + dmLimitAll, limits,
             size, size);
    event.what = evKeyDown;
    event.keyDown.keyCode = kbEsc;
    doPin = True;
  }
}

// newPinBox wird von execute aufgerufen, um eine neue
// Pinbox zu erstellen. Will jemand Nachfahren der hier
// implementierten Klassen zchten, so mu er auf jeden Fall
// alle drei newSubView-Methoden sowie newPinBox
// berschreiben.

TMenuBox* TRipMenuBox::newPinBox(TRect& bounds,
                                 TMenu *aMenu)
{
  return new TPinMenuBox (bounds, aMenu);
}

// newSubView liefert fr den Fall von verschachtelten
// Untermens eine Menbox vom eigenen Typ, damit auch
// solche Mens abreibar sind.

TMenuView *TRipMenuBox::newSubView (const TRect& bounds,
                                    TMenu *aMenu,
                                    TMenuView *aParentMenu)
{
  return new TRipMenuBox (bounds, aMenu, aParentMenu);
}

// TPinMenuBox hat wie Fenster ein Schliefeld, ber das
// die Box abgeschossen werden kann, und falls
// cmReceivedFocus eintrifft und der Aussender ein nicht
// modales Objekt ist, drngt sich die Menbox ber
// makeFirst in den Vordergrund.
// Die beiden neuen Felder dieser Klasse sind fr zwei
// mgliche Flle zustndig:
// loaded wird gesetzt, falls das Objekt vom Stream
// gelesen wird. Dann ist nmlich menu eine eigene Kopie
// und mu vom Destruktor selbst gelscht werden; im
// Normalfall ist menu nur ein Zeiger in die Men-
// struktur, die zu TProgram::menuBar gehrt.
// In growMode wird gfGrowAll gesetzt, so da die
// Menboxen beim Umschalten von einem Bildschirmmodus in
// den anderen einen konstanten Abstand von der rechten
// unteren Bildschirmecke halten (aber aufgepat:
// so knnen kleine Mens, die z.B. im 80x50-Modus am oberen
// Bildschirmrand kleben, beim Umschalten in den 80x25-Modus
// verschwinden!).

TPinMenuBox::TPinMenuBox (TRect& bounds, TMenu *aMenu) :
    TRipMenuBox (bounds, aMenu)
{
  loaded = False;
  growMode |= gfGrowAll;
}

TPinMenuBox::~TPinMenuBox ()
{
    if (loaded) delete menu;
}

void TPinMenuBox::draw (void)
{
    TRipMenuBox::draw ();
    writeChar (3, 0, '[', 1, 1);
    writeChar (4, 0, '', 3, 1);
    writeChar (5, 0, ']', 1, 1);
}

// handleEvent mu auf die Schliebox und auf
// cmReceivedFocus reagieren.

void TPinMenuBox::handleEvent (TEvent& event)
{
  TPoint mouse = makeLocal (event.mouse.where);
  if (event.what == evMouseDown &&
      mouse.x > 2 && mouse.x < 6 && mouse.y == 0)
  {
    destroy (this);
    clearEvent (event);
  }
  else if (event.what == evBroadcast)
    switch (event.message.command)
    {
      case cmReceivedFocus:
        if (!((TView*)event.message.infoPtr)->
                            getState (sfModal))
          makeFirst ();
        break;
      default:
        break;
    }
  TRipMenuBox::handleEvent (event);
}

// newPinBox wird von TRipMenuBox::execute aufgerufen,
// sobald die Box verschoben wurde. Da nun aber eine
// TPinMenuBox verschoben wurde, ist es berflssig, eine
// ebensolche neu zu erstellen. Daher gibt newPinBox den
// Focus auf, den es beim Verschieben erhielt, und gibt
// einen NULL-Zeiger zurck.

#pragma argsused
TMenuBox* TPinMenuBox::newPinBox
          (TRect& bounds, TMenu *aMenu)
{
  setState (sfFocused, False);
  drawView ();
  return NULL;
}

TMenuView *TPinMenuBox::newSubView (const TRect& bounds,
                                    TMenu *aMenu,
                                    TMenuView *aParentMenu)
{
    return new TRipMenuBox (bounds, aMenu, aParentMenu);
}

TStreamable *TPinMenuBox::build (void)
{
    return new TPinMenuBox (streamableInit);
}

// da updateMenu(TMenu*) privat in TMenuView ist,
// schickt sich read selbst den Rundruf
// cmCommandSetChanged, damit handleEvent doUpdate
// aufruft, denn es knnte sein, da der Zustand der Befehle
// (enabled oder nicht) nicht mehr der gleiche ist wie zum
// Zeitpunkt des Speicherns.

void *TPinMenuBox::read (ipstream& is)
{
    TMenuBox::read (is);
    loaded = True;
    message (this, evBroadcast, cmCommandSetChanged, NULL);
    return this;
}

const char * const TPinMenuBox::name = "TPinMenuBox";

TStreamableClass RPinMenuBox (TPinMenuBox::name,
                              TPinMenuBox::build,
                              __DELTA (TPinMenuBox));
/* ====================================================== */
/*                 Ende von RIPMENU.CPP                   */

