/* ====================================================== */
/*                      RIPMENU.H                         */
/*                                                        */
/* nach der Turbo-Pascal-Version von Christian Ohr, DOS-  */
/* toolbox 2'93; Klassen fr abreibare Mens.            */
/*                                                        */
/* Die Klasse TRipMenu sorgt dafr, da Boxen vom Typ   */
/* TRipMenuBox in Umlauf kommen, die abreibar und auf  */
/* dem Desktop frei plazierbar sind, allerdings nur ber  */
/* die Maus (denn wie sollte man ber die Tastatur ein    */
/* solches abgerissenes Men wieder aktivieren?). Hat man */
/* das Men fallengelassen, so wird an derselben Stelle   */
/* eine TPinMenuBox installiert, eine Menbox, die auf  */
/* dem Desktop bleibt und so Shortcuts mit der Maus       */
/* ermglicht. TRipMenuBar und TPinMenuBox sind beide */
/* streambar, fr TRipMenuBox ist diese Fhigkeit       */
/* unntig, denn dieses Objekt ist quasi ein temporres */
/* Objekt, das nur zwischenzeitlich benutzt wird.         */                                 */
/*                                                        */
/* Die Hierarchie sieht das so aus:                       */
/*   TMenuView                                            */
/*       TMenuBar                                         */
/*           TRipMenuBar                                  */
/*       TMenuBox                                         */
/*           TRipMenuBox                                  */
/*               TPinMenuBox                              */
/*                                                        */
/*       (c) 1993 Raimond Reichert & toolbox              */
/* ====================================================== */
#define Uses_TEvent
#define Uses_TRect
#define Uses_TMenu
#define Uses_TMenuItem
#define Uses_TMenuBar
#define Uses_TMenuBox
#define Uses_TSubMenu
#define Uses_TProgram
#define Uses_TDeskTop
#define Uses_TStreamableClass
#define Uses_TKeys

#if !defined (__TV_H)
#include <tv.h>
#endif

#if defined (Uses_TRipMenuBar) && !defined (__TRipMenuBar)
#define __TRipMenuBar

class TRipMenuBar : public TMenuBar
{
public:
  TRipMenuBar (const TRect& bounds, TSubMenu& aMenu) :
    TMenuBar (bounds, aMenu) {}
  TRipMenuBar (const TRect& bounds, TMenu *aMenu) :
    TMenuBar (bounds, aMenu) {}
  TRipMenuBar (StreamableInit) :
    TMenuBar (streamableInit) {};

  virtual TMenuView *newSubView (const TRect& bounds,
                                 TMenu *aMenu,
                                 TMenuView *aParentMenu);

  static const char * const name;
  static TStreamable *build (void);

private:
  virtual const char *streamableName (void) const
    { return name; }
};

inline ipstream& operator >> (ipstream& is,TRipMenuBar& cl)
    { return is >> (TStreamable&)cl; }
inline ipstream& operator >>(ipstream& is,TRipMenuBar*& cl)
    { return is >> (void *&)cl; }

inline opstream& operator << (opstream& os,TRipMenuBar& cl)
    { return os << (TStreamable&)cl; }
inline opstream& operator << (opstream& os,TRipMenuBar* cl)
    { return os << (TStreamable *)cl; }

#endif // __TRipMenuBar


#if defined (Uses_TRipMenuBox) && !defined (__TRipMenuBox)
#define __TRipMenuBox

class TRipMenuBox : public TMenuBox
{
protected:
    Boolean doPin;

public:
    TRipMenuBox (const TRect& bounds, TMenu *aMenu) :
      TMenuBox (bounds, aMenu, 0) { doPin = False; }
    TRipMenuBox (const TRect& bounds,
                 TMenu *aMenu,
                 TMenuView *aParentMenu) :
      TMenuBox (bounds, aMenu, aParentMenu)
      { doPin = False; }
    TRipMenuBox (StreamableInit) :
      TMenuBox (streamableInit) {}

    virtual ushort execute (void);
    virtual void getEvent (TEvent& event);
    virtual TMenuBox* newPinBox (TRect& bounds,
                                 TMenu *aMenu);
    virtual TMenuView *newSubView (const TRect& bounds,
                                   TMenu *aMenu,
                                   TMenuView *aParentMenu);
};

#endif // __TRipMenuBox


#if defined (Uses_TPinMenuBox) && !defined (__TPinMenuBox)
#define __TPinMenuBox

class TPinMenuBox : public TRipMenuBox
{
    Boolean loaded;
public:
    TPinMenuBox (TRect& bounds, TMenu *aMenu );
    TPinMenuBox (StreamableInit) :
      TRipMenuBox (streamableInit) {}
    ~TPinMenuBox ();

    virtual void draw (void);
    virtual void handleEvent (TEvent& event);
    virtual TMenuBox* newPinBox (TRect& bounds,
                                 TMenu *aMenu);
    virtual TMenuView *newSubView (const TRect& bounds,
                                   TMenu *aMenu,
                                   TMenuView *aParentMenu);

    static const char * const name;
    static TStreamable *build (void);

protected:
    virtual void *read (ipstream&);

private:
    virtual const char *streamableName (void) const
      { return name; }
};

#endif // __TPinMenuBox
/* ====================================================== */
/*                      RIPMENU.H                         */

