/* ----------------------------------------------------------------------- */
/*                                tinstdlg.cpp                             */
/*                                                                         */
/*                         (c) Raimond Reichert '93                        */
/*                                                                         */
/* Eine Anmerkung zum Speichern und Laden von TInstDialog'en: Da es keinen */
/* Sinn ergibt, die Kollektion mit ihren TViewData's zu speichern, schreibt*/
/* TVDCollection::writeItem nichts und gibt ::readItem einen NULL-Zeiger   */
/* zurck. Es obliegt dem Besitzer der Kollektion, in diesem Fall          */
/* TInstViewer bzw dessen Besitzer TInstDialog, dafr zu sorgen, dass nach */
/* dem Laden des Dialoges eine neue Kollektion erstellt wird !             */
/*                                                                         */
/* ----------------------------------------------------------------------- */
#define Uses_MsgBox
#define Uses_TButton
#define Uses_TCheckBoxes
#define Uses_TGroup
#define Uses_TInputLine
#define Uses_TInstDialog
#define Uses_TLabel
#define Uses_TPoint
#define Uses_TProgram
#define Uses_TRect
#define Uses_TScrollGroup
#define Uses_TSItem

#include "tinstdlg.h"
#include "tscrgrp.h"

#if !defined (__DOS_H)
#include <dos.h>
#endif

const cmSetData    = 10001;    // nur innerhalb von TInstDialog gltig
const cmDeleteView = 10002;    // dito

/* -----------------------------------------------------------------------
                                TViewData
   ----------------------------------------------------------------------- */
// Der Konstruktor kopiert alle Werte aus der View nach TViewData, damit
// eine manipulierbare Kopie fr den Dialog existiert; ushort-Werte mssen
// dabei in Strings konvertiert werden. Da name nur ein Zeiger ist, wird
// dieses Feld auch nicht vom Destruktor freigegeben.
// -----------------------------------------------------------------------
TViewData::TViewData (TView* aView, short aLevel) :
    view (aView),
    level (aLevel),
    state (view->state),
    options (view->options),
    eventMask (view->eventMask),
    growMode (view->growMode),
    dragMode (view->dragMode)
{
    itoa (view->origin.x, originX, 10);
    itoa (view->origin.y, originY, 10);
    itoa (view->size.x, sizeX, 10);
    itoa (view->size.y, sizeY, 10);
    itoa (view->helpCtx, helpCtx, 10);
    name = (char*) view->streamableName ();
}

/* -----------------------------------------------------------------------
                                TVDCollection
   ----------------------------------------------------------------------- */
#pragma argsused
void *TVDCollection::readItem (ipstream& is)
{
    return NULL;
}

#pragma argsused
void TVDCollection::writeItem (void *ptr, opstream& os)
{

}

TStreamable *TVDCollection::build (void)
{
    return new TVDCollection (streamableInit);
}

const char * const TVDCollection::name = "TVDCollection";

TStreamableClass RVDCollection (TVDCollection::name,
                                TVDCollection::build,
                                __DELTA (TVDCollection));

/* -----------------------------------------------------------------------
                             getInstTree
   ----------------------------------------------------------------------- */
TCollection* getInstTree (TView* view)
{
    TCollection* viewColl = new TVDCollection (100, 10);
    collectViews (view, 0, viewColl);
    return viewColl;
}

/* -----------------------------------------------------------------------
                             collectViews
   ----------------------------------------------------------------------- */
void collectViews (TView *view, int level, TCollection *viewColl)
{
    if (view)
    {
        TView* last = view;
        do
        {
            viewColl->insert (new TViewData (view, level));
            if (((TGroup*)view)->last->owner == (TGroup*)view)
                collectViews (((TGroup*)view)->last, level+1, viewColl);
            view = view->next;
        } while (view != last && view != NULL);
    }
}

/* -----------------------------------------------------------------------
                                TInstViewer
   ----------------------------------------------------------------------- */
TInstViewer::TInstViewer (const TRect& bounds,
                          TScrollBar *aHScrollBar,
                          TScrollBar *aVScrollBar,
                          TView *startView) :
    TLevelViewer (bounds, aHScrollBar, aVScrollBar)
{
    newList (getInstTree (startView));
}

// -----------------------------------------------------------------------
// focusItem verschickt cmInstFocused mit einem Zeiger auf die gerade
// fokussierte TViewData.
// -----------------------------------------------------------------------
void TInstViewer::focusItem (short item)
{
    TLevelViewer::focusItem (item);
    message (owner, evBroadcast, cmInstFocused, list()->at (item));
}

int TInstViewer::getLevel (short item)
{
    return ((TViewData*)list()->at (item))->level;
}

char* TInstViewer::getName (short item)
{
    return ((TViewData*)list()->at (item))->name;
}

void *TInstViewer::read (ipstream& is)
{
    TListBox::read (is);
    is >> levelWidth;
    return this;
}

void TInstViewer::write (opstream& os)
{
    TListBox::write (os);
    os << levelWidth;
}

TStreamable *TInstViewer::build (void)
{
    return new TInstViewer (streamableInit);
}

const char * const TInstViewer::name = "TInstViewer";

TStreamableClass RInstViewer (TInstViewer::name,
                              TInstViewer::build,
                              __DELTA (TInstViewer));

/* -----------------------------------------------------------------------
                                TInstDialog
   ----------------------------------------------------------------------- */
// Der Konstruktor von TInstDialog kmmert sich um alle seine usserlich-
// keiten - Grsse, Beschriftung und vor allem sein komplexes Inneres,
// aus TInstViewer, einer TScrollGroup mit TCheckBoxes, TLabels und
// TInputLines, und zwei TButtons. Damit auch von Beginn an die Daten der
// in TInstViewer fokussierten View angezeigt werden, wird instViewer->
// focusItem (0) aufgerufen, was zu cmInstFocused fhrt und von handleEvent
// mit setData () beantwortet wird.
// -----------------------------------------------------------------------
TInstDialog::TInstDialog () :
    TDialog (TRect (1, 0, 78, 23), "Instanzen - Monitor"),
    TWindowInit (&TInstDialog::initFrame)
{
    TRect ivr (1, 1, 75, 6);
    TRect vsb (ivr.b.x, ivr.a.y, ivr.b.x+1, ivr.b.y);
    TScrollBar *vScrollBar = new TScrollBar (vsb);
    insert (vScrollBar);
    instViewer = newInstViewer (ivr, NULL, vScrollBar);
    insert (instViewer);

    TPoint sgMaxSize;
    sgMaxSize.x = 73;  sgMaxSize.y = 22;
    TRect sgr (2, 7, 75, 19);
    TScrollGroup *sg = new TScrollGroup (TRect (2, 7, 75, 19),
                                         sgMaxSize, this);
    sg->insert (new TCheckBoxes (TRect (1, 0, 20, 12),
                                 new TSItem ("sfVisible",
                                 new TSItem ("sfCursorVis",
                                 new TSItem ("sfCursorIns",
                                 new TSItem ("sfShadow",
                                 new TSItem ("sfActive",
                                 new TSItem ("sfSelected",
                                 new TSItem ("sfFocused",
                                 new TSItem ("sfDragging",
                                 new TSItem ("sfDisabled",
                                 new TSItem ("sfModal",
                                 new TSItem ("sfDefault",
                                 new TSItem ("sfExposed", NULL))))))))))))));
    sg->insert (new TCheckBoxes (TRect (27, 0, 46, 10),
                                 new TSItem ("ofSelectable",
                                 new TSItem ("ofTopSelect",
                                 new TSItem ("ofFirstClick",
                                 new TSItem ("ofFramed",
                                 new TSItem ("ofPreProcess",
                                 new TSItem ("ofPostProcess",
                                 new TSItem ("ofBuffered",
                                 new TSItem ("ofTileable",
                                 new TSItem ("ofCenterX",
                                 new TSItem ("ofCenterY", NULL))))))))))));
    sg->insert (new TCheckBoxes (TRect (53, 0, 72, 10),
                                 new TSItem ("evMouseDown",
                                 new TSItem ("evMouseUp",
                                 new TSItem ("evMouseMove",
                                 new TSItem ("evMouseAuto",
                                 new TSItem ("evKeyDown",
                                 new TSItem ("(undefined)",
                                 new TSItem ("(undefined)",
                                 new TSItem ("(undefined)",
                                 new TSItem ("evCommand",
                                 new TSItem ("evBroadcast", NULL))))))))))));
    sg->insert (new TCheckBoxes (TRect (1, 13, 20, 19),
                                 new TSItem ("dmDragMove",
                                 new TSItem ("dmDragGrow",
                                 new TSItem ("dmLimitLoX",
                                 new TSItem ("dmLimitLoY",
                                 new TSItem ("dmLimitHiX",
                                 new TSItem ("dmLimitHiY", NULL))))))));
    sg->insert (new TCheckBoxes (TRect (27, 13, 46, 17),
                                 new TSItem ("gfGrowLoX",
                                 new TSItem ("gfGrowLoY",
                                 new TSItem ("gfGrowHiX",
                                 new TSItem ("gfGrowHiY", NULL))))));

    TInputLine *il;
    sg->insert (il = new TInputLine (TRect (64, 13, 69, 14), 4));
    sg->insert (new TLabel (TRect (53, 13, 63, 14), "origin.x:", il));

    sg->insert (il = new TInputLine (TRect (64, 14, 69, 15), 4));
    sg->insert (new TLabel (TRect (53, 14, 63, 15), "origin.y:", il));

    sg->insert (il = new TInputLine (TRect (64, 16, 69, 17), 4));
    sg->insert (new TLabel (TRect (53, 16, 63, 17), "size.x:", il));

    sg->insert (il = new TInputLine (TRect (64, 17, 69, 18), 4));
    sg->insert (new TLabel (TRect (53, 17, 63, 18), "size.y:", il));

    sg->insert (il = new TInputLine (TRect (64, 19, 69, 20), 6));
    sg->insert (new TLabel (TRect (53, 19, 63, 20), "helpCtx:", il));
    insert (sg);

    insert (new TButton (TRect (10, 20, 20, 22), "~C~lose", cmOK,
                         bfNormal+bfDefault));
    insert (new TButton (TRect (30, 20, 45, 22), "~S~et Data",
                         cmSetData, bfNormal));
    insert (new TButton (TRect (55, 20, 70, 22), "~D~elete View",
                         cmDeleteView, bfNormal));
    selectNext (False);

    instViewer->focusItem (0);  // Daten auch schn anzeigen
}

TView* TInstDialog::getStartView ()
{
    return TProgram::application;
}

TInstViewer* TInstDialog::newInstViewer (TRect& bounds,
                                         TScrollBar *aHScrollBar,
                                         TScrollBar *aVScrollBar)
{
    return new TInstViewer (bounds,
                            aHScrollBar,
                            aVScrollBar,
                            getStartView ());
}

// -----------------------------------------------------------------------
// Auf den Broadcast cmInstFocused reagiert handleEvent mit dem Setzen von
// currentInst und dem Aufruf von setData, wobei zu beachten ist, dass
// dieses mit der Addresse von currentInst->state aufgerufen wird, was
// bedingt, dass die Reihenfolge und Grsse der Felder in TViewData
// peinlich genau mit derjenigen der Dialogelemente bereinstimmt.
// Auf den Befehl "cmSetData" hin, der vom Button ausgesandt wird, werden
// alle Daten aus currentInst in die entsprechende View bertragen und
// deren Neudarstellung veranlasst, um so die Wirkung direkt nachvoll-
// ziehen zu knnen. Dabei darf nicht vergessen werden, dass auch die
// Daten in der Kollektion angepasst werden mssen, was durch die
// monsterhafte Ermittlung von viewData als Zeiger auf das Kollektions-
// element und anschliessendem getData (&(viewData->state)); geschieht.
// -----------------------------------------------------------------------
void TInstDialog::handleEvent (TEvent& event)
{
    TDialog::handleEvent (event);

    if (event.what == evBroadcast &&
        event.message.command == cmInstFocused &&
        event.message.infoPtr != currentInst)
    {
        currentInst = (TViewData*) event.message.infoPtr;
        setData (&(currentInst->state));
    }

    if (event.what == evCommand)
        switch (event.message.command)
        {
            case cmDeleteView:
                // beim Lschen gibt es ein paar Verbote, gegen die
                // im Interesse der Applikation nicht verstossen werden
                // darf:
                if (currentInst->view == TProgram::application)
                    sayNoNo ("Applikationsview darf nicht gelscht werden!",
                             owner);
                else if (currentInst->view == TProgram::deskTop &&
                         owner == TProgram::deskTop)
                    sayNoNo ("Der eigene Besitzer kann nicht "
                             "gelscht werden!",
                              owner);
                else if (owner == currentInst->view->owner)
                    // aus unerfindlichen Grnden ist es nicht mglich, Views
                    // zu lschen, die den gleichen Besitzer wie der Dialog
                    // haben - auch wenn es innerhalb der IDE mit dem
                    // "List all"-Dialog problemlos funktioniert:
                    sayNoNo ("Kann nicht View aus der eigenen Gruppe "
                             "lschen!", owner);
                else
                {
                    destroy (currentInst->view);
                    // Die Kollektion neu einlesen, damit auch die
                    // gelschten Subviews aus der Darstellung verschwinden;
                    // doch der Dialog muss zuvor aus der Gruppe entfernt
                    // werden, da er sonst auch mit eingelesen wird:
                    TGroup *theOwner = owner;
                    owner->lock ();
                    owner->remove (this);
                    instViewer->newList (getInstTree (getStartView ()));
                    theOwner->insert (this);
                    owner->unlock ();
                }
                clearEvent (event);
                break;

            case cmSetData:
                if (currentInst->view != TProgram::application)
                {
                    TViewData *viewData = (TViewData*) instViewer->list()->
                                            at (instViewer->list()->
                                                indexOf (currentInst));
                    // Die Daten in der Kollektion anpassen:
                    getData (&(viewData->state));
                    // Die View selbst manipulieren:
                    currentInst->view->state = 0;
                    currentInst->view->setState (viewData->state, True);
                    currentInst->view->options = viewData->options;
                    currentInst->view->eventMask = viewData->eventMask;
                    currentInst->view->growMode = viewData->growMode;
                    currentInst->view->dragMode = viewData->dragMode;
                    TRect bounds (atol (viewData->originX),
                                  atol (viewData->originY),
                                  atol (viewData->originX)+
                                  atol (viewData->sizeX),
                                  atol (viewData->originY)+
                                  atol (viewData->sizeY));
                    currentInst->view->changeBounds (bounds);
                    currentInst->view->helpCtx = atol (viewData->helpCtx);
                }
                else
                    sayNoNo ("Applikation darf wegen Stabilitt nicht"
                             "manipuliert werden!", owner);
                clearEvent (event);
                break;
            default:
                break;
        }
}

// -----------------------------------------------------------------------
// sayNoNo wird von handleEvent benutzt, um dem Benutzer mitzuteilen, das
// er gerade etwas befehlen wollte, was verboten ist. Allerdings
// funktioniert die Messagebox nicht, falls der Instanzendialog auf
// Applikationsebene eingefgt wurde, daher als Alternative der
// "Soundeffekt".
// -----------------------------------------------------------------------
void TInstDialog::sayNoNo (char *msg, TGroup *theOwner)
{
    if (theOwner != TProgram::application)
        messageBox (msg, mfOKButton);
    else
    {
        sound (1000); delay (25); sound (2000); delay (25);
        sound (1000); delay (25);
        nosound ();
    }
}

void *TInstDialog::read (ipstream& is)
{
    TDialog::read (is);
    is >> instViewer;
    instViewer->newList (getInstTree (getStartView ()));
    return this;
}

void TInstDialog::write (opstream& os)
{
    TDialog::write (os);
    os << instViewer;
}

TStreamable *TInstDialog::build (void)
{
    return new TInstDialog (streamableInit);
}

const char * const TInstDialog::name = "TInstDialog";

TStreamableClass RInstDialog (TInstDialog::name,
                              TInstDialog::build,
                              __DELTA (TInstDialog));

