/* ------------------------------------------------------ */
/*                      tinstdlg.h                        */
/*                                                        */
/*                (c) Raimond Reichert '93                */
/*                                                        */
/* Nach dem Instanzen-Monitor aus "Turbo Pascal 6.0, Turbo*/
/* Vision" von A.Ertl, R.Machholz und A.Schallmeier,      */
/* Addison-Wesley 1991.                                   */
/*                                                        */
/* tinstdlg.h definiert die Objekte, die TInstDialog aus- */
/* machen:                                                */
/* - TViewData speichert alle relevanten Daten einer View,*/
/* - TVDCollection sammelt TViewData in einer Kollektion, */
/* - TInstViewer ist der konkrete Nachfahre von TLevel-   */
/*   Viewer und liefert ber getLevel und getName die     */
/*   von getText gewnschten Informationen,               */
/* - TInstDialog schliesslich ist der Dialog, der ber    */
/*   TInstDialog::TInstDialog() initialisiert wird und    */
/*   streamable ist (siehe zu diesem Punkt Kommentar in   */
/*   tinstdlg.cpp).                                       */
/* getInstanceTree liefert eine Kollektion mit der bei    */
/* dem Parameter view startenden View, wobei collectViews */
/* die eigentliche rekursive Sammelarbeit bernimmt.      */
/*                                                        */
/* ------------------------------------------------------ */
#define Uses_TCollection
#define Uses_TDialog
#define Uses_TLevelViewer
#define Uses_TObject
#define Uses_TStreamable
#define Uses_TView

#if defined (Uses_TInstDialog)
#define Uses_TInstViewer
#define Uses_TVDCollection
#define Uses_TViewData
#endif

#if !defined (__TLevelViewer)
#include "tlevlvwr.h"
#endif

#if !defined (__TV_H)
#include <tv.h>
#endif

#if !defined (__STDLIB_H)
#include <stdlib.h>
#endif

const cmInstFocused = 10000;

#if defined (Uses_TViewData) && !defined (__TViewData)
#define __TViewData

class TViewData : public TObject
{
public:
    TView* view;
    short level;
    char* name;
    ushort state, options, eventMask;
    ushort dragMode, growMode;
    char originX [4];
    char originY [4];
    char sizeX [4];
    char sizeY [4];
    char helpCtx [6];

    TViewData (TView *aView, short aLevel);
    TViewData (int nothing) { }
};

#endif // __TViewData


#if defined (Uses_TVDCollection) && \
   !defined (__TVDCollection)
#define __TVDCollection

class TVDCollection : public TCollection
{
public:
    TVDCollection (ccIndex aLimit, ccIndex aDelta) :
        TCollection (aLimit, aDelta) { }
    TVDCollection (StreamableInit) :
        TCollection (streamableInit) {}

private:
    virtual void *readItem (ipstream& is);
    virtual void writeItem (void *item, opstream& os);

public:
    static const char * const name;
    static TStreamable *build (void);

private:
    virtual const char *streamableName (void) const
        { return name; }
};

TCollection* getInstTree (TView* view);

void collectViews (TView *view, int level,
                   TCollection *viewColl);

#endif // __TVDCollection


#if defined (Uses_TInstViewer) && !defined (__TInstViewer)
#define __TInstViewer

class TInstViewer: public TLevelViewer
{
public:
    TInstViewer (const TRect& bounds,
                 TScrollBar *aHScrollBar,
                 TScrollBar *aVScrollBar,
                 TView *startView);
    TInstViewer (StreamableInit) :
        TLevelViewer (streamableInit) {}

    virtual ushort dataSize () { return 0; }
    virtual void focusItem (short item);
    virtual void getData (void *rec)
        { /* kein Datenaustausch ! */ }
    virtual int getLevel (short item);
    virtual char* getName (short item);
    virtual void setData (void *rec)
        { /* kein Datenaustausch ! */ }

    static const char * const name;
    static TStreamable *build (void);

protected:
    virtual void *read (ipstream& is);
    virtual void write (opstream& os);

private:
    virtual const char *streamableName (void) const
        { return name; }
};

inline ipstream& operator >> (ipstream& is, TInstViewer& cl)
    { return is >> (TStreamable&)cl; }
inline ipstream& operator >> (ipstream& is,TInstViewer*& cl)
    { return is >> (void *&)cl; }

inline opstream& operator << (opstream& os, TInstViewer& cl)
    { return os << (TStreamable&)cl; }
inline opstream& operator << (opstream& os, TInstViewer* cl)
    { return os << (TStreamable *)cl; }

#endif // __TInstViewer


#if defined (Uses_TInstDialog) && !defined (__TInstDialog)
#define __TInstDialog

class TInstDialog: public TDialog
{
protected:
    TViewData *currentInst;
    TInstViewer *instViewer;

public:
    TInstDialog ();
    TInstDialog (StreamableInit) :
        TDialog (streamableInit),
        TWindowInit (&TInstDialog::initFrame) { }

    virtual TInstViewer* newInstViewer (TRect& bounds,
        TScrollBar *aHScrollBar, TScrollBar *aVScrollBar);
    virtual TView* getStartView ();
    virtual void handleEvent (TEvent& event);
    static void sayNoNo (char *msg, TGroup *theOwner);

    static const char * const name;
    static TStreamable *build (void);

protected:
    virtual void *read (ipstream& is);
    virtual void write (opstream& os);

private:
    virtual const char *streamableName (void) const
        { return name; }
};

inline ipstream& operator >> (ipstream& is, TInstDialog& cl)
    { return is >> (TStreamable&)cl; }
inline ipstream& operator >> (ipstream& is,TInstDialog*& cl)
    { return is >> (void *&)cl; }

inline opstream& operator << (opstream& os, TInstDialog& cl)
    { return os << (TStreamable&)cl; }
inline opstream& operator << (opstream& os, TInstDialog* cl)
    { return os << (TStreamable *)cl; }

#endif // __TInstDialog
