/* ----------------------------------------------------------------------- */
/*                                tlevlvwr.cpp                             */
/*                                                                         */
/*                         (c) Raimond Reichert '93                        */
/*                                                                         */
/* ----------------------------------------------------------------------- */
#define Uses_TLevelViewer

#include <tlevlvwr.h>

/* -----------------------------------------------------------------------
                                TLevelViewer
   ----------------------------------------------------------------------- */
TLevelViewer::TLevelViewer (const TRect& bounds,
                            TScrollBar *aHScrollBar,
                            TScrollBar *aVScrollBar) :
    TListBox (bounds, 1, aVScrollBar)
{
    hScrollBar = aHScrollBar;
    setWidth (LEVEL_WIDTH, X_WIDTH);
}

void TLevelViewer::getText (char *dest, short item, short maxLen)
{
    for (int i=0; i < maxLen; i++) dest[i] = ' ';

    int iLevel = getLevel (item);
    if (iLevel > 0)
    {
        for (int j = 1; j < iLevel; j++)
        {
            dest [(j-1)*levelWidth] = getVertical (item, j);
        }
        dest [levelWidth*(iLevel-1)] = getTreeChar (item);
        strnset (dest+1+levelWidth*(iLevel-1), '', levelWidth-2);
    }
    strcpy (dest+levelWidth*iLevel, "");
    strcat (dest, getName (item));

    dest [maxLen] = '\0';
}

char TLevelViewer::getTreeChar (int item)
{
    int l = getLevel (item);
    while (++item < range)
    {
       if (getLevel (item) < l )
           break;
       else
           if (getLevel (item) == l )
               return '';
    }
    return '';
}

char TLevelViewer::getVertical (int item, int levelLine)
{
    while (++item < range && getLevel (item) > levelLine) ;
    if (item == range || getLevel (item) < levelLine)
        return ' ';
    else
        return '';
}

// -----------------------------------------------------------------------
// Der direkte Vorfahre vernachlssigt den horizontalen Scrollbalken, so
// dass sich handleEvent im Falle des Vorhandensein eines solchen selbst
// um ihn zu kmmern hat.
// -----------------------------------------------------------------------
void TLevelViewer::handleEvent (TEvent& event)
{
    TListViewer::handleEvent (event);

    if (event.what == evKeyDown &&
        (event.keyDown.keyCode == kbRight ||
         event.keyDown.keyCode == kbLeft) &&
        hScrollBar)
        hScrollBar->handleEvent (event);
}

void TLevelViewer::setWidth (short aLevelWidth,
                             short aXWidth)
{
    if (hScrollBar) hScrollBar->setRange (0, aXWidth);
    levelWidth = aLevelWidth;
    drawView ();
}

