/* ------------------------------------------------------ */
/*                      tlevlvwr.h                        */
/*                                                        */
/*                (c) Raimond Reichert                    */
/*                                                        */
/* TLevelViewer ist ein abstraktes Objekt, das von        */
/* TListBox abgeleitet ist und vor allem dessen getText-  */
/* Methode berschreibt, die sich zum korrekten Erstellen */
/* einer Zeile in einer hierarchieartigen Liste auf die   */
/* abstrakten Methoden getName (liefert Beschriftung) und */
/* getLevel (liefert Hierarchietiefe) sttzt. Auf diese   */
/* Weise braucht TLevelViewer weder die Kollektion noch   */
/* deren Eintrge zu kennen, ist also von der Speicherung */
/* der dargestellten Information unabhngig. Da es ein    */
/* abstraktes Objekt ist, besitzt es keine Methoden, um   */
/* sich auf dem Stream zu speichern oder von dort zu      */
/* laden, doch damit der Nachkomme dies kann, muss        */
/* TLevelViewer::TLevelViewer (StreamableInit) definiert  */
/* werden, damit die Aufrufkette der Konstruktoren nicht  */
/* unterbrochen wird, was der Compiler nicht mag. Der     */
/* Destruktor entfernt noch die Kollektion aus dem        */
/* Speicher, da davon ausgegangen wird, dass diese dem    */
/* darstellenden Objekt gehrt.                           */
/*                                                        */
/* ------------------------------------------------------ */
#define Uses_TEvent
#define Uses_TKeys
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TStreamableClass

#if !defined (__TV_H)
#include <tv.h>
#endif

#if !defined (__STRING_H)
#include <string.h>
#endif

#if defined (Uses_TLevelViewer) && !defined (__TLevelViewer)
#define __TLevelViewer
                                // Standardwerte fr
static short LEVEL_WIDTH = 5;   // TLevelViewer::level_Width
static short X_WIDTH     = 150; // ...::hScrollBar::range

class TLevelViewer: public TListBox
{
protected:
    int levelWidth;

public:
    TLevelViewer (const TRect& bounds,
                  TScrollBar *aHScrollBar,
                  TScrollBar *aVScrollBar);
    TLevelViewer (StreamableInit) :
        TListBox (streamableInit) {}
    ~TLevelViewer () { newList (NULL); }

    virtual int getLevel  (short item) = 0;
    virtual char* getName (short item) = 0;
    virtual void getText (char *dest,
                          short item,
                          short maxLen);

  protected:
    char getTreeChar (int item);
    char getVertical (int item, int No);

  public:
    virtual void handleEvent (TEvent& event);
    virtual void setWidth (short aLevelWidth,
                           short aXWidth);
};

#endif // __TLevelViewer
