(* ====================================================== *)
(*                       TVTETRIS.PAS                     *)
(*      Copyright (C) 1993 Hartmut Gnther & toolbox      *)
(*   Compiler: Turbo Pascal ab V. 6.0, Real-Mode Target   *)
(* ====================================================== *)
{$A+,B-,D-,E-,F-,G-,I-,L-,N-,O-,P-,Q-,R-,S-,T-,V-,X+,Y-}
{$M 4096,20000,640000}
PROGRAM TV_TETRIS;

USES
  Crt, Objects, Drivers, App, Menus, Dialogs, Views,
  Gadgets;
  (* Hinweis: Gadgets befindet sich im TVDEMO-Verzeichnis *)

CONST
{$IFNDEF Ver70}
  Seg0040 = $0040;
{$ENDIF}
  cminfo  = 100;
  cmSetup = 101;
  cmGame  = 102;
  cmDown  = 103;

(* Im Zweispieler-Modus befindet sich aus technischen     *)
(* Grnden Spieler 1 auf der rechten und Spieler 2 auf    *)
(* der linken Seite. Dies kann man ber die Variable      *)
(* gamedlg.Spieler nachvollziehen                         *)

  cmRight = 104;               (* Kommandos fr Spieler 1 *)
  cmLeft  = 105;
  cmRotate= 106;
  cmDrop  = 107;
  cmRi    = 108;               (* Kommandos fr Spieler 2 *)
  cmLe    = 109;
  cmRo    = 110;
  cmDr    = 111;

(* Das Kommando cmnxfig1 wird von Spieler 1 abgeschickt,*)
(* um bei Sp2 das nchste Teil zu verndern. cmnxfig2     *)
(* ndert das nchste Teil von Sp1.                       *)
(* Diese Aktion macht nur bei eingeschalteter             *)
(* Figurenvorschau Sinn.                                  *)
  cmnxFig1 = 112;
  cmnxFig2 = 113;
  cmIdle   = 114;   (* Damit werden die Gadgets aktiviert *)

TYPE
  tFigure  = ARRAY[1..4,  1.. 5] OF WORD;
  tField   = ARRAY[1..20, 1..20] OF WORD;
  tTop     = ARRAY[1..20] OF RECORD
               Name    : STRING[10];
               pktzahl : WORD;
             END;
  tDlgData = RECORD
               StartLevelFlag,
               ExtSetFlag,
               SoundFlag,                 (* Ton an / aus *)
               NextFlag,      (* Figurenvorschau an / aus *)
               TwoFlag,       (* Ein- / Zweispieler-Modus *)
               spnr    : WORD;      (* Nummer des Spieler *)
              END;

  tMainApp = OBJECT(tApplication)
    Name    : STRING[10];
    DlgData : tDlgData;
    heap    : pHeapView;                  (* Zwei Gadgets *)
    Clock   : pClockView;
    CONSTRUCTOR Init;
    PROCEDURE   InitMenuBar;                     VIRTUAL;
    PROCEDURE   InitStatusLine;                  VIRTUAL;
    PROCEDURE   Information;
    PROCEDURE   HandleEvent(VAR Event : tEvent); VIRTUAL;
    PROCEDURE   Idle;                            VIRTUAL;
  END;

  pSetupDlg = ^tSetupDlg;
  tSetupDlg = OBJECT(tDialog)
    CONSTRUCTOR Init;
  END;

  pHighScoreDlg = ^tHighScoreDlg;(* Die Highscore-Tabelle *)
  tHighScoreDlg = OBJECT(tDialog)
    Top_Twenty : tTop;
    CONSTRUCTOR Init(sn : STRING; pkt : WORD);
  END;

  pInfoview = ^tInfoView;
  tInfoView = OBJECT(tView)
    sc, cd : WORD;
    sl, fl : BYTE;
    CONSTRUCTOR Init(Bounds : tRect);
    PROCEDURE   Showinfo(Score, Credits       : WORD;
                         Fullines, Startlevel : BYTE);
    PROCEDURE   Draw; VIRTUAL;
  END;

  pFieldview = ^tFieldView;
  tFieldView = OBJECT(tView)
    Field : tField;
    PROCEDURE Showfield(Data : tField);
    PROCEDURE Draw; VIRTUAL;
  END;

  (* Die Figurenvorschau   *)
  pNextview = ^tNextView;
  tNextView = OBJECT(tView)
    NextFig : BYTE;
    CONSTRUCTOR Init(Bounds : tRect);
    PROCEDURE   ShowNext(nt : BYTE);
    PROCEDURE   Draw; VIRTUAL;
  END;

  (* Eingabebox fr den Namen des Spielers *)
  pNameDlg = ^tNameDlg;
  tNameDlg = OBJECT(tDialog)
    CONSTRUCTOR Init(s : STRING; pkt : WORD);
  END;

  pGameDlg2 = ^tGameDlg;        (* Es gibt zwei Instanzen *)
  pGameDlg1 = ^tGameDlg;        (* des Objekts tGameDlg  *)
  tGameDlg  = OBJECT(tWindow)
    GameFieldView : pFieldview;
    GameInfoView  : pInfoview;
    GameNextView  : pNextview;
    CreditView,
    GameOverview  : pStaticText;
    DosTime       : ^LongInt;
    oct,                       (* Beginn der Creditchance *)
    OldTime       : LongInt;
    Field         : tField;
    Fig           : tFigure;
    col           : ShortInt;
    Level         : BYTE;
    Startlevel    : BYTE;
    SoundEff      : BYTE;
    NextShow      : BYTE;
    Spieler, rot  : BYTE;
    Line          : BYTE;
    LineDropped   : BYTE;
    MaxFigures    : BYTE;
    Fullines      : BYTE;
    num, NextNum  : BYTE;
    Credits       : WORD;                  (* Bonuspunkte *)
    Score         : WORD;
    c_Aktiv       : BOOLEAN;
    GameOver      : BOOLEAN;
    CONSTRUCTOR Init(DlgData          : tDlgData);
    PROCEDURE   HandleEvent(VAR Event : tEvent);    VIRTUAL;
    PROCEDURE   GetFigure;
    PROCEDURE   MoveFigure(Key        : WORD);
    PROCEDURE   SFX_Row;                 (* Sound-Effekte *)
    PROCEDURE   SFX_Credit;
    PROCEDURE   SFX_Gameovr;
    FUNCTION    Check(ShowF : BOOLEAN) : BOOLEAN;
  END;

  pStart = ^tStart;
  tStart = OBJECT(tDialog)
    CONSTRUCTOR Init(DlgData : tDlgData);
    PROCEDURE   HandleEvent(VAR Event : tEvent);    VIRTUAL;
  END;

VAR
  Main    : tMainApp;
  TwoMode : BOOLEAN;
  Punkte1,
  Punkte2 : WORD;           (* Punkte von Spieler 1 und 2 *)

CONSTRUCTOR tMainApp.Init;
VAR
  r : tRect;
BEGIN
  tApplication.Init;
  WITH DlgData DO BEGIN
    StartLevelFlag := 5;
    ExtSetFlag     := 0;
    SoundFlag      := 1;
    NextFlag       := 0;
    TwoFlag        := 0;
  END;

  (* Gadgets einfgen *)
  GetExtent(r);
  Dec(r.B.X);
  r.A.X := r.B.X - 19; r.B.Y := r.A.Y + 1;
  r.B.X := r.A.X + 8;
  heap := New(pHeapView, Init(r));
  GetExtent(r);
  r.A.X := r.B.X - 9; r.B.Y := r.A.Y+1;
  Clock := New(pClockView, Init(r));
  Insert(heap);
  Insert(Clock);
  Information;
  NoSound;
END;

PROCEDURE tMainApp.InitMenuBar;
VAR
  r : tRect;
BEGIN
  GetExtent(r);
  r.B.Y := r.A.Y+1;
  MenuBar := New(pMenuBar, Init(r,
    NewMenu(
      NewSubMenu('~T~V-Tetris', hcNoContext,
        NewMenu(
          NewItem('~I~nformation', 'F1', kbF1, cminfo,
                  hcNoContext,
          NewItem('~V~oreinstellungen', 'F2', kbF2, cmSetup,
                  hcNoContext,
          NewItem('~S~piel', 'F3', kbF3, cmGame,hcNoContext,
          NewItem('~E~nde', 'F4', kbF4, cmQuit, hcNoContext,
      NIL))))),
    NIL))));
END;

PROCEDURE tMainApp.InitStatusLine;
VAR
  r : tRect;
BEGIN
  GetExtent(r);
  r.A.Y := r.B.Y-1;
  StatusLine := New(pStatusLine, Init(r, NewStatusDef(0, 0,
    NewStatusKey('~F1~ Info',      kbF1,    cminfo,
    NewStatusKey('~F2~ Setup',     kbF2,    cmSetup,
    NewStatusKey('~F3~ Spiel',     kbF3,    cmGame,
    NewStatusKey('~F4~ Ende',      kbF4,    cmQuit,
    NewStatusKey('~'#27'~ Links',  kbLeft,  cmLeft,
    NewStatusKey('~'#24'~ Rot',    kbUp,    cmRotate,
    NewStatusKey('~'#26'~ Rechts', kbRight, cmRight,
    NewStatusKey('~'#25'~ Runter', kbDown,  cmDrop,
    NewStatusKey('  ', kbF10, cmMenu, (*  Lckenfller    *)
    (* es folgen die Kommandos fr Spieler 2, es werden   *)
    (* sowohl Klein- als auch Grobuchstaben              *)
    (* bercksichtigt                                     *)
    NewStatusKey('~'#27'~ Li2', $1E61, cmLe,
    NewStatusKey('~'#27'~ Li2', $1E41, cmLe,
    NewStatusKey('~'#24'~ Ro2', $1177, cmRo,
    NewStatusKey('~'#24'~ Ro2', $1157, cmRo,
    NewStatusKey('~'#26'~ Re2', $2064, cmRi,
    NewStatusKey('~'#26'~ Re2', $2044, cmRi,
    NewStatusKey('~'#25'~ Ru2', $1F73, cmDr,
    NewStatusKey('~'#25'~ Ru2', $1F53, cmDr,
    NewStatusKey('', $011B, cmCancel,
                            (* wegen des Zweispielermodus *)
  NIL)))))))))))))))))), NIL)));
  DisableCommands([cmRotate, cmLeft, cmRight, cmDrop]);
END;

PROCEDURE tMainApp.Information;
VAR
  d     : pDialog;
  r     : tRect;
  dummy : WORD;
BEGIN
  r.Assign(0, 0, 54, 18);
  d := New(pDialog, Init(r, 'Information'));
  WITH d^ DO BEGIN
    Options := Options OR ofCentered;
    r.Grow(-1, -1);
    Dec(r.B.Y, 3);
    Insert(New(pStaticText, Init(r,
        ^C'TV-TETRIS V2.2'#13#13#13 +
        ^C'Geschrieben mit Turbo Pascal'#13 +
        ^C'von Hartmut Gnther'#13 +
        ^C'Copyright (C) 1993 H. Gnther & toolbox')));
    r.Assign(2, 10, 53, 14);
    Insert(New(pStaticText, Init(r,
        ^C'Einstellungen mit [F2]'#13#13 +
        ^C'Im Zweispieler-Modus werden die Tasten'+#13+
        ^C'a, s, d und e von Spieler 2 bentzt.')));

    r.Assign(19, 15, 29, 17);
    Insert(New(pButton, Init(r, 'O~K', cmOk, bfDefault)));
  END;
  dummy := Desktop^.ExecView(d);
  Dispose(d, Done);
END;

PROCEDURE tMainApp.HandleEvent(VAR Event : tEvent);

  PROCEDURE Setup(VAR DlgData : tDlgData);
  VAR
    P : pSetupDlg;
  BEGIN
    P := New(pSetupDlg,Init);
    IF ValidView(P) <> NIL THEN BEGIN
      P^.SetData(DlgData);
      IF Desktop^.ExecView(P) <> cmCancel THEN
        P^.GetData(DlgData);
      Dispose(P, Done);
    END;
  END;


(* Gibt im Zwei-Spielermodus den Sieger bekannt           *)
  PROCEDURE Info_Sieger;
  VAR
    d        : pDialog;
    r        : tRect;
    dummy    : WORD;
    sp,
    ps1, ps2 : STRING;
  BEGIN
    IF Punkte1 > Punkte2      THEN sp := '1'
    ELSE IF Punkte2 > Punkte1 THEN sp := '2'
                              ELSE sp := '1 und Spieler 2';
    Str(Punkte1:5, ps1);
    Str(Punkte2:5, ps2);
    r.Assign(0, 0, 50, 11);
    d := New(pDialog, Init(r, ''));
    WITH d^ DO BEGIN
      Options := Options OR ofCentered;
      r.Grow(-1, -1);
      Dec(r.B.Y, 3);
      Insert(New(pStaticText, Init(r,
                 #13'  Spieler 1 hat ' + ps1 +
                 ' Punkte'#13'  Spieler 2 hat ' + ps2 +
                 ' Punkte erreicht.'#13#13 +
                 '  Damit hat Spieler ' + sp +
                 ' gewonnen!!'#13)));

      r.Assign(19, 8, 29, 10);
      Insert(New(pButton, Init(r, 'O~K', cmOk, bfDefault)));
    END;
    dummy := Desktop^.ExecView(d);
    Dispose(d, Done);
  END;

  PROCEDURE Auswertung(SNr : BYTE;pkt : WORD);
  (*  Ermittelt den Namen des Spielers und zeigt ggf. die *)
  (* Highscoretabelle an. Im Ein-Spielermodus wird snr = 0*)
  (* bergeben                                            *)
  VAR
    h : pHighScoreDlg;
    n : pNameDlg;
    d : WORD;                           (* Dummy-Variable *)
    s : STRING;
  BEGIN
    IF SNr > 0 THEN BEGIN
      Str(SNr,s);
      s := 'Spieler ' + s;
    END ELSE s := '';
    n := New(pNameDlg, Init(s, pkt));
    IF ValidView(n) <> NIL THEN BEGIN
      IF Desktop^.ExecView(n) = cmCancel THEN BEGIN
        Dispose(n, Done);
        Exit;
      END;
      n^.GetData(s);
      Dispose(n, Done);
      IF s = '' THEN Exit;
      h := New(pHighScoreDlg, Init(s, pkt));
      IF ValidView(h) <> NIL THEN BEGIN
        IF (SNr <> 1) OR (Punkte2 = 0) THEN
          d := Desktop^.ExecView(h);
        Dispose(h,Done);
      END;
    END;
  END;

  PROCEDURE Game(DlgData : tDlgData);
  VAR
    P : pStart;
    d : WORD;                           (* Dummy-Variable *)
  BEGIN
    P := New(pStart, Init(DlgData));
    IF ValidView(P) <> NIL THEN BEGIN
      DisableCommands([cmQuit, cminfo, cmSetup, cmGame]);
      EnableCommands([cmRotate, cmLeft, cmRight, cmDrop,
                      cmLe, cmRi, cmDr, cmRo]);
      d := Desktop^.ExecView(P);
      Dispose(P,Done);
      DisableCommands([cmRotate, cmLeft, cmRight, cmDrop,
                       cmLe, cmRi, cmDr, cmRo]);
      EnableCommands([cmQuit, cminfo, cmSetup, cmGame]);
      IF (Punkte1 = 0) AND (Punkte2 = 0) THEN Exit;
      IF DlgData.TwoFlag = 0 THEN Auswertung(0, Punkte1)
      ELSE BEGIN
        Info_Sieger;
        IF Punkte1 > 0 THEN Auswertung(1, Punkte1);
        IF Punkte2 > 0 THEN Auswertung(2, Punkte2);
      END;
    END;
  END;

BEGIN
  tApplication.HandleEvent(Event);
  IF Event.What = evCommand THEN BEGIN
    CASE Event.Command OF
      cminfo  : Information;
      cmSetup : Setup(DlgData);
      cmGame  : Game(DlgData);
      ELSE Exit;
    END;
    ClearEvent(Event);
  END;
END;

PROCEDURE tMainApp.Idle;
BEGIN
  tApplication.Idle;
  heap^.Update;                        (* Gadgets updaten *)
  Clock^.Update
END;


(*====================== tSetupDlg =======================*)

CONSTRUCTOR tSetupDlg.Init;
VAR
  DlgSubview : pView;
  r          : tRect;
BEGIN
  r.Assign(0, 0, 79, 23);
  tDialog.Init(r, 'Voreinstellungen');
  Options := Options OR ofCentered;
  r.Grow(-1, -1);
  r.Assign(3, 2, 13, 12);
  DlgSubview := New(pRadioButtons,Init(r,
                    NewSItem(' ~0~', NewSItem(' ~1~',
                    NewSItem(' ~2~', NewSItem(' ~3~',
                    NewSItem(' ~4~', NewSItem(' ~5~',
                    NewSItem(' ~6~', NewSItem(' ~7~',
                    NewSItem(' ~8~', NewSItem(' ~9~',
                    NIL))))))))))));
  Insert(DlgSubview);
  r.Assign(2, 1, 13, 2);
  Insert(New(pLabel, Init(r, '~S~tartlevel', DlgSubview)));
  r.Assign(17, 2, 46, 3);
  Insert(New(pCheckBoxes, Init(r,
         NewSItem('~E~rweiterter Figurensatz', NIL))));
  r.Assign(17, 3, 46, 4);
  Insert(New(pCheckBoxes, Init(r,
         NewSItem('~G~erusche', NIL))));
  r.Assign(17, 4, 46, 5);
  Insert(New(pCheckBoxes, Init(r,
         NewSItem('~N~chstes Teil zeigen', NIL))));
  r.Assign(17, 5, 46, 6);
  Insert(New(pCheckBoxes, Init(r,
         NewSItem('~Z~wei Spieler-Modus', NIL))));
  r.Assign(17, 7, 78, 14);
  Insert(New(pStaticText, Init(r,
        'Tasten beim 2 Spieler-Modus:'#13 +
        'Spieler 2:'#13 +
        ' A = nach links, D = nach rechts,'#13             +
        ' W = rotieren,   S = fallenlassen'#13             +
        'bei 8 Credits: X = fallenlassen bei Spieler 1'#13 +
        'bei 4 Credits: C = rotieren bei Spieler 1'#13     +
        'bei 4 Credits: V = nderung der nchsten Figur '  +
        'bei Sp. 1')));
  r.Assign(17, 14, 78, 18);
  Insert(New(pStaticText, Init(r,
        'Spieler 1:'#13                                    +
        'bei 8 Credits: Del  = fallenlassen bei '          +
        'Spieler 2'#13                                     +
        'bei 4 Credits: End  = rotieren bei Spieler 2'#13  +
        'bei 4 Credits: Pgdn = nderung der nchsten '     +
        'Figur bei Sp. 2')));
  r.Assign(17, 19, 78, 22);
  Insert(New(pStaticText, Init(r,
        'Erscheint "Versenke eine Reihe", so gibt es 5 '   +
        'Credits, wenn '#13                                +
        'man schnell genug eine Reihe ausfllt.  Tip: Man' +
        ' mu die Credits im richtigen Moment gegen den '  +
        'Gegner einsetzen!')));
  r.Assign(21 + 38, 16 - 14, 32 + 38, 18 - 14);
  Insert(New(pButton, Init(r, '~A~bbruch', cmCancel,
                           bfNormal)));
  r.Assign(11 + 38, 16 - 14, 19 + 38,18 - 14);
  Insert(New(pButton, Init(r, '~O~K', cmOk, bfDefault)));
END;

CONSTRUCTOR tHighScoreDlg.Init;
VAR
  r         : tRect;
  IO_Fehler : WORD;
  i         : BYTE;
  dVar      : File OF tTop;
  TopName,
  S_Punkte  : STRING[10];
  TopFName  : STRING[79];
  Rang      : STRING[3];
  Sp_Ins    : BOOLEAN;
BEGIN
  r.Assign(0, 0, 51, 19);
  tDialog.Init(r, 'Highscore');
  TopFName := ParamStr(0);
  System.Delete(TopFName, Length(TopFName) - 3, 4);
  TopFName := TopFName + '.TOP';
  Assign(dVar, TopFName);
{$I-}                         (* Highscore-Tabelle ffnen *)
  Reset(dVar);
  IO_Fehler := IOResult;
  IF IO_Fehler = 0 THEN BEGIN
    Read(dVar,Top_Twenty);
    IO_Fehler := IOResult;
    System.Close(dVar);
    IO_Fehler := IOResult;
  END;
{$I+}
  Options := Options OR ofCentered;
  r.Grow(-1, -1);
  r.Assign(8, 2, 35, 3);
  Insert(New(pStaticText,
             Init(r, 'Die 20 besten Spieler:')));

  Sp_Ins := FALSE;
  FOR i := 1 TO 20 DO BEGIN
    IF (pkt > Top_Twenty[i].pktzahl) AND (NOT Sp_Ins) THEN
    BEGIN
      Move(Top_Twenty[i], Top_Twenty[i + 1],
           SizeOf(Top_Twenty[i]) * (20 - i));
      Top_Twenty[i].Name    := sn;
      Top_Twenty[i].pktzahl := pkt;
      Sp_Ins                := TRUE;
    END;
    IF i > 10 THEN r.Assign(28, i - 10 + 4, 49, i - 10 + 5)
              ELSE r.Assign(2, i + 4, 23, i + 5);
    Str(Top_Twenty[i].pktzahl:5, S_Punkte);
    FillChar(TopName, SizeOf(TopName), 0);
    TopName    := Top_Twenty[i].Name;
    TopName[0] := Chr(SizeOf(TopName) - 1);
    Str(i:2, Rang);
    Rang := Rang + '.';
    Insert(New(pStaticText, Init(r, Rang +
           TopName + '  ' + S_Punkte)));
  END;
  r.Assign(20, 16, 30, 18);
  Insert(New(pButton, Init(r, '~O~K', cmOk, bfDefault)));
  IF (IO_Fehler = 0) OR (IO_Fehler = 2) THEN BEGIN
{$I-}
  Rewrite(dVar);
  IO_Fehler := IOResult;
  IF IO_Fehler <> 0 THEN Exit;
  Write(dVar, Top_Twenty);
  IO_Fehler := IOResult;
  System.Close(dVar);
  IO_Fehler := IOResult;
{$I+}
  END;
END;

CONSTRUCTOR tNameDlg.Init;
VAR
  r             : tRect;
  Txt, S_Punkte : STRING[10];
  NSubview      : pInputLine;
BEGIN
  r.Assign(0, 0, 48, 7);
  tDialog.Init(r, s);
  Options := Options OR ofCentered;
  CASE pkt OF
    0   .. 999 : Txt := '';
    1000..2999 : Txt := 'Bravo! ';
    3000..4999 : Txt := 'Super! ';
    ELSE         Txt := 'Spitze! ';
  END;
  Str(pkt, S_Punkte);
  r.Assign(3, 1, 45, 2);
  Insert(New(pStaticText, Init(r, #3 + Txt + 'Sie haben '
             + S_Punkte + ' Punkte erreicht!')));
  r.Assign(25, 4, 33, 6);
  Insert(New(pButton, Init(r, '~O~K', cmOk, bfDefault)));
  r.Assign(35, 4, 46, 6);
  Insert(New(pButton, Init(r, '~A~bbruch',
             cmCancel, bfNormal)));
  r.Assign(3, 4, 15, 5);
  NSubview := New(pInputLine, Init(r, 10));
  Insert(NSubview);
  r.Assign(2, 3, 15, 4);
  Insert(New(pLabel, Init(r, 'Ihr Name:', NSubview)));
END;

CONSTRUCTOR tStart.Init;
VAR
  r : tRect;
BEGIN
  r.Assign(0, 0, 50 + Byte(DlgData.TwoFlag) * 30, 22);
  tDialog.Init(r, 'Tetris');
  IF DlgData.TwoFlag = 1 THEN BEGIN
    DlgData.spnr := 2;
    Insert(New(pGameDlg2, Init(DlgData)));
  END;
  DlgData.spnr := 1;
  Insert(New(pGameDlg1, Init(DlgData)));
  Options := Options OR ofPreProcess
                     OR ofPostProcess OR ofCentered;
END;

PROCEDURE tStart.HandleEvent;
VAR
  r : tRect;
BEGIN
  IF Event.What = evKeyboard THEN
    IF Event.CharCode = #13 THEN BEGIN
      Event.What := evCommand;        (* Ende mit [ENTER] *)
      Event.Command := cmCancel;
    END;

  IF Event.What = evNothing THEN BEGIN
    Event.What := evCommand; (* Damit wird tGameDlg.Han-  *)
    Event.Command := cmIdle; (* dleEv ausgefhrt          *)
  END;
  tDialog.HandleEvent(Event);
END;

(* ====================== tGameDlg ====================== *)

CONSTRUCTOR tGameDlg.Init;
VAR
  CTxt, sn : STRING;
  r        : tRect;
BEGIN
  TwoMode := (DlgData.TwoFlag = 1);
  Spieler := DlgData.spnr;
  Str(Spieler, sn);
  IF TwoMode THEN BEGIN
    r.Assign((2 - Spieler) * 43, 0,
              37 + (2 - Spieler) * 43, 22);
    sn := 'Spieler ' + sn;
  END ELSE BEGIN
    r.Assign(0, 0, 50, 22);
    sn := 'Tetris';
  END;
  tWindow.Init(r, sn, 0);
  Spieler := DlgData.spnr;
  r.Assign(1, 1, 36 + (1 - Byte(TwoMode)) * 13, 21);
  Insert(New(pView,Init(r)));
  r.Assign(1, 1, 21, 21);
  GameFieldView := New(pFieldview, Init(r));
  Insert(GameFieldView);
  IF DlgData.NextFlag > 0 THEN BEGIN
    r.Assign(22, 10, 31, 11);
    Insert(New(pStaticText, Init(r, 'Nchstes:')));
    r.Assign(22, 11, 28, 14);
    GameNextView := New(pNextview, Init(r));
    Insert(GameNextView);
  END;
  IF TwoMode THEN CTxt := #13#13'Credits:'
             ELSE CTxt := '';
  r.Assign(22, 2, 30, 9);
  Insert(New(pStaticText, Init(r,
         'Punkte:'#13#13'Linien:'#13#13'Level:' + CTxt)));
  r.Assign(30, 2, 35, 9);
  GameInfoView := New(pInfoview, Init(r));
  Insert(GameInfoView);
  r.Assign(24, 15, 34, 16);
  GameOverview := New(pStaticText, Init(r, 'Spielende!'));
  Insert(GameOverview);
  GameOverview^.Hide;
  r.Assign(24, 18, 35, 20);
  CreditView := New(pStaticText,
                    Init(r, 'Versenke eine Reihe!'));
  Insert(CreditView);
  CreditView^.Hide;
  Flags      := 0;
  EventMask  := $FFFF;
  DosTime    := Ptr(Seg0040, $6C);
  FillChar(Field, SizeOf(Field), 0);
  Startlevel := DlgData.StartLevelFlag;
  SoundEff   := DlgData.SoundFlag;
  NextShow   := DlgData.NextFlag;
  GameOver   := FALSE;
  Level      := 0;
  MaxFigures := 7;
  Punkte1    := 0;
  Punkte2    := 0;
  Score      := 0;
  Inc(MaxFigures, Byte(DlgData.ExtSetFlag AND 1 = 1) * 3);
  Randomize;
  NextNum := Random(MaxFigures) + 1;
  GetFigure;
  Options := Options OR ofPreProcess OR ofPostProcess;
                                  (* -->  alle Ereignisse *)
  State := State AND NOT sfShadow;
END;

PROCEDURE tGameDlg.HandleEvent;

  PROCEDURE Calc_Credits;
  BEGIN
    IF NOT TwoMode THEN Exit;
    IF c_Aktiv THEN BEGIN
      IF DosTime^ - oct > 40 THEN BEGIN
                (* die Chance ist nach 40 Ticks vorbei... *)
        CreditView^.Hide;
        c_Aktiv :=  FALSE;
      END;
    END ELSE IF (Random(50000) = 1) OR
                (DosTime^ - oct > 720) THEN BEGIN
      oct := DosTime^;
      CreditView^.Show;
      c_Aktiv := TRUE;
    END;
  END;

  PROCEDURE Check_Keys1;
  (* Diese Prozedur prft, ob Spieler 1 eine Aktion gegen *)
  (* Spieler 2 startet. Sind genug Credits vorhanden,     *)
  (* wird ber eine Message das Kommando an alle Desktop- *)
  (* Objekte geschickt. Das View-Objekt von Sp2           *)
  (* verarbeitet dann die Botschaft und lt z.B. ein     *)
  (* Teil fallen.                                         *)
  VAR
    DummyP : Pointer;
  BEGIN
    CASE Event.ScanCode OF
      $53 : IF Credits >= 8 THEN BEGIN
              Dec(Credits,8);
              DummyP := Message(Desktop, evCommand,
                                cmDr, NIL);
            END;
      $4F : IF Credits >= 4 THEN BEGIN
              Dec(Credits,4);
              DummyP := Message(Desktop, evCommand, cmRo,
                                NIL);
            END;
      $51 : IF Credits >= 4 THEN BEGIN
              Dec(Credits,4);
              DummyP := Message(Desktop, evCommand,
                                cmnxFig1, NIL);
            END;
      ELSE  Exit;
    END;
    ClearEvent(Event);
    GameInfoView^.Showinfo(Score, Credits, Fullines,
                           Startlevel);
  END;


  PROCEDURE Check_Keys2;     (* Gegenstck zu Check_keys1 *)
  VAR
    DummyP : Pointer;
  BEGIN
    CASE UpCase(Event.CharCode) OF
      'X': IF Credits >= 8 THEN BEGIN
             Dec(Credits,8);
             DummyP := Message(Desktop, evCommand, cmDrop,
                               NIL);
           END;
      'C': IF Credits >= 4 THEN BEGIN
             Dec(Credits,4);
             DummyP := Message(Desktop, evCommand,
                               cmRotate, NIL);
           END;
      'V': IF Credits >= 4 THEN BEGIN
             Dec(Credits,4);
             DummyP := Message(Desktop, evCommand,
                               cmnxFig2, NIL);
           END;
      ELSE Exit;
    END;
    ClearEvent(Event);
    GameInfoView^.Showinfo(Score, Credits, Fullines,
                           Startlevel);
  END;

VAR
  r : tRect;
BEGIN
  tWindow.HandleEvent(Event);
  IF NOT GameOver THEN BEGIN
    Calc_Credits;
    IF DosTime^ - (28 - Startlevel * 3) > OldTime THEN BEGIN
      Inc(OldTime,28-Startlevel*3);
      MoveFigure(cmDown);
    END ELSE IF Event.What = evCommand THEN
      CASE Event.Command OF
        cmLeft,
        cmRight,
        cmRotate,
        cmDrop    : IF Spieler = 2 THEN
                      Exit
                    ELSE
                      MoveFigure(Event.Command);
        cmLe, cmRi,
        cmRo, cmDr: IF Spieler = 1 THEN Exit
                    ELSE BEGIN
                    (* movefigure versteht nur die        *)
                    (* Befehle cmdrop,cmleft usw. deshalb *)
                    (* Konvertierung von z.B cmri zu      *)
                    (* cmright                            *)
                      Event.Command := Event.Command - 4;
                      MoveFigure(Event.Command);
                    END;
        cmnxFig1:   BEGIN
                      IF Spieler = 1 THEN Exit;
                      NextNum := Random(MaxFigures) + 1;
                      IF NextShow > 0 THEN
                        GameNextView^.ShowNext(NextNum);
                    END;
        cmnxFig2:   BEGIN
                      IF Spieler = 2 THEN Exit;
                      NextNum := Random(MaxFigures) + 1;
                      IF NextShow > 0 THEN
                        GameNextView^.ShowNext(NextNum);
                    END;
                    ELSE Exit;
      END
    ELSE IF (Event.What = evKeyboard) AND TwoMode THEN BEGIN
      IF Spieler = 1 THEN Check_Keys1 ELSE Check_Keys2;
      Exit;
    END;
    ClearEvent(Event);
    IF GameOver THEN BEGIN
      GameOverview^.Show;
      SFX_Gameovr;
      IF Spieler = 1 THEN Punkte1 := Score
                     ELSE Punkte2 := Score;
    END;
  END;
END;

PROCEDURE tGameDlg.GetFigure;
CONST
  Figures: ARRAY[1..10, 1..4] OF WORD =
    (($0066, $0066, $0066, $0066),
     ($0036, $0462, $0036, $0462),
     ($0063, $0132, $0063, $0132),
     ($000F, $4444, $000F, $4444),
     ($0074, $0223, $0170, $0311),
     ($0072, $0232, $0270, $0262),
     ($0071, $0322, $0470, $0226),
     ($0057, $0313, $0750, $0646),
     ($0272, $0272, $0272, $0272),
     ($0117, $0711, $0744, $0447));

  Value: ARRAY[0..15] OF WORD =
    ($0001, $0002, $0004, $0008, $0010, $0020,
     $0040, $0080, $0100, $0200, $0400, $0800,
     $1000, $2000, $4000, $8000);

VAR
  i, j, k : INTEGER;

BEGIN
  num     := NextNum;
  NextNum :=Random(MaxFigures)+1;
  IF NextShow > 0 THEN GameNextView^.ShowNext(NextNum);
  FOR i := 1 TO 4 DO BEGIN
    j := 1;
    FOR k := 0 TO 15 DO BEGIN
      IF Figures[num, i] AND VALUE[k] = VALUE[k] THEN BEGIN
        Fig[i,j] := k MOD 4+(k SHR 2) SHL 8;
        Inc(j);
      END;
      IF j = 5 THEN Fig[i, j] := Fig[i, j - 1];
    END;
  END;
  col := 7; Line := 0; rot := 1; LineDropped := 20;
  MoveFigure(cmDown);
  OldTime := DosTime^;
END;

PROCEDURE tGameDlg.MoveFigure;
VAR
  DummyCol   : ShortInt;
  DummyRot,
  DummyLine  : BYTE;
  OneCube    : LongInt;
  LineFilled : BOOLEAN;
BEGIN
  IF Key = cmDrop THEN LineDropped := Line;
  REPEAT
    DummyCol  := col;
    DummyLine := Line;
    DummyRot  := rot;
    CASE Key OF
      cmLeft   : Dec(col,2);
      cmRight  : Inc(col,2);
      cmDown,
      cmDrop   : Inc(Line);
      cmRotate : Inc(rot, 1 - Byte(rot = 4) SHL 2);
    END;
    IF NOT Check(Key <> cmDrop) THEN BEGIN
      col  := DummyCol;
      Line := DummyLine;
      rot  := DummyRot;
      IF Key IN [cmDown, cmDrop] THEN BEGIN
        IF Line > 0 THEN BEGIN
          Inc(Score, Startlevel + num + Byte(num > 7)
                     SHL 6 + (20 - LineDropped) SHR 2);
          IF Spieler = 1 THEN Punkte1 := Score
                         ELSE Punkte2 := Score;
          OneCube := 219 + Word((num + Byte(num > 7)))
                     SHL 8 + 219 SHL 16 + LongInt((num +
                     Byte(num > 7))) SHL 24;
          FOR DummyCol := 1 TO 5 DO
            Move(OneCube, Field[Line +
                 Hi(Fig[rot, DummyCol]), col +
                 Lo(Fig[rot, DummyCol]) SHL 1],
                 SizeOf(OneCube));
          FOR DummyLine := 2 TO 20 DO BEGIN
            LineFilled := TRUE;
            FOR DummyCol := 1 TO 20 DO
              IF Lo(Field[DummyLine, DummyCol]) <> 219 THEN
                LineFilled := FALSE;
            IF LineFilled THEN BEGIN
              IF NOT c_Aktiv THEN SFX_Row
              ELSE BEGIN
                Inc(Credits, 5);
                SFX_Credit;
              END;
              Inc(Fullines);
              IF Fullines MOD (10) = 0 THEN BEGIN
                IF (Level < 6) OR (Fullines MOD 20 = 0) THEN
                  Inc(Level, Byte(Level < 9));
                IF Level > Startlevel THEN
                  Startlevel := Level;
              END;
              Move(Field[1, 1], Field[2, 1],
                   40 * (DummyLine - 1));
              FillChar(Field[1, 1], 40, 0);
              GameFieldView^.Showfield(Field);
            END;
          END;
          GetFigure;
        END ELSE GameOver := TRUE;
        GameInfoView^.Showinfo(Score, Credits, Fullines,
                               Startlevel);
      END;
    END;
  UNTIL (Key <> cmDrop) OR (Line = 1) OR GameOver;
END;

PROCEDURE tGameDlg.SFX_Row;
(* Sound-FX wenn eine Reihe versenkt wird *)
BEGIN
  IF SoundEff <> 0 THEN BEGIN
    Sound(300); Delay(15);
    Sound(200); Delay(15);
    Sound(400); Delay(20);
    NoSound;
  END;
END;

PROCEDURE tGameDlg.SFX_Credit;
(* Sound-FX wenn man 5 Credits bekommen hat *)
BEGIN
  IF SoundEff <> 0 THEN BEGIN
    Sound(742 DIV (3 - Spieler));  Delay(20);
    NoSound;                       Delay(70);
    Sound(883 DIV (3 - Spieler));  Delay(20);
    NoSound;                       Delay(70);
    Sound(1409 DIV (3 - Spieler)); Delay(20);
    NoSound;
  END;
END;

PROCEDURE tGameDlg.SFX_Gameovr;
(* Sound-FX wenn das Spiel vorbei ist *)
BEGIN
  IF SoundEff <> 0 THEN BEGIN
    Sound(150); Delay(45);
    Sound(60);  Delay(45);
    Sound(30);  Delay(45);
    NoSound;
  END;
END;

FUNCTION tGameDlg.Check;
VAR
  i:      INTEGER;
  Buf:    tField;
  X:      ShortInt;
  Y:      BYTE;
  MoveOk: BOOLEAN;

BEGIN
  MoveOk := TRUE;
  Move(Field, Buf, SizeOf(Field));
  FOR i := 1 TO 5 DO BEGIN
    X := col  + Lo(Fig[rot, i]) SHL 1;
    Y := Line + Hi(Fig[rot, i]);
    IF (X > 0) AND (X < 20) AND (Y < 21)
               AND (Lo(Field[Y,X]) <> 219) THEN
      MoveChar(Buf[Y, X], #219, num + Byte(num > 7), 2)
    ELSE
      MoveOk := FALSE;
  END;
  IF MoveOk THEN IF ShowF THEN
    GameFieldView^.Showfield(Buf);
  Check := MoveOk;
END;

CONSTRUCTOR tInfoView.Init;
BEGIN
  tView.Init(Bounds);
  Showinfo(0, 0, 0, 0);
END;

PROCEDURE tInfoView.Showinfo;
BEGIN
  sc := Score;
  fl := Fullines;
  sl := Startlevel;
  cd := Credits;
  Draw;
END;

PROCEDURE tInfoView.Draw;
VAR
  r     : tRect;
  dummy : STRING;
BEGIN
  tView.Draw;
  GetExtent(r);
  Str(sc:5, dummy);
  WriteStr(r.A.X, r.A.Y, dummy, 1);
  Str(fl:5, dummy);
  WriteStr(r.A.X, r.A.Y + 2, dummy, 1);
  Str(sl:5, dummy);
  WriteStr(r.A.X, r.A.Y + 4, dummy, 1);
  IF TwoMode THEN BEGIN
    Str(cd:5, dummy);
    WriteStr(r.A.X, r.A.Y + 6, dummy, 1);
  END;
END;

PROCEDURE tFieldView.Showfield;
BEGIN
  Field := Data;
  Draw;
END;

PROCEDURE tFieldView.Draw;
VAR
  r : tRect;
BEGIN
  GetExtent(r);
  WriteBuf(r.A.X, r.A.Y, r.B.X, r.B.Y, Field);
END;

CONSTRUCTOR tNextView.Init;
VAR
  n : BYTE;
BEGIN
  tView.Init(Bounds);
  ShowNext(0);
END;

PROCEDURE tNextView.ShowNext;
BEGIN
  NextFig := nt;
  Draw;
END;

PROCEDURE tNextView.Draw;
(* Diese Prozedur zeigt das Teil an, das als nchstes     *)
(* herunterfllt. Die Felder Fi und Fi_Ext enthalten die  *)
(* Icons der verschiedenen Figuren                        *)
CONST
  Fi     : ARRAY[1..10, 1..4] OF WORD =
           (($0100, $01DB, $01DB, $0100),
            ($02DC, $02DB, $02DF, $0200),
            ($03DF, $03DB, $03DC, $0300),
            ($04DC, $04DC, $04DC, $04DC),
            ($05DC, $05DC, $05DB, $0500),
            ($06DC, $06DB, $06DC, $0600),
            ($07DB, $07DC, $07DC, $0700),
            ($09DB, $09DF, $09DB, $0900),
            ($0ADF, $0ADB, $0ADF, $0A00),
            ($0BDB, $0B00, $0B00, $0B00));
  Fi_Ext : ARRAY[9..10,1..4] OF WORD =
           (($0A00, $0ADC, $0A00, $0A00),
            ($0BDC, $0BDC, $0BDC, $0B00));
VAR
  r    : tRect;
  Patt : ARRAY[1..18] OF WORD;
BEGIN
  tView.Draw;
  GetExtent(r);
  FillChar(Patt, SizeOf(Patt), 0);
  IF NextFig > 0 THEN Move(Fi[NextFig,  1], Patt[8], 8);
  IF NextFig > 8 THEN Move(Fi_Ext[NextFig], Patt[2], 8);
  WriteBuf(r.A.X, r.A.Y, r.B.X, r.B.Y, Patt);
END;

BEGIN
  Main.Init;
  Main.Run;
  Main.Done;
END.

(* ====================================================== *)
(*                   Ende von TVTETRIS.PAS                *)
