(* ====================================================== *)
(*                      EINHEIT.PAS                       *)
(*                  Objekte fr Unilook                   *)
(*     (c) 1993 Jeanette Winzenburg & toolbox             *)
(* ====================================================== *)
UNIT Einheit;

INTERFACE

USES
  Objects, Validate, MsgBox, Dialogs, Views,
  Drivers, Menus;

  FUNCTION GetOk(CONST Frage: STRING): BOOLEAN;
  { NB: Result = true, falls Frage mit Ja beantwortet }

TYPE
  pNoCase = ^tNoCase;
  tNoCase = OBJECT(tStringCollection)
    FUNCTION Compare(Key1, Key2: Pointer): INTEGER; VIRTUAL;
    { sortiert ohne Bercksichtigung
      von Gro- und Kleinschreibung }
  END;

CONST
  rNoCase: tStreamRec = (
    ObjType : 1050;
    VMTLink : Ofs(TypeOf(tNoCase)^);
    Load    : @tNoCase.Load;
    Store   : @tNoCase.Store);

  PROCEDURE RegisterNoCase;

TYPE
  pWortValidator = ^tWortValidator;
  tWortValidator = OBJECT (tValidator)
    Wort : pString;
    CONSTRUCTOR Init(CONST aWort: STRING);
    DESTRUCTOR  Done;                               VIRTUAL;
    PROCEDURE   SetWort(CONST aWort: STRING);       VIRTUAL;
    FUNCTION    GetWort: STRING;                    VIRTUAL;
    FUNCTION    IsValid(CONST aWort: STRING): BOOLEAN;
                                                    VIRTUAL;
      { NB: result = TRUE fr upstr(aWort) = upstr(Wort^) }
    PROCEDURE Error;                                VIRTUAL;
  END;

  pWortInput = ^tWortInput;
  tWortInput = OBJECT (tInputline)
    CONSTRUCTOR Init(VAR R: tRect; Wort: STRING);
    PROCEDURE   SetData(VAR Rec);                   VIRTUAL;
    { Eingabezeile mit WortValidator }
  END;

  pAnzeigeText = ^tAnzeigeText;
  tAnzeigeText = OBJECT (tStaticText)
    PROCEDURE NewText(CONST aText: STRING);         VIRTUAL;
  END;

  pEditWortDialog = ^tEditWortDialog;
  tEditWortDialog = OBJECT (tDialog)
    Muster : pAnzeigeText;
    CONSTRUCTOR Init(Wort: STRING);
    PROCEDURE   SetData(VAR Rec);                   VIRTUAL;
    FUNCTION    Valid(Command: WORD): BOOLEAN;      VIRTUAL;
  END;

  pAnzeigeZeile = ^tAnzeigeZeile;
  tAnzeigeZeile = OBJECT (tAnzeigeText)
    FUNCTION GetPalette: pPalette;                  VIRTUAL;
  END;

IMPLEMENTATION

  FUNCTION GetOk(CONST Frage: STRING): BOOLEAN;
  BEGIN
    GetOk := MessageBox(Frage, NIL, mfConfirmation OR
                        mfYesButton OR mfNoButton) = cmYes;
  END;

  { ******* tNoCase ******* }

  FUNCTION tNoCase.Compare(Key1, Key2: Pointer): INTEGER;
    { aus: Toolbox 4/93, S. 122 }
  VAR
    Wert, EndPos, i : INTEGER;
  BEGIN
    IF (Key1 = NIL) OR (Key2 = NIL) THEN BEGIN
      IF Key1 = Key2 THEN
        Compare := 0
      ELSE IF Key1 = NIL THEN
        Compare := -1
      ELSE
        Compare := 1;
    END ELSE BEGIN
      IF Length(pString(Key1)^) >=
         Length(pString(Key2)^) THEN
        EndPos := Length(pString(Key2)^)
      ELSE
         EndPos := Length(pString(Key1)^);
      Wert := 0;
      i := 1;
      WHILE (i <= EndPos) AND (Wert = 0) DO BEGIN
        Wert := Ord(UpCase(pString(Key1)^[i])) -
                Ord(UpCase(pString(Key2)^[i]));
        Inc(i);
      END;
      IF Wert = 0 THEN
        Wert := Length(pString(Key1)^) -
                Length(pString(Key2)^);
      IF Wert = 0 THEN
        Compare := 0
      ELSE IF Wert < 0 THEN
        Compare := -1
      ELSE
        Compare := 1;
    END;
  END;

  PROCEDURE RegisterNoCase;
  BEGIN
    RegisterType(rNoCase);
  END;

  { ******* tWortValidator ******* }

  CONSTRUCTOR tWortValidator.Init(CONST aWort: STRING);
  BEGIN
    INHERITED Init;
    SetWort(aWort);
  END;

  DESTRUCTOR tWortValidator.Done;
  BEGIN
    SetWort('');
    INHERITED Done;
  END;

  PROCEDURE tWortValidator.SetWort(CONST aWort: STRING);
  BEGIN
    DisposeStr(Wort);
    Wort := NewStr(aWort);
  END;

  FUNCTION tWortValidator.GetWort: STRING;
  BEGIN
    IF Wort <> NIL THEN GetWort := Wort^
                   ELSE GetWort := '';
  END;

  FUNCTION tWortValidator.IsValid
           (CONST aWort: STRING): BOOLEAN;
  VAR
    Ok : BOOLEAN;
    i  : INTEGER;
  BEGIN
    i := 1;
    Ok := Length(aWort) = Length(GetWort);
    WHILE Ok AND (i <= Length(aWort)) DO BEGIN
      Ok := UpCase(aWort[i]) = UpCase(Wort^[i]);
      Inc(i);
    END;
    IsValid := Ok;
  END;

  PROCEDURE tWortValidator.Error;
  BEGIN
    MessageBox('Nur Gro/Kleinschreibung von %s ' +
               'vernderbar', @Wort, mfError OR mfOkButton);
  END;

  { ******* tWortInput ******* }

  CONSTRUCTOR tWortInput.Init(VAR R: tRect; Wort: STRING);
  BEGIN
    INHERITED Init(R, SizeOf(STRING) - 1);
    SetValidator(New(pWortValidator, Init('')));
    Options := Options OR ofValidate;
    SetData(Wort);
    BlockCursor;
  END;

  PROCEDURE tWortInput.SetData(VAR Rec);
  BEGIN
    IF Validator <> NIL THEN
      pWortValidator(Validator)^.SetWort(STRING(Rec));
    INHERITED SetData(Rec);
  END;

  { ******* tAnzeigeText ******* }

  PROCEDURE tAnzeigeText.NewText(CONST aText: STRING);
  BEGIN
    DisposeStr(Text);
    Text := NewStr(aText);
    DrawView;
  END;

  { ******* tEditWortDialog ******* }

  CONSTRUCTOR tEditWortDialog.Init(Wort: STRING);

    PROCEDURE DialogInit;
    VAR
      V, LB : pView;
      R     : tRect;
    BEGIN
      R.Assign(17, 6, 57, 16);
      INHERITED Init(R, 'Schreibweise festlegen fr:');
      R.Assign(3, 2, 3+32, 3);
      Muster := New(pAnzeigeText, Init(R, Wort));
      Insert(Muster);
      R.Assign(3, 5, 37, 6);
      V := New(pWortInput, Init(R, Wort));
      Insert(V);
      R.Assign(2, 4, 21, 5);
      Insert(New(pLabel, Init(R, 'Neue Schreibweise:', V)));
      R.Assign(12, 7, 26, 9);
      V := New(pButton, Init(R, '~O~k', cmOk, bfDefault));
      V^.Options := V^.Options OR ofCenterX;
      Insert(V);
      SelectNext(FALSE);
    END;

  BEGIN
    DialogInit;
    Options := Options OR ofCentered;
  END;

  PROCEDURE tEditWortDialog.SetData(VAR Rec);
  BEGIN
    IF Muster <> NIL THEN Muster^.NewText(STRING(Rec));
    INHERITED SetData(Rec);
  END;

  FUNCTION tEditWortDialog.Valid(Command: WORD): BOOLEAN;
  VAR
    Ok : BOOLEAN;
  BEGIN
    Ok := TRUE;
    IF (Command = cmCancel) THEN
      Ok := GetOk('Bearbeitung abbrechen?')
    ELSE
      Ok := INHERITED Valid(Command);
    Valid := Ok;
  END;

  { ******* tAnzeigeZeile ******* }

  FUNCTION tAnzeigeZeile.GetPalette: pPalette;
  CONST
    MyPal : STRING[Length(cMenuView)] = cMenuView;
  BEGIN
    GetPalette := @MyPal;
  END;

END.
(* ====================================================== *)
(*                 Ende von EINHEIT.PAS                   *)


