(* ====================================================== *)
(*                      UNILOOK.PAS                       *)
(* Filter fr einheitliche Schreibweise aller Bezeichner  *)
(* eingesetzt am "Druckerausgang" der BP 7.0 DOS-IDE.     *)
(* Kommandozeilenparamter:                                *)
(*  $EDNAME    Dateiname des aktuellen Editorfensters     *)
(*  /Rx        Schreiboption fr reservierte Worte:       *)
(*             x = 0 : alles klein (Vorgabe)              *)
(*             x = 1 : alles gro                         *)
(*                                                        *)
(*     (c) 1993 Jeanette Winzenburg & toolbox             *)
(* ====================================================== *)
{$A+,B-,D+,E+,F-,G+,I+,L+,N-,O-,P-,Q+,R+,S+,T-,V-,X+,Y+}
{$M 16384,0,655360}

PROGRAM UniLook;

USES
  Objects, Drivers, Dialogs, Views, Dos,
  MsgBox, App, Memory, Diverses, Einheit;

CONST
  { Dateimanagement }
  ListenName : STRING = 'LookList.Jlt';
  TempExt    : STRING = '.J$$';
  BakExt     : STRING = '.JBK';
  StreamSize : WORD   = 2048;
  { Optionen fr reservierte Worte }
  rsKlein  = 0;      { alles klein }
  rsGross  = 1;      { alles gro  }
  { Syntax-Attribute }
  Esc             = #27;
  syLeerraum      = '1';
  syKommentar     = '2';
  syReserviert    = '3';
  syBezeichner    = '4';
  sySymbol        = '5';
  syString        = '6';
  syZahl          = '7';
  syAssembler     = '8';
  SyntaxCodes : SET OF CHAR = [syLeerraum..syAssembler];
  { Kommando-Konstanten }
  cmDoProcess     = 255;
  { Fehler-Konstanten }
  erDoQuit        = -1000;
  erNoName        = -1001;
  erMemory        = -255;

TYPE
  pEinheitslook = ^tEinheitslook;
  tEinheitslook = OBJECT (tApplication)
    Error        : INTEGER;
    ReserveOp    : WORD;        { Option reservierte Worte }
    WortListe    : pSortedCollection;  { Muster Bezeichner }
    ListModified : BOOLEAN;
    DateiName    : pString;                { Originaldatei }
    OutDat       : pStream;           { temp. Ausgabedatei }
    Anzeige      : pAnzeigeZeile;{ Anzeige statt Menuzeile }

    CONSTRUCTOR Init;
    DESTRUCTOR  Done;               VIRTUAL;
    PROCEDURE   ProcessCommandLine; VIRTUAL;
                { Kommandozeile auswerten }
    PROCEDURE   ReadListe;          VIRTUAL;
                { vorhandene Wortliste einlesen }
    PROCEDURE   InitMenuBar;        VIRTUAL;
                { Anzeige einfgen, Menubar = NIL! }
    PROCEDURE   InitListe;          VIRTUAL;
                { neue Wortliste }
    PROCEDURE   InitOutput;         VIRTUAL;
                { Ausgabedatei geffnet }
    PROCEDURE   CloseOutput;        VIRTUAL;
                { falls fehlerfrei,
                  Original durch OutDat ersetzen }
    PROCEDURE   SaveListe;          VIRTUAL;
                { vernderte Wortliste speichern }
    PROCEDURE   ProcessReserved(VAR Wort: STRING); VIRTUAL;
                { reserviertes Wort bearbeiten }
    PROCEDURE   ProcessBezeichner(VAR Wort: STRING);
                                                   VIRTUAL;
                { Bezeichner bearbeiten }
    PROCEDURE   DoAusgabe(Ausgabe: STRING);        VIRTUAL;
                { Ausgabe + linefeed in OutDat schreiben }
    FUNCTION    GetEingabe
                (VAR Eingabe: STRING): BOOLEAN;    VIRTUAL;
                { result = true, falls fehlerfrei gelesen }
    PROCEDURE   ProcessText;                       VIRTUAL;
                { zentrale Bearbeitungsschleife }
    PROCEDURE   HandleEvent(VAR Event: tEvent);    VIRTUAL;
                { cmDoProcess : bearbeiten,
                                Dateien sichern, beenden }
    PROCEDURE   OutOfMemory;                       VIRTUAL;
                { Fehlermeldung mit erMemory }
    FUNCTION    CalcFileName
                (aName: FNameStr): FNameStr;       VIRTUAL;
                { Result = Pfad der Anwendung + aName }
    FUNCTION    OpenStream
              (Name: STRING; Mode: WORD): Pointer; VIRTUAL;
                { gltigen Stream ffnen }
    PROCEDURE   PostBackup(FileName: FNameStr); VIRTUAL;
         { Originaldatei durch OutDat mit Backup ersetzen }
    PROCEDURE   FehlerMeldung(Index: INTEGER); VIRTUAL;
                { Fehlermeldung mit Index }
  END;


  CONSTRUCTOR tEinheitslook.Init;
  VAR
    Ok    : BOOLEAN;
    Event : tEvent;
  BEGIN
    INHERITED Init;
    RegisterObjects;
    RegisterNoCase;
    ProcessCommandLine;
    IF Error <> 0 THEN BEGIN
      FehlerMeldung(Error);
      Fail;
    END;
    Anzeige^.NewText(DateiName^ + '...');
    ReadListe;
    Ok := FALSE;
    IF Error <> 0 THEN BEGIN
      CASE Error OF
        stInitError : Ok := GetOk('Bezeichner-Datei nicht' +
          ' gefunden. Neue Datei anlegen?');
        stGetError  : Ok := GetOk('Bezeichner-Datei leer ' +
          'oder unbekannte Listenart. Neue Datei anlegen?');
      END;
    END;
    IF Ok THEN InitListe;
    IF Error = 0 THEN InitOutput;
    IF Error = 0 THEN BEGIN
      Event.What    := evCommand;
      Event.Command := cmDoProcess;
      PutEvent(Event);
    END ELSE BEGIN
      FehlerMeldung(Error);
      Done;  Fail;
    END;
  END;

  DESTRUCTOR tEinheitslook.Done;
  BEGIN
    IF WortListe <> NIL THEN
      Dispose(WortListe, Done);
    IF OutDat <> NIL THEN
      Dispose(OutDat, Done);
    DisposeStr(DateiName);
    INHERITED Done;
  END;

  PROCEDURE tEinheitslook.ProcessCommandLine;
  VAR
    Param   : STRING;
    I, N, E : INTEGER;

    FUNCTION pVal(VAR P: STRING; Def: WORD): WORD;
    BEGIN
      System.Delete(P, 1, 1);
      Val(P, N, E);
      IF E = 0 THEN pVal := N
               ELSE pVal := Def;
    END;

  BEGIN
    FOR I := 1 TO ParamCount DO BEGIN
      Param := ParamStr(I);
      IF (Length(Param) >= 2) THEN
        CASE UpCase(Param[1]) OF
          '/', '-': BEGIN
             System.Delete(Param, 1, 1);
             IF UpCase(Param[1]) = 'R' THEN
               ReserveOp := pVal(Param, rsKlein)
          END;
        ELSE
          IF Param[2] = ':' THEN
            DateiName := NewStr(Param);
        END;
    END;
    IF DateiName = NIL THEN Error := erNoName;
  END;

  PROCEDURE tEinheitslook.ReadListe;
  VAR
    Stream : pStream;
  BEGIN
    Stream := OpenStream(CalcFileName(ListenName), stOpen);
    IF Stream <> NIL THEN BEGIN
      WortListe := Pointer(Stream^.Get);
      Error := Stream^.Status;
      IF Error <> stOk THEN
        IF WortListe <> NIL THEN BEGIN
          Dispose(WortListe, Done);
          WortListe := NIL;
        END;
      Dispose(Stream, Done);
    END;
  END;

  PROCEDURE tEinheitslook.InitListe;
  BEGIN
    Error := 0;
    WortListe := New(pNoCase, Init(100, 100));
  END;

  PROCEDURE tEinheitslook.InitOutput;
  VAR
    D, N, E : STRING;
  BEGIN
    FSplit(DateiName^, D, N, E);
    OutDat := OpenStream(D + N + TempExt, stCreate);
  END;

  PROCEDURE tEinheitslook.InitMenuBar;
  VAR
    R : tRect;
  BEGIN
    GetExtent(R);  R.B.Y := Succ(R.A.Y);
    Anzeige := New(pAnzeigeZeile, Init(R, ''));
    Insert(Anzeige);
  END;

  PROCEDURE tEinheitslook.CloseOutput;
  BEGIN
    IF OutDat <> NIL THEN Dispose(OutDat, Done);
    OutDat := NIL;
    PostBackup(DateiName^);
    Anzeige^.NewText('');
  END;

  PROCEDURE tEinheitslook.SaveListe;
  VAR
    Stream : pStream;
  BEGIN
    IF NOT ListModified THEN Exit;
    Stream := OpenStream(CalcFileName(ListenName),
                         stCreate);
    IF Stream <> NIL THEN BEGIN
      Stream^.Put(WortListe);
      Error := Stream^.Status;
      Dispose(Stream, Done);
      IF Error = stOk THEN ListModified := FALSE;
    END;
  END;

  PROCEDURE tEinheitslook.ProcessReserved(VAR Wort: STRING);
  VAR
    I : INTEGER;
  BEGIN
    CASE ReserveOp OF
      rsKlein : FOR I := 1 TO Length(Wort) DO
                  Wort[I] := LoCase(Wort[I]);
      rsGross : FOR I := 1 TO Length(Wort) DO
                  Wort[I] := UpCase(Wort[I]);
    END;
  END;

  PROCEDURE tEinheitslook.ProcessBezeichner
            (VAR Wort: STRING);
  VAR
    I : INTEGER;
    D : pDialog;
  BEGIN
    IF Error <> 0 THEN Exit;
    IF Length(Wort) = 1 THEN
      Wort[1] := UpCase(Wort[1])
    ELSE BEGIN
      IF WortListe^.Search(@Wort, I) THEN
        Wort := pString(WortListe^.At(I))^
      ELSE BEGIN
        D := New(pEditWortDialog, Init(''));
        IF ExecuteDialog(D, @Wort) = cmOk THEN BEGIN
          ListModified := TRUE;
          WortListe^.AtInsert(I, NewStr(Wort));
        END ELSE
          Error := erDoQuit;
      END;
    END;
  END;

  PROCEDURE tEinheitslook.DoAusgabe(Ausgabe: STRING);
  CONST
    LineFeed : STRING[2] = #13#10;
  BEGIN
    IF Error <> 0 THEN Exit;
    OutDat^.Write(Ausgabe[1], Length(Ausgabe));
    OutDat^.Write(LineFeed[1], Length(LineFeed));
    Error := OutDat^.Status;
  END;

  FUNCTION tEinheitslook.GetEingabe
                             (VAR Eingabe: STRING): BOOLEAN;
  BEGIN
     ReadLn(Eingabe);
     GetEingabe := (Eingabe <> '') OR NOT EoF;
  END;

  PROCEDURE tEinheitslook.ProcessText;
  VAR
    Eingabe, Ausgabe, Wort : STRING;
    C                      : INTEGER;

    PROCEDURE GetWort;
    BEGIN
      Inc(C);
      WHILE (C <= Length(Eingabe))  AND
            (Eingabe[C] <> Esc) DO BEGIN
        Inc(Wort[0]);
        Wort[Length(Wort)] := Eingabe[C];
        Inc(C);
      END;
    END;

  BEGIN
    WHILE (Error = 0) AND GetEingabe(Eingabe) DO BEGIN
      Ausgabe := '';
      C       := 1;
      WHILE (C <= Length(Eingabe)) DO BEGIN
        IF (Eingabe[C] = Esc) AND
           (Eingabe[Succ(C)] IN SyntaxCodes) THEN BEGIN
          Inc(C);
          Wort := '';
          CASE Eingabe[C] OF
            syReserviert : BEGIN
               GetWort;
               ProcessReserved(Wort);
            END;
            syBezeichner : BEGIN
               GetWort;
               ProcessBezeichner(Wort);
            END;
          ELSE
            Inc(C);
          END;
          Ausgabe := Ausgabe + Wort;
        END ELSE BEGIN
          Inc(Ausgabe[0]);
          Ausgabe[Length(Ausgabe)] := Eingabe[C];
          Inc(C);
        END;
      END;
      DoAusgabe(Ausgabe);
    END;
  END;

  PROCEDURE tEinheitslook.HandleEvent(VAR Event: tEvent);
  BEGIN
    INHERITED HandleEvent(Event);
    IF Event.What = evCommand THEN BEGIN
      CASE Event.Command OF
        cmDoProcess : BEGIN
          ProcessText;
          CloseOutput;
          SaveListe;
          IF Error <> 0 THEN FehlerMeldung(Error);
          EndModal(cmQuit);
        END;
      ELSE
        Exit;
      END;
      ClearEvent(Event);
    END;
  END;

  FUNCTION tEinheitslook.CalcFileName
                                (aName: FNameStr): FNameStr;
  VAR
    ExeName, Dir, Name, Ext: FNameStr;
  BEGIN
    ExeName := ParamStr(0);
    FSplit(ExeName, Dir, Name, Ext);
    CalcFileName := Dir + aName;
  END;

  FUNCTION tEinheitslook.OpenStream
                        (Name: STRING; Mode: WORD): Pointer;
  VAR
    B : pStream;
  BEGIN
    B     := NIL;
    Error := erMemory;
    IF MaxAvail >= StreamSize THEN BEGIN
      B := New(pBufStream, Init(Name, Mode, StreamSize));
      IF NOT LowMemory THEN Error := B^.Status ;
      IF Error <> 0 THEN BEGIN
        Dispose(B, Done);
        B := NIL;
      END;
    END;
    OpenStream := B;
  END;

  PROCEDURE tEinheitslook.PostBackup(FileName: FNameStr);
  VAR
    E : ExtStr;
    F : FILE;
  BEGIN
    CutExt(FileName, E);
    IF Error = 0 THEN BEGIN        {$I-}
      FileErased(FileName + BakExt);
      IF FileExists(FileName + E) THEN BEGIN
        Assign(F, FileName + E);
        Rename(F, FileName + BakExt);
      END;
      IF FileExists(FileName + TempExt) THEN BEGIN
        Assign(F, FileName + TempExt);
        Rename(F, FileName + E);
      END;                       {$I+}
      Error := IOResult;
    END;
  END;

  PROCEDURE tEinheitslook.OutOfMemory;
  BEGIN
    FehlerMeldung(erMemory);
  END;

CONST
  Ft : ARRAY[1..-stPutError] OF STRING[15] =
       ('stError', 'stInitError', 'stReadError',
        'stWriteError', 'stGetError', 'stPutError');

  PROCEDURE tEinheitslook.FehlerMeldung(Index: INTEGER);
  VAR
    InfoText : STRING;
  BEGIN
    CASE Index OF
      erMemory : InfoText :=
                 'Nicht gengend Speicher fr Operation ';
      erNoName : InfoText :=
                 'Kein Dateiname angegeben';
      erDoQuit : InfoText := 'Bearbeitung abgebrochen';
      stPutError..stError :
                 InfoText := 'Stream-Fehler: ' + Ft[-Index];
    ELSE
      InfoText := 'Nicht identifizierter Fehler ';
    END;
    MessageBox(InfoText, NIL, mfError OR mfOkButton);
  END;

VAR
  Prog : pEinheitslook;

BEGIN
  New(Prog, Init);
  IF Prog <> NIL THEN BEGIN
    Prog^.Run;
    Dispose(Prog, Done);
  END;
END.
(* ====================================================== *)
(*                  Ende von UNILOOK.PAS                  *)


