(* ====================================================== *)
(*                       CHKVESA.PAS                      *)
(*             (C) 1994 J. Braun & DMV-Verlag             *)
(*            Compiler: Borland/Turbo Pascal 7.0          *)
(* ====================================================== *)
{$A+,B-,D-,E-,F-,G-,I-,L-,N-,O-,P-,Q-,R-,S-,T-,V-,X+,Y-}
{$M 16384,0,$400}
{$IFDEF DPMI} Error: Nur im Real-Mode compilieren!! {$ENDIF}

PROGRAM CheckVESA;

USES
  Crt, WinDos, Objects, Printer, NumberConversion;

TYPE
  pModeInfoBlock = ^tModeInfoBlock;
  tModeInfoBlock = RECORD
    ModeAttributes     : WORD;
    WinA_Attributes    : BYTE;
    WinB_Attributes    : BYTE;
    WinGranularity     : WORD;
    WinSize            : WORD;
    WinA_Segment       : WORD;
    WinB_Segment       : WORD;
    WinFunctPtr        : LongRec; (* eigentl. ein Pointer *)
    BytesPerScanLine   : WORD;
    (* Optional bis Version 1.2 der VESA-Spezifikation *)
    XResolution        : WORD;
    YResolution        : WORD;
    XCharSize          : BYTE;
    YCharSize          : BYTE;
    NumberOfPlanes     : BYTE;
    BitsPerPixel       : BYTE;
    NumberOfBanks      : BYTE;
    MemoryModel        : BYTE;
    BankSize           : BYTE;
    (* Erst ab Version 2.0 der VESA-Spezifikation.      *)
    (* Ab hier wird nicht mehr im Programm ausgewertet: *)
    NumberofImagePages : BYTE;
    Reserved_1         : BYTE;
    RedMaskSize        : BYTE;
    RedFieldPosition   : BYTE;
    GreenMaskSize      : BYTE;
    GreenFieldPosition : BYTE;
    BlueMaskSize       : BYTE;
    BlueFieldPosition  : BYTE;
    RsvdMaskSize       : BYTE;
    RsvdFieldPosition  : BYTE;
    DirectColorModeInfo: BYTE;
    Reserved_2         : ARRAY[0..215] OF BYTE;
  END;

  pVgaInfoBlock = ^ VgaInfoBlock;
  VgaInfoBlock = RECORD
    VESASignature : ARRAY[0..3] OF CHAR;
    VESAVersion   : BYTE;
    VESARevision  : BYTE;
    OEMStringPtr  : pChar;
    Capabilities  : ARRAY[0..3] OF BYTE;
    VideoModePtr  : ^WORD;
    TotalMemory   : WORD;
    reserved      : ARRAY[0..242] OF BYTE;
  END;

  List   = ARRAY[0..50] OF WORD;

CONST
  Print      : BOOLEAN = FALSE;
  FileOut    : BOOLEAN = FALSE;

VAR
  OutFile    : STRING[79];
  BIOSString : STRING;

(* ------------------------------------------------------ *)

PROCEDURE Wait(VAR ch: CHAR);
BEGIN
  IF NOT FileOut THEN
  REPEAT
    ch := Readkey;
  UNTIL ch IN [^C, #13, #27, #32];
  IF ch = ^C  THEN ch := #27; (* auch mit ^C abbrechen!   *)
  IF ch = #32 THEN ch := #13; (* Die Leertaste tut's auch *)
END;

(* ------------------------------------------------------ *)

PROCEDURE GetCh;
CONST
  s =  'Weiter mit ['#17'], Ende mit [ESC]';
VAR
  ch : CHAR;
BEGIN
  GotoXY(80 - Length(s), 25);
  IF NOT FileOut THEN Write(s);
  Wait(ch);
  IF ch = #27 THEN BEGIN
    GotoXY(1, 25);
    ClrEol;
    GotoXY(1, 25);
    IF FileOut THEN Close(Lst);
    Halt;             (* letzten Bildschirm nicht lschen *)
  END;
  ClrScr;
  IF Print THEN Write(Lst, #12); (* Seitenvorschub *)
END;

(* ------------------------------------------------------ *)

PROCEDURE QuickSort(VAR a: List; Lo, Hi: WORD);
(* by Borland Inc., QSORT.PAS, Demoprogramm zu BP/TP 7.0 *)

  PROCEDURE Sort(l, r: WORD);
  VAR
    i, j, x, y: INTEGER;
  BEGIN
    i := l; j := r; x := a[(l + r) DIV 2];
    REPEAT
      WHILE a[i] < x DO Inc(i);
      WHILE x < a[j] DO Dec(j);
      IF i <= j THEN BEGIN
        y := a[i]; a[i] := a[j]; a[j] := y;
        Inc(i); Dec(j);
      END;
    UNTIL i > j;
    IF l < j THEN Sort(l, j);
    IF i < r THEN Sort(i, r);
  END;

BEGIN
  Sort(Lo, Hi);
END;

(* ------------------------------------------------------ *)

PROCEDURE TestVESAMode(ModeName : STRING; Mode : WORD);
VAR
  VgaPtr         : pModeInfoBlock;
  i              : WORD;
  ModeInfoBuffer : tModeInfoBlock;
  Regs           : tRegisters;
BEGIN
  WriteLn('Test auf ', ModeName);
  IF Print THEN WriteLn(Lst, ^M^J'Test auf ', ModeName);
  FOR i := 1 TO Length(ModeName) + 9 DO BEGIN
    Write('');
    IF Print THEN Write(Lst, '');
  END;
  WITH Regs DO BEGIN
    ES := Seg(ModeInfoBuffer);
    DI := Ofs(ModeInfoBuffer);
    AX := $4F01;
    CX := Mode;
    Intr($10, Regs);
  END;
  IF Regs.AX <> $4F THEN BEGIN
    WriteLn(^M^J'Modus ', HexWordStr(Mode),
            'h wird nicht untersttzt'^J);
    IF Print THEN
      WriteLn(Lst, ^M^J'Modus ', HexWordStr(Mode),
              'h wird nicht untersttzt'^M^J);
    GetCh;
    Exit;
  END;
  VgaPtr := pModeInfoBlock(@ModeInfoBuffer);
  WriteLn(^M^J, ModeName, ' Nummer ', HexWordStr(Mode),
          'h wird untersttzt'^J);
  IF Print THEN
    WriteLn(Lst, ^M^J, ModeName, ' Nummer ',
            HexWordStr(Mode), 'h wird untersttzt'^J);

  WriteLn('  Das Modusattribut ',
          HexWordStr(VgaPtr^.ModeAttributes),
          'h ist folgendermaen aufgebaut:');
  IF Print THEN
  WriteLn(Lst, '  Das Modusattribut ',
          HexWordStr(VgaPtr^.ModeAttributes),
          'h ist folgendermaen aufgebaut:');
  Write('  Modus wird ');
  IF Print THEN Write(Lst, '  Modus wird ');
  IF (VgaPtr^.ModeAttributes AND 1) = 0 THEN BEGIN
    Write('nicht ');
    IF Print THEN Write(Lst, 'nicht ');
  END;
  WriteLn('von der Hardware untersttzt');
  IF Print THEN
    WriteLn(Lst, 'von der Hardware untersttzt');

  Write('  Extended Info fr diesen Modus ist ');
  IF Print THEN
   Write(Lst, '  Extended Info fr diesen Modus ist ');
  IF (VgaPtr^.ModeAttributes AND 2) = 0 THEN BEGIN
    Write( 'nicht ');
    IF Print THEN Write(Lst, 'nicht ');
  END;
  WriteLn('verfgbar');
  IF Print THEN WriteLn(Lst, 'verfgbar');
  Write('  BIOS-Ausgabefunktionen fr diesen Modus werden');
  IF Print THEN
    Write(Lst, '  BIOS-Ausgabefunktionen fr diesen '+
               'Modus werden');
  IF (VgaPtr^.ModeAttributes AND 4) = 0 THEN BEGIN
    Write( ' nicht');
    IF Print THEN Write(Lst, ' nicht');
  END;
  WriteLn(' untersttzt');
  IF Print THEN WriteLn(Lst, ' untersttzt');

  Write('  Die Bildschirmausgabe erfolgt ');
  IF Print THEN
    Write(Lst, '  Die Bildschirmausgabe erfolgt ');
  IF (VgaPtr^.ModeAttributes AND 8) = 0 THEN BEGIN
    WriteLn('monochrom');
    IF Print THEN WriteLn(Lst, 'monochrom')
  END ELSE BEGIN
    WriteLn('in Farbe');
    IF Print THEN WriteLn(Lst, 'in Farbe');
  END;
  Write('  Es handelt sich um einen ');
  IF Print THEN
    Write(Lst, '  Es handelt sich um einen ');
  IF (VgaPtr^.ModeAttributes AND 16) = 0 THEN BEGIN
    Write('Text');
    IF Print THEN Write(Lst, 'Text')
  END ELSE BEGIN
    Write('Grafik');
    IF Print THEN Write(Lst, 'Grafik');
  END;
  WriteLn('modus');
  IF Print THEN WriteLn(Lst, 'modus');
  WriteLn(^M^J'  Fenster A-Attribute:  ',
          HexWordStr(VgaPtr^.WinA_Attributes),
          'h  Fenster B-Attribute:  ',
          HexWordStr(VgaPtr^.WinB_Attributes), 'h');
  Write  ('  Fenster Auflsung:    ',
          HexWordStr(VgaPtr^.WinGranularity));
  WriteLn('h  Fenstergre:         ',
          HexWordStr(VgaPtr^.WinSize), 'h');
  WriteLn('  Fenster A-Segment:    ',
          HexWordStr(VgaPtr^.WinA_Segment),
          'h  Fenster B-Segment:    ',
          HexWordStr(VgaPtr^.WinB_Segment), 'h');
  WriteLn('  Fensterfunktions-Ptr: ',
          HexWordStr(VgaPtr^.WinFunctPtr.Hi),
          ':', HexWordStr(VgaPtr^.WinFunctPtr.Lo));
  WriteLn('  Bytes pro Scanzeile:  ',
          VgaPtr^.BytesPerScanLine:4, ^J);
  WriteLn('  Die restlichen Bereiche der Struktur ' +
          'sind optional:');
  WriteLn('  X-Auflsung:          ',
          VgaPtr^.XResolution:4,
          '   Y-Auflsung:          ',
          VgaPtr^.YResolution:4);
  WriteLn('  Zeichengre (X):     ',
          VgaPtr^.XCharSize:4,
          '   Zeichengre (Y):     ',
          VgaPtr^.YCharSize:4);
  Write  ('  Zahl der Bitplanes:   ',
          VgaPtr^.NumberOfPlanes:4);
  WriteLn('   Bits pro Pixel:       ',
          VgaPtr^.BitsPerPixel:4);
  Write  ('  Zahl der Bnke:       ',
          VgaPtr^.NumberOfBanks:4);
  WriteLn('   Bank-Gre:           ',
          VgaPtr^.BankSize:4);
  Write  ('  Speichermodell:       ',
          VgaPtr^.MemoryModel:4, ^J);
  IF Print THEN BEGIN
    WriteLn(Lst, ^M^J'  Fenster A-Attribute:  ',
            HexWordStr(VgaPtr^.WinA_Attributes),
            'h  Fenster B-Attribute:  ',
            HexWordStr(VgaPtr^.WinB_Attributes), 'h');
    Write(Lst, '  Fenster Auflsung:    ',
            HexWordStr(VgaPtr^.WinGranularity), 'h');
    WriteLn(Lst, '  Fenstergre:         ',
            HexWordStr(VgaPtr^.WinSize), 'h');
    WriteLn(Lst, '  Fenster A-Segment:    ',
            HexWordStr(VgaPtr^.WinA_Segment),
            'h  Fenster B-Segment:    ',
            HexWordStr(VgaPtr^.WinB_Segment), 'h');
    WriteLn(Lst, '  Fensterfunktions-Ptr: ',
            HexWordStr(VgaPtr^.WinFunctPtr.Hi),
            ':', HexWordStr(VgaPtr^.WinFunctPtr.Lo));
    WriteLn(Lst, '  Bytes pro Scanzeile:  ',
            VgaPtr^.BytesPerScanLine:4, ^J);
    WriteLn(Lst, '  Die restlichen Bereiche der Struktur '
          + 'sind optional:');
    WriteLn(Lst, '  X-Auflsung:          ',
            VgaPtr^.XResolution:4,
            '   Y-Auflsung:          ',
            VgaPtr^.YResolution:4);
    WriteLn(Lst, '  Zeichengre (X):     ',
            VgaPtr^.XCharSize:4,
            '   Zeichengre (Y):     ',
            VgaPtr^.YCharSize:4);
    Write  (Lst, '  Zahl der Bitplanes:   ',
            VgaPtr^.NumberOfPlanes:4);
    WriteLn(Lst, '   Bits pro Pixel:       ',
            VgaPtr^.BitsPerPixel:4);
    Write  (Lst, '  Zahl der Bnke:       ',
            VgaPtr^.NumberOfBanks:4);
    WriteLn(Lst, '   Bank-Gre:           ',
            VgaPtr^.BankSize:4);
    WriteLn(Lst, '  Speichermodell:       ',
            VgaPtr^.MemoryModel:4, ^M^J);
  END;
  GetCh;
END;

(* ------------------------------------------------------ *)

PROCEDURE CheckBIOS(VAR Name: STRING);

  PROCEDURE SearchCopyRight(CONST Segment, Offset: WORD);
  VAR
    Len,
    num,
    i         : WORD;
    s         : ARRAY[0..2] OF STRING;     (* ID-Strings  *)
    f         : Text;             (* Datei mit ID-Strings *)
    IDs       : ARRAY[0..60] OF STRING[50];(* max. 50 IDs *)
    fName     : STRING[79];                (* Findfirst   *)
    sr        : tSearchRec;
    fm, DosErr: INTEGER;
  BEGIN
    Len := (Byte(Ptr(Segment, Offset + 2)^) * 512 );

    WriteLn(^M^J'BIOS-Erweiterung gefunden bei:   ',
            HexWordStr(Segment), ':',
            HexWordStr(Offset));
    WriteLn('Lnge des ROM-Bereichs:          ', Len,
            ' Bytes', ' (= ',
            Byte(Ptr(Segment, Offset + 2)^),
            ' Blcke  512 Bytes)');
    Write('Gefundene IDs in diesem Bereich: ');
    IF Print THEN BEGIN
      WriteLn(Lst, ^M^J'BIOS-Erweiterung gefunden bei:   ',
              HexWordStr(Segment), ':',
              HexWordStr(Offset));
      WriteLn(Lst, 'Lnge des ROM-Bereichs:          ', Len,
              ' Bytes', ' (= ',
              Byte(Ptr(Segment, Offset + 2)^),
              ' Blcke  512 Bytes)');
      Write(Lst, 'Gefundene IDs in diesem Bereich: ');
    END;
    (* Byte 3 und Byte 4 --> Initroutine *)
    s[0] := ''; s[1] := ''; s[2] := '';
    FOR i := Offset + 5 TO $100 DO BEGIN
     IF (Byte(Ptr(Segment, Offset + i)^) >  31) AND
        (Byte(Ptr(Segment, Offset + i)^) < 127) THEN
       s[0] := s[0] + Char(Ptr(Segment, Offset + i)^);
    END;
    FOR i := 1 TO Length(s[0]) DO
      s[1] := s[1] + UpCase(s[0][i]);
    fName := ParamStr(0);
    Delete(FName, Length(FName) - 2, 3);
    FName := FName + 'DAT'#0;      (* Jetzt wird's wst:  *)
    FindFirst(@FName[1], $27, sr); (* WinDos.FindFirst    *)
    Dec(FName[0]);                 (* arbeitet mit pChars *)
    DosErr := DosError;
    IF DosErr = 18 THEN BEGIN      (* Datei fehlt         *)
    (* vordefinierte Liste verwenden, wenn die Datei      *)
    (* chkvesa.dat fehlt                                *)
      IDs[00] := 'HEADLAND';
      IDs[01] := 'TRIDENT';
      IDs[02] := 'TRIDENT TVGA';
      IDs[03] := 'TRIDENT TVGA 8800 BR';
      IDs[04] := 'TRIDENT TVGA 8800 CS';
      IDs[05] := 'TRIDENT TVGA 8900';
      IDs[06] := 'OAK';
      IDs[07] := 'OTI';
      IDs[08] := 'PHOENIX';
      IDs[09] := 'TSENG';
      IDs[10] := 'TSENG LAB';
      IDs[11] := 'TSENG LABORATORIES';
      IDs[12] := 'WESTERN';
      IDs[13] := 'WESTERN DIGITAL';
      IDs[14] := 'PARADISE';
      IDs[15] := 'CHIPS';
      IDs[16] := 'SIGMA';
      IDs[17] := 'SIGMA DESIGN';
      IDs[18] := 'CIRRUS';
      IDs[19] := 'AHEAD';
      IDs[20] := 'EVEREX';
      IDs[21] := 'GENOA';
      IDs[22] := 'VIDEO 7';
      IDs[23] := 'VIDEO SEVEN';
      IDs[24] := 'ZYMOS';
      IDs[25] := 'ZYMOS POACH 51';
      IDs[26] := 'NCR';
      IDs[27] := 'NCR 77C22';
      IDs[28] := 'HERCULES';
      IDs[29] := 'HERCULES GRAPHICS';
      IDs[30] := 'HERCULES GRAPHICS STATION';
      IDs[31] := '761295520';               (* ATI Wonder *)
      IDs[32] := 'TANDY';
      IDs[33] := 'AT&T';
      Num     := 33;
      (* vordefinierte Daten in die Datei abspeichern: *)
      Assign(f, FName);
      {$IFOPT I+} {$I-} {$ENDIF}
      Rewrite(f);
      {$I+}
      DosErr := IoResult;
      IF DosErr <> 0 THEN BEGIN
        WriteLn('Programmfehler: Die Datei ', FName,
                ' fehlt und kann nicht angelegt werden!',
                ^M^J,'Dos-Fehler: ', DosErr);
        Halt;
      END;
      WriteLn(f, '; ***** Liste der Grafikkarten fr CHK'  +
                 'VESA ******');
      WriteLn(f, '; -------------------------------------' +
                 '----------');
      WriteLn(f, '; Bei der Eingabe neuer Werte ist zu be' +
                 'achten,');
      WriteLn(f, '; da Teilstrings zuerst angegeben werd' +
                 'en mssen.');
      WriteLn(f, '; Kommentarzeilen mssen als erstes Zei' +
                 'chen einen');
      WriteLn(f, '; Strichpunkt besitzen. Bezeichner drf' +
                 'en nicht');
      WriteLn(f, '; lnger als 50 Zeichen sein. Leerzeile' +
                 'n sind');
      WriteLn(f, '; nicht erlaubt.');
      WriteLn(f, '; -------------------------------------' +
                 '----------');
      FOR i := 0 TO Num DO WriteLn(f, IDs[i]);
      WriteLn(f, '');
      Close(f);
    END ELSE IF DosErr = 0 THEN BEGIN    (* Datei gibt es *)
      fm := FileMode;
      FileMode := 0;
      Assign(f, FName);
      {$IFOPT I+} {$I-} {$ENDIF}
      Reset(f);
      {$I+}
      DosErr := IoResult;
      IF DosErr <> 0 THEN BEGIN
        WriteLn('Programmfehler: Die Datei ', FName,
                ' kann nicht eingelesen werden!',
                ^M^J,'Dos-Fehler: ', DosErr);
        Halt;
      END;
      FOR i := 0 TO 50 DO IDs[i] := '';
      Num := 0;
      WHILE NOT Eof(f) DO BEGIN
        ReadLn(f, IDs[Num]);
        FOR i := 1 TO Length(IDs[num]) DO
          IDs[num][i] := Upcase(IDs[num][i]);
        IF IDs[Num][1] <> ';' THEN Inc(Num);
      END;
      IF IDs[num] = '' THEN Dec(Num);
      FileMode := fm;
    END ELSE BEGIN
      Write(^M^J'Dateizugriffsfehler!'^M^J,
            'Programm wird abgebrochen.'^M^J,
            'Dos-Fehler: ', DosErr);

      Halt(1);
    END;
    FOR i := 1 TO Num DO
      IF Pos(IDs[i], s[1]) > 0 THEN s[2] := IDs[i];
    IF s[2] <> '' THEN BEGIN
      IF s[2] = '761295520' THEN s[2] := 'ATI';
      Write(s[2]);
      IF Print THEN Write(Lst, s[2]);
      IF Pos('S3', s[1]) > 0 THEN BEGIN
        Write(' (S3-Kennung gefunden)');
        IF Print THEN Write(Lst, ' (S3-Kennung gefunden)');
      END;
      WriteLn;
      IF Print THEN WriteLn(Lst, '');
    END ELSE BEGIN
      WriteLn('keine! Der Text im Bereich sieht so aus: ');
      WriteLn(s[0], ^J);
      IF Print THEN BEGIN
        WriteLn(Lst, 'keine! Der Text im Bereich sieht ' +
                     'so aus: ');
        WriteLn(Lst, s[0], ^M^J);
      END;
    END;
    Name := s[2];
  END;

CONST
  Segment : WORD = $C000;      (* Beginn des ROM-Bereichs *)
BEGIN
  WHILE Segment < $F600 DO BEGIN
    IF WORD(Ptr(Segment, 0)^) = $AA55 THEN
      SearchCopyRight(Segment, 0);
    Inc(Segment, $200);      (* Suche in 512 Byte-Blcken *)
  END;
END;

(* ------------------------------------------------------ *)

PROCEDURE TestInt1F;
VAR
  Int1FVect: Pointer;
BEGIN
  GetIntVec($1F, Int1FVect);
  Write(^M^J'Der Interrupt 1Fh (Grafikzeichensatz) ');
  IF Print THEN
    Write(Lst,^M^J'Der Interrupt 1Fh (Grafikzeichensatz) ');
  IF Int1FVect <> NIL THEN BEGIN
    WriteLn('zeigt auf ',
            HexWordStr(Seg(Int1FVect^)), ':',
            HexWordStr(Ofs(Int1FVect^)));
  IF Print THEN
    WriteLn(Lst, 'zeigt auf ',
                 HexWordStr(Seg(Int1FVect^)), ':',
                 HexWordStr(Ofs(Int1FVect^)));
  END ELSE BEGIN
    WriteLn(' ist nicht installiert');
    IF Print THEN
      WriteLn(Lst, ' ist nicht installiert');
  END;
END;

(* ------------------------------------------------------ *)

PROCEDURE CheckSVGAModes;
VAR
  InitMode,                             (* zurckschalten *)
  i         : BYTE;                     (* Zhlervariabl. *)
  FName     : STRING;                   (* Dateiname      *)
  f         : Text;                     (* Dateivariable  *)
  ch        : CHAR;                     (* Tastenabfrage  *)
  RegBX     : WORD;
  Int7FVect : Pointer;
BEGIN
  InitMode := Byte(Ptr(Seg0040, $0049)^);
  IF BIOSString <> '' THEN BEGIN
    FName := BIOSString;
    IF Pos(' ', FName) > 0 THEN
      FName[0] := Char(Pos(' ', FName) - 1);    (* cut it *)
    IF FName = 'TSENG' THEN BEGIN
      IF Port[$3CD] AND $C0 = 0 THEN FName := 'ET4000'
                                ELSE FName := 'ET3000';
    END;
  END ELSE BEGIN
  (* Check auf VIDEO SEVEN *)
    ASM
      MOV  AX, $1A00
      INT  10h
      CMP  AL, 1Ah
      JNE  @NoV7
      MOV  AX, 6F00h
      XOR  BX, BX
      INT  10h
      JMP  @exit
      @NoV7:
      XOR  BX, BX
      @exit:
      MOV  RegBX, BX
    END;
    IF RegBX = $5637 THEN
      FName := 'VIDEO7'
    ELSE BEGIN
      GetIntVec($7F, Int7FVect); (* 8514/A-Vektor *)
      IF Int7FVect <> NIL THEN FName := 'IBM8514'
                          ELSE FName := 'UNKNOWN';
    END;
  END;
  WriteLn(^M^J'Es werden alle von der Karte erlaubten ',
          'Bildschirmmodi ermittelt');
  WriteLn('und anschlieend in die Datei ' + FName +
          '.DAT geschrieben. Hierzu'^M^J'werden alle Modi' +
          ' bis zur Nummer 0FFh durchprobiert.');
  GetCh;
  Assign(f, FName + '.DAT');
  {$IFOPT I+} {$I-} {$ENDIF}
  Rewrite(f);
  {$I+}
  IF IOResult <> 0 THEN BEGIN
    WriteLn('Datei kann nicht geschrieben werden.'
            + ^M^J'Programm wird abgebrochen!');
    Exit;
  END;
  Write(f, 'Grafikmodi der ', FName, '-Karte ');
  WriteLn(f, '(Standard- und SVGA-Modi):');
  FOR i := $0 TO $FF DO BEGIN
    IF i IN [$08..$0C] THEN
       WriteLn(f, ^I, i, 'd,', ^I, HexByteStr(i), 'h',
                  ^I'-------- reserviert ----------')
    ELSE BEGIN
      ASM
        MOV AL, i
        XOR AH, AH
        INT 10h;
      END;
      IF Byte(Ptr(Seg0040, $49)^) = i THEN BEGIN
        WriteLn(HexByteStr(i), '      ', i);
        Write(f, ^I, Byte(Ptr(Seg0040, $0049)^), 'd,',
                 ^I,
                 HexByteStr(Byte(Ptr(Seg0040, $0049)^)),
                            'h');
        IF i < $14 THEN BEGIN
          Write(f, ^I);
          CASE i OF
            0: WriteLn(f,' 40 x 25 CGA  Text    ' +
                         '16 Graustufen');
            1: WriteLn(f,' 40 x 25 CGA  Text    16 Farben');
            2: WriteLn(f,' 80 x 25 CGA  Text    ' +
                         '16 Graustufen');
            3: WriteLn(f,' 80 x 25 CGA  Text    16 Farben');
            4: WriteLn(f,' 320x200 CGA  Grafik   ' +
                         '4 Graustufen');
            5: WriteLn(f,' 320x200 CGA  Grafik   4 Farben');
            6: WriteLn(f,' 640x200 CGA  Grafik   2 Farben');
            7: WriteLn(f,' 720x348 MDA/Hercules Monochrom');
           $D: WriteLn(f,' 320x200 EGA  Grafik  16 Farben');
           $E: WriteLn(f,' 640x200 EGA  Grafik  16 Farben');
           $F: WriteLn(f,' 640x350 EGA  Grafik  Monochrom');
          $10: WriteLn(f,' 640x350 EGA  Grafik  16 Farben');
          $11: WriteLn(f,' 640x480 MCGA Grafik   2 Farben');
          $12: WriteLn(f,' 640x480 VGA  Grafik  16 Farben');
          $13: WriteLn(f,' 320x200 MCGA Grafik 256 Farben');
          END;
          IF i = $13 THEN
             WriteLn(f, '--------------------------------',
                        '---------------------------');
        END ELSE IF i = $6A THEN
          WriteLn(f, ^I, 8 * Byte(Ptr(Seg0040, $004A)^): 4,
                    'x', 6 * Byte(Ptr(Seg0040, $004A)^): 3,
                    ' VESA-Grafik  16 Farben')
        ELSE BEGIN
          IF Byte(Ptr(SegB800, $0000)^) = Byte('$') THEN
            WriteLn(f, ^I, Byte(Ptr(Seg0040, $4A)^): 4,
                       'x',
                       Succ(Byte(Ptr(Seg0040, $84)^)): 3,
                       ' SVGA Text    16 Farben')
          ELSE IF Byte(Ptr(SegB000, $0)^) = Byte('$') THEN
            WriteLn(f, Byte(Ptr(Seg0040, $004a)^): 4,
                       'x ?? SVGA Text    Monochrom')
          ELSE BEGIN (* Grafikmodus ! *)
            Write(f, ^I, 8 * Byte(Ptr(Seg0040, $004A)^): 4,
                     'x');
            CASE Byte(Ptr(Seg0040, $004A)^) OF
              80: Write(f, '480');
             100: Write(f, '600');
             128: Write(f, '768');
             160: Write(f, '960');
             ELSE Write(f, '???');
            END;
            WriteLn(f,   ' SVGA Grafik ??? Farben')
          END;
        END;
      END;
      ASM
        MOV AL, InitMode
        XOR AH, AH
        INT 10h
      END;
    END;
  END;
  Close(f);
  WriteLn('GrafikModi als Datei ', FName, '.DAT',
          ' gespeichert.'^M^J'Programm ist beendet.');
END;

(* ------------------------------------------------------ *)

PROCEDURE DisplayHeader;
BEGIN
  ClrScr;
  HighVideo;
  WriteLn('VESA-Test, toolbox 1''94'^M^J +
          '');
  NormVideo;
  IF Print THEN
    WriteLn(Lst, ^M^J'VESA-Test, toolbox 1''94'^M^J +
                 '');
END;

(* ------------------------------------------------------ *)

PROCEDURE Help;
BEGIN
  HighVideo;
  WriteLn('VESA-Check v1.0, (C) 1994 J. Braun & DMV'^J);
  NormVideo;
  WriteLn('Das Programm ermittelt ROM-BIOS-Erweiterungen' +
          ' und die'^M^J'Verfgbarkeit eines VESA-'       +
          'Grafiktreibers.'^J);
  WriteLn('Durch Angabe des Kommandozeilenparameters '    +
          '/p wer-'^M^J'den die'                        +
          ' Informationen zustzlich auch ausgedruckt,'   +
          ^M^J'mit /d in die Datei chkvesa.lst '      +
          'geschrieben.');
  WriteLn('Auer dem Programm selbst wird zustzlich '    +
          'die Datei'^M^J'chkvesa.dat mit der Liste '   +
          'der BIOS-Copyrights im ');
  WriteLn('selben Verzeichnis wie das Programm bentigt'  +
          '. Diese'^M^J'Textdatei kann durch eigene'      +
          ' Eintrge ergnzt werden.'^J);
END;

(* ------------------------------------------------------ *)

PROCEDURE TestVESAModes;
VAR
  RegAX : WORD;
  ch    : CHAR;
BEGIN
  (* Testen der von der VESA-Kommision definierten        *)
  (* Grafikmodi in 16 und 256 Farben:                     *)

  TestVESAMode('Grafik 640*400 in 256 Farben',     $100);
  TestVESAMode('Grafik 640*480 in 256 Farben',     $101);
  TestVESAMode('Grafik 800*600 in 16 Farben',      $102);
  TestVESAMode('Grafik 800*600 in 256 Farben',     $103);
  TestVESAMode('Grafik 1024*768 in 16 Farben',     $104);
  TestVESAMode('Grafik 1024*768 in 256 Farben',    $105);
  TestVESAMode('Grafik 1280*1024 in 16 Farben',    $106);
  TestVESAMode('Grafik 1280*1024 in 256 Farben',   $107);

  (* Testen der definierten VESA-Textmodi:                *)
  TestVESAMode('Text  80*60 Zeichen',              $108);
  TestVESAMode('Text 132*25 Zeichen',              $109);
  TestVESAMode('Text 132*43 Zeichen',              $10A);
  TestVESAMode('Text 132*50 Zeichen',              $10B);
  TestVESAMode('Text 132*60 Zeichen',              $10C);

  (* Testen der VESA-Modi in Hi-Color (15, 16 und 24 Bit: *)
  TestVESAMode('Grafik 320*200 in 32768 Farben',     $10D);
  TestVESAMode('Grafik 320*200 in 65536 Farben',     $10E);
  TestVESAMode('Grafik 320*200 in 16,8 Millionen Farben',
                                                     $10F);
  TestVESAMode('Grafik 640*480 in 32768 Farben',     $110);
  TestVESAMode('Grafik 640*480 in 65536 Farben',     $111);
  TestVESAMode('Grafik 640*480 in 16,8 Millionen Farben',
                                                     $112);
  TestVESAMode('Grafik 800*600 in 32768 Farben',     $113);
  TestVESAMode('Grafik 800*600 in 65536 Farben',     $114);
  TestVESAMode('Grafik 800*600 in 16,8 Millionen Farben',
                                                     $115);
  TestVESAMode('Grafik 1024*768 in 32768 Farben',    $116);
  TestVESAMode('Grafik 1024*768 in 65536 Farben',    $117);
  TestVESAMode('Grafik 1024*768 in 16,8 Millionen Farben',
                                                     $118);
  TestVESAMode('Grafik 1280*1024 in 32768 Farben',   $119);
  TestVESAMode('Grafik 1280*1024 in 65536 Farben',   $11A);
  TestVESAMode('Grafik 1280*1024 in 16,8 Millionen Farben',
                                                     $11B);

  (* Schalten in einen Standard-Modus per VESA-Aufruf:    *)
  ASM
    MOV  AX,    4F02h
    MOV  BX,    13h
    INT  10h
    MOV  RegAX, AX
  END;
  IF RegAX <> $4F THEN BEGIN
    WriteLn('Grafikkarte ist nicht 100% VESA-kompatibel');
    IF Print THEN
      WriteLn(Lst,
              'Grafikkarte ist nicht 100% VESA-kompatibel');
    Halt(1);
  END;

  (* Der Rest wird nicht mehr auf dem Drucker ausgegeben: *)
  Write(^J^J'Es wurde in den Modus 13h geschaltet'^J^J);
  Wait(ch);

  ASM (* Zurckschalten in den Textmodus per VESA-Aufruf: *)
    MOV AX,    4F02h
    MOV BX,    LastMode
    INT 10h
    MOV RegAX, AX
  END;
  IF RegAX <> $4F THEN BEGIN
    WriteLn('Grafikkarte ist nicht 100% VESA-kompatibel');
    Halt(1);
  END ELSE
    WriteLn('... zurck im normalen Textmodus.');
  (* .... und fertig *)
  IF FileOut THEN Close(Lst);        (* Datei schlieen ! *)
END;

(* ====================== Main ========================== *)

CONST
  WeekDay : ARRAY[0..6] OF pChar = { kompakter als Strings }
            ('Sonntag', 'Montag', 'Dienstag', 'Mittwoch',
             'Donnerstag', 'Freitag', 'Samstag');
VAR
  VgaPtr           : pVgaInfoBlock;
  ModePtr          : ^WORD;
  ch               : CHAR;
  GlobalInfoBuffer : ARRAY[0..255] OF CHAR;
  aModePtr         : List;
  yetdone          : ARRAY[0..3] OF BOOLEAN;
  Index, Year,
  Month, Day,
  DayOfWeek, Hour,
  Minute, Second,
  Sec100, RegAX    : WORD;
BEGIN
  DirectVideo := FALSE;  (* sonst kein Write im Modus 13h *)
  CheckBreak  := FALSE;  (* ESC und ^C reichen            *)
  IF Boolean(Pos('?', ParamStr(1))) THEN BEGIN
    Help;
    Halt(0);
  END;
  Print :=   (Pos('/p', ParamStr(1)) > 0) OR
             (Pos('/P', ParamStr(1)) > 0);
  FileOut := (Pos('/d', ParamStr(1)) > 0) OR
             (Pos('/D', ParamStr(1)) > 0);
  IF FileOut THEN BEGIN
    Print   := TRUE;
    OutFile := ParamStr(0);
    Delete(OutFile, Length(OutFile) - 2, 3);
    OutFile := OutFile + 'LST';
    Assign(Lst, OutFile);
    Rewrite(Lst);
  END;
  IF Print AND NOT FileOut THEN BEGIN
    WriteLn(^M^J'Bitte den Drucker einschalten und mit ' +
          'Papier bestcken!');
    Write('Wenn alles klar ist, ' +
          'weiter mit ['#17']; Ende mit [ESC] ');
    Wait(ch);
    IF ch = #27 THEN BEGIN
      WriteLn(^M^J^J'VESA-Check abgebrochen ...');
      IF FileOut THEN Close(Lst);
      Halt(1);
    END ELSE ClrScr;
  END;
  DisplayHeader;
  IF Print THEN BEGIN
    GetDate(Year, Month, Day, DayOfWeek);
    GetTime(Hour, Minute, Second, Sec100);
    Write  (Lst, 'Ermittelt am ', WeekDay[DayOfWeek],
                 ' den ', Day, '.', Month, '.', Year,
                 ' um ', Hour, ':');
    IF Minute < 10 THEN Write(Lst, '0');
    WriteLn(Lst, Minute, ' h'^M^J);
  END;
  WriteLn(^J'BIOS-Check:');
  IF Print THEN WriteLn(Lst, ^M^J'BIOS-Check:');
  CheckBIOS(BIOSString);(* Suche nach ROMs im ROM-Bereich *)
  TestInt1F;            (* Wohin zeigt der Interrupt 1Fh  *)
  IF Print THEN WriteLn(Lst, ^M^J);
  Getch;
  FOR Index := 0 TO 50 DO aModePtr[Index] := 0;
  DisplayHeader;
  (* Testen der VESA-Konfiguration. Es werden alle von    *)
  (* Karte bzw. dem VESA-Treiber untersttzen Grafik- und *)
  (* Textmodi angezeigt                                   *)
  ASM
    MOV   AX,    Seg GlobalInfoBuffer
    MOV   ES,    AX
    MOV   AX,    Offset GlobalInfoBuffer
    MOV   DI,    AX
    MOV   AX,    4F00h
    INT   10h
    MOV   RegAX, AX
  END;
  IF RegAX <> $4F THEN BEGIN
    WriteLn(^J);
    HighVideo;
    WriteLn('Die Grafikkarte ist nicht VESA-kompatibel');
    NormVideo;
    IF Print THEN
      WriteLn(Lst, ^M^J^M^J,
             'Die Grafikkarte ist nicht VESA-kompatibel');
    CheckSVGAModes;
    Halt(1);
  END;
  WriteLn(^M^J'Die Grafikkarte ist VESA-kompatibel'^J^J);
  IF Print THEN
    WriteLn(Lst,
       ^M^J'Die Grafikkarte ist VESA-kompatibel'^M^J^M^J);
  VgaPtr := pVgaInfoBlock(@GlobalInfoBuffer);
  Write('Die VESA-Signatur ist: ');
  IF Print THEN Write(Lst, 'Die VESA-Signatur ist: ');
  FOR Index := 0 TO 3 DO
    Write(VgaPtr^.VESASignature[Index]);
  WriteLn(^M^J'Versionsnummer:        ',
          VgaPtr^.VESAVersion, '.',
          VgaPtr^.VESARevision, ' '^J);
  Write('Untersttzte VESA-Modi sind (in Hex):'^J);
  IF Print THEN BEGIN
    FOR Index := 0 TO 3 DO
      Write(Lst, VgaPtr^.VESASignature[Index]);
    WriteLn(Lst, ^M^J'Versionsnummer:        ',
          VgaPtr^.VESAVersion, '.',
          VgaPtr^.VESARevision, ' '^J);
    Write(Lst, 'Untersttzte VESA-Modi sind (in Hex):'^J);
  END;
  ModePtr := Pointer(VgaPtr^.VideoModePtr);
  Index   := 0;
  WHILE (ModePtr^ <> $FFFF) DO BEGIN
    aModePtr[Index] := ModePtr^;
    Inc(ModePtr);
    Inc(Index);
  END;
  (* Sortieren der ermittelten Modi, da diese nicht     *)
  (* unbedingt in einer logischen Reihenfolge ange-     *)
  (* gegeben sein mssen:                               *)
  QuickSort(aModePtr, 0, 50);
  FOR Index := 0 TO 3 DO yetdone[Index] := FALSE;
  FOR Index := 0 TO 50 DO BEGIN
    IF aModePtr[Index] < $100 THEN
      IF NOT yetdone[0] THEN BEGIN
       yetdone[0] := TRUE;
       Write(^M^J'untersttzte SVGA-Modi: ');
       IF Print THEN
         Write(Lst, ^M^J'SVGA-Sondermodi:       ');
      END;
      IF (aModePtr[Index] >= $100) AND
         (aModePtr[Index] <= $107) THEN
        IF NOT yetdone[1] THEN BEGIN
          Write(^M^J'VESA-Grafik-Modi:       ');
          IF Print THEN
            Write(Lst, ^M^J'VESA-Grafik-Modi:       ');
          yetdone[1] := TRUE;
        END;
      IF (aModePtr[Index] >= $108) AND
         (aModePtr[Index] <= $10C) THEN
        IF NOT yetdone[2] THEN BEGIN
          Write(^M^J'VESA-Text-Modi:         ');
          IF PRINT THEN
            Write(Lst, ^M^J'VESA-Text-Modi:         ');
          yetdone[2] := TRUE;
        END;
      IF aModePtr[Index] >= $10D THEN
        IF NOT yetdone[3] THEN BEGIN
          Write(^M^J'VESA-Hi-Color-Grafik:   ');
          IF Print THEN
            Write(Lst, ^M^J'VESA-Hi-Color-Grafik:   ');
          yetdone[3] := TRUE;
        END;
    IF aModePtr[Index] <> 0 THEN BEGIN
      Write(HexWordStr(aModePtr[Index]), '  ');
      IF Print THEN
         Write(Lst, HexWordStr(aModePtr[Index]), '  ');
    END;
    IF WhereX > 73 THEN
      Write(^M^J'                        ');
  END;
  IF Print THEN WriteLn(Lst, ^M^J);
  GetCh;

  (* Anzeigen der Sondermodi (falls die Karte solche      *)
  (* untersttzt                                          *)
  FOR Index := 0 TO 50 DO
   IF aModePtr[Index] > 0 THEN
     IF aModePtr[Index] < $100 THEN BEGIN
       TestVESAMode('SVGA-Modus',
                    aModePtr[Index]);
     END;
  TestVESAModes;
END.

(* ====================================================== *)
(*                  Ende von CHKVESA.PAS                  *)
