(*  *)
(*                      NUMBERCO.PAS                      *)
(*       (C) 1993, 1994 J. Braun und DMV-Verlag           *)
(*       Compiler:  Turbo/Borland Pascal ab 7.0           *)
(*  *)
(* Konvertierungen Dez->Hex, Dez->Oct, Dez->Bin nach      *)
(* String und pChar fr Byte-, Word- und Longint-Zahlen.  *)
(* Alle String-Konvertierungen sind als Funktionen        *)
(* implementiert, die pChars teilweise als Funktionen und *)
(* als Prozeduren, da maximal vier Zeichen als Parameter  *)
(* pChar-Funktionen bergeben werden knnen               *)
(*  *)
{$A+,B-,D-,E-,F-,I-,L-,N-,O-,P-,Q-,R-,S-,T-,V-}
{$IFDEF Ver70}  {$X+,Y-,G-} {$ENDIF}
{$IFDEF DPMI}   {$G+}       {$ENDIF}
{$IFDEF Windows {$G+}       {$ENDIF}

UNIT NumberConversion;

INTERFACE

(*  Dezimal nach Hexadezimal  *)

FUNCTION HexByteStr({$IFDEF Ver70}CONST{$ENDIF} b: BYTE):
                                                     STRING;
FUNCTION HexWordStr({$IFDEF Ver70}CONST{$ENDIF} w: WORD):
                                                     STRING;
FUNCTION HexLongStr({$IFDEF Ver70}CONST{$ENDIF} L: LongInt):
                                                     STRING;
{$IFDEF Ver70}
FUNCTION HexByteChr(CONST b: BYTE): pChar;
FUNCTION HexWordChr(CONST w: WORD): pChar;
PROCEDURE HexLongChr(CONST L: LongInt; VAR s: pChar);
{$ENDIF}

(*  Dezimal nach Octal  *)

FUNCTION OctByteStr({$IFDEF Ver70}CONST{$ENDIF} b: BYTE):
                                                     STRING;
FUNCTION OctWordStr({$IFDEF Ver70}CONST{$ENDIF} w: WORD):
                                                     STRING;
FUNCTION OctLongStr({$IFDEF Ver70}CONST{$ENDIF} L: LongInt):
                                                     STRING;
{$IFDEF Ver70}
FUNCTION OctByteChr(CONST b: BYTE): pChar;
PROCEDURE OctWordChr(CONST w: WORD; VAR s: pChar);
PROCEDURE OctLongChr(CONST L: LongInt; VAR s: pChar);
{$ENDIF}

(*  Dezimal nach Binr  *)

FUNCTION BinByteStr({$IFDEF Ver70}CONST{$ENDIF} b: BYTE):
                                                     STRING;
FUNCTION BinWordStr({$IFDEF Ver70}CONST{$ENDIF} w: WORD):
                                                     STRING;
FUNCTION BinLongStr({$IFDEF Ver70}CONST{$ENDIF} L: LongInt):
                                                     STRING;
{$IFDEF Ver70}
PROCEDURE BinByteChr(CONST b: BYTE; VAR s: pChar);
PROCEDURE BinWordChr(CONST w: WORD; VAR s: pChar);
PROCEDURE BinLongChr(CONST L: LongInt; VAR s: pChar);
{$ENDIF}

(*  Entfernen fhrender Nullen  *)

PROCEDURE DeleteLeadingZeroStr(VAR s: STRING);
{$IFDEF Ver70}
PROCEDURE DeleteLeadingZeroChr(VAR s: pChar);
{$ENDIF}

(*  *)

IMPLEMENTATION

{$IFDEF Ver70}
USES
  Objects { LongRec }, Strings { StrCopy, StrCat };
{$ELSE}
TYPE
  LongRec = RECORD
    Lo, Hi: WORD;
  END;
{$ENDIF}

CONST
  NumChr: ARRAY[0..$F] OF CHAR = '0123456789ABCDEF';

FUNCTION HexByteStr({$IFDEF Ver70}CONST{$ENDIF} b: BYTE):
                                                     STRING;
BEGIN
  HexByteStr := NumChr[b SHR   4]
              + NumChr[b AND $0F];
END;

FUNCTION HexWordStr({$IFDEF Ver70}CONST{$ENDIF} w: WORD):
                                                     STRING;
BEGIN
  HexWordStr := NumChr[(Hi(w) SHR   4)]
              + NumChr[(Hi(w) AND $0F)]
              + NumChr[(Lo(w) SHR   4)]
              + NumChr[(Lo(w) AND $0F)];
END;

FUNCTION HexLongStr({$IFDEF Ver70}CONST{$ENDIF} L: LongInt):
                                                     STRING;
VAR
  lr: LongRec ABSOLUTE L;
BEGIN
  HexLongStr := HexWordStr(lr.Hi) + HexWordStr(lr.Lo);
END;

{$IFDEF Ver70}
FUNCTION HexByteChr(CONST b: BYTE): pChar;
VAR
  s: pChar;
BEGIN
  GetMem(s, 3);
  StrCopy(s, '00');
  s[0] := NumChr[b SHR 4];
  s[1] := NumChr[b AND $0F];
  HexByteChr := s;
  FreeMem(s, 3);
END;

FUNCTION HexWordChr(CONST w: WORD): pChar;
VAR
 s: pChar;
BEGIN
  GetMem(s, 5);
  StrCopy(s, '0000');
  s[0] := NumChr[(Hi(w) SHR   4)];
  s[1] := NumChr[(Hi(w) AND $0F)];
  s[2] := NumChr[(Lo(w) SHR   4)];
  s[3] := NumChr[(Lo(w) AND $0F)];
  s[4] := #0;
  HexWordChr := s;
  FreeMem(s, 5);
END;

PROCEDURE HexLongChr(CONST L: LongInt; VAR s: pChar);
VAR
  lr: LongRec ABSOLUTE L;
BEGIN
  s :=  HexWordChr(lr.Hi);
  StrCat(s, HexWordChr(lr.Lo));
END;

{$ENDIF}

(*  *)

FUNCTION OctByteStr({$IFDEF Ver70}CONST{$ENDIF} b: BYTE):
                                                     STRING;
BEGIN
  OctByteStr := NumChr[b  SHR 6]
              + NumChr[(b SHR 3) MOD 8]
              + NumChr[b  MOD 8];
END;

FUNCTION OctWordStr({$IFDEF Ver70}CONST{$ENDIF} w: WORD):
                                                     STRING;
BEGIN
  OctWordStr := NumChr[Hi(w)  SHR 6]
              + NumChr[(Hi(w) SHR 3) MOD 8]
              + NumChr[Hi(w)  MOD 8]
              + NumChr[Lo(w)  SHR 6]
              + NumChr[(Lo(w) SHR 3) MOD 8]
              + NumChr[Lo(w)  MOD 8];
END;

FUNCTION OctLongStr({$IFDEF Ver70}CONST{$ENDIF} L: LongInt):
                                                     STRING;
VAR
  lr: LongRec ABSOLUTE L;
BEGIN
  OctLongStr := OctWordStr(lr.Hi) + OctWordStr(lr.Lo);
END;

{$IFDEF Ver70}
FUNCTION OctByteChr(CONST b: BYTE): pChar;
VAR
  s: pChar;
BEGIN
  GetMem(s, 4);
  StrCopy(s, '000');
  s[0] := NumChr[b  SHR 6];
  s[1] := NumChr[(b SHR 3) MOD 8];
  s[2] := NumChr[b  MOD 8];
  OctByteChr := s;
  FreeMem(s, 4);
END;

PROCEDURE OctWordChr(CONST w: WORD; VAR s: pChar);
BEGIN
  StrCopy(s, '000000');
  s[0] := NumChr[Hi(w)  SHR 6];
  s[1] := NumChr[(Hi(w) SHR 3) MOD 8];
  s[2] := NumChr[Hi(w)  MOD 8];
  s[3] := NumChr[Lo(w)  SHR 6];
  s[4] := NumChr[(Lo(w) SHR 3) MOD 8];
  s[5] := NumChr[Lo(w)  MOD 8];
END;

PROCEDURE OctLongChr(CONST L: LongInt; VAR s: pChar);
VAR
  s1: pChar;
  lr: LongRec ABSOLUTE L;
BEGIN
  GetMem(s1, 7);
  OctWordChr(lr.Hi, s);
  OctWordChr(lr.Lo, s1);
  StrCat(s, s1);
  FreeMem(s1, 7);
END;
{$ENDIF}

(*  *)

FUNCTION BinByteStr({$IFDEF Ver70}CONST{$ENDIF} b: BYTE):
                                                     STRING;
VAR
  s: STRING;
BEGIN
  s := '00000000';
  IF (b AND $01) = $01 THEN s[8] := '1';
  IF (b AND $02) = $02 THEN s[7] := '1';
  IF (b AND $04) = $04 THEN s[6] := '1';
  IF (b AND $08) = $08 THEN s[5] := '1';
  IF (b AND $10) = $10 THEN s[4] := '1';
  IF (b AND $20) = $20 THEN s[3] := '1';
  IF (b AND $40) = $40 THEN s[2] := '1';
  IF (b AND $80) = $80 THEN s[1] := '1';
  BinByteStr := s;
END;

FUNCTION BinWordStr({$IFDEF Ver70}CONST{$ENDIF} w: WORD):
                                                     STRING;
VAR
  s : STRING;
BEGIN
  IF w > $00FF THEN s := BinByteStr(Hi(w))
               ELSE s := '00000000';
  BinWordStr := s + BinByteStr(Lo(w));
END;

FUNCTION BinLongStr({$IFDEF Ver70}CONST{$ENDIF} L: LongInt):
                                                     STRING;
VAR
  lr: LongRec ABSOLUTE L;
BEGIN
  BinLongStr := BinByteStr(Hi(lr.Hi))
              + BinByteStr(Lo(lr.Hi))
              + BinByteStr(Hi(lr.Lo))
              + BinByteStr(Lo(lr.Lo))
END;

{$IFDEF Ver70}
PROCEDURE BinByteChr(CONST b: BYTE; VAR s: pChar);
BEGIN
  StrCopy(s, '00000000');
  IF (b AND $01) = $01 THEN s[7] := '1';
  IF (b AND $02) = $02 THEN s[6] := '1';
  IF (b AND $04) = $04 THEN s[5] := '1';
  IF (b AND $08) = $08 THEN s[4] := '1';
  IF (b AND $10) = $10 THEN s[3] := '1';
  IF (b AND $20) = $20 THEN s[2] := '1';
  IF (b AND $40) = $40 THEN s[1] := '1';
  IF (b AND $80) = $80 THEN s[0] := '1';
END;

PROCEDURE BinWordChr(CONST w: WORD; VAR s: pChar);
VAR
  s1: pChar;
BEGIN
  GetMem(s1, 9);
  IF w > $00FF THEN BinByteChr(Hi(w), s)
               ELSE StrCopy(s, '00000000');
  BinByteChr(Lo(w), s1);
  StrCat(s, s1);
  FreeMem(s1, 9);
END;

PROCEDURE BinLongChr(CONST L: LongInt; VAR s: pChar);
VAR
  lr: LongRec ABSOLUTE L;
  s1: pChar;
BEGIN
  GetMem(s1, 9);
  BinByteChr(Hi(lr.Hi), s);
  BinByteChr(Lo(lr.Hi), s1);
  StrCat(s, s1);
  BinByteChr(Hi(lr.Lo), s1);
  StrCat(s, s1);
  BinByteChr(Lo(lr.Lo), s1);
  StrCat(s, s1);
  FreeMem(s1, 9);
END;
{$ENDIF}

(*  *)

PROCEDURE DeleteLeadingZeroStr(VAR s: STRING);
BEGIN
  WHILE s[1] = '0' DO Delete(s, 1, 1);
END;

{$IFDEF Ver70}
PROCEDURE DeleteLeadingZeroChr(VAR s: pChar);
BEGIN
  WHILE s[0] = '0' DO Inc(s);
END;
{$ENDIF}

END.

(*  *)
(*                  Ende von NUMBERCO.PAS                 *)
