(* ====================================================== *)
(*                      VPCX.PAS                          *)
(* PCX-Dateien auf VESA-VGA-Karten anzeigen;              *)
(* z.Z. werden nur 4 und 8 Bit (entspricht 16 und 256     *)
(* Farben) PCX-Dateien untersttzt                        *)
(*           (c) 1993 Peter Meerwald & toolbox            *)
(* ====================================================== *)
{$A-,B+,D+,E+,F-,G+,I-,L+,N-,O-,P-,Q-,R+,S-,T-,V+,X+,Y+}
{$M 16384,0,655360}

PROGRAM VPCX;

USES Vesa;

CONST
  Sequencer = $3C4;
             (* Adresse des Sequencer-Ports der VGA-Karte *)

TYPE
  tRGBColor = RECORD
    R, G, B : BYTE;
  END;

  tPCXHeader = RECORD     { Dateikopf einer PCX-Datei }
    Magic,	          { 0ah = ZSoft PCX }
    Version     : BYTE;	  { 0 = V2.5,
                            2 = V2.8 mit Palette,
                            3 = V2.8 ohne Palette,
                            5 = V3.0 }
    Compressed,           { 1 = PCX RLE }
    BPP         : BYTE;	  { Bits/Pixel pro Ebene }
    xMin,	          { Bildgre}
    yMin,
    xMax,
    yMax,
    HRes,
    VRes        : WORD;
    Pal         : ARRAY [0..15] OF tRGBColor;
    Reserved0   : BYTE;
    Planes      : BYTE;	  { Anzahl der Farbebenen }
    BytesLine,	          { Bytes pro Zeile }
    PalInfo,              { Palette Info (1 = Farb/SW,
                            2 = Graustufen) }
    DPI         : WORD;
    Reserved1   : ARRAY [0..55] OF BYTE;
  END;

  pByteBuffer = ^tByteBuffer;
  tByteBuffer = ARRAY [0..$FFFE] OF BYTE;

  String80    = STRING [80];
  String8     = STRING [8];

  tDACColors  = ARRAY [0..255] OF tRGBColor;

VAR
  VesaInfo      : tVesaInformation;
  ModeInfo      : tModeInformation;
  SavedVesaMode : WORD;
  VesaMode      : WORD;
  VesaBank      : WORD;
  ScreenBuffer  : pByteBuffer;
  WriteBank     : BYTE;
  ScreenOfs     : WORD;
  MapMask       : BYTE;
  PCXHeader     : tPCXHeader;
  F             : FILE;
  PCXBuffer     : pByteBuffer;
  DisplayBuffer : pByteBuffer;
  PCXIndex      : WORD;
  DisplayIndex  : WORD;
  Scan          : LongInt;
  ScanRun       : WORD;
  FName         : String80;
  FIndex        : LongInt;
  i, j          : WORD;
  DAC           : tDACColors;
  Colors        : WORD;
  Pal           : ARRAY [0..16] OF BYTE;

  FUNCTION Min(a, b : LongInt) : LongInt;
  BEGIN
    IF a < b THEN Min := a
             ELSE Min := b;
  END;

  PROCEDURE FindVesaMode(  xSize, ySize : WORD;
                           MemModel     : BYTE;
                           BitPixel     : BYTE;
                       VAR BestMode     : WORD;
                       VAR BestModeInfo : tModeInformation);
  (* sucht passenden VESA-Modus fr das Bild;             *)
  (* Achtung: Extended Information wird fr die Modi      *)
  (* erwartet (ist nach VESA-Standard nicht zwingend      *)
  (* erforderlich) -> evt. VESA-Treiber laden             *)
  VAR
    TmpModeInfo : tModeInformation;
    Mode        : WORD;
    Error       : BOOLEAN;
  BEGIN
    i        := 0;
    BestMode := $FFFF;
    Error    := FALSE;
    WHILE ((VesaInfo.VideoModes^[i] <> $FFFF) AND
           NOT (Error)) DO BEGIN
      Mode := VesaInfo.VideoModes^[i];
      Error := NOT ReturnModeInformation(Mode, TmpModeInfo);
      IF TmpModeInfo.Attributes AND
         vmExtendedInformation = vmExtendedInformation THEN
        WITH TmpModeInfo DO BEGIN
          IF ((Width >= xSize) AND
              (Height >= ySize) AND
              (TmpModeInfo.MemoryModel = MemModel) AND
              (TmpModeInfo.BitsPixel = BitPixel) AND
              (TmpModeInfo.Attributes AND
               vmSupported = vmSupported)) THEN BEGIN
            IF BestMode = $FFFF THEN BEGIN
              BestMode     := Mode;
	      BestModeInfo := TmpModeInfo;
            END ELSE IF ((Width >= xSize) AND
                (Width <= BestModeInfo.Width)) AND
                ((Height >= ySize) AND
                (Height <= BestModeInfo.Height)) THEN BEGIN
              BestModeInfo := TmpModeInfo;
              BestMode     := Mode;
            END;
          END;
      END ELSE
        WriteLn('Warnung: Modus ', Mode,
                ' weist keine Extended Information auf.');
      Inc(i);
    END;
  END;

  PROCEDURE ShiftIntensity(Colors: WORD; Shift: ShortInt;
    VAR Buffer); ASSEMBLER;
    (* Farben sind in PCX-Dateien RGB-codiert             *)
    (* (8 Bits/Farbkomponente), VGA-Karten knnen nur 6   *)
    (* Bits/Farbkomponente darstellen -> jede             *)
    (* Farbkomponente wird um 2 Bit nach rechts geschoben *)
  ASM
    PUSH    DS
    LDS     SI, Buffer
    LES     DI, Buffer
    MOV     BX, Colors
    MOV     AX, BX
    SHL     AX, 1
    ADD     BX, AX
    MOV     CL, Shift
    CLD
    CMP     cl, 0
    JS      @@ShiftRight
  @@ShiftLeft:
    LODSB
    SHL     AL, CL
    STOSB
    DEC     BX
    JNZ     @@ShiftLeft
    JMP     @@Exit
  @@ShiftRight:
    NEG     CL
  @@1:
    LODSB
    SHR     AL, CL
    STOSB
    DEC     BX
    JNZ     @@1
  @@Exit:
    POP     DS
  END;

  PROCEDURE DecodeColors(VAR F: FILE);
    (* Laden der Farbwerte aus PCX-Datei und Programmie-  *)
    (* ren des DAC ber das BIOS; enthlt das Bild nur 16 *)
    (* Farben, so sind diese im Dateikopf gespeichert,    *)
    (* ansonsten 769 Bytes vor Dateiende (auf das         *)
    (* "Magic-Byte" $0C folgen die Farbwerte)             *)
  BEGIN
    Seek(F, FileSize(F) - SizeOf(DAC) - 1);
    i := 0;
    BlockRead(F, i, 1);
    IF i = $0C THEN BEGIN
                       (* Magic-Byte fr 256-Farb-Tabelle *)
      Colors := 256;
      BlockRead(F, DAC, SizeOf(DAC));
    END ELSE BEGIN
      Colors := 16;
      Move(PCXHeader.Pal, DAC, SizeOf(PCXHeader.Pal));
      ASM
        LES    DI, Pal.DWord
        CLD
        XOR    AX, AX
      @@1:
        STOSB
        INC    AX
        CMP    AL, 15
        JBE    @@1
        XOR    AX, AX
        STOSB
        MOV    AX, 1002h
        MOV    DX, DI
        INT    10h
      END;
    END;
    ShiftIntensity(Colors, -2, DAC);
    ASM
      MOV   AX, 1012h
      MOV   BX, 0000h
      MOV   CX, Colors
      LEA   DX, DAC
      PUSH  DS
      POP   ES
      INT   10h
    END;
  END;

  PROCEDURE FigureOutBanks(VAR ScreenBuffer : pByteBuffer;
                           VAR WriteBank    : BYTE);
    (* Ermitteln, welches Bildschirmspeicherwindow fr    *)
    (* Schreibzugriffe vorgesehen ist                     *)
  BEGIN
    IF ModeInfo.WindowAAttributes AND
       vwWriteable = vwWriteable THEN BEGIN
      WriteBank    := $00;
      ScreenBuffer := Ptr(ModeInfo.WindowASegment, $0000);
    END ELSE IF ModeInfo.WindowBAttributes AND
                vwWriteable = vwWriteable THEN BEGIN
      WriteBank    := $01;
      ScreenBuffer := Ptr(ModeInfo.WindowBSegment, $0000);
    END ELSE BEGIN
      WriteBank    := $00;
      ScreenBuffer := Ptr($A000, $0000);
    END;
  END;

  PROCEDURE DecodePCXLine;
    (* Einlesen und entkomprimieren einer Bildzeile;      *)
    (* das PCX-Format verwendet eine einfache             *)
    (* Run-Length-Kodierung (RLE)                         *)
  BEGIN
    PCXIndex     := 0;
    DisplayIndex := 0;

    BlockRead(F, PCXBuffer^,
              Min(PCXHeader.BytesLine *2* PCXHeader.Planes,
              FileSize(F) - FIndex));

    WHILE DisplayIndex < PCXHeader.BytesLine *
                         PCXHeader.Planes DO BEGIN
      IF PCXBuffer^[PCXIndex] AND $C0 = $C0 THEN BEGIN
        FOR i := 0 TO (PCXBuffer^[PCXIndex] AND $3f) - 1 DO
          DisplayBuffer^[DisplayIndex + i] :=
                         PCXBuffer^[PCXIndex + 1];
        DisplayIndex :=
              DisplayIndex + (PCXBuffer^[PCXIndex] AND $3F);
        PCXIndex := PCXIndex + 2;
      END ELSE BEGIN
        DisplayBuffer^[DisplayIndex] :=
                                       PCXBuffer^[PCXIndex];
        Inc(DisplayIndex);
        Inc(PCXIndex);
      END;
    END;
    FIndex := FIndex + PCXIndex;
    Seek(F, FIndex);
  END;

  PROCEDURE DisplayLine;
  (* Ausgabe einer Bildzeile; dabei wird auf die Gre    *)
  (* der Bildschirmspeicherbnke Rcksicht genommen       *)
  BEGIN
    VesaBank := (Scan * ModeInfo.BytesPerLine) DIV
               (LongInt(ModeInfo.WindowGranularity) * 1024);
    SetVesaBank(WriteBank, VesaBank);
    ScreenOfs := (Scan * ModeInfo.BytesPerLine) MOD
               (LongInt(ModeInfo.WindowGranularity) * 1024);

    ScanRun := Min((PCXHeader.xMax - PCXHeader.xMin + 1) DIV
          (1 SHL (PCXHeader.Planes - 1)), ModeInfo.Width DIV
          (1 SHL (PCXHeader.Planes - 1)));

    IF LongInt(ScreenOfs) + ScanRun >
       LongInt(ModeInfo.WindowGranularity) * 1024 THEN BEGIN
      FOR i := 0 TO LongInt(ModeInfo.WindowGranularity) *
                    1024 - ScreenOfs - 1 DO
        IF i + ScreenOfs < $FFFF THEN
          ScreenBuffer^[ScreenOfs + i] :=
          DisplayBuffer^[MapMask * PCXHeader.BytesLine + i];
      j := i + 1;
      Inc(VesaBank);
      SetVesaBank(WriteBank, VesaBank);
      ScanRun := LongInt(ScreenOfs) + ScanRun - 1 -
               (LongInt(ModeInfo.WindowGranularity) * 1024);
      ScreenOfs := $0000;
      FOR i := 0 TO ScanRun DO
        ScreenBuffer^[ScreenOfs + i] :=
        DisplayBuffer^[MapMask*PCXHeader.BytesLine + j + i];
    END ELSE
      FOR i := 0 TO ScanRun - 1 DO
        ScreenBuffer^[ScreenOfs + i] :=
          DisplayBuffer^[MapMask * PCXHeader.BytesLine + i];
  END;

  PROCEDURE DisplayPCX4(VAR F        : FILE;
                            PCXHeader: TPCXHeader;
                            ModeInfo: tModeInformation);
    (* Ausgabe einer 4 Bit (16 Farben) PCX-Datei          *)
  BEGIN
    GetMem(PCXBuffer,     PCXHeader.BytesLine * 8);
    GetMem(DisplayBuffer, PCXHeader.BytesLine * 4);

    DecodeColors(F);
    FigureOutBanks(ScreenBuffer, WriteBank);

    Seek(F, SizeOf(PCXHeader));
    FIndex := FilePos(F);

    FOR Scan := 0 TO PCXHeader.yMax-PCXHeader.yMin DO BEGIN
      DecodePCXLine;
      FOR MapMask := 0 TO 3 DO BEGIN
        Port[Sequencer] := $02;
        Port[Sequencer + 1] := $01 SHL MapMask;
        DisplayLine;
      END;
    END;
  END;

  PROCEDURE DisplayPCX8(VAR F         : FILE;
                            PCXHeader : TPCXHeader;
                            ModeInfo  : tModeInformation);
    (* Ausgabe einer 8 Bit (256 Farben) PCX-Datei *)
  BEGIN
    GetMem(PCXBuffer, PCXHeader.BytesLine * 2);
    GetMem(DisplayBuffer, PCXHeader.BytesLine);

    DecodeColors(F);
    FigureOutBanks(ScreenBuffer, WriteBank);

    Seek(F, SizeOf(PCXHeader));
    FIndex := FilePos(F);

    MapMask := 0;

    FOR Scan := 0 TO PCXHeader.yMax-PCXHeader.yMin DO BEGIN
      DecodePCXLine;
      IF ModeInfo.MemoryModel = $04 THEN
        DisplayLine
      ELSE BEGIN
        ScreenOfs := LongInt(Scan) * ModeInfo.BytesPerLine;
        ScanRun := Min(PCXHeader.xMax - PCXHeader.xMin,
                       ModeInfo.Width);
        FOR i := 0 TO ScanRun - 1 DO BEGIN
	  Port[Sequencer] := $02;
	  Port[Sequencer + 1] := ($01 SHL (i MOD 4));

	  j := ScreenBuffer^[ScreenOfs + (i DIV 4)];
	  ScreenBuffer^[ScreenOfs + (i DIV 4)] :=
                                          DisplayBuffer^[i];
        END;
      END;
    END;
  END;

  PROCEDURE WaitForKey; ASSEMBLER;
  ASM
    MOV     AH, 00h
    INT     16h
  END;

BEGIN
  WriteLn;
  WriteLn('VPCX PCX Dateien anzeigen');
  WriteLn('VESA BIOS Demonstration');
  WriteLn('(c) 1993 Peter Meerwald & DMV-Verlag');
  WriteLn('Syntax: VPCX filename');
  WriteLn;

  IF NOT ReturnVesaInformation(VesaInfo) THEN BEGIN
    (* VESA Erweiterung vorhanden? *)
    WriteLn('Fehler: VESA Erweiterung nicht gefunden.');
    Halt(1);
  END;

  IF ParamCount <> 1 THEN BEGIN
    (* wurde ein Dateiname angegeben? *)
    WriteLn('Fehler: PCX-Datei nicht angegeben.');
    Halt(2);
  END ELSE
    FName := ParamStr(1);

  Assign(F, FName);
  Reset(F, 1);
  IF IOResult <> 0 THEN BEGIN
    WriteLn('Fehler: PCX-Datei nicht gefunden.');
    Halt(2);
  END;

  BlockRead(F, PCXHeader, SizeOf(PCXHeader));

  IF PCXHeader.Magic <> $0a THEN BEGIN
    WriteLn('Fehler: Datei hat ungltiges Format.');
    Halt(2);
  END;

  IF (PCXHeader.Version <> 5) OR
     (PCXHeader.Compressed <> 1) THEN BEGIN
    WriteLn('Fehler: PCX-Format wird nicht untersttzt.');
    Halt(2);
  END;

  WriteLn;
  WITH PCXHeader DO BEGIN
    IF (PCXHeader.BPP = 1) AND
       (PCXHeader.Planes = 4) THEN BEGIN
      FindVesaMode(xMax - xMin + 1, yMax - yMin + 1, $03,
                   4, VesaMode, ModeInfo);
      IF VesaMode = $FFFF THEN BEGIN
	WriteLn('Fehler: Keinen passenden 16-Farb-Modus',
                'fr das PCX-Bild gefunden.');
	Halt(3);
      END ELSE  BEGIN
	SavedVesaMode := ReturnVesaMode;
                 (* aktuellen Video-Modus sichern *)
	SetVesaMode(VesaMode);
                 (* ermittelten VESA Modus setzen *)
	DisplayPCX4(F, PCXHeader, ModeInfo);
                 (* Bild anzeigen                 *)
        WaitForKey;           (* auf Taste warten *)
	SetVesaMode(SavedVesaMode);
          (* gesicherten Video-Modus zurcksetzen *)
      END;
    END ELSE IF (PCXHeader.BPP = 8) AND
                (PCXHeader.Planes = 1) THEN BEGIN
        FindVesaMode(xMax - xMin + 1, yMax - yMin + 1,
                     $04, 8, VesaMode, ModeInfo);
        IF VesaMode = $FFFF THEN
          FindVesaMode(xMax - xMin + 1, yMax - yMin + 1,
                       $05, 8, VesaMode, ModeInfo);
        IF VesaMode = $FFFF THEN BEGIN
     	  WriteLn('Fehler: Keinen passenden',
               '256-Farb-Modus fr das PCX-Bild gefunden.');
	  Halt(3);
        END ELSE BEGIN
	  SavedVesaMode := ReturnVesaMode;
	  SetVesaMode(VesaMode);
	  DisplayPCX8(F, PCXHeader, ModeInfo);
          WaitForKey;
	  SetVesaMode(SavedVesaMode);
        END;
      END ELSE BEGIN
        WriteLn('Fehler: VPCX untersttzt nur 16 und',
          '256 Farben in PCX-Dateien.');
	Halt(3);
      END;
  END;
END.
(* ====================================================== *)
(*                   Ende von VPCX.PAS                    *)
