(* ====================================================== *)
(*                       GETFONTS.PAS                     *)
(* Programm zum Kopieren der ROM-Zeichenstze einer       *)
(* EGA/VGA-Karte zur Benutzung mit VGAFONT.PAS            *)
(*          (c) 1993 Torsten Schubert & toolbox           *)
(* ====================================================== *)

{$I+}
PROGRAM GetFonts;

USES Dos;

TYPE
  BoundArray = ARRAY [0..255] OF BYTE;

VAR
  F        : FILE;
  Regs     : Registers;
  P        : Pointer;
  B        : BoundArray;
  FontPath : STRING;

  PROCEDURE Fehler;
    (* Gibt einen Fehlertext mit der Fehlernummer aus und *)
    (* beendet das Programm *)
  BEGIN
    WriteLn('Fehler Nr : ', InOutRes:3,
            ' aufgetreten, Programm beendet');
    Halt(IOResult);
  END;


  PROCEDURE CalcBounds(Size : BYTE);
    (* Berechungen fr Proportionalschrift *)
  VAR
    CP       : Pointer;
    i, j, x,
    tFirst,
    tWidth   : BYTE;
  BEGIN
    CP := P;
    FOR i := 0 TO 255 DO BEGIN       (* Fr jedes Zeichen *)
      tFirst := 8;
      tWidth := 0;
      FOR j := 1 TO Size DO BEGIN
        X := Byte(CP^);    (* Zeile des Zeichens auslesen *)
        ASM
          XOR    AX,AX
          MOV    AL,[x]
        @Loop1:
          INC    AH
          SHL    AL,1
          JC     @Found
          JNZ    @Loop1
          JMP    @NotFound
        @Found:
          CMP    AH,[tFirst]
          JGE    @NotFound
          MOV    [tFirst],ah
        @NotFound:
          XOR    AX,AX
          MOV    AH,9
          SUB    AH,[tFirst]
          MOV    AL,[x]
        END;
        Inc(LongInt(CP));
      END;
      IF tFirst = 8 THEN tFirst := 3;
      Dec(tFirst);
      B[i] := tFirst;
    END;
  END;

BEGIN
  (*  Zuerst der 8x16 Zeichensatz : *)
  FontPath := GetEnv('BGIPATH') + '\';
  WITH Regs DO BEGIN
    AX := $1130;        (* Funktion 11h Unterfunktion 30h *)
    BH := 6;            (* 8x16 Zeichensatz               *)
    Intr($10, Regs);    (* Videointerrupt aufrufen        *)
    P := Ptr(ES, BP);   (* Zeiger auf Font ermitteln      *)
  END;
  Assign(F, FontPath + 'CHAR8x16.PIX');
                              (* Zeichensatzdatei anlegen *)
  Rewrite(F, 16*256);
  IF InOutRes <> 0 THEN Fehler;
  BlockWrite(F, P^, 1);       (* und speichern des Fonts  *)
  IF InOutRes <> 0 THEN Fehler;
  Close(F);
  CalcBounds(16);             (* Zeichenbreiten berechnen *)
  Assign(F, FontPath + 'CHAR8x16.BND');  (* Datei anlegen *)
  Rewrite(F, SizeOf(B));
  IF InOutRes <> 0 THEN Fehler;
  BlockWrite(F, B, 1);(* und speichern der Zeichenbreiten *)
  IF InOutRes <> 0 THEN Fehler;
  Close(F);

  (*  Nun der 8x14 Zeichensatz : *)
  WITH Regs DO BEGIN
    AX := $1130;        (* Funktion 11h Unterfunktion 30h *)
    BH := 2;            (* 8x14 Zeichensatz               *)
    Intr($10, Regs);    (* Videointerrupt aufrufen        *)
    P := Ptr(ES, BP);   (* Zeiger auf Font ermitteln      *)
  END;
  Assign(F, FontPath + 'CHAR8x14.PIX');
                              (* Zeichensatzdatei anlegen *)
  Rewrite(F, 14*256);
  IF InOutRes <> 0 THEN Fehler;
  BlockWrite(F, P^, 1);       (* und speichern des Fonts  *)
  IF InOutRes <> 0 THEN Fehler;
  Close(F);
  CalcBounds(14);             (* Zeichenbreiten berechnen *)
  Assign(F, FontPath + 'CHAR8x14.BND');  (* Datei anlegen *)
  Rewrite(F, SizeOf(B));
  IF InOutRes <> 0 THEN Fehler;
  BlockWrite(F, B, 1);(* und speichern der Zeichenbreiten *)
  IF InOutRes <> 0 THEN Fehler;
  Close(F);

  (*  Zuletzt der 8x8 Zeichensatz : *)
  WITH Regs DO BEGIN
    AX := $1130;        (* Funktion 11h Unterfunktion 30h *)
    BH := 3;            (* 8x8 Zeichensatz                *)
    Intr($10, Regs);    (* Videointerupt aufrufen         *)
    P := Ptr(ES, BP);   (* Zeiger auf Font ermitteln      *)
  END;
  Assign(F, FontPath + 'CHAR8x8.PIX');
                              (* Zeichensatzdatei anlegen *)
  Rewrite(F, 8*256);
  IF InOutRes <> 0 THEN Fehler;
  BlockWrite(F, P^, 1);       (* und speichern des Fonts  *)
  IF InOutRes <> 0 THEN Fehler;
  Close(F);
  CalcBounds(8);              (* Zeichenbreiten berechnen *)
  Assign(F, FontPath + 'CHAR8x8.BND');   (* Datei anlegen *)
  Rewrite(F, SizeOf(B));
  IF InOutRes <> 0 THEN Fehler;
  BlockWrite(F, B, 1);(* und speichern der Zeichenbreiten *)
  IF InOutRes <> 0 THEN Fehler;
  Close(F);
END.
(* ====================================================== *)
(*                Ende von GETFONTS.PAS                   *)

