(* ====================================================== *)
(*                       VGAFONT.PAS                      *)
(*   Untersttzung von VGA (8x16) oder EGA (8x14) Fonts   *)
(*          (c) 1993 Torsten Schubert & toolbox           *)
(* ====================================================== *)
UNIT VGAFont;

INTERFACE

CONST
  Pix8x8Font  = 0;     (* Fr den 8x8 Zeichensatz  *)
  Pix8x14Font = 1;     (* Fr den 8x14 Zeichensatz *)
  Pix8x16Font = 2;     (* Fr den 8x16 Zeichensatz *)

  tsNormal    = 0;     (* Schrift Normal           *)
  tsKursive   = 1;     (* Kursive (italic) Schrift *)
  tsOutline   = 2;     (* Umrandete Schrift        *)
  tsBold      = 4;     (* Fettschrift              *)
  tsUnderline = 8;     (* Unterstrichen            *)
  tsPro       = 16;    (* Proportionalschrift      *)

TYPE
  TextSettingsType = RECORD
    Font    : BYTE;    (* Pix8x8 .. Pix8x16        *)
    Style   : BYTE;    (* tsNormal  | tsKursive |
                          tsOutline | tsBold    |
                          tsUnderline              *)
    OC      : BYTE;    (* Randfarbe bei Outline    *)
  END;

(* CONST
  FontPath : STRING = ''; *)
               (* Verzeichnis fr die Zeichenstze *)


  PROCEDURE InitGraph(VAR GraphDriver  : INTEGER;
                      VAR GraphMode    : INTEGER;
                          PathToDriver : STRING);
    (* Speicherplatz fr den Zeichensatz reservieren und  *)
    (* Zeichensatz laden. Diese Routine darf erst nach    *)
    (* InitGraph aufgerufen werden.                     *)

  PROCEDURE SetTextStyle(TS : TextSettingsType);
    (* Definieren der Textgre und des Textstils fr die *)
    (* neuen Zeichensatzroutinen.                         *)

  PROCEDURE GetTextSettings(VAR TS: TextSettingsType);
    (* Liefert die zur Zeit gltigen Textvorgaben fr die *)
    (* neuen Zeichensatzroutinen.                         *)

  PROCEDURE OutText(x,y : INTEGER; TextString : STRING);
    (* Gibt Text an der bergebenen Position aus, wobei   *)
    (* die mit NewSetTextStyle gesetzten Parameter und    *)
    (* aktuelle Zeichenfarbe verwendet werden.            *)

  FUNCTION  TextHeight : BYTE;
    (* Liefert die Texthhe des aktuellen Zeichenssatz    *)
    (* in Pixel.                                          *)

  FUNCTION  TextWidth(TextString : STRING) : WORD;
    (* Liefert die Textbreite des bergebene Strings      *)
    (* in Pixel.                                          *)

IMPLEMENTATION

USES Graph;

TYPE
  pByteArray = ^tByteArray;
  tByteArray = ARRAY [0..$FFF0] OF BYTE;

  BoundArray = ARRAY [0..255] OF BYTE;

  pPixFont   = ^tPixFont;

  tPixFont   = RECORD
    Count    : BYTE;    (* Anzahl der Zeichen - 1         *)
    Height   : BYTE;    (* Hhe der Zeichenbox in Pixel   *)
    BaseLine : BYTE;    (* Grundlinie des Fonts von unten *)
    Data     : pByteArray;
    b        : BoundArray;
  END;

  String8    = STRING[8];

  DrawProc   = PROCEDURE(x, y  : INTEGER; Ch   : CHAR;
                         Color : BYTE;    Flag : BYTE);

VAR
  Font8x8,
  Font8x14,
  Font8x16     : tPixFont;

  CurrentFont  : pPixFont;
  CurrentTS    : TextSettingsType;

  MaxXperPlane : WORD;    (* Variablen zur Zeichenausgabe *)
  MaxX         : INTEGER;
  FontPath     : STRING;

  PROCEDURE LoadFont(VAR Font          : tPixFont;
                         Name          : String8;
                         Count, Height : BYTE);
  VAR
    F : FILE;
    i : WORD;
  BEGIN
    Assign(F, FontPath + Name + '.PIX');
    Font.Count  := Count;
    Font.Height := Height;
    i := (Count+1) * Height;
    GetMem(Font.Data, i);
    {$I-}
    Reset(F, i);
    IF IOResult > 0 THEN Halt(300);
    BlockRead(F, Font.Data^, 1);
    IF IOResult > 0 THEN Halt(300);
    {$I+}
    Close(F);
    Assign(F, FontPath + Name + '.BND');
    {$I-}
    Reset(F, SizeOf(BoundArray));
    IF IOResult > 0 THEN Halt(300);
    BlockRead(F, Font.b, 1);
    IF IOResult > 0 THEN Halt(300);
    {$I+}
    Close(F);
  END;

  PROCEDURE InitGraph(VAR GraphDriver  : INTEGER;
                      VAR GraphMode    : INTEGER;
                          PathToDriver : STRING);
  VAR
    i : BYTE;
  BEGIN
    Graph.InitGraph(GraphDriver, GraphMode,
                    PathToDriver);
    FontPath := PathToDriver + '\';
    LoadFont(Font8x8,  'Char8x8',  255,  8);
    Font8x8.BaseLine  :=  6;
    LoadFont(Font8x14, 'Char8x14', 255, 14);
    Font8x14.BaseLine := 12;
    LoadFont(Font8x16, 'Char8x16', 255, 16);
    Font8x16.BaseLine := 13;
    IF (Succ(GetMaxY) DIV 25) < 15 THEN BEGIN
      CurrentTS.Font := Pix8x14Font;
      CurrentFont    := @Font8x14
    END ELSE BEGIN
      CurrentTS.Font := Pix8x14Font;
      CurrentFont    := @Font8x16;
    END;
    MaxX := GetMaxX;
    MaxXperPlane := (MaxX+1) DIV 8;
    Dec(MaxX, 7);
    CurrentTS.Style := tsNormal;
    CurrentTS.OC    := GetBkColor;
  END;

  PROCEDURE SetTextStyle(TS : TextSettingsType);
  BEGIN
    CurrentTS := TS;
    CASE TS.Font OF
      Pix8x8Font  : CurrentFont := @Font8x8;
      Pix8x14Font : CurrentFont := @Font8x14;
      Pix8x16Font : CurrentFont := @Font8x16;
    END;
  END;

  PROCEDURE GetTextSettings(VAR TS: TextSettingsType);
  BEGIN
    TS := CurrentTS;
  END;

{$F+}
  PROCEDURE OutCharXY(x, y        : INTEGER; c : CHAR;
                      Color, Flag : BYTE); ASSEMBLER;
  VAR
    CharOfs  : WORD;
    Height   : BYTE;
    BaseLine : BYTE;
  ASM
    PUSH    DS
    XOR     AX,AX              (* CharOfs fr c berechnen *)
    XOR     CX,CX
    MOV     AL,[c]
    LES     SI, DWORD Ptr [CurrentFont]
    MOV     CL,tPixFont(ES:[SI]).BaseLine
    MOV     [BaseLine],CL
    MOV     CL,tPixFont(ES:[SI]).Height
    MOV     [Height],CL
    MUL     CX                   (* ord(c)*NewFont.Height *)
    LES     SI,tPixFont(ES:[SI]).Data
                   (* Ptr auf erstes Byte des Buchstabens *)
    ADD     SI,AX; (* ermitteln                           *)
    MOV     bx,MaxXperPlane
    MOV     AX,[y]
    MUL     bx
    XCHG    AX,DI
    MOV     AX,[x]
    MOV     CX,AX
    SHR     AX,1
    SHR     AX,1
    SHR     AX,1
    ADD     DI,AX
    MOV     AX,0A000H
    MOV     DS,AX     (* ds = $A000; BX := MaxXperPlane;  *)
    MOV     DX,3CEH   (* Enable Set/Reset Register setzen *)
    MOV     AX,0F01H  (* Alle Ebenen bernehmen           *)
    OUT     DX,AX
    MOV     AL,0
                 (* Farbe in das Set/Reset Register laden *)
    MOV     AH,[Color]
    OUT     DX,AX

  @Loop1:
    MOV     CX,[x]
    AND     CL,7                (* X mod 8 *)
    MOV     AH,ES:[SI]
    TEST    [Flag],2            (* Unterstreichen *)
    JZ      @NoUnderLine        (* nein dann berspringen *)
    Dec     [BaseLine]
    JNZ     @NoUnderLine
    MOV     AH,$FF              (* dann Unterstreichen *)
  @NoUnderLine:
    TEST    [Flag],1            (* Fett *)
    JZ      @NoBold
    SHR     AH,1
    OR      AH,ES:[SI]
  @NoBold:
    MOV     AL,08
    SHR     AH,CL
    OUT     DX,AX
    ADD     [DI],AL

  @Weiter:
    Inc     DI
    XCHG    Ch,CL
    MOV     CL,08
    SUB     CL,Ch

    MOV     AH,ES:[SI]
    TEST    [Flag],2            (* Unterstreichen *)
    JZ      @NoUnderLine2
    CMP     [BaseLine],0
    JNZ     @NoUnderLine2
    MOV     AH,$FF              (* dann Linie *)
  @NoUnderLine2:
    TEST    [Flag],1            (* Fett *)
    JZ      @NoBold2
    SHR     AH,1
    OR      AH,ES:[SI]
  @NoBold2:
    SHL     AH,CL
    OUT     DX,AX
    XCHG    [DI],AL

    Inc     SI
    ADD     DI,bx
    Dec     DI
    Dec     [Height]
    JG      @Loop1

  @Weiter2:
    MOV     DX,3CEH
    MOV     AX,0001H
    OUT     DX,AX
    MOV     AX,0FF08H
    OUT     DX,AX
    POP     DS
  END;

  PROCEDURE OutKCharXY(x, y : INTEGER; c : CHAR;
                       Color, Flag : BYTE); ASSEMBLER;
  VAR
    CharOfs   : WORD;
    Height    : BYTE;
    BaseLine  : BYTE;
  ASM
    PUSH    DS
    XOR     AX,AX              (* Charofs fr c berechnen *)
    XOR     CX,CX
    MOV     AL,[c]
    LES     SI, DWORD Ptr [CurrentFont]
    MOV     CL,tPixFont(ES:[SI]).BaseLine
    MOV     [BaseLine],CL
    MOV     CL,tPixFont(ES:[SI]).Height
    MOV     [Height],CL
    MUL     CX                   (* ord(c)*NewFont.Height *)
    LES     SI,tPixFont(ES:[SI]).Data
                    (* Ptr auf erstes Byte des Buchstaben *)
    ADD     SI,AX;  (* ermitteln                          *)
    MOV     BX,MaxXperPlane
    MOV     AX,[y]
    MUL     bx
    XCHG    AX,DI
    MOV     AX,[x]
    MOV     CX,AX
    SHR     AX,1
    SHR     AX,1
    SHR     AX,1
    ADD     DI,AX
    MOV     AX,0A000H
    MOV     DS,AX     (* ds = $A000; BX := MaxXperPlane;  *)
    MOV     DX,3CEH   (* Enable Set/Reset Register Setzen *)
    MOV     AX,0F01H            (* Alle Ebenen bernehmen *)
    OUT     DX,AX
    MOV     AL,0 (* Farbe in das Set/Reset Register laden *)
    MOV     AH,[Color]
    OUT     DX,AX

  @Loop1:
    AND     CL,7                (* X mod 8 *)
    MOV     AH,ES:[SI]
    TEST    [Flag],2            (* Unterstreichen *)
    JZ      @NoUnderLine        (* nein dann berspringen *)
    Dec     [BaseLine]
    JNZ     @NoUnderLine
    MOV     AH,$FF              (* dann Unterstreichen *)
  @NoUnderLine:
    TEST    [Flag],1            (* Fett *)
    JZ      @NoBold
    SHR     AH,1
    OR      AH,ES:[SI]
  @NoBold:
    MOV     AL,08
    SHR     AH,CL
    OUT     DX,AX
    ADD     [DI],AL

  @Weiter:
    Inc     DI
    XCHG    Ch,CL
    MOV     CL,08
    SUB     CL,Ch

    MOV     AH,ES:[SI]
    TEST    [Flag],2            (* Unterstreichen *)
    JZ      @NoUnderLine2
    CMP     [BaseLine],0
    JNZ     @NoUnderLine2
    MOV     AH,$FF              (* dann Linie *)
  @NoUnderLine2:
    TEST    [Flag],1            (* Fett *)
    JZ      @NoBold2
    SHR     AH,1
    OR      AH,ES:[SI]
  @NoBold2:
    SHL     AH,CL
    OUT     DX,AX
    XCHG    [DI],AL

    Inc     SI
    ADD     DI,bx
    Dec     DI
    MOV     CX,[x]
    TEST    CX,7
    JNZ     @W1
    Dec     DI
  @W1:
    Dec     CX
    MOV     [x],CX
    Dec     [Height]
    JG      @Loop1

  @Weiter2:
    MOV     DX,3CEH
    MOV     AX,0001H
    OUT     DX,AX
    MOV     AX,0FF08H
    OUT     DX,AX
    POP     DS
  END;
{$F-}

  PROCEDURE OutText(x, y : INTEGER; TextString : STRING);
  VAR
    TS          : Graph.TextSettingsType;
    i, ic, Flag : BYTE;
    Proc        : DrawProc;
    Prop        : BOOLEAN;
    A           : CHAR;
  BEGIN
    Graph.GetTextSettings(TS);
    WITH TS DO BEGIN
      IF Horiz = CenterText THEN
        Dec(x, Graph.TextWidth(TextString) DIV 2)
      ELSE IF Horiz = RightText THEN
        Dec(x, Graph.TextWidth(TextString));
      IF Vert = CenterText THEN
        Dec(y, CurrentFont^.Height DIV 2)
      ELSE IF Vert = BottomText THEN
        Dec(y, CurrentFont^.Height);
    END;
    WITH CurrentTS DO BEGIN
      IF (Style AND tsKursive) > 0 THEN BEGIN
        Inc(x, CurrentFont^.BaseLine);
        Proc := OutKCharXY
      END ELSE
        Proc := OutCharXY;
      IF (y < 0) OR (y > GetMaxY) THEN Exit;
      Flag := 0;
      IF (Style AND tsBold) <> 0 THEN Flag := 1;
      IF (Style AND tsUnderline) <> 0 THEN Flag := Flag + 2;
      Prop := (Style AND tsPro) <> 0;
      IF (Style AND tsOutline) = 0 THEN
        FOR i := 1 TO Length(TextString) DO BEGIN
          A := TextString[i];
          IF Prop THEN Dec(x, CurrentFont^.b[Ord(A)]);
          IF (x >= 0) AND (x <= MaxX) THEN
            Proc(x, y, A, GetColor, Flag);
          Inc(x, 8);
        END
      ELSE
        FOR i := 1 TO Length(TextString) DO BEGIN
          IF (x >= 0) AND (x <= MaxX) THEN BEGIN
            Proc(x-1, y,   TextString[i], OC, Flag);
            Proc(x-1, y-1, TextString[i], OC, Flag);
            Proc(x  , y-1, TextString[i], OC,Flag);
            Proc(x+1, y-1, TextString[i], OC,Flag);
            Proc(x+1, y,   TextString[i], OC,Flag);
            Proc(x+1, y+1, TextString[i], OC,Flag);
            Proc(x  , y+1, TextString[i], OC,Flag);
            Proc(x-1, y+1, TextString[i], OC,Flag);
            Proc(x,   y,   TextString[i], GetColor, Flag);
          END;
          Inc(x,8);
        END;
    END;
  END;

  FUNCTION TextHeight : BYTE;
  BEGIN
    TextHeight := CurrentFont^.Height;
  END;

  FUNCTION TextWidth(TextString : STRING) : WORD;
  VAR
    L : WORD;
    i : BYTE;
  BEGIN
    IF (CurrentTS.Style AND tsPro) = 0 THEN
                             (* Keine Proportionalschrift *)
      TextWidth := Length(TextString) SHL 3
                             (* dann Breite := Lnge * 8  *)
    ELSE BEGIN
      L := 0;
      FOR i := 1 TO Length(TextString) DO BEGIN
          (* sonst die Breite jedes Buchstabens ermitteln *)
        Inc(L, 8);
        Dec(L, CurrentFont^.b[Ord(TextString[i])]);
      END;
      TextWidth := L;
    END;
  END;

END.
(* ====================================================== *)
(*                  Ende von VGAFONT.PAS                  *)

