// name: winver.c
// fxn:  Das Modul enthaelt ein Programm zur Feststellung der
//       aktuellen Windows-Version
//       Es ist sowohl mit dem 16bit VC++ als auch mit dem 
//       32bit VC++ zu bersetzen.
//       Die 16bit Version liefert:
//       - Betrieb unter WOW (NT)
//       - Betrieb unter "native" windows 16 bit
//       - Vorhandensein und Version von Win32s
//       Die 32bit Version liefert:
//       - Betrieb unter Win32s
//       - Betrieb unter Win32
//
// Compiler-Optionen:
//         16bit VC++: /nologo /W3 /AL /O1 /D "NDEBUG" /GA 
//         32bit VC++: /nologo /W3 /YX /O2 /D "_X86_" 
//                     /D "NDEBUG" /D "_WINDOWS" /FR /ML /Fp"WINVER.PCH"
// 
// Linker-Optionen:
//         16bit VC++: /NOLOGO /LIB:"oldnames" /LIB:"libw" /LIB:"llibcew" /NOD 
//                     /PACKC:61440 /STACK:10240 /ALIGN:16 /ONERROR:NOEXE 
//                     /LIB:"commdlg.lib" /LIB:"olecli.lib" 
//                     /LIB:"olesvr.lib" /LIB:"shell.lib" 
//         32bit VC++: /NOLOGO /SUBSYSTEM:windows user32.lib 
//                     gdi32.lib winspool.lib comdlg32.lib 
//                     advapi32.lib olecli32.lib olesvr32.lib shell32.lib
//          
// update: 1.00 5.09.93 tw          
//         1.01 7.09.93 tw kleinere Fehler im buffering beseitigt
//         1.02 6.11.93 tw win32s info hinzugefgt, 32bit Version hinzugefgt

#include <windows.h>    
#include <io.h>         
// #define         PROG_NAME       "Windows Version"

// Flag fuer WOW 
#define WF_WINNT   0x4000          
// Bit fuer Win32s  
#define WV_WIN32S  0x80000000

const char * pszW32SysDllName = "W32SYS.DLL";

// Sturktur zur Aufnahme der win32s information
typedef struct
{
   BYTE verMajor;
   BYTE verMinor;
   WORD idBuild;
   BOOL fDebug;
} WIN32SINFO;

int PASCAL 
WinMain( 
   HINSTANCE hInstance, 
   HINSTANCE hPrevInstance,
   LPSTR lpszCmdLine, 
   int nCmdShow )
{
   DWORD dwWinVer;      // Windows Version
   char * pszAppName;   
   #ifndef _WIN32
   DWORD dwWinFlags;    // Windows Version Flags
   char  szWin32sMsgBuf[512];
   #endif
   
   char szMsgBuf[1024];

   dwWinVer = GetVersion(); // Version ermitteln        
   
   // 32bit Teil
   #ifdef _WIN32     
   pszAppName = "WinVer 32bit Version";
   if( dwWinVer & WV_WIN32S )
   {
      wsprintf( szMsgBuf, 
                "Systemumgebung: Win32s - 16Bit Windows Version: %d.%02d.",
                LOBYTE( LOWORD( dwWinVer)),
                HIBYTE( LOWORD( dwWinVer)));
   }
   else
   {
      wsprintf( szMsgBuf, 
                "Systemumgebung: Windows NT - Version: %d.%02d (Build-ID: %d)",
                 LOBYTE( LOWORD( dwWinVer)),
                 HIBYTE( LOWORD( dwWinVer)),
                 HIWORD( dwWinVer) & 0x7FFF);
   }
   #else // ende des 32bit teils und begin des 16bit Teils

   pszAppName = "WinVer 16bit Version";
   
   dwWinFlags = GetWinFlags();
   if( dwWinFlags & WF_WINNT )
   {
      wsprintf( szMsgBuf, 
                "Systemumgebung: 16bit Windows on Windows - Window NT Version %d.%02d",
                LOBYTE( LOWORD( dwWinVer)),
                HIBYTE( LOWORD( dwWinVer)));
   }
   else
   {
      FARPROC     lpfnWin32sInfo;
      HANDLE      hWin32s;
      WIN32SINFO  cbWin32sInfo;

      SetErrorMode( SEM_NOOPENFILEERRORBOX ); 
      hWin32s = LoadLibrary( pszW32SysDllName );
      if( hWin32s > HINSTANCE_ERROR )
      {
         lpfnWin32sInfo = GetProcAddress( hWin32s, "GETWIN32SINFO" );
         if( lpfnWin32sInfo )
         {
            (*lpfnWin32sInfo)( &cbWin32sInfo );
            wsprintf( szWin32sMsgBuf,
                      "Version von Win32s: %d.%d (Build: %d)",            
                      cbWin32sInfo.verMajor,
                      cbWin32sInfo.verMinor,
                      cbWin32sInfo.idBuild);
         }
         else
         {
            wsprintf( szWin32sMsgBuf,
                      "Version von Win32s: 1.0" );
         }                      
         
         FreeLibrary( hWin32s );
      }
      else
      {
         wsprintf( szWin32sMsgBuf, "Win32s nicht vorhanden" );
      }         
       
      wsprintf( szMsgBuf, 
                "Systemumgebung: 16bit Windows - Version: %d.%02d\n%s",
                LOBYTE( LOWORD( dwWinVer)),
                HIBYTE( LOWORD( dwWinVer)),
                szWin32sMsgBuf);
   }

   #endif // #ifdef _WIN32
   
   MessageBox( NULL,
               szMsgBuf,
               pszAppName,
               MB_OK);
   return FALSE;               

}
   
   
