(* ====================================================== *)
(*                          XREN.MOD                      *)
(*                        Version 1.01                    *)
(*              (C) 1993 Jrg Braun & DMV-Verlag          *)
(*    Umbenennen und Verschieben von Verzeichnissen und   *)
(*     Dateien auf einem Laufwerk via DOS-Funktion 56h    *)
(* ------------------------------------------------------ *)
(*                Compiler: Fitted Modula 2.0             *)
(* ====================================================== *)
(* $L- *) (* $A+ *) (* $S- *) (* $T- *) (* $R- *)
MODULE XRen;

FROM Strings          IMPORT Append, Assign, Length, 
			     Pos, CompareStr;
FROM NumberConversion IMPORT NumToString;
FROM Directories      IMPORT DirEntry, GetFirstDir, 
			     NORMAL, READONLY, HIDDEN, 
			     SYSTEM, DIRECTORY, ARCHIVE;
FROM InOut            IMPORT WriteLn, WriteString;
FROM SYSTEM           IMPORT SEG, OFS, ADR;
FROM System           IMPORT DOSVersion, XTrap, GetArg, 
			     Terminate, AX, BX, CX, DX, SI, 
			     DI, BP, DS, ES, FLAGS, 
			     carryFlag;
IMPORT ASCII;

CONST
  NOERROR     = 0;        (* Ausfhrung war fehlerfrei    *)
  WRONGDOS    = 0FFF0H;   (* Es wird DOS ab 3.x bentigt  *)
  PARAMERROR  = 0FFF1H;   (* Kommandozeilen-Fehler        *)
  NOTFOUND    = 0FFF2H;   (* Quelldatei nicht vorhanden   *)
  SAMENAMES   = 0FFF3H;   (* Quelle = Ziel                *)

TYPE
  PathStr = ARRAY[0..79] OF CHAR;

(* ------------------------------------------------------ *)

PROCEDURE DosMessage(ErrNum: CARDINAL;
		     Source, Target: PathStr): CARDINAL;
(* Smtliche Ausgaben des Programms laufen ber diese     *)
(* Prozedur.                                              *)
VAR
  NewLine : ARRAY[0..1] OF CHAR;
  NumStr  : ARRAY[0..6] OF CHAR;
  MsgStr  : ARRAY[0..255] OF CHAR;
BEGIN
  NewLine[0] :=  ASCII.cr; (* 15C; *)
  NewLine[1] :=  ASCII.lf; (* 12C; *)
  MsgStr     := "Fehler: ";
  CASE ErrNum OF
    NOERROR:     (* Ausfhrung fehlerfrei: Erfolgsmeldung *)
       MsgStr := "Eintrag ";
       Append(MsgStr, Source );
       Append(MsgStr, " nach ");
       Append(MsgStr, Target);
       Append(MsgStr, " umbenannt.");
	      (* Liste der erweiterten DOS-Fehler-Nummern *)
 |  1: Append(MsgStr, "Unerlaubte Funktion");
 |  2: Append(MsgStr, "Datei nicht gefunden");
 |  3: Append(MsgStr, "Verzeichnis nicht gefunden");
 |  5: Append(MsgStr, "Zugriff verweigert");
 |  6: Append(MsgStr, "Ungltiger Handle");
 |  8: Append(MsgStr, "Zu wenig freier Speicher");
 | 15: Append(MsgStr, "Ungltiges Laufwerk");
 | 16: Append(MsgStr, "Versuch, das aktive Verzeichnis");
       Append(MsgStr, " zu lschen");
 | 18: Append(MsgStr, "Keine weiteren Dateien");
 | 19: Append(MsgStr, "Diskette ist schreibgeschtzt");
 | 20: Append(MsgStr, "Falsche Disketteneinheit");
 | 21: Append(MsgStr, "Laufwerk nicht bereit");
 | 23: Append(MsgStr, "CRC-Fehler");
 | 25: Append(MsgStr, "Suchfehler");
 | 27: Append(MsgStr, "Sektor nicht gefunden");
 | 31: Append(MsgStr, "Allgemeiner Fehler");
			    (* Benutzerdefinierte Fehler: *)
 |  WRONGDOS:
       Append(MsgStr, "Falsche DOS-Version");
       Append(MsgStr, NewLine);
       Append(MsgStr, "XREN bentigt DOS 3.x oder hher");
 |  PARAMERROR:
       Assign(NewLine, MsgStr);
       MsgStr := "XREN Version 1.0";
       Append(MsgStr, NewLine);
       Append(MsgStr, "Aufruf: XREN Quelle Ziel");
       Append(MsgStr, NewLine);
       Append(MsgStr, NewLine);
       Append(MsgStr, "XREN benennt Verzeichnisse und");
       Append(MsgStr, " Dateien um und verschiebt");
       Append(MsgStr, NewLine);
       Append(MsgStr, "sie bei Angabe zweier Pfade.");
       Append(MsgStr, NewLine);
       Append(MsgStr, "Die Wildcards * und  ? ");
       Append(MsgStr, "drfen nicht angegeben werden.");
       Append(MsgStr, NewLine);
       Append(MsgStr, "Verschieben ber verschiedene ");
       Append(MsgStr, "Laufwerke ist nicht mglich");
 |  SAMENAMES:
       Append(MsgStr, "Gleicher Quell- und Zielnamen: ");
       Append(MsgStr, Source);
       Append(MsgStr, " = ");
       Append(MsgStr, Target);
       Append(MsgStr,  "");
 |  NOTFOUND:
       Append(MsgStr, "Dateieintrag ");
       Append(MsgStr, Source);
       Append(MsgStr, " nicht gefunden");
  ELSE
    NumToString(ErrNum, 10, NumStr, 4);
    Append(MsgStr, "Unerwarteter Fehler (Nummer ");
    Append(MsgStr, NumStr);
    Append(MsgStr, ")");
  END;
  IF ErrNum <> NOERROR THEN 
    Append(MsgStr, "!"); 
  END;
  Append(MsgStr, NewLine);
  WriteString(MsgStr); (* <- nur hier erfolgt die Ausgabe *)
  RETURN ErrNum;
END DosMessage;

(* ------------------------------------------------------ *)

PROCEDURE XRename(Source, Target: PathStr): CARDINAL;
VAR
  SourceAddr, 
  TargetAddr : POINTER TO CHAR;
BEGIN
  SourceAddr := ADR(Source); 
  TargetAddr := ADR(Target);
  DS         := SourceAddr.SEG;
  DX         := SourceAddr.OFS; (* DS:DX <= Quelldateiname*)
  ES         := TargetAddr.SEG;
  DI         := TargetAddr.OFS; (* ES:DI <= Zieldateiname *)
  AX         := 5600H;          (* AH    <= 56h           *)
  XTrap(21H);
  IF FLAGS = {carryFlag} THEN
    IF AX = 3 THEN 
      RETURN 5;       (* Path not found --> Access denied *)
    END;
    RETURN AX;(* Fehler in AX falls carryFlag gesetzt ist *)
  END;        
  RETURN NOERROR;
END XRename;

(* ------------------------------------------------------ *)

PROCEDURE StrUpper(VAR s: ARRAY OF CHAR);
VAR
  i : CARDINAL;
BEGIN
  FOR i := 1 TO Length(s) DO 
    s[i] := CAP(s[i]);
  END;  
END StrUpper;

(* ------------------------------------------------------ *)

PROCEDURE StrLower(VAR s: ARRAY OF CHAR);
VAR
  i : CARDINAL;
BEGIN
  FOR i := 1 TO Length(s) DO
    IF (s[i] > "A") THEN 
      IF (s[i] < "Z") THEN 
	INC(s[i], 32);
      END;  
    END;
  END;  
END StrLower;

(* ====================================================== *)

VAR
  Entry               : DirEntry;
  Source, Target      : PathStr;
  ErrNum, ParamLength : CARDINAL;
  ok                  : BOOLEAN;
BEGIN
  ErrNum := NOERROR;
  IF DOSVersion < 300 THEN 
    ErrNum := DosMessage(WRONGDOS, "","");
  ELSE
    GetArg(Source, ParamLength);
    GetArg(Target, ParamLength);
    IF Length(Target) = 0 THEN
      ErrNum := DosMessage(PARAMERROR, "","")
    ELSE
      StrUpper(Source);
      StrUpper(Target);
      IF (Pos("?",  Source) <= Length(Source)) OR
	 (Pos("?",  Target) <= Length(Target)) OR
	 (Pos("/H", Source) <= Length(Source)) OR
	 (Pos("/H", Target) <= Length(Target)) OR
	 (Pos("*",  Source) <= Length(Source)) OR
	 (Pos("*",  Target) <= Length(Target)) THEN
	ErrNum := DosMessage(PARAMERROR, "", "")
      ELSE 
	IF CompareStr(Source, Target) = 0 THEN
	  StrLower(Source); 
	  StrLower(Target);
	  ErrNum := DosMessage(SAMENAMES, Source, Target);
	ELSE
	  GetFirstDir(Source, NORMAL + READONLY + HIDDEN +
			      SYSTEM + DIRECTORY + ARCHIVE,
			      Entry, ok);
	  IF ok THEN
	    StrLower(Source);
	    StrLower(Target);
	    ErrNum := DosMessage(XRename(Source, Target),
				 Source, Target);
	  ELSE
	    StrLower(Source);
	    ErrNum := DosMessage(NOTFOUND, Source, "");
	  END;
	END;
      END;
    END;
  END;
  Terminate(ErrNum);
END XRen.

(*========================================================*)
(*                    Ende von XREN.MOD                   *)
