(* ====================================================== *)
(*                          XREN.PAS                      *)
(*                        Version 1.01                    *)
(*              (C) 1993 Jrg Braun & DMV-Verlag          *)
(*    Umbenennen und Verschieben von Verzeichnissen und   *)
(*     Dateien auf einem Laufwerk via DOS-Funktion 56h    *)
(* Diese Version von XREN verwendet die Standard-Unit DOS *)
(* ------------------------------------------------------ *)
(* Compiler: Turbo Pascal ab 5.0, Stony Brook Pascal+ 6.0 *)
(* ====================================================== *)
{.$DEFINE DEBUG}

{$A+,B-,E-,F+,I-,O-,T-,V-,M $FFF1,0,$1000}
{$IFDEF DEBUG}
  {$D+,L+,S+,R+,Q+}
{$ELSE}
  {$D-,S-,R-,Q-}
{$ENDIF}

PROGRAM XRen;

USES Dos, PStrings;

CONST
  NOERROR     = 0;        (* Ausfhrung war fehlerfrei    *)
  WRONGDOS    = $FFF0;    (* Es wird DOS ab 3.x bentigt  *)
  PARAMERROR  = $FFF1;    (* Kommandozeilen-Fehler        *)
  NOTFOUND    = $FFF2;    (* Quelldatei nicht vorhanden   *)
  SAMENAMES   = $FFF3;    (* Quelle = Ziel                *)

FUNCTION DosMessage(ErrNum: WORD;
                    Source, Target: PathStr): WORD;
(* Smtliche Ausgaben des Programms laufen ber diese     *)
(* Prozedur.                                              *)
VAR
  NumStr : STRING[6];
  MsgStr : STRING;
BEGIN
  MsgStr := 'Fehler: ';
  CASE ErrNum OF
    NOERROR:     (* Ausfhrung fehlerfrei: Erfolgsmeldung *)
       MsgStr := 'Eintrag ' + Source + ' nach ' + Target
               +  ' umbenannt.';
              (* Liste der erweiterten DOS-Fehler-Nummern *)
    1: MsgStr := MsgStr + 'Unerlaubte Funktion';
    2: MsgStr := MsgStr + 'Datei nicht gefunden';
    3: MsgStr := MsgStr + 'Verzeichnis nicht gefunden';
    5: MsgStr := MsgStr + 'Zugriff verweigert';
    6: MsgStr := MsgStr + 'Ungltiger Handle';
    8: MsgStr := MsgStr + 'Zu wenig freier Speicher';
   15: MsgStr := MsgStr + 'Ungltiges Laufwerk';
   16: MsgStr := MsgStr + 'Versuch, das aktive Verzeichnis'
                        + ' zu lschen';
   18: MsgStr := MsgStr + 'Keine weiteren Dateien';
   19: MsgStr := MsgStr + 'Diskette ist schreibgeschtzt';
   20: MsgStr := MsgStr + 'Falsche Disketteneinheit';
   21: MsgStr := MsgStr + 'Laufwerk nicht bereit';
   23: MsgStr := MsgStr + 'CRC-Fehler';
   25: MsgStr := MsgStr + 'Suchfehler';
   27: MsgStr := MsgStr + 'Sektor nicht gefunden';
   31: MsgStr := MsgStr + 'Allgemeiner Fehler';
                            (* Benutzerdefinierte Fehler: *)
    WRONGDOS:
       MsgStr := MsgStr + 'Falsche DOS-Version'^M^J +
                 'XREN bentigt DOS 3.x oder hher';
    PARAMERROR:
       MsgStr :=  ^M^J'XREN Version 1.0'^M^J
               +  'Aufruf: XREN Quelle Ziel'^M^J
               +  ^J'XREN benennt Verzeichnisse und'
               +  ' Dateien um und verschiebt'
               +  ^M^J'sie bei Angabe zweier Pfade.'
               +  ^M^J'Die Wildcards * und  ? '
               +  'drfen nicht angegeben werden.'^M
               +  ^J'Verschieben ber verschiedene '
               +  'Laufwerke ist nicht mglich';
    SAMENAMES:
       MsgStr :=  MsgStr
               +  'Gleicher Quell- und Zielnamen: '
               + Source +  ' = ' +  Target +  '';
    NOTFOUND:
       MsgStr :=  MsgStr + 'Dateieintrag ' + Source
               +  ' nicht gefunden';
  ELSE
    Str(ErrNum, NumStr);
    MsgStr := MsgStr + 'Unerwarteter Fehler (Nummer ' +
              NumStr + ')';
  END;
  IF ErrNum <> NOERROR THEN MsgStr := MsgStr + '!';
  WriteLn(MsgStr);    (* <-- nur hier erfolgt die Ausgabe *)
  DosMessage := ErrNum;
END;

FUNCTION XRename(Source, Target: PathStr): WORD;
VAR
  Regs : Registers;
  s, t : PathStr;
BEGIN
  s := Source + #0;
  t := Target + #0;
  XRename := NOERROR;
  WITH Regs DO BEGIN         (* DOS-Funktion 56h: Rename  *)
    DS := Seg(s[1]);
    DX := Ofs(s[1]);         (* DS:DX  := Quelldateiname  *)
    ES := Seg(t[1]);
    DI := Ofs(t[1]);         (* ES:DI  := Zieldateiname   *)
    AH := $56;               (* AH     := 56h             *)
    MsDos(Regs);
    IF Odd(Flags) THEN BEGIN
      IF AX = 3 THEN AX := 5;
                      (* Path not found --> Access denied *)
      XRename := AX;
    END;
  END;        (* Fehler := AX falls CarryFlag gesetzt ist *)
END;

VAR
  sr             : SearchRec;
  Source, Target : PathStr;
  ErrNum         : WORD;
BEGIN
  ErrNum := NOERROR;
  IF Lo(DOSVersion) < 3 THEN BEGIN
    ErrNum := DosMessage(WRONGDOS, '','');
  END ELSE BEGIN
    IF ParamCount < 2 THEN
      ErrNum := DosMessage(PARAMERROR, '','')
    ELSE BEGIN
      Source := StrUpper(ParamStr(1));
      Target := StrUpper(ParamStr(2));
      IF (Boolean(Pos('?',  Source))) OR
         (Boolean(Pos('?',  Target))) OR
         (Boolean(Pos('/H', Source))) OR
         (Boolean(Pos('/H', Target))) OR
         (Boolean(Pos('*',  Source))) OR
         (Boolean(Pos('*',  Target))) THEN
        ErrNum := DosMessage(PARAMERROR, '', '')
      ELSE BEGIN
        IF Source = Target THEN
          ErrNum := DosMessage(SAMENAMES, StrLower(Source),
                                          StrLower(Target))
        ELSE BEGIN
          FindFirst(Source, AnyFile - VolumeID, sr);
          IF DOSError = 0 THEN
            ErrNum := DosMessage(XRename(Source, Target),
                                 StrLower(Source),
                                 StrLower(Target))
          ELSE
            ErrNum := DosMessage(NOTFOUND,
                                 StrLower(Source),
                                 '');
        END;
      END;
    END;
  END;
  Halt(ErrNum);
END.

(*========================================================*)
(*                    Ende von XREN.PAS                   *)
